#line 1 "ad_x86.cpp"
//
// Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//

// Machine Generated File.  Do Not Edit!

#include "precompiled.hpp"
#include "adfiles/adGlobals_x86.hpp"
#include "adfiles/ad_x86.hpp"
#include "memory/allocation.inline.hpp"
#include "code/codeCache.hpp"
#include "code/compiledIC.hpp"
#include "code/nativeInst.hpp"
#include "code/vmreg.inline.hpp"
#include "gc/shared/collectedHeap.inline.hpp"
#include "oops/compiledICHolder.hpp"
#include "oops/compressedOops.hpp"
#include "oops/markWord.hpp"
#include "oops/method.hpp"
#include "oops/oop.inline.hpp"
#include "opto/c2_MacroAssembler.hpp"
#include "opto/cfgnode.hpp"
#include "opto/intrinsicnode.hpp"
#include "opto/locknode.hpp"
#include "opto/opcodes.hpp"
#include "opto/regalloc.hpp"
#include "opto/regmask.hpp"
#include "opto/runtime.hpp"
#include "runtime/biasedLocking.hpp"
#include "runtime/safepointMechanism.hpp"
#include "runtime/sharedRuntime.hpp"
#include "runtime/stubRoutines.hpp"
#include "utilities/growableArray.hpp"
#include "utilities/powerOfTwo.hpp"

//SourceForm

#line 250 "src/hotspot/cpu/x86/x86_32.ad"

#define   RELOC_IMM32    Assembler::imm_operand
#define   RELOC_DISP32   Assembler::disp32_operand

#define __ _masm.

// How to find the high register of a Long pair, given the low register
#define   HIGH_FROM_LOW(x) ((x)+2)

// These masks are used to provide 128-bit aligned bitmasks to the XMM
// instructions, to allow sign-masking or sign-bit flipping.  They allow
// fast versions of NegF/NegD and AbsF/AbsD.

// Note: 'double' and 'long long' have 32-bits alignment on x86.
static jlong* double_quadword(jlong *adr, jlong lo, jlong hi) {
  // Use the expression (adr)&(~0xF) to provide 128-bits aligned address
  // of 128-bits operands for SSE instructions.
  jlong *operand = (jlong*)(((uintptr_t)adr)&((uintptr_t)(~0xF)));
  // Store the value to a 128-bits operand.
  operand[0] = lo;
  operand[1] = hi;
  return operand;
}

// Buffer for 128-bits masks used by SSE instructions.
static jlong fp_signmask_pool[(4+1)*2]; // 4*128bits(data) + 128bits(alignment)

// Static initialization during VM startup.
static jlong *float_signmask_pool  = double_quadword(&fp_signmask_pool[1*2], CONST64(0x7FFFFFFF7FFFFFFF), CONST64(0x7FFFFFFF7FFFFFFF));
static jlong *double_signmask_pool = double_quadword(&fp_signmask_pool[2*2], CONST64(0x7FFFFFFFFFFFFFFF), CONST64(0x7FFFFFFFFFFFFFFF));
static jlong *float_signflip_pool  = double_quadword(&fp_signmask_pool[3*2], CONST64(0x8000000080000000), CONST64(0x8000000080000000));
static jlong *double_signflip_pool = double_quadword(&fp_signmask_pool[4*2], CONST64(0x8000000000000000), CONST64(0x8000000000000000));

// Offset hacking within calls.
static int pre_call_resets_size() {
  int size = 0;
  Compile* C = Compile::current();
  if (C->in_24_bit_fp_mode()) {
    size += 6; // fldcw
  }
  if (VM_Version::supports_vzeroupper()) {
    size += 3; // vzeroupper
  }
  return size;
}

// !!!!! Special hack to get all type of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.
int MachCallStaticJavaNode::ret_addr_offset() {
  return 5 + pre_call_resets_size();  // 5 bytes from start of call to where return address points
}

int MachCallDynamicJavaNode::ret_addr_offset() {
  return 10 + pre_call_resets_size();  // 10 bytes from start of call to where return address points
}

static int sizeof_FFree_Float_Stack_All = -1;

int MachCallRuntimeNode::ret_addr_offset() {
  assert(sizeof_FFree_Float_Stack_All != -1, "must have been emitted already");
  return 5 + pre_call_resets_size() + (_leaf_no_fp ? 0 : sizeof_FFree_Float_Stack_All);
}

int MachCallNativeNode::ret_addr_offset() {
  ShouldNotCallThis();
  return -1;
}

//
// Compute padding required for nodes which need alignment
//

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaDirectNode::compute_padding(int current_offset) const {
  current_offset += pre_call_resets_size();  // skip fldcw, if any
  current_offset += 1;      // skip call opcode byte
  return align_up(current_offset, alignment_required()) - current_offset;
}

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallDynamicJavaDirectNode::compute_padding(int current_offset) const {
  current_offset += pre_call_resets_size();  // skip fldcw, if any
  current_offset += 5;      // skip MOV instruction
  current_offset += 1;      // skip call opcode byte
  return align_up(current_offset, alignment_required()) - current_offset;
}

// EMIT_RM()
void emit_rm(CodeBuffer &cbuf, int f1, int f2, int f3) {
  unsigned char c = (unsigned char)((f1 << 6) | (f2 << 3) | f3);
  cbuf.insts()->emit_int8(c);
}

// EMIT_CC()
void emit_cc(CodeBuffer &cbuf, int f1, int f2) {
  unsigned char c = (unsigned char)( f1 | f2 );
  cbuf.insts()->emit_int8(c);
}

// EMIT_OPCODE()
void emit_opcode(CodeBuffer &cbuf, int code) {
  cbuf.insts()->emit_int8((unsigned char) code);
}

// EMIT_OPCODE() w/ relocation information
void emit_opcode(CodeBuffer &cbuf, int code, relocInfo::relocType reloc, int offset = 0) {
  cbuf.relocate(cbuf.insts_mark() + offset, reloc);
  emit_opcode(cbuf, code);
}

// EMIT_D8()
void emit_d8(CodeBuffer &cbuf, int d8) {
  cbuf.insts()->emit_int8((unsigned char) d8);
}

// EMIT_D16()
void emit_d16(CodeBuffer &cbuf, int d16) {
  cbuf.insts()->emit_int16(d16);
}

// EMIT_D32()
void emit_d32(CodeBuffer &cbuf, int d32) {
  cbuf.insts()->emit_int32(d32);
}

// emit 32 bit value and construct relocation entry from relocInfo::relocType
void emit_d32_reloc(CodeBuffer &cbuf, int d32, relocInfo::relocType reloc,
        int format) {
  cbuf.relocate(cbuf.insts_mark(), reloc, format);
  cbuf.insts()->emit_int32(d32);
}

// emit 32 bit value and construct relocation entry from RelocationHolder
void emit_d32_reloc(CodeBuffer &cbuf, int d32, RelocationHolder const& rspec,
        int format) {
#ifdef ASSERT
  if (rspec.reloc()->type() == relocInfo::oop_type && d32 != 0 && d32 != (int)Universe::non_oop_word()) {
    assert(oopDesc::is_oop(cast_to_oop(d32)), "cannot embed broken oops in code");
  }
#endif
  cbuf.relocate(cbuf.insts_mark(), rspec, format);
  cbuf.insts()->emit_int32(d32);
}

// Access stack slot for load or store
void store_to_stackslot(CodeBuffer &cbuf, int opcode, int rm_field, int disp) {
  emit_opcode( cbuf, opcode );               // (e.g., FILD   [ESP+src])
  if( -128 <= disp && disp <= 127 ) {
    emit_rm( cbuf, 0x01, rm_field, ESP_enc );  // R/M byte
    emit_rm( cbuf, 0x00, ESP_enc, ESP_enc);    // SIB byte
    emit_d8 (cbuf, disp);     // Displacement  // R/M byte
  } else {
    emit_rm( cbuf, 0x02, rm_field, ESP_enc );  // R/M byte
    emit_rm( cbuf, 0x00, ESP_enc, ESP_enc);    // SIB byte
    emit_d32(cbuf, disp);     // Displacement  // R/M byte
  }
}

   // rRegI ereg, memory mem) %{    // emit_reg_mem
void encode_RegMem( CodeBuffer &cbuf, int reg_encoding, int base, int index, int scale, int displace, relocInfo::relocType disp_reloc ) {
  // There is no index & no scale, use form without SIB byte
  if ((index == 0x4) &&
      (scale == 0) && (base != ESP_enc)) {
    // If no displacement, mode is 0x0; unless base is [EBP]
    if ( (displace == 0) && (base != EBP_enc) ) {
      emit_rm(cbuf, 0x0, reg_encoding, base);
    }
    else {                    // If 8-bit displacement, mode 0x1
      if ((displace >= -128) && (displace <= 127)
          && (disp_reloc == relocInfo::none) ) {
        emit_rm(cbuf, 0x1, reg_encoding, base);
        emit_d8(cbuf, displace);
      }
      else {                  // If 32-bit displacement
        if (base == -1) { // Special flag for absolute address
          emit_rm(cbuf, 0x0, reg_encoding, 0x5);
          // (manual lies; no SIB needed here)
          if ( disp_reloc != relocInfo::none ) {
            emit_d32_reloc(cbuf, displace, disp_reloc, 1);
          } else {
            emit_d32      (cbuf, displace);
          }
        }
        else {                // Normal base + offset
          emit_rm(cbuf, 0x2, reg_encoding, base);
          if ( disp_reloc != relocInfo::none ) {
            emit_d32_reloc(cbuf, displace, disp_reloc, 1);
          } else {
            emit_d32      (cbuf, displace);
          }
        }
      }
    }
  }
  else {                      // Else, encode with the SIB byte
    // If no displacement, mode is 0x0; unless base is [EBP]
    if (displace == 0 && (base != EBP_enc)) {  // If no displacement
      emit_rm(cbuf, 0x0, reg_encoding, 0x4);
      emit_rm(cbuf, scale, index, base);
    }
    else {                    // If 8-bit displacement, mode 0x1
      if ((displace >= -128) && (displace <= 127)
          && (disp_reloc == relocInfo::none) ) {
        emit_rm(cbuf, 0x1, reg_encoding, 0x4);
        emit_rm(cbuf, scale, index, base);
        emit_d8(cbuf, displace);
      }
      else {                  // If 32-bit displacement
        if (base == 0x04 ) {
          emit_rm(cbuf, 0x2, reg_encoding, 0x4);
          emit_rm(cbuf, scale, index, 0x04);
        } else {
          emit_rm(cbuf, 0x2, reg_encoding, 0x4);
          emit_rm(cbuf, scale, index, base);
        }
        if ( disp_reloc != relocInfo::none ) {
          emit_d32_reloc(cbuf, displace, disp_reloc, 1);
        } else {
          emit_d32      (cbuf, displace);
        }
      }
    }
  }
}


void encode_Copy( CodeBuffer &cbuf, int dst_encoding, int src_encoding ) {
  if( dst_encoding == src_encoding ) {
    // reg-reg copy, use an empty encoding
  } else {
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, dst_encoding, src_encoding );
  }
}

void emit_cmpfp_fixup(MacroAssembler& _masm) {
  Label exit;
  __ jccb(Assembler::noParity, exit);
  __ pushf();
  //
  // comiss/ucomiss instructions set ZF,PF,CF flags and
  // zero OF,AF,SF for NaN values.
  // Fixup flags by zeroing ZF,PF so that compare of NaN
  // values returns 'less than' result (CF is set).
  // Leave the rest of flags unchanged.
  //
  //    7 6 5 4 3 2 1 0
  //   |S|Z|r|A|r|P|r|C|  (r - reserved bit)
  //    0 0 1 0 1 0 1 1   (0x2B)
  //
  __ andl(Address(rsp, 0), 0xffffff2b);
  __ popf();
  __ bind(exit);
}

void emit_cmpfp3(MacroAssembler& _masm, Register dst) {
  Label done;
  __ movl(dst, -1);
  __ jcc(Assembler::parity, done);
  __ jcc(Assembler::below, done);
  __ setb(Assembler::notEqual, dst);
  __ movzbl(dst, dst);
  __ bind(done);
}


//=============================================================================
const RegMask& MachConstantBaseNode::_out_RegMask = RegMask::Empty;

int ConstantTable::calculate_table_base_offset() const {
  return 0;  // absolute addressing, no offset
}

bool MachConstantBaseNode::requires_postalloc_expand() const { return false; }
void MachConstantBaseNode::postalloc_expand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  ShouldNotReachHere();
}

void MachConstantBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  // Empty encoding
}

uint MachConstantBaseNode::size(PhaseRegAlloc* ra_) const {
  return 0;
}

#ifndef PRODUCT
void MachConstantBaseNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  st->print("# MachConstantBaseNode (empty encoding)");
}
#endif


//=============================================================================
#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  Compile* C = ra_->C;

  int framesize = C->output()->frame_size_in_bytes();
  int bangsize = C->output()->bang_size_in_bytes();
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove wordSize for return addr which is already pushed.
  framesize -= wordSize;

  if (C->output()->need_stack_bang(bangsize)) {
    framesize -= wordSize;
    st->print("# stack bang (%d bytes)", bangsize);
    st->print("\n\t");
    st->print("PUSH   EBP\t# Save EBP");
    if (PreserveFramePointer) {
      st->print("\n\t");
      st->print("MOV    EBP, ESP\t# Save the caller's SP into EBP");
    }
    if (framesize) {
      st->print("\n\t");
      st->print("SUB    ESP, #%d\t# Create frame",framesize);
    }
  } else {
    st->print("SUB    ESP, #%d\t# Create frame",framesize);
    st->print("\n\t");
    framesize -= wordSize;
    st->print("MOV    [ESP + #%d], EBP\t# Save EBP",framesize);
    if (PreserveFramePointer) {
      st->print("\n\t");
      st->print("MOV    EBP, ESP\t# Save the caller's SP into EBP");
      if (framesize > 0) {
        st->print("\n\t");
        st->print("ADD    EBP, #%d", framesize);
      }
    }
  }

  if (VerifyStackAtCalls) {
    st->print("\n\t");
    framesize -= wordSize;
    st->print("MOV    [ESP + #%d], 0xBADB100D\t# Majik cookie for stack depth check",framesize);
  }

  if( C->in_24_bit_fp_mode() ) {
    st->print("\n\t");
    st->print("FLDCW  \t# load 24 bit fpu control word");
  }
  if (UseSSE >= 2 && VerifyFPU) {
    st->print("\n\t");
    st->print("# verify FPU stack (must be clean on entry)");
  }

#ifdef ASSERT
  if (VerifyStackAtCalls) {
    st->print("\n\t");
    st->print("# stack alignment check");
  }
#endif
  st->cr();
}
#endif


void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  MacroAssembler _masm(&cbuf);

  int framesize = C->output()->frame_size_in_bytes();
  int bangsize = C->output()->bang_size_in_bytes();

  __ verified_entry(framesize, C->output()->need_stack_bang(bangsize)?bangsize:0, C->in_24_bit_fp_mode(), C->stub_function() != NULL);

  C->output()->set_frame_complete(cbuf.insts_size());

  if (C->has_mach_constant_base_node()) {
    // NOTE: We set the table base offset here because users might be
    // emitted before MachConstantBaseNode.
    ConstantTable& constant_table = C->output()->constant_table();
    constant_table.set_table_base_offset(constant_table.calculate_table_base_offset());
  }
}

uint MachPrologNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_); // too many variables; just compute it the hard way
}

int MachPrologNode::reloc() const {
  return 0; // a large enough number
}

//=============================================================================
#ifndef PRODUCT
void MachEpilogNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  Compile *C = ra_->C;
  int framesize = C->output()->frame_size_in_bytes();
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  if (C->max_vector_size() > 16) {
    st->print("VZEROUPPER");
    st->cr(); st->print("\t");
  }
  if (C->in_24_bit_fp_mode()) {
    st->print("FLDCW  standard control word");
    st->cr(); st->print("\t");
  }
  if (framesize) {
    st->print("ADD    ESP,%d\t# Destroy frame",framesize);
    st->cr(); st->print("\t");
  }
  st->print_cr("POPL   EBP"); st->print("\t");
  if (do_polling() && C->is_method_compilation()) {
    st->print("CMPL     rsp, poll_offset[thread]  \n\t"
              "JA       #safepoint_stub\t"
              "# Safepoint: poll for GC");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile *C = ra_->C;
  MacroAssembler _masm(&cbuf);

  if (C->max_vector_size() > 16) {
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    _masm.vzeroupper();
  }
  // If method set FPU control word, restore to standard control word
  if (C->in_24_bit_fp_mode()) {
    _masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
  }

  int framesize = C->output()->frame_size_in_bytes();
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  // Note that VerifyStackAtCalls' Majik cookie does not change the frame size popped here

  if (framesize >= 128) {
    emit_opcode(cbuf, 0x81); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d32(cbuf, framesize);
  } else if (framesize) {
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, framesize);
  }

  emit_opcode(cbuf, 0x58 | EBP_enc);

  if (StackReservedPages > 0 && C->has_reserved_stack_access()) {
    __ reserved_stack_check();
  }

  if (do_polling() && C->is_method_compilation()) {
    Register thread = as_Register(EBX_enc);
    MacroAssembler masm(&cbuf);
    __ get_thread(thread);
    Label dummy_label;
    Label* code_stub = &dummy_label;
    if (!C->output()->in_scratch_emit_size()) {
      code_stub = &C->output()->safepoint_poll_table()->add_safepoint(__ offset());
    }
    __ relocate(relocInfo::poll_return_type);
    __ safepoint_poll(*code_stub, thread, true /* at_return */, true /* in_nmethod */);
  }
}

uint MachEpilogNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachEpilogNode::reloc() const {
  return 0; // a large enough number
}

const Pipeline * MachEpilogNode::pipeline() const {
  return MachNode::pipeline_class();
}

//=============================================================================

enum RC { rc_bad, rc_int, rc_float, rc_xmm, rc_stack };
static enum RC rc_class( OptoReg::Name reg ) {

  if( !OptoReg::is_valid(reg)  ) return rc_bad;
  if (OptoReg::is_stack(reg)) return rc_stack;

  VMReg r = OptoReg::as_VMReg(reg);
  if (r->is_Register()) return rc_int;
  if (r->is_FloatRegister()) {
    assert(UseSSE < 2, "shouldn't be used in SSE2+ mode");
    return rc_float;
  }
  assert(r->is_XMMRegister(), "must be");
  return rc_xmm;
}

static int impl_helper( CodeBuffer *cbuf, bool do_size, bool is_load, int offset, int reg,
                        int opcode, const char *op_str, int size, outputStream* st ) {
  if( cbuf ) {
    emit_opcode  (*cbuf, opcode );
    encode_RegMem(*cbuf, Matcher::_regEncode[reg], ESP_enc, 0x4, 0, offset, relocInfo::none);
#ifndef PRODUCT
  } else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    if( opcode == 0x8B || opcode == 0x89 ) { // MOV
      if( is_load ) st->print("%s   %s,[ESP + #%d]",op_str,Matcher::regName[reg],offset);
      else          st->print("%s   [ESP + #%d],%s",op_str,offset,Matcher::regName[reg]);
    } else { // FLD, FST, PUSH, POP
      st->print("%s [ESP + #%d]",op_str,offset);
    }
#endif
  }
  int offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
  return size+3+offset_size;
}

// Helper for XMM registers.  Extra opcode bits, limited syntax.
static int impl_x_helper( CodeBuffer *cbuf, bool do_size, bool is_load,
                         int offset, int reg_lo, int reg_hi, int size, outputStream* st ) {
  int in_size_in_bits = Assembler::EVEX_32bit;
  int evex_encoding = 0;
  if (reg_lo+1 == reg_hi) {
    in_size_in_bits = Assembler::EVEX_64bit;
    evex_encoding = Assembler::VEX_W;
  }
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: Compressed displacemement is better than AVX on spill mem operations,
    //                          it maps more cases to single byte displacement
    _masm.set_managed();
    if (reg_lo+1 == reg_hi) { // double move?
      if (is_load) {
        __ movdbl(as_XMMRegister(Matcher::_regEncode[reg_lo]), Address(rsp, offset));
      } else {
        __ movdbl(Address(rsp, offset), as_XMMRegister(Matcher::_regEncode[reg_lo]));
      }
    } else {
      if (is_load) {
        __ movflt(as_XMMRegister(Matcher::_regEncode[reg_lo]), Address(rsp, offset));
      } else {
        __ movflt(Address(rsp, offset), as_XMMRegister(Matcher::_regEncode[reg_lo]));
      }
    }
#ifndef PRODUCT
  } else if (!do_size) {
    if (size != 0) st->print("\n\t");
    if (reg_lo+1 == reg_hi) { // double move?
      if (is_load) st->print("%s %s,[ESP + #%d]",
                              UseXmmLoadAndClearUpper ? "MOVSD " : "MOVLPD",
                              Matcher::regName[reg_lo], offset);
      else         st->print("MOVSD  [ESP + #%d],%s",
                              offset, Matcher::regName[reg_lo]);
    } else {
      if (is_load) st->print("MOVSS  %s,[ESP + #%d]",
                              Matcher::regName[reg_lo], offset);
      else         st->print("MOVSS  [ESP + #%d],%s",
                              offset, Matcher::regName[reg_lo]);
    }
#endif
  }
  bool is_single_byte = false;
  if ((UseAVX > 2) && (offset != 0)) {
    is_single_byte = Assembler::query_compressed_disp_byte(offset, true, 0, Assembler::EVEX_T1S, in_size_in_bits, evex_encoding);
  }
  int offset_size = 0;
  if (UseAVX > 2 ) {
    offset_size = (offset == 0) ? 0 : ((is_single_byte) ? 1 : 4);
  } else {
    offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
  }
  size += (UseAVX > 2) ? 2 : 0; // Need an additional two bytes for EVEX
  // VEX_2bytes prefix is used if UseAVX > 0, so it takes the same 2 bytes as SIMD prefix.
  return size+5+offset_size;
}


static int impl_movx_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                            int src_hi, int dst_hi, int size, outputStream* st ) {
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: logic complex between full EVEX, partial and AVX, manage EVEX spill code one way.
    _masm.set_managed();
    if (src_lo+1 == src_hi && dst_lo+1 == dst_hi) { // double move?
      __ movdbl(as_XMMRegister(Matcher::_regEncode[dst_lo]),
                as_XMMRegister(Matcher::_regEncode[src_lo]));
    } else {
      __ movflt(as_XMMRegister(Matcher::_regEncode[dst_lo]),
                as_XMMRegister(Matcher::_regEncode[src_lo]));
    }
#ifndef PRODUCT
  } else if (!do_size) {
    if (size != 0) st->print("\n\t");
    if (UseXmmRegToRegMoveAll) {//Use movaps,movapd to move between xmm registers
      if (src_lo+1 == src_hi && dst_lo+1 == dst_hi) { // double move?
        st->print("MOVAPD %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      } else {
        st->print("MOVAPS %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      }
    } else {
      if( src_lo+1 == src_hi && dst_lo+1 == dst_hi ) { // double move?
        st->print("MOVSD  %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      } else {
        st->print("MOVSS  %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      }
    }
#endif
  }
  // VEX_2bytes prefix is used if UseAVX > 0, and it takes the same 2 bytes as SIMD prefix.
  // Only MOVAPS SSE prefix uses 1 byte.  EVEX uses an additional 2 bytes.
  int sz = (UseAVX > 2) ? 6 : 4;
  if (!(src_lo+1 == src_hi && dst_lo+1 == dst_hi) &&
      UseXmmRegToRegMoveAll && (UseAVX == 0)) sz = 3;
  return size + sz;
}

static int impl_movgpr2x_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                            int src_hi, int dst_hi, int size, outputStream* st ) {
  // 32-bit
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: logic complex between full EVEX, partial and AVX, manage EVEX spill code one way.
    _masm.set_managed();
    __ movdl(as_XMMRegister(Matcher::_regEncode[dst_lo]),
             as_Register(Matcher::_regEncode[src_lo]));
#ifndef PRODUCT
  } else if (!do_size) {
    st->print("movdl   %s, %s\t# spill", Matcher::regName[dst_lo], Matcher::regName[src_lo]);
#endif
  }
  return (UseAVX> 2) ? 6 : 4;
}


static int impl_movx2gpr_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                                 int src_hi, int dst_hi, int size, outputStream* st ) {
  // 32-bit
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: logic complex between full EVEX, partial and AVX, manage EVEX spill code one way.
    _masm.set_managed();
    __ movdl(as_Register(Matcher::_regEncode[dst_lo]),
             as_XMMRegister(Matcher::_regEncode[src_lo]));
#ifndef PRODUCT
  } else if (!do_size) {
    st->print("movdl   %s, %s\t# spill", Matcher::regName[dst_lo], Matcher::regName[src_lo]);
#endif
  }
  return (UseAVX> 2) ? 6 : 4;
}

static int impl_mov_helper( CodeBuffer *cbuf, bool do_size, int src, int dst, int size, outputStream* st ) {
  if( cbuf ) {
    emit_opcode(*cbuf, 0x8B );
    emit_rm    (*cbuf, 0x3, Matcher::_regEncode[dst], Matcher::_regEncode[src] );
#ifndef PRODUCT
  } else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    st->print("MOV    %s,%s",Matcher::regName[dst],Matcher::regName[src]);
#endif
  }
  return size+2;
}

static int impl_fp_store_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int src_hi, int dst_lo, int dst_hi,
                                 int offset, int size, outputStream* st ) {
  if( src_lo != FPR1L_num ) {      // Move value to top of FP stack, if not already there
    if( cbuf ) {
      emit_opcode( *cbuf, 0xD9 );  // FLD (i.e., push it)
      emit_d8( *cbuf, 0xC0-1+Matcher::_regEncode[src_lo] );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("FLD    %s",Matcher::regName[src_lo]);
#endif
    }
    size += 2;
  }

  int st_op = (src_lo != FPR1L_num) ? EBX_num /*store & pop*/ : EDX_num /*store no pop*/;
  const char *op_str;
  int op;
  if( src_lo+1 == src_hi && dst_lo+1 == dst_hi ) { // double store?
    op_str = (src_lo != FPR1L_num) ? "FSTP_D" : "FST_D ";
    op = 0xDD;
  } else {                   // 32-bit store
    op_str = (src_lo != FPR1L_num) ? "FSTP_S" : "FST_S ";
    op = 0xD9;
    assert( !OptoReg::is_valid(src_hi) && !OptoReg::is_valid(dst_hi), "no non-adjacent float-stores" );
  }

  return impl_helper(cbuf,do_size,false,offset,st_op,op,op_str,size, st);
}

// Next two methods are shared by 32- and 64-bit VM. They are defined in x86.ad.
static void vec_mov_helper(CodeBuffer *cbuf, int src_lo, int dst_lo,
                          int src_hi, int dst_hi, uint ireg, outputStream* st);

void vec_spill_helper(CodeBuffer *cbuf, bool is_load,
                            int stack_offset, int reg, uint ireg, outputStream* st);

static void vec_stack_to_stack_helper(CodeBuffer *cbuf, int src_offset,
                                     int dst_offset, uint ireg, outputStream* st) {
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    switch (ireg) {
    case Op_VecS:
      __ pushl(Address(rsp, src_offset));
      __ popl (Address(rsp, dst_offset));
      break;
    case Op_VecD:
      __ pushl(Address(rsp, src_offset));
      __ popl (Address(rsp, dst_offset));
      __ pushl(Address(rsp, src_offset+4));
      __ popl (Address(rsp, dst_offset+4));
      break;
    case Op_VecX:
      __ movdqu(Address(rsp, -16), xmm0);
      __ movdqu(xmm0, Address(rsp, src_offset));
      __ movdqu(Address(rsp, dst_offset), xmm0);
      __ movdqu(xmm0, Address(rsp, -16));
      break;
    case Op_VecY:
      __ vmovdqu(Address(rsp, -32), xmm0);
      __ vmovdqu(xmm0, Address(rsp, src_offset));
      __ vmovdqu(Address(rsp, dst_offset), xmm0);
      __ vmovdqu(xmm0, Address(rsp, -32));
      break;
    case Op_VecZ:
      __ evmovdquq(Address(rsp, -64), xmm0, 2);
      __ evmovdquq(xmm0, Address(rsp, src_offset), 2);
      __ evmovdquq(Address(rsp, dst_offset), xmm0, 2);
      __ evmovdquq(xmm0, Address(rsp, -64), 2);
      break;
    default:
      ShouldNotReachHere();
    }
#ifndef PRODUCT
  } else {
    switch (ireg) {
    case Op_VecS:
      st->print("pushl   [rsp + #%d]\t# 32-bit mem-mem spill\n\t"
                "popl    [rsp + #%d]",
                src_offset, dst_offset);
      break;
    case Op_VecD:
      st->print("pushl   [rsp + #%d]\t# 64-bit mem-mem spill\n\t"
                "popq    [rsp + #%d]\n\t"
                "pushl   [rsp + #%d]\n\t"
                "popq    [rsp + #%d]",
                src_offset, dst_offset, src_offset+4, dst_offset+4);
      break;
     case Op_VecX:
      st->print("movdqu  [rsp - #16], xmm0\t# 128-bit mem-mem spill\n\t"
                "movdqu  xmm0, [rsp + #%d]\n\t"
                "movdqu  [rsp + #%d], xmm0\n\t"
                "movdqu  xmm0, [rsp - #16]",
                src_offset, dst_offset);
      break;
    case Op_VecY:
      st->print("vmovdqu [rsp - #32], xmm0\t# 256-bit mem-mem spill\n\t"
                "vmovdqu xmm0, [rsp + #%d]\n\t"
                "vmovdqu [rsp + #%d], xmm0\n\t"
                "vmovdqu xmm0, [rsp - #32]",
                src_offset, dst_offset);
      break;
    case Op_VecZ:
      st->print("vmovdqu [rsp - #64], xmm0\t# 512-bit mem-mem spill\n\t"
                "vmovdqu xmm0, [rsp + #%d]\n\t"
                "vmovdqu [rsp + #%d], xmm0\n\t"
                "vmovdqu xmm0, [rsp - #64]",
                src_offset, dst_offset);
      break;
    default:
      ShouldNotReachHere();
    }
#endif
  }
}

uint MachSpillCopyNode::implementation( CodeBuffer *cbuf, PhaseRegAlloc *ra_, bool do_size, outputStream* st ) const {
  // Get registers to move
  OptoReg::Name src_second = ra_->get_reg_second(in(1));
  OptoReg::Name src_first = ra_->get_reg_first(in(1));
  OptoReg::Name dst_second = ra_->get_reg_second(this );
  OptoReg::Name dst_first = ra_->get_reg_first(this );

  enum RC src_second_rc = rc_class(src_second);
  enum RC src_first_rc = rc_class(src_first);
  enum RC dst_second_rc = rc_class(dst_second);
  enum RC dst_first_rc = rc_class(dst_first);

  assert( OptoReg::is_valid(src_first) && OptoReg::is_valid(dst_first), "must move at least 1 register" );

  // Generate spill code!
  int size = 0;

  if( src_first == dst_first && src_second == dst_second )
    return size;            // Self copy, no move

  if (bottom_type()->isa_vect() != NULL) {
    uint ireg = ideal_reg();
    assert((src_first_rc != rc_int && dst_first_rc != rc_int), "sanity");
    assert((src_first_rc != rc_float && dst_first_rc != rc_float), "sanity");
    assert((ireg == Op_VecS || ireg == Op_VecD || ireg == Op_VecX || ireg == Op_VecY || ireg == Op_VecZ ), "sanity");
    if( src_first_rc == rc_stack && dst_first_rc == rc_stack ) {
      // mem -> mem
      int src_offset = ra_->reg2offset(src_first);
      int dst_offset = ra_->reg2offset(dst_first);
      vec_stack_to_stack_helper(cbuf, src_offset, dst_offset, ireg, st);
    } else if (src_first_rc == rc_xmm && dst_first_rc == rc_xmm ) {
      vec_mov_helper(cbuf, src_first, dst_first, src_second, dst_second, ireg, st);
    } else if (src_first_rc == rc_xmm && dst_first_rc == rc_stack ) {
      int stack_offset = ra_->reg2offset(dst_first);
      vec_spill_helper(cbuf, false, stack_offset, src_first, ireg, st);
    } else if (src_first_rc == rc_stack && dst_first_rc == rc_xmm ) {
      int stack_offset = ra_->reg2offset(src_first);
      vec_spill_helper(cbuf, true,  stack_offset, dst_first, ireg, st);
    } else {
      ShouldNotReachHere();
    }
    return 0;
  }

  // --------------------------------------
  // Check for mem-mem move.  push/pop to move.
  if( src_first_rc == rc_stack && dst_first_rc == rc_stack ) {
    if( src_second == dst_first ) { // overlapping stack copy ranges
      assert( src_second_rc == rc_stack && dst_second_rc == rc_stack, "we only expect a stk-stk copy here" );
      size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),ESI_num,0xFF,"PUSH  ",size, st);
      size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),EAX_num,0x8F,"POP   ",size, st);
      src_second_rc = dst_second_rc = rc_bad;  // flag as already moved the second bits
    }
    // move low bits
    size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),ESI_num,0xFF,"PUSH  ",size, st);
    size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),EAX_num,0x8F,"POP   ",size, st);
    if( src_second_rc == rc_stack && dst_second_rc == rc_stack ) { // mov second bits
      size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),ESI_num,0xFF,"PUSH  ",size, st);
      size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),EAX_num,0x8F,"POP   ",size, st);
    }
    return size;
  }

  // --------------------------------------
  // Check for integer reg-reg copy
  if( src_first_rc == rc_int && dst_first_rc == rc_int )
    size = impl_mov_helper(cbuf,do_size,src_first,dst_first,size, st);

  // Check for integer store
  if( src_first_rc == rc_int && dst_first_rc == rc_stack )
    size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),src_first,0x89,"MOV ",size, st);

  // Check for integer load
  if( dst_first_rc == rc_int && src_first_rc == rc_stack )
    size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),dst_first,0x8B,"MOV ",size, st);

  // Check for integer reg-xmm reg copy
  if( src_first_rc == rc_int && dst_first_rc == rc_xmm ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad),
            "no 64 bit integer-float reg moves" );
    return impl_movgpr2x_helper(cbuf,do_size,src_first,dst_first,src_second, dst_second, size, st);
  }
  // --------------------------------------
  // Check for float reg-reg copy
  if( src_first_rc == rc_float && dst_first_rc == rc_float ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad) ||
            (src_first+1 == src_second && dst_first+1 == dst_second), "no non-adjacent float-moves" );
    if( cbuf ) {

      // Note the mucking with the register encode to compensate for the 0/1
      // indexing issue mentioned in a comment in the reg_def sections
      // for FPR registers many lines above here.

      if( src_first != FPR1L_num ) {
        emit_opcode  (*cbuf, 0xD9 );           // FLD    ST(i)
        emit_d8      (*cbuf, 0xC0+Matcher::_regEncode[src_first]-1 );
        emit_opcode  (*cbuf, 0xDD );           // FSTP   ST(i)
        emit_d8      (*cbuf, 0xD8+Matcher::_regEncode[dst_first] );
     } else {
        emit_opcode  (*cbuf, 0xDD );           // FST    ST(i)
        emit_d8      (*cbuf, 0xD0+Matcher::_regEncode[dst_first]-1 );
     }
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      if( src_first != FPR1L_num ) st->print("FLD    %s\n\tFSTP   %s",Matcher::regName[src_first],Matcher::regName[dst_first]);
      else                      st->print(             "FST    %s",                            Matcher::regName[dst_first]);
#endif
    }
    return size + ((src_first != FPR1L_num) ? 2+2 : 2);
  }

  // Check for float store
  if( src_first_rc == rc_float && dst_first_rc == rc_stack ) {
    return impl_fp_store_helper(cbuf,do_size,src_first,src_second,dst_first,dst_second,ra_->reg2offset(dst_first),size, st);
  }

  // Check for float load
  if( dst_first_rc == rc_float && src_first_rc == rc_stack ) {
    int offset = ra_->reg2offset(src_first);
    const char *op_str;
    int op;
    if( src_first+1 == src_second && dst_first+1 == dst_second ) { // double load?
      op_str = "FLD_D";
      op = 0xDD;
    } else {                   // 32-bit load
      op_str = "FLD_S";
      op = 0xD9;
      assert( src_second_rc == rc_bad && dst_second_rc == rc_bad, "no non-adjacent float-loads" );
    }
    if( cbuf ) {
      emit_opcode  (*cbuf, op );
      encode_RegMem(*cbuf, 0x0, ESP_enc, 0x4, 0, offset, relocInfo::none);
      emit_opcode  (*cbuf, 0xDD );           // FSTP   ST(i)
      emit_d8      (*cbuf, 0xD8+Matcher::_regEncode[dst_first] );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("%s  ST,[ESP + #%d]\n\tFSTP   %s",op_str, offset,Matcher::regName[dst_first]);
#endif
    }
    int offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
    return size + 3+offset_size+2;
  }

  // Check for xmm reg-reg copy
  if( src_first_rc == rc_xmm && dst_first_rc == rc_xmm ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad) ||
            (src_first+1 == src_second && dst_first+1 == dst_second),
            "no non-adjacent float-moves" );
    return impl_movx_helper(cbuf,do_size,src_first,dst_first,src_second, dst_second, size, st);
  }

  // Check for xmm reg-integer reg copy
  if( src_first_rc == rc_xmm && dst_first_rc == rc_int ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad),
            "no 64 bit float-integer reg moves" );
    return impl_movx2gpr_helper(cbuf,do_size,src_first,dst_first,src_second, dst_second, size, st);
  }

  // Check for xmm store
  if( src_first_rc == rc_xmm && dst_first_rc == rc_stack ) {
    return impl_x_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),src_first, src_second, size, st);
  }

  // Check for float xmm load
  if( dst_first_rc == rc_xmm && src_first_rc == rc_stack ) {
    return impl_x_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),dst_first, dst_second, size, st);
  }

  // Copy from float reg to xmm reg
  if( dst_first_rc == rc_xmm && src_first_rc == rc_float ) {
    // copy to the top of stack from floating point reg
    // and use LEA to preserve flags
    if( cbuf ) {
      emit_opcode(*cbuf,0x8D);  // LEA  ESP,[ESP-8]
      emit_rm(*cbuf, 0x1, ESP_enc, 0x04);
      emit_rm(*cbuf, 0x0, 0x04, ESP_enc);
      emit_d8(*cbuf,0xF8);
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("LEA    ESP,[ESP-8]");
#endif
    }
    size += 4;

    size = impl_fp_store_helper(cbuf,do_size,src_first,src_second,dst_first,dst_second,0,size, st);

    // Copy from the temp memory to the xmm reg.
    size = impl_x_helper(cbuf,do_size,true ,0,dst_first, dst_second, size, st);

    if( cbuf ) {
      emit_opcode(*cbuf,0x8D);  // LEA  ESP,[ESP+8]
      emit_rm(*cbuf, 0x1, ESP_enc, 0x04);
      emit_rm(*cbuf, 0x0, 0x04, ESP_enc);
      emit_d8(*cbuf,0x08);
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("LEA    ESP,[ESP+8]");
#endif
    }
    size += 4;
    return size;
  }

  assert( size > 0, "missed a case" );

  // --------------------------------------------------------------------
  // Check for second bits still needing moving.
  if( src_second == dst_second )
    return size;               // Self copy; no move
  assert( src_second_rc != rc_bad && dst_second_rc != rc_bad, "src_second & dst_second cannot be Bad" );

  // Check for second word int-int move
  if( src_second_rc == rc_int && dst_second_rc == rc_int )
    return impl_mov_helper(cbuf,do_size,src_second,dst_second,size, st);

  // Check for second word integer store
  if( src_second_rc == rc_int && dst_second_rc == rc_stack )
    return impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),src_second,0x89,"MOV ",size, st);

  // Check for second word integer load
  if( dst_second_rc == rc_int && src_second_rc == rc_stack )
    return impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),dst_second,0x8B,"MOV ",size, st);


  Unimplemented();
  return 0; // Mute compiler
}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_, outputStream* st) const {
  implementation( NULL, ra_, false, st );
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation( &cbuf, ra_, false, NULL );
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}


//=============================================================================
#ifndef PRODUCT
void BoxLockNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("LEA    %s,[ESP + #%d]",Matcher::regName[reg],offset);
}
#endif

void BoxLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_encode(this);
  if( offset >= 128 ) {
    emit_opcode(cbuf, 0x8D);      // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x2, reg, 0x04);
    emit_rm(cbuf, 0x0, 0x04, ESP_enc);
    emit_d32(cbuf, offset);
  }
  else {
    emit_opcode(cbuf, 0x8D);      // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x1, reg, 0x04);
    emit_rm(cbuf, 0x0, 0x04, ESP_enc);
    emit_d8(cbuf, offset);
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  if( offset >= 128 ) {
    return 7;
  }
  else {
    return 4;
  }
}

//=============================================================================
#ifndef PRODUCT
void MachUEPNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  st->print_cr(  "CMP    EAX,[ECX+4]\t# Inline cache check");
  st->print_cr("\tJNE    SharedRuntime::handle_ic_miss_stub");
  st->print_cr("\tNOP");
  st->print_cr("\tNOP");
  if( !OptoBreakpoint )
    st->print_cr("\tNOP");
}
#endif

void MachUEPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler masm(&cbuf);
#ifdef ASSERT
  uint insts_size = cbuf.insts_size();
#endif
  masm.cmpptr(rax, Address(rcx, oopDesc::klass_offset_in_bytes()));
  masm.jump_cc(Assembler::notEqual,
               RuntimeAddress(SharedRuntime::get_ic_miss_stub()));
  /* WARNING these NOPs are critical so that verified entry point is properly
     aligned for patching by NativeJump::patch_verified_entry() */
  int nops_cnt = 2;
  if( !OptoBreakpoint ) // Leave space for int3
     nops_cnt += 1;
  masm.nop(nops_cnt);

  assert(cbuf.insts_size() - insts_size == size(ra_), "checking code size of inline cache node");
}

uint MachUEPNode::size(PhaseRegAlloc *ra_) const {
  return OptoBreakpoint ? 11 : 12;
}


//=============================================================================

// This is UltraSparc specific, true just means we have fast l2f conversion
const bool Matcher::convL2FSupported(void) {
  return true;
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
bool Matcher::is_short_branch_offset(int rule, int br_size, int offset) {
  // The passed offset is relative to address of the branch.
  // On 86 a branch displacement is calculated relative to address
  // of a next instruction.
  offset -= br_size;

  // the short version of jmpConUCF2 contains multiple branches,
  // making the reach slightly less
  if (rule == jmpConUCF2_rule)
    return (-126 <= offset && offset <= 125);
  return (-128 <= offset && offset <= 127);
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Will one (StoreL ConL) be cheaper than two (StoreI ConI)?.
  return false;
}

// The ecx parameter to rep stos for the ClearArray node is in dwords.
const bool Matcher::init_array_count_is_in_bytes = false;

// Needs 2 CMOV's for longs.
const int Matcher::long_cmove_cost() { return 1; }

// No CMOVF/CMOVD with SSE/SSE2
const int Matcher::float_cmove_cost() { return (UseSSE>=1) ? ConditionalMoveLimit : 0; }

// Does the CPU require late expand (see block.cpp for description of late expand)?
const bool Matcher::require_postalloc_expand = false;

// Do we need to mask the count passed to shift instructions or does
// the cpu only look at the lower 5/6 bits anyway?
const bool Matcher::need_masked_shift_count = false;

bool Matcher::narrow_oop_use_complex_address() {
  ShouldNotCallThis();
  return true;
}

bool Matcher::narrow_klass_use_complex_address() {
  ShouldNotCallThis();
  return true;
}

bool Matcher::const_oop_prefer_decode() {
  ShouldNotCallThis();
  return true;
}

bool Matcher::const_klass_prefer_decode() {
  ShouldNotCallThis();
  return true;
}

// Is it better to copy float constants, or load them directly from memory?
// Intel can load a float constant from a direct address, requiring no
// extra registers.  Most RISCs will have to materialize an address into a
// register first, so they would do better to copy the constant from stack.
const bool Matcher::rematerialize_float_constants = true;

// If CPU can load and store mis-aligned doubles directly then no fixup is
// needed.  Else we split the double into 2 integer pieces and move it
// piece-by-piece.  Only happens when passing doubles into C code as the
// Java calling convention forces doubles to be aligned.
const bool Matcher::misaligned_doubles_ok = true;

// Advertise here if the CPU requires explicit rounding operations to implement strictfp mode.
const bool Matcher::strict_fp_requires_explicit_rounding = true;

// Are floats conerted to double when stored to stack during deoptimization?
// On x32 it is stored with convertion only when FPU is used for floats.
bool Matcher::float_in_double() { return (UseSSE == 0); }

// Do ints take an entire long register or just half?
const bool Matcher::int_in_long = false;

// Return whether or not this register is ever used as an argument.  This
// function is used on startup to build the trampoline stubs in generateOptoStub.
// Registers not mentioned will be killed by the VM call in the trampoline, and
// arguments in those registers not be available to the callee.
bool Matcher::can_be_java_arg( int reg ) {
  if(  reg == ECX_num   || reg == EDX_num   ) return true;
  if( (reg == XMM0_num  || reg == XMM1_num ) && UseSSE>=1 ) return true;
  if( (reg == XMM0b_num || reg == XMM1b_num) && UseSSE>=2 ) return true;
  return false;
}

bool Matcher::is_spillable_arg( int reg ) {
  return can_be_java_arg(reg);
}

bool Matcher::use_asm_for_ldiv_by_con( jlong divisor ) {
  // Use hardware integer DIV instruction when
  // it is faster than a code which use multiply.
  // Only when constant divisor fits into 32 bit
  // (min_jint is excluded to get only correct
  // positive 32 bit values from negative).
  return VM_Version::has_fast_idiv() &&
         (divisor == (int)divisor && divisor != min_jint);
}

// Register for DIVI projection of divmodI
RegMask Matcher::divI_proj_mask() {
  return EAX_REG_mask();
}

// Register for MODI projection of divmodI
RegMask Matcher::modI_proj_mask() {
  return EDX_REG_mask();
}

// Register for DIVL projection of divmodL
RegMask Matcher::divL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODL projection of divmodL
RegMask Matcher::modL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

const RegMask Matcher::method_handle_invoke_SP_save_mask() {
  return NO_REG_mask();
}

// Returns true if the high 32 bits of the value is known to be zero.
bool is_operand_hi32_zero(Node* n) {
  int opc = n->Opcode();
  if (opc == Op_AndL) {
    Node* o2 = n->in(2);
    if (o2->is_Con() && (o2->get_long() & 0xFFFFFFFF00000000LL) == 0LL) {
      return true;
    }
  }
  if (opc == Op_ConL && (n->get_long() & 0xFFFFFFFF00000000LL) == 0LL) {
    return true;
  }
  return false;
}


#line 1315 "ad_x86.cpp"


//SourceForm

#line 1237 "src/hotspot/cpu/x86/x86.ad"


#include "opto/addnode.hpp"
#include "c2_intelJccErratum_x86.hpp"

void PhaseOutput::pd_perform_mach_node_analysis() {
  if (VM_Version::has_intel_jcc_erratum()) {
    int extra_padding = IntelJccErratum::tag_affected_machnodes(C, C->cfg(), C->regalloc());
    _buf_sizes._code += extra_padding;
  }
}

int MachNode::pd_alignment_required() const {
  if (VM_Version::has_intel_jcc_erratum() && IntelJccErratum::is_jcc_erratum_branch(this)) {
    // Conservatively add worst case padding. We assume that relocInfo::addr_unit() is 1 on x86.
    return IntelJccErratum::largest_jcc_size() + 1;
  } else {
    return 1;
  }
}

int MachNode::compute_padding(int current_offset) const {
  if (flags() & Node::PD::Flag_intel_jcc_erratum) {
    Compile* C = Compile::current();
    PhaseOutput* output = C->output();
    Block* block = output->block();
    int index = output->index();
    return IntelJccErratum::compute_padding(current_offset, this, block, index, C->regalloc());
  } else {
    return 0;
  }
}

// Emit exception handler code.
// Stuff framesize into a register and call a VM stub routine.
int HandlerImpl::emit_exception_handler(CodeBuffer& cbuf) {

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  C2_MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_exception_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();
  __ jump(RuntimeAddress(OptoRuntime::exception_blob()->entry_point()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

// Emit deopt handler code.
int HandlerImpl::emit_deopt_handler(CodeBuffer& cbuf) {

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  C2_MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_deopt_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();

#ifdef _LP64
  address the_pc = (address) __ pc();
  Label next;
  // push a "the_pc" on the stack without destroying any registers
  // as they all may be live.

  // push address of "next"
  __ call(next, relocInfo::none); // reloc none is fine since it is a disp32
  __ bind(next);
  // adjust it so it matches "the_pc"
  __ subptr(Address(rsp, 0), __ offset() - offset);
#else
  InternalAddress here(__ pc());
  __ pushptr(here.addr());
#endif

  __ jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));
  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow %d", (__ offset() - offset));
  __ end_a_stub();
  return offset;
}

Assembler::Width widthForType(BasicType bt) {
  if (bt == T_BYTE) {
    return Assembler::B;
  } else if (bt == T_SHORT) {
    return Assembler::W;
  } else if (bt == T_INT) {
    return Assembler::D;
  } else {
    assert(bt == T_LONG, "not a long: %s", type2name(bt));
    return Assembler::Q;
  }
}

//=============================================================================

  // Float masks come from different places depending on platform.
#ifdef _LP64
  static address float_signmask()  { return StubRoutines::x86::float_sign_mask(); }
  static address float_signflip()  { return StubRoutines::x86::float_sign_flip(); }
  static address double_signmask() { return StubRoutines::x86::double_sign_mask(); }
  static address double_signflip() { return StubRoutines::x86::double_sign_flip(); }
#else
  static address float_signmask()  { return (address)float_signmask_pool; }
  static address float_signflip()  { return (address)float_signflip_pool; }
  static address double_signmask() { return (address)double_signmask_pool; }
  static address double_signflip() { return (address)double_signflip_pool; }
#endif
  static address vector_short_to_byte_mask() { return StubRoutines::x86::vector_short_to_byte_mask(); }
  static address vector_int_to_byte_mask() { return StubRoutines::x86::vector_int_to_byte_mask(); }
  static address vector_byte_perm_mask() { return StubRoutines::x86::vector_byte_perm_mask(); }
  static address vector_long_sign_mask() { return StubRoutines::x86::vector_long_sign_mask(); }
  static address vector_all_bits_set() { return StubRoutines::x86::vector_all_bits_set(); }
  static address vector_int_to_short_mask() { return StubRoutines::x86::vector_int_to_short_mask(); }
  static address vector_short_shufflemask() { return StubRoutines::x86::vector_short_shuffle_mask(); }
  static address vector_int_shufflemask() { return StubRoutines::x86::vector_int_shuffle_mask(); }
  static address vector_long_shufflemask() { return StubRoutines::x86::vector_long_shuffle_mask(); }
  static address vector_32_bit_mask() { return StubRoutines::x86::vector_32_bit_mask(); }
  static address vector_64_bit_mask() { return StubRoutines::x86::vector_64_bit_mask(); }

//=============================================================================
const bool Matcher::match_rule_supported(int opcode) {
  if (!has_match_rule(opcode)) {
    return false; // no match rule present
  }
  switch (opcode) {
    case Op_AbsVL:
    case Op_StoreVectorScatter:
      if (UseAVX < 3) {
        return false;
      }
      break;
    case Op_PopCountI:
    case Op_PopCountL:
      if (!UsePopCountInstruction) {
        return false;
      }
      break;
    case Op_PopCountVI:
      if (!UsePopCountInstruction || !VM_Version::supports_avx512_vpopcntdq()) {
        return false;
      }
      break;
    case Op_MulVI:
      if ((UseSSE < 4) && (UseAVX < 1)) { // only with SSE4_1 or AVX
        return false;
      }
      break;
    case Op_MulVL:
      if (UseSSE < 4) { // only with SSE4_1 or AVX
        return false;
      }
      break;
    case Op_MulReductionVL:
      if (VM_Version::supports_avx512dq() == false) {
        return false;
      }
      break;
    case Op_AddReductionVL:
      if (UseSSE < 2) { // requires at least SSE2
        return false;
      }
      break;
    case Op_AbsVB:
    case Op_AbsVS:
    case Op_AbsVI:
    case Op_AddReductionVI:
    case Op_AndReductionV:
    case Op_OrReductionV:
    case Op_XorReductionV:
      if (UseSSE < 3) { // requires at least SSSE3
        return false;
      }
      break;
    case Op_VectorLoadShuffle:
    case Op_VectorRearrange:
    case Op_MulReductionVI:
      if (UseSSE < 4) { // requires at least SSE4
        return false;
      }
      break;
    case Op_SqrtVD:
    case Op_SqrtVF:
    case Op_VectorMaskCmp:
    case Op_VectorCastB2X:
    case Op_VectorCastS2X:
    case Op_VectorCastI2X:
    case Op_VectorCastL2X:
    case Op_VectorCastF2X:
    case Op_VectorCastD2X:
      if (UseAVX < 1) { // enabled for AVX only
        return false;
      }
      break;
    case Op_CompareAndSwapL:
#ifdef _LP64
    case Op_CompareAndSwapP:
#endif
      if (!VM_Version::supports_cx8()) {
        return false;
      }
      break;
    case Op_CMoveVF:
    case Op_CMoveVD:
      if (UseAVX < 1) { // enabled for AVX only
        return false;
      }
      break;
    case Op_StrIndexOf:
      if (!UseSSE42Intrinsics) {
        return false;
      }
      break;
    case Op_StrIndexOfChar:
      if (!UseSSE42Intrinsics) {
        return false;
      }
      break;
    case Op_OnSpinWait:
      if (VM_Version::supports_on_spin_wait() == false) {
        return false;
      }
      break;
    case Op_MulVB:
    case Op_LShiftVB:
    case Op_RShiftVB:
    case Op_URShiftVB:
    case Op_VectorInsert:
    case Op_VectorLoadMask:
    case Op_VectorStoreMask:
    case Op_VectorBlend:
      if (UseSSE < 4) {
        return false;
      }
      break;
#ifdef _LP64
    case Op_MaxD:
    case Op_MaxF:
    case Op_MinD:
    case Op_MinF:
      if (UseAVX < 1) { // enabled for AVX only
        return false;
      }
      break;
#endif
    case Op_CacheWB:
    case Op_CacheWBPreSync:
    case Op_CacheWBPostSync:
      if (!VM_Version::supports_data_cache_line_flush()) {
        return false;
      }
      break;
    case Op_ExtractB:
    case Op_ExtractL:
    case Op_ExtractI:
    case Op_RoundDoubleMode:
      if (UseSSE < 4) {
        return false;
      }
      break;
    case Op_RoundDoubleModeV:
      if (VM_Version::supports_avx() == false) {
        return false; // 128bit vroundpd is not available
      }
      break;
    case Op_LoadVectorGather:
      if (UseAVX < 2) {
        return false;
      }
      break;
    case Op_FmaVD:
    case Op_FmaVF:
      if (!UseFMA) {
        return false;
      }
      break;
    case Op_MacroLogicV:
      if (UseAVX < 3 || !UseVectorMacroLogic) {
        return false;
      }
      break;
    case Op_VectorMaskGen:
    case Op_LoadVectorMasked:
    case Op_StoreVectorMasked:
      if (UseAVX < 3) {
        return false;
      }
      break;
#ifndef _LP64
    case Op_AddReductionVF:
    case Op_AddReductionVD:
    case Op_MulReductionVF:
    case Op_MulReductionVD:
      if (UseSSE < 1) { // requires at least SSE
        return false;
      }
      break;
    case Op_MulAddVS2VI:
    case Op_RShiftVL:
    case Op_AbsVD:
    case Op_NegVD:
      if (UseSSE < 2) {
        return false;
      }
      break;
#endif // !LP64
  }
  return true;  // Match rules are supported by default.
}

//------------------------------------------------------------------------

// Identify extra cases that we might want to provide match rules for vector nodes and
// other intrinsics guarded with vector length (vlen) and element type (bt).
const bool Matcher::match_rule_supported_vector(int opcode, int vlen, BasicType bt) {
  if (!match_rule_supported(opcode)) {
    return false;
  }
  // Matcher::vector_size_supported() restricts vector sizes in the following way (see Matcher::vector_width_in_bytes):
  //   * SSE2 supports 128bit vectors for all types;
  //   * AVX1 supports 256bit vectors only for FLOAT and DOUBLE types;
  //   * AVX2 supports 256bit vectors for all types;
  //   * AVX512F supports 512bit vectors only for INT, FLOAT, and DOUBLE types;
  //   * AVX512BW supports 512bit vectors for BYTE, SHORT, and CHAR types.
  // There's also a limit on minimum vector size supported: 2 elements (or 4 bytes for BYTE).
  // And MaxVectorSize is taken into account as well.
  if (!vector_size_supported(bt, vlen)) {
    return false;
  }
  // Special cases which require vector length follow:
  //   * implementation limitations
  //   * some 512bit vector operations on FLOAT and DOUBLE types require AVX512DQ
  //   * 128bit vroundpd instruction is present only in AVX1
  int size_in_bits = vlen * type2aelembytes(bt) * BitsPerByte;
  switch (opcode) {
    case Op_AbsVF:
    case Op_NegVF:
      if ((vlen == 16) && (VM_Version::supports_avx512dq() == false)) {
        return false; // 512bit vandps and vxorps are not available
      }
      break;
    case Op_AbsVD:
    case Op_NegVD:
    case Op_MulVL:
      if ((vlen == 8) && (VM_Version::supports_avx512dq() == false)) {
        return false; // 512bit vpmullq, vandpd and vxorpd are not available
      }
      break;
    case Op_CMoveVF:
      if (vlen != 8) {
        return false; // implementation limitation (only vcmov8F_reg is present)
      }
      break;
    case Op_RotateRightV:
    case Op_RotateLeftV:
    case Op_MacroLogicV:
      if (!VM_Version::supports_evex() ||
          ((size_in_bits != 512) && !VM_Version::supports_avx512vl())) {
        return false;
      }
      break;
    case Op_VectorMaskGen:
    case Op_LoadVectorMasked:
    case Op_StoreVectorMasked:
      if (!VM_Version::supports_avx512bw()) {
        return false;
      }
      if ((size_in_bits != 512) && !VM_Version::supports_avx512vl()) {
        return false;
      }
      break;
    case Op_CMoveVD:
      if (vlen != 4) {
        return false; // implementation limitation (only vcmov4D_reg is present)
      }
      break;
    case Op_MaxV:
    case Op_MinV:
      if (UseSSE < 4 && is_integral_type(bt)) {
        return false;
      }
      if ((bt == T_FLOAT || bt == T_DOUBLE)) {
          // Float/Double intrinsics are enabled for AVX family currently.
          if (UseAVX == 0) {
            return false;
          }
          if (UseAVX > 2 && (!VM_Version::supports_avx512dq() && size_in_bits == 512)) { // 512 bit Float/Double intrinsics need AVX512DQ
            return false;
          }
      }
      break;
    case Op_AddReductionVI:
      if (bt == T_INT && (UseSSE < 3 || !VM_Version::supports_ssse3())) {
        return false;
      }
      // fallthrough
    case Op_AndReductionV:
    case Op_OrReductionV:
    case Op_XorReductionV:
      if (is_subword_type(bt) && (UseSSE < 4)) {
        return false;
      }
#ifndef _LP64
      if (bt == T_BYTE || bt == T_LONG) {
        return false;
      }
#endif
      break;
#ifndef _LP64
    case Op_VectorInsert:
      if (bt == T_LONG || bt == T_DOUBLE) {
        return false;
      }
      break;
#endif
    case Op_MinReductionV:
    case Op_MaxReductionV:
      if ((bt == T_INT || is_subword_type(bt)) && UseSSE < 4) {
        return false;
      } else if (bt == T_LONG && (UseAVX < 3 || !VM_Version::supports_avx512vlbwdq())) {
        return false;
      }
      // Float/Double intrinsics enabled for AVX family.
      if (UseAVX == 0 && (bt == T_FLOAT || bt == T_DOUBLE)) {
        return false;
      }
      if (UseAVX > 2 && (!VM_Version::supports_avx512dq() && size_in_bits == 512)) {
        return false;
      }
#ifndef _LP64
      if (bt == T_BYTE || bt == T_LONG) {
        return false;
      }
#endif
      break;
    case Op_VectorTest:
      if (UseSSE < 4) {
        return false; // Implementation limitation
      } else if (size_in_bits < 32) {
        return false; // Implementation limitation
      } else if (size_in_bits == 512 && (VM_Version::supports_avx512bw() == false)) {
        return false; // Implementation limitation
      }
      break;
    case Op_VectorLoadShuffle:
    case Op_VectorRearrange:
      if(vlen == 2) {
        return false; // Implementation limitation due to how shuffle is loaded
      } else if (size_in_bits == 256 && UseAVX < 2) {
        return false; // Implementation limitation
      } else if (bt == T_BYTE && size_in_bits >= 256 && !VM_Version::supports_avx512_vbmi())  {
        return false; // Implementation limitation
      } else if (bt == T_SHORT && size_in_bits >= 256 && !VM_Version::supports_avx512bw())  {
        return false; // Implementation limitation
      }
      break;
    case Op_VectorLoadMask:
      if (size_in_bits == 256 && UseAVX < 2) {
        return false; // Implementation limitation
      }
      // fallthrough
    case Op_VectorStoreMask:
      if (vlen == 2) {
        return false; // Implementation limitation
      }
      break;
    case Op_VectorCastB2X:
      if (size_in_bits == 256 && UseAVX < 2) {
        return false; // Implementation limitation
      }
      break;
    case Op_VectorCastS2X:
      if (is_integral_type(bt) && size_in_bits == 256 && UseAVX < 2) {
        return false;
      }
      break;
    case Op_VectorCastI2X:
      if (is_integral_type(bt) && size_in_bits == 256 && UseAVX < 2) {
        return false;
      }
      break;
    case Op_VectorCastL2X:
      if (is_integral_type(bt) && size_in_bits == 256 && UseAVX < 2) {
        return false;
      } else if (!is_integral_type(bt) && !VM_Version::supports_avx512dq()) {
        return false;
      }
      break;
    case Op_VectorCastF2X:
    case Op_VectorCastD2X:
      if (is_integral_type(bt)) {
        // Casts from FP to integral types require special fixup logic not easily
        // implementable with vectors.
        return false; // Implementation limitation
      }
    case Op_MulReductionVI:
      if (bt == T_BYTE && size_in_bits == 512 && !VM_Version::supports_avx512bw()) {
        return false;
      }
      break;
    case Op_StoreVectorScatter:
      if(bt == T_BYTE || bt == T_SHORT) {
        return false;
      } else if (size_in_bits < 512 && !VM_Version::supports_avx512vl()) {
        return false;
      }
      // fallthrough
    case Op_LoadVectorGather:
      if (size_in_bits == 64 ) {
        return false;
      }
      break;
  }
  return true;  // Per default match rules are supported.
}

// x86 supports generic vector operands: vec and legVec.
const bool Matcher::supports_generic_vector_operands = true;

MachOper* Matcher::pd_specialize_generic_vector_operand(MachOper* generic_opnd, uint ideal_reg, bool is_temp) {
  assert(Matcher::is_generic_vector(generic_opnd), "not generic");
  bool legacy = (generic_opnd->opcode() == LEGVEC);
  if (!VM_Version::supports_avx512vlbwdq() && // KNL
      is_temp && !legacy && (ideal_reg == Op_VecZ)) {
    // Conservatively specialize 512bit vec TEMP operands to legVecZ (zmm0-15) on KNL.
    return new legVecZOper();
  }
  if (legacy) {
    switch (ideal_reg) {
      case Op_VecS: return new legVecSOper();
      case Op_VecD: return new legVecDOper();
      case Op_VecX: return new legVecXOper();
      case Op_VecY: return new legVecYOper();
      case Op_VecZ: return new legVecZOper();
    }
  } else {
    switch (ideal_reg) {
      case Op_VecS: return new vecSOper();
      case Op_VecD: return new vecDOper();
      case Op_VecX: return new vecXOper();
      case Op_VecY: return new vecYOper();
      case Op_VecZ: return new vecZOper();
    }
  }
  ShouldNotReachHere();
  return NULL;
}

bool Matcher::is_generic_reg2reg_move(MachNode* m) {
  switch (m->rule()) {
    case MoveVec2Leg_rule:
    case MoveLeg2Vec_rule:
      return true;
    default:
      return false;
  }
}

bool Matcher::is_generic_vector(MachOper* opnd) {
  switch (opnd->opcode()) {
    case VEC:
    case LEGVEC:
      return true;
    default:
      return false;
  }
}

//------------------------------------------------------------------------

bool Matcher::supports_vector_variable_shifts(void) {
  return (UseAVX >= 2);
}

bool Matcher::supports_vector_variable_rotates(void) {
  return true;
}

const bool Matcher::has_predicated_vectors(void) {
  bool ret_value = false;
  if (UseAVX > 2) {
    ret_value = VM_Version::supports_avx512vl();
  }

  return ret_value;
}

const int Matcher::float_pressure(int default_pressure_threshold) {
  int float_pressure_threshold = default_pressure_threshold;
#ifdef _LP64
  if (UseAVX > 2) {
    // Increase pressure threshold on machines with AVX3 which have
    // 2x more XMM registers.
    float_pressure_threshold = default_pressure_threshold * 2;
  }
#endif
  return float_pressure_threshold;
}

// Max vector size in bytes. 0 if not supported.
const int Matcher::vector_width_in_bytes(BasicType bt) {
  assert(is_java_primitive(bt), "only primitive type vectors");
  if (UseSSE < 2) return 0;
  // SSE2 supports 128bit vectors for all types.
  // AVX2 supports 256bit vectors for all types.
  // AVX2/EVEX supports 512bit vectors for all types.
  int size = (UseAVX > 1) ? (1 << UseAVX) * 8 : 16;
  // AVX1 supports 256bit vectors only for FLOAT and DOUBLE.
  if (UseAVX > 0 && (bt == T_FLOAT || bt == T_DOUBLE))
    size = (UseAVX > 2) ? 64 : 32;
  if (UseAVX > 2 && (bt == T_BYTE || bt == T_SHORT || bt == T_CHAR))
    size = (VM_Version::supports_avx512bw()) ? 64 : 32;
  // Use flag to limit vector size.
  size = MIN2(size,(int)MaxVectorSize);
  // Minimum 2 values in vector (or 4 for bytes).
  switch (bt) {
  case T_DOUBLE:
  case T_LONG:
    if (size < 16) return 0;
    break;
  case T_FLOAT:
  case T_INT:
    if (size < 8) return 0;
    break;
  case T_BOOLEAN:
    if (size < 4) return 0;
    break;
  case T_CHAR:
    if (size < 4) return 0;
    break;
  case T_BYTE:
    if (size < 4) return 0;
    break;
  case T_SHORT:
    if (size < 4) return 0;
    break;
  default:
    ShouldNotReachHere();
  }
  return size;
}

// Limits on vector size (number of elements) loaded into vector.
const int Matcher::max_vector_size(const BasicType bt) {
  return vector_width_in_bytes(bt)/type2aelembytes(bt);
}
const int Matcher::min_vector_size(const BasicType bt) {
  int max_size = max_vector_size(bt);
  // Min size which can be loaded into vector is 4 bytes.
  int size = (type2aelembytes(bt) == 1) ? 4 : 2;
  return MIN2(size,max_size);
}

const bool Matcher::supports_scalable_vector() {
  return false;
}

const int Matcher::scalable_vector_reg_size(const BasicType bt) {
  return -1;
}

// Vector ideal reg corresponding to specified size in bytes
const uint Matcher::vector_ideal_reg(int size) {
  assert(MaxVectorSize >= size, "");
  switch(size) {
    case  4: return Op_VecS;
    case  8: return Op_VecD;
    case 16: return Op_VecX;
    case 32: return Op_VecY;
    case 64: return Op_VecZ;
  }
  ShouldNotReachHere();
  return 0;
}

// x86 supports misaligned vectors store/load.
const bool Matcher::misaligned_vectors_ok() {
  return true;
}


const bool Matcher::convi2l_type_required = true;

// Check for shift by small constant as well
static bool clone_shift(Node* shift, Matcher* matcher, Matcher::MStack& mstack, VectorSet& address_visited) {
  if (shift->Opcode() == Op_LShiftX && shift->in(2)->is_Con() &&
      shift->in(2)->get_int() <= 3 &&
      // Are there other uses besides address expressions?
      !matcher->is_visited(shift)) {
    address_visited.set(shift->_idx); // Flag as address_visited
    mstack.push(shift->in(2), Matcher::Visit);
    Node *conv = shift->in(1);
#ifdef _LP64
    // Allow Matcher to match the rule which bypass
    // ConvI2L operation for an array index on LP64
    // if the index value is positive.
    if (conv->Opcode() == Op_ConvI2L &&
        conv->as_Type()->type()->is_long()->_lo >= 0 &&
        // Are there other uses besides address expressions?
        !matcher->is_visited(conv)) {
      address_visited.set(conv->_idx); // Flag as address_visited
      mstack.push(conv->in(1), Matcher::Pre_Visit);
    } else
#endif
      mstack.push(conv, Matcher::Pre_Visit);
    return true;
  }
  return false;
}

// This function identifies sub-graphs in which a 'load' node is
// input to two different nodes, and such that it can be matched
// with BMI instructions like blsi, blsr, etc.
// Example : for b = -a[i] & a[i] can be matched to blsi r32, m32.
// The graph is (AndL (SubL Con0 LoadL*) LoadL*), where LoadL*
// refers to the same node.
//
// Match the generic fused operations pattern (op1 (op2 Con{ConType} mop) mop)
// This is a temporary solution until we make DAGs expressible in ADL.
template<typename ConType>
class FusedPatternMatcher {
  Node* _op1_node;
  Node* _mop_node;
  int _con_op;

  static int match_next(Node* n, int next_op, int next_op_idx) {
    if (n->in(1) == NULL || n->in(2) == NULL) {
      return -1;
    }

    if (next_op_idx == -1) { // n is commutative, try rotations
      if (n->in(1)->Opcode() == next_op) {
        return 1;
      } else if (n->in(2)->Opcode() == next_op) {
        return 2;
      }
    } else {
      assert(next_op_idx > 0 && next_op_idx <= 2, "Bad argument index");
      if (n->in(next_op_idx)->Opcode() == next_op) {
        return next_op_idx;
      }
    }
    return -1;
  }

 public:
  FusedPatternMatcher(Node* op1_node, Node* mop_node, int con_op) :
    _op1_node(op1_node), _mop_node(mop_node), _con_op(con_op) { }

  bool match(int op1, int op1_op2_idx,  // op1 and the index of the op1->op2 edge, -1 if op1 is commutative
             int op2, int op2_con_idx,  // op2 and the index of the op2->con edge, -1 if op2 is commutative
             typename ConType::NativeType con_value) {
    if (_op1_node->Opcode() != op1) {
      return false;
    }
    if (_mop_node->outcnt() > 2) {
      return false;
    }
    op1_op2_idx = match_next(_op1_node, op2, op1_op2_idx);
    if (op1_op2_idx == -1) {
      return false;
    }
    // Memory operation must be the other edge
    int op1_mop_idx = (op1_op2_idx & 1) + 1;

    // Check that the mop node is really what we want
    if (_op1_node->in(op1_mop_idx) == _mop_node) {
      Node* op2_node = _op1_node->in(op1_op2_idx);
      if (op2_node->outcnt() > 1) {
        return false;
      }
      assert(op2_node->Opcode() == op2, "Should be");
      op2_con_idx = match_next(op2_node, _con_op, op2_con_idx);
      if (op2_con_idx == -1) {
        return false;
      }
      // Memory operation must be the other edge
      int op2_mop_idx = (op2_con_idx & 1) + 1;
      // Check that the memory operation is the same node
      if (op2_node->in(op2_mop_idx) == _mop_node) {
        // Now check the constant
        const Type* con_type = op2_node->in(op2_con_idx)->bottom_type();
        if (con_type != Type::TOP && ConType::as_self(con_type)->get_con() == con_value) {
          return true;
        }
      }
    }
    return false;
  }
};

static bool is_bmi_pattern(Node* n, Node* m) {
  assert(UseBMI1Instructions, "sanity");
  if (n != NULL && m != NULL) {
    if (m->Opcode() == Op_LoadI) {
      FusedPatternMatcher<TypeInt> bmii(n, m, Op_ConI);
      return bmii.match(Op_AndI, -1, Op_SubI,  1,  0)  ||
             bmii.match(Op_AndI, -1, Op_AddI, -1, -1)  ||
             bmii.match(Op_XorI, -1, Op_AddI, -1, -1);
    } else if (m->Opcode() == Op_LoadL) {
      FusedPatternMatcher<TypeLong> bmil(n, m, Op_ConL);
      return bmil.match(Op_AndL, -1, Op_SubL,  1,  0) ||
             bmil.match(Op_AndL, -1, Op_AddL, -1, -1) ||
             bmil.match(Op_XorL, -1, Op_AddL, -1, -1);
    }
  }
  return false;
}

// Should the matcher clone input 'm' of node 'n'?
bool Matcher::pd_clone_node(Node* n, Node* m, Matcher::MStack& mstack) {
  // If 'n' and 'm' are part of a graph for BMI instruction, clone the input 'm'.
  if (UseBMI1Instructions && is_bmi_pattern(n, m)) {
    mstack.push(m, Visit);
    return true;
  }
  return false;
}

// Should the Matcher clone shifts on addressing modes, expecting them
// to be subsumed into complex addressing expressions or compute them
// into registers?
bool Matcher::pd_clone_address_expressions(AddPNode* m, Matcher::MStack& mstack, VectorSet& address_visited) {
  Node *off = m->in(AddPNode::Offset);
  if (off->is_Con()) {
    address_visited.test_set(m->_idx); // Flag as address_visited
    Node *adr = m->in(AddPNode::Address);

    // Intel can handle 2 adds in addressing mode
    // AtomicAdd is not an addressing expression.
    // Cheap to find it by looking for screwy base.
    if (adr->is_AddP() &&
        !adr->in(AddPNode::Base)->is_top() &&
        LP64_ONLY( off->get_long() == (int) (off->get_long()) && ) // immL32
        // Are there other uses besides address expressions?
        !is_visited(adr)) {
      address_visited.set(adr->_idx); // Flag as address_visited
      Node *shift = adr->in(AddPNode::Offset);
      if (!clone_shift(shift, this, mstack, address_visited)) {
        mstack.push(shift, Pre_Visit);
      }
      mstack.push(adr->in(AddPNode::Address), Pre_Visit);
      mstack.push(adr->in(AddPNode::Base), Pre_Visit);
    } else {
      mstack.push(adr, Pre_Visit);
    }

    // Clone X+offset as it also folds into most addressing expressions
    mstack.push(off, Visit);
    mstack.push(m->in(AddPNode::Base), Pre_Visit);
    return true;
  } else if (clone_shift(off, this, mstack, address_visited)) {
    address_visited.test_set(m->_idx); // Flag as address_visited
    mstack.push(m->in(AddPNode::Address), Pre_Visit);
    mstack.push(m->in(AddPNode::Base), Pre_Visit);
    return true;
  }
  return false;
}

void Compile::reshape_address(AddPNode* addp) {
}

static inline Assembler::ComparisonPredicate booltest_pred_to_comparison_pred(int bt) {
  switch (bt) {
    case BoolTest::eq: return Assembler::eq;
    case BoolTest::ne: return Assembler::neq;
    case BoolTest::le: return Assembler::le;
    case BoolTest::ge: return Assembler::nlt;
    case BoolTest::lt: return Assembler::lt;
    case BoolTest::gt: return Assembler::nle;
    default : ShouldNotReachHere(); return Assembler::_false;
  }
}

static inline Assembler::ComparisonPredicateFP booltest_pred_to_comparison_pred_fp(int bt) {
  switch (bt) {
  case BoolTest::eq: return Assembler::EQ_OQ;  // ordered non-signaling
  // As per JLS 15.21.1, != of NaNs is true. Thus use unordered compare.
  case BoolTest::ne: return Assembler::NEQ_UQ; // unordered non-signaling
  case BoolTest::le: return Assembler::LE_OQ;  // ordered non-signaling
  case BoolTest::ge: return Assembler::GE_OQ;  // ordered non-signaling
  case BoolTest::lt: return Assembler::LT_OQ;  // ordered non-signaling
  case BoolTest::gt: return Assembler::GT_OQ;  // ordered non-signaling
  default: ShouldNotReachHere(); return Assembler::FALSE_OS;
  }
}

// Helper methods for MachSpillCopyNode::implementation().
static void vec_mov_helper(CodeBuffer *cbuf, int src_lo, int dst_lo,
                          int src_hi, int dst_hi, uint ireg, outputStream* st) {
  assert(ireg == Op_VecS || // 32bit vector
         (src_lo & 1) == 0 && (src_lo + 1) == src_hi &&
         (dst_lo & 1) == 0 && (dst_lo + 1) == dst_hi,
         "no non-adjacent vector moves" );
  if (cbuf) {
    C2_MacroAssembler _masm(cbuf);
    switch (ireg) {
    case Op_VecS: // copy whole register
    case Op_VecD:
    case Op_VecX:
#ifndef _LP64
      __ movdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
#else
      if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
        __ movdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
      } else {
        __ vextractf32x4(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]), 0x0);
     }
#endif
      break;
    case Op_VecY:
#ifndef _LP64
      __ vmovdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
#else
      if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
        __ vmovdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
      } else {
        __ vextractf64x4(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]), 0x0);
     }
#endif
      break;
    case Op_VecZ:
      __ evmovdquq(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]), 2);
      break;
    default:
      ShouldNotReachHere();
    }
#ifndef PRODUCT
  } else {
    switch (ireg) {
    case Op_VecS:
    case Op_VecD:
    case Op_VecX:
      st->print("movdqu  %s,%s\t# spill",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      break;
    case Op_VecY:
    case Op_VecZ:
      st->print("vmovdqu %s,%s\t# spill",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      break;
    default:
      ShouldNotReachHere();
    }
#endif
  }
}

void vec_spill_helper(CodeBuffer *cbuf, bool is_load,
                     int stack_offset, int reg, uint ireg, outputStream* st) {
  if (cbuf) {
    C2_MacroAssembler _masm(cbuf);
    if (is_load) {
      switch (ireg) {
      case Op_VecS:
        __ movdl(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        break;
      case Op_VecD:
        __ movq(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        break;
      case Op_VecX:
#ifndef _LP64
        __ movdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ movdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        } else {
          __ vpxor(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), 2);
          __ vinsertf32x4(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset),0x0);
        }
#endif
        break;
      case Op_VecY:
#ifndef _LP64
        __ vmovdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ vmovdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        } else {
          __ vpxor(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), 2);
          __ vinsertf64x4(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset),0x0);
        }
#endif
        break;
      case Op_VecZ:
        __ evmovdquq(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset), 2);
        break;
      default:
        ShouldNotReachHere();
      }
    } else { // store
      switch (ireg) {
      case Op_VecS:
        __ movdl(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        break;
      case Op_VecD:
        __ movq(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        break;
      case Op_VecX:
#ifndef _LP64
        __ movdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ movdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        }
        else {
          __ vextractf32x4(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]), 0x0);
        }
#endif
        break;
      case Op_VecY:
#ifndef _LP64
        __ vmovdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ vmovdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        }
        else {
          __ vextractf64x4(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]), 0x0);
        }
#endif
        break;
      case Op_VecZ:
        __ evmovdquq(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]), 2);
        break;
      default:
        ShouldNotReachHere();
      }
    }
#ifndef PRODUCT
  } else {
    if (is_load) {
      switch (ireg) {
      case Op_VecS:
        st->print("movd    %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      case Op_VecD:
        st->print("movq    %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
       case Op_VecX:
        st->print("movdqu  %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      case Op_VecY:
      case Op_VecZ:
        st->print("vmovdqu %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      default:
        ShouldNotReachHere();
      }
    } else { // store
      switch (ireg) {
      case Op_VecS:
        st->print("movd    [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      case Op_VecD:
        st->print("movq    [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
       case Op_VecX:
        st->print("movdqu  [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      case Op_VecY:
      case Op_VecZ:
        st->print("vmovdqu [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      default:
        ShouldNotReachHere();
      }
    }
#endif
  }
}

static inline jlong replicate8_imm(int con, int width) {
  // Load a constant of "width" (in bytes) and replicate it to fill 64bit.
  assert(width == 1 || width == 2 || width == 4, "only byte, short or int types here");
  int bit_width = width * 8;
  jlong val = con;
  val &= (((jlong) 1) << bit_width) - 1;  // mask off sign bits
  while(bit_width < 64) {
    val |= (val << bit_width);
    bit_width <<= 1;
  }
  return val;
}

#ifndef PRODUCT
  void MachNopNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("nop \t# %d bytes pad for loops and calls", _count);
  }
#endif

  void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc*) const {
    C2_MacroAssembler _masm(&cbuf);
    __ nop(_count);
  }

  uint MachNopNode::size(PhaseRegAlloc*) const {
    return _count;
  }

#ifndef PRODUCT
  void MachBreakpointNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("# breakpoint");
  }
#endif

  void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc* ra_) const {
    C2_MacroAssembler _masm(&cbuf);
    __ int3();
  }

  uint MachBreakpointNode::size(PhaseRegAlloc* ra_) const {
    return MachNode::size(ra_);
  }


#line 2442 "ad_x86.cpp"


#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C', // ECX
  'C', // EBX
  'N', // EBP
  'C', // EDI
  'C', // EAX
  'C', // EDX
  'C', // ESI
  'N', // ESP
  'C', // FPR0L
  'C', // FPR0H
  'C', // FPR1L
  'C', // FPR1H
  'C', // FPR2L
  'C', // FPR2H
  'C', // FPR3L
  'C', // FPR3H
  'C', // FPR4L
  'C', // FPR4H
  'C', // FPR5L
  'C', // FPR5H
  'C', // FPR6L
  'C', // FPR6H
  'C', // FPR7L
  'C', // FPR7H
  'C', // FILL0
  'C', // FILL1
  'C', // FILL2
  'C', // FILL3
  'C', // FILL4
  'C', // FILL5
  'C', // FILL6
  'C', // FILL7
  'C', // XMM0
  'C', // XMM0b
  'C', // XMM0c
  'C', // XMM0d
  'C', // XMM0e
  'C', // XMM0f
  'C', // XMM0g
  'C', // XMM0h
  'C', // XMM0i
  'C', // XMM0j
  'C', // XMM0k
  'C', // XMM0l
  'C', // XMM0m
  'C', // XMM0n
  'C', // XMM0o
  'C', // XMM0p
  'C', // XMM1
  'C', // XMM1b
  'C', // XMM1c
  'C', // XMM1d
  'C', // XMM1e
  'C', // XMM1f
  'C', // XMM1g
  'C', // XMM1h
  'C', // XMM1i
  'C', // XMM1j
  'C', // XMM1k
  'C', // XMM1l
  'C', // XMM1m
  'C', // XMM1n
  'C', // XMM1o
  'C', // XMM1p
  'C', // XMM2
  'C', // XMM2b
  'C', // XMM2c
  'C', // XMM2d
  'C', // XMM2e
  'C', // XMM2f
  'C', // XMM2g
  'C', // XMM2h
  'C', // XMM2i
  'C', // XMM2j
  'C', // XMM2k
  'C', // XMM2l
  'C', // XMM2m
  'C', // XMM2n
  'C', // XMM2o
  'C', // XMM2p
  'C', // XMM3
  'C', // XMM3b
  'C', // XMM3c
  'C', // XMM3d
  'C', // XMM3e
  'C', // XMM3f
  'C', // XMM3g
  'C', // XMM3h
  'C', // XMM3i
  'C', // XMM3j
  'C', // XMM3k
  'C', // XMM3l
  'C', // XMM3m
  'C', // XMM3n
  'C', // XMM3o
  'C', // XMM3p
  'C', // XMM4
  'C', // XMM4b
  'C', // XMM4c
  'C', // XMM4d
  'C', // XMM4e
  'C', // XMM4f
  'C', // XMM4g
  'C', // XMM4h
  'C', // XMM4i
  'C', // XMM4j
  'C', // XMM4k
  'C', // XMM4l
  'C', // XMM4m
  'C', // XMM4n
  'C', // XMM4o
  'C', // XMM4p
  'C', // XMM5
  'C', // XMM5b
  'C', // XMM5c
  'C', // XMM5d
  'C', // XMM5e
  'C', // XMM5f
  'C', // XMM5g
  'C', // XMM5h
  'C', // XMM5i
  'C', // XMM5j
  'C', // XMM5k
  'C', // XMM5l
  'C', // XMM5m
  'C', // XMM5n
  'C', // XMM5o
  'C', // XMM5p
  'C', // XMM6
  'C', // XMM6b
  'C', // XMM6c
  'C', // XMM6d
  'C', // XMM6e
  'C', // XMM6f
  'C', // XMM6g
  'C', // XMM6h
  'C', // XMM6i
  'C', // XMM6j
  'C', // XMM6k
  'C', // XMM6l
  'C', // XMM6m
  'C', // XMM6n
  'C', // XMM6o
  'C', // XMM6p
  'C', // XMM7
  'C', // XMM7b
  'C', // XMM7c
  'C', // XMM7d
  'C', // XMM7e
  'C', // XMM7f
  'C', // XMM7g
  'C', // XMM7h
  'C', // XMM7i
  'C', // XMM7j
  'C', // XMM7k
  'C', // XMM7l
  'C', // XMM7m
  'C', // XMM7n
  'C', // XMM7o
  'C', // XMM7p
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C', // ECX
  'E', // EBX
  'E', // EBP
  'E', // EDI
  'C', // EAX
  'C', // EDX
  'E', // ESI
  'N', // ESP
  'C', // FPR0L
  'C', // FPR0H
  'C', // FPR1L
  'C', // FPR1H
  'C', // FPR2L
  'C', // FPR2H
  'C', // FPR3L
  'C', // FPR3H
  'C', // FPR4L
  'C', // FPR4H
  'C', // FPR5L
  'C', // FPR5H
  'C', // FPR6L
  'C', // FPR6H
  'C', // FPR7L
  'C', // FPR7H
  'C', // FILL0
  'C', // FILL1
  'C', // FILL2
  'C', // FILL3
  'C', // FILL4
  'C', // FILL5
  'C', // FILL6
  'C', // FILL7
  'C', // XMM0
  'C', // XMM0b
  'C', // XMM0c
  'C', // XMM0d
  'C', // XMM0e
  'C', // XMM0f
  'C', // XMM0g
  'C', // XMM0h
  'C', // XMM0i
  'C', // XMM0j
  'C', // XMM0k
  'C', // XMM0l
  'C', // XMM0m
  'C', // XMM0n
  'C', // XMM0o
  'C', // XMM0p
  'C', // XMM1
  'C', // XMM1b
  'C', // XMM1c
  'C', // XMM1d
  'C', // XMM1e
  'C', // XMM1f
  'C', // XMM1g
  'C', // XMM1h
  'C', // XMM1i
  'C', // XMM1j
  'C', // XMM1k
  'C', // XMM1l
  'C', // XMM1m
  'C', // XMM1n
  'C', // XMM1o
  'C', // XMM1p
  'C', // XMM2
  'C', // XMM2b
  'C', // XMM2c
  'C', // XMM2d
  'C', // XMM2e
  'C', // XMM2f
  'C', // XMM2g
  'C', // XMM2h
  'C', // XMM2i
  'C', // XMM2j
  'C', // XMM2k
  'C', // XMM2l
  'C', // XMM2m
  'C', // XMM2n
  'C', // XMM2o
  'C', // XMM2p
  'C', // XMM3
  'C', // XMM3b
  'C', // XMM3c
  'C', // XMM3d
  'C', // XMM3e
  'C', // XMM3f
  'C', // XMM3g
  'C', // XMM3h
  'C', // XMM3i
  'C', // XMM3j
  'C', // XMM3k
  'C', // XMM3l
  'C', // XMM3m
  'C', // XMM3n
  'C', // XMM3o
  'C', // XMM3p
  'C', // XMM4
  'C', // XMM4b
  'C', // XMM4c
  'C', // XMM4d
  'C', // XMM4e
  'C', // XMM4f
  'C', // XMM4g
  'C', // XMM4h
  'C', // XMM4i
  'C', // XMM4j
  'C', // XMM4k
  'C', // XMM4l
  'C', // XMM4m
  'C', // XMM4n
  'C', // XMM4o
  'C', // XMM4p
  'C', // XMM5
  'C', // XMM5b
  'C', // XMM5c
  'C', // XMM5d
  'C', // XMM5e
  'C', // XMM5f
  'C', // XMM5g
  'C', // XMM5h
  'C', // XMM5i
  'C', // XMM5j
  'C', // XMM5k
  'C', // XMM5l
  'C', // XMM5m
  'C', // XMM5n
  'C', // XMM5o
  'C', // XMM5p
  'C', // XMM6
  'C', // XMM6b
  'C', // XMM6c
  'C', // XMM6d
  'C', // XMM6e
  'C', // XMM6f
  'C', // XMM6g
  'C', // XMM6h
  'C', // XMM6i
  'C', // XMM6j
  'C', // XMM6k
  'C', // XMM6l
  'C', // XMM6m
  'C', // XMM6n
  'C', // XMM6o
  'C', // XMM6p
  'C', // XMM7
  'C', // XMM7b
  'C', // XMM7c
  'C', // XMM7d
  'C', // XMM7e
  'C', // XMM7f
  'C', // XMM7g
  'C', // XMM7h
  'C', // XMM7i
  'C', // XMM7j
  'C', // XMM7k
  'C', // XMM7l
  'C', // XMM7m
  'C', // XMM7n
  'C', // XMM7o
  'C', // XMM7p
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  0 // no trailing comma
};


const        int   reduceOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  immI_rule,
  /*    9 */  immI_0_rule,
  /*   10 */  immI_1_rule,
  /*   11 */  immI_M1_rule,
  /*   12 */  immI2_rule,
  /*   13 */  immI8_rule,
  /*   14 */  immU8_rule,
  /*   15 */  immI16_rule,
  /*   16 */  immU31_rule,
  /*   17 */  immI_32_rule,
  /*   18 */  immI_1_31_rule,
  /*   19 */  immI_32_63_rule,
  /*   20 */  immI_2_rule,
  /*   21 */  immI_3_rule,
  /*   22 */  immI_4_rule,
  /*   23 */  immI_8_rule,
  /*   24 */  immP_rule,
  /*   25 */  immP0_rule,
  /*   26 */  immL_rule,
  /*   27 */  immL0_rule,
  /*   28 */  immL_M1_rule,
  /*   29 */  immL_127_rule,
  /*   30 */  immL_32bits_rule,
  /*   31 */  immL32_rule,
  /*   32 */  immDPR0_rule,
  /*   33 */  immDPR1_rule,
  /*   34 */  immDPR_rule,
  /*   35 */  immD_rule,
  /*   36 */  immD0_rule,
  /*   37 */  immFPR0_rule,
  /*   38 */  immFPR1_rule,
  /*   39 */  immFPR_rule,
  /*   40 */  immF_rule,
  /*   41 */  immF0_rule,
  /*   42 */  immI_16_rule,
  /*   43 */  immI_24_rule,
  /*   44 */  immI_255_rule,
  /*   45 */  immI_65535_rule,
  /*   46 */  rRegI_rule,
  /*   47 */  xRegI_rule,
  /*   48 */  eAXRegI_rule,
  /*   49 */  eBXRegI_rule,
  /*   50 */  eCXRegI_rule,
  /*   51 */  eDXRegI_rule,
  /*   52 */  eDIRegI_rule,
  /*   53 */  naxRegI_rule,
  /*   54 */  nadxRegI_rule,
  /*   55 */  ncxRegI_rule,
  /*   56 */  eSIRegI_rule,
  /*   57 */  anyRegP_rule,
  /*   58 */  eRegP_rule,
  /*   59 */  rRegP_rule,
  /*   60 */  eRegP_no_EBP_rule,
  /*   61 */  naxRegP_rule,
  /*   62 */  nabxRegP_rule,
  /*   63 */  pRegP_rule,
  /*   64 */  eAXRegP_rule,
  /*   65 */  eBXRegP_rule,
  /*   66 */  eCXRegP_rule,
  /*   67 */  eDXRegP_rule,
  /*   68 */  eSIRegP_rule,
  /*   69 */  eDIRegP_rule,
  /*   70 */  eRegL_rule,
  /*   71 */  eADXRegL_rule,
  /*   72 */  eBCXRegL_rule,
  /*   73 */  eADXRegL_low_only_rule,
  /*   74 */  rFlagsReg_rule,
  /*   75 */  eFlagsReg_rule,
  /*   76 */  eFlagsRegU_rule,
  /*   77 */  eFlagsRegUCF_rule,
  /*   78 */  flagsReg_long_LTGE_rule,
  /*   79 */  flagsReg_long_EQNE_rule,
  /*   80 */  flagsReg_long_LEGT_rule,
  /*   81 */  flagsReg_ulong_LTGE_rule,
  /*   82 */  flagsReg_ulong_EQNE_rule,
  /*   83 */  flagsReg_ulong_LEGT_rule,
  /*   84 */  regDPR_rule,
  /*   85 */  regDPR1_rule,
  /*   86 */  regDPR2_rule,
  /*   87 */  regnotDPR1_rule,
  /*   88 */  regFPR_rule,
  /*   89 */  regFPR1_rule,
  /*   90 */  regF_rule,
  /*   91 */  legRegF_rule,
  /*   92 */  vlRegF_rule,
  /*   93 */  regD_rule,
  /*   94 */  legRegD_rule,
  /*   95 */  vlRegD_rule,
  /*   96 */  direct_rule,
  /*   97 */  indirect_rule,
  /*   98 */  indOffset8_rule,
  /*   99 */  indOffset32_rule,
  /*  100 */  indOffset32X_rule,
  /*  101 */  indIndexOffset_rule,
  /*  102 */  indIndex_rule,
  /*  103 */  indIndexScale_rule,
  /*  104 */  indIndexScaleOffset_rule,
  /*  105 */  load_long_RegP_rule,
  /*  106 */  load_long_indirect_rule,
  /*  107 */  load_long_indOffset32_rule,
  /*  108 */  stackSlotP_rule,
  /*  109 */  stackSlotI_rule,
  /*  110 */  stackSlotF_rule,
  /*  111 */  stackSlotD_rule,
  /*  112 */  stackSlotL_rule,
  /*  113 */  cmpOp_rule,
  /*  114 */  cmpOpU_rule,
  /*  115 */  cmpOpUCF_rule,
  /*  116 */  cmpOpUCF2_rule,
  /*  117 */  cmpOp_fcmov_rule,
  /*  118 */  cmpOp_commute_rule,
  /*  119 */  cmpOpU_commute_rule,
  /*  120 */  rxmm0_rule,
  /*  121 */  vec_rule,
  /*  122 */  legVec_rule,
  /*  123 */  vecS_rule,
  /*  124 */  legVecS_rule,
  /*  125 */  vecD_rule,
  /*  126 */  legVecD_rule,
  /*  127 */  vecX_rule,
  /*  128 */  legVecX_rule,
  /*  129 */  vecY_rule,
  /*  130 */  legVecY_rule,
  /*  131 */  vecZ_rule,
  /*  132 */  legVecZ_rule,
  /*  133 */  cmpOp_vcmppd_rule,
  // last operand
  /*  134 */  load_long_memory_rule,
  /*  135 */  memory_rule,
  /*  136 */  long_memory_rule,
  // last operand class
  /*  137 */  _AddP_eRegP_rRegI_rule,
  /*  138 */  _LShiftI_rRegI_immI2_rule,
  /*  139 */  _AddP_eRegP__LShiftI_rRegI_immI2_rule,
  /*  140 */  _LoadI_memory__rule,
  /*  141 */  _LoadL_memory__rule,
  /*  142 */  _LoadB_memory__rule,
  /*  143 */  _LoadUB_memory__rule,
  /*  144 */  _AndI__LoadUB_memory__immI_rule,
  /*  145 */  _LoadS_memory__rule,
  /*  146 */  _LShiftI__LoadS_memory__immI_24_rule,
  /*  147 */  _LoadUS_memory__rule,
  /*  148 */  _LShiftI__LoadUS_memory__immI_24_rule,
  /*  149 */  _AndI__LoadUS_memory__immI_255_rule,
  /*  150 */  _AndI__LoadUS_memory__immI_rule,
  /*  151 */  _LShiftI__LoadI_memory__immI_24_rule,
  /*  152 */  _LShiftI__LoadI_memory__immI_16_rule,
  /*  153 */  _AndI__LoadI_memory__immI_255_rule,
  /*  154 */  _AndI__LoadI_memory__immI_65535_rule,
  /*  155 */  _AndI__LoadI_memory__immU31_rule,
  /*  156 */  _ConvI2L__LoadI_memory___rule,
  /*  157 */  _ConvL2I_eRegL__rule,
  /*  158 */  _RoundDouble_regDPR1__rule,
  /*  159 */  _RoundFloat_regFPR1__rule,
  /*  160 */  _ConvD2F_regDPR1__rule,
  /*  161 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  162 */  _Binary_rRegI_rRegI_rule,
  /*  163 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  164 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  165 */  _Binary_rRegI__LoadI_memory__rule,
  /*  166 */  _Binary_eRegP_eRegP_rule,
  /*  167 */  _Binary_cmpOp_fcmov_eFlagsRegU_rule,
  /*  168 */  _Binary_regDPR1_regDPR_rule,
  /*  169 */  _Binary_regFPR1_regFPR_rule,
  /*  170 */  _Binary_regDPR_regDPR_rule,
  /*  171 */  _Binary_regFPR_regFPR_rule,
  /*  172 */  _Binary_regF_regF_rule,
  /*  173 */  _Binary_regD_regD_rule,
  /*  174 */  _Binary_eRegL_eRegL_rule,
  /*  175 */  _AddI__LoadI_memory__rRegI_rule,
  /*  176 */  _AddI_rRegI__LoadI_memory__rule,
  /*  177 */  _AddI__LoadI_memory__immI_rule,
  /*  178 */  _AddI__LoadI_memory__immI_1_rule,
  /*  179 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  180 */  _Binary_eAXRegP_eRegP_rule,
  /*  181 */  _Binary_eAXRegI_rRegI_rule,
  /*  182 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  183 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  184 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  185 */  _SubI__LoadI_memory__rRegI_rule,
  /*  186 */  _SubI_immI_0_rRegI_rule,
  /*  187 */  _ConvI2L_nadxRegI__rule,
  /*  188 */  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule,
  /*  189 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_rule,
  /*  190 */  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule,
  /*  191 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_rule,
  /*  192 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63_rule,
  /*  193 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63_rule,
  /*  194 */  _ConvI2L_eAXRegI__rule,
  /*  195 */  _AndL__ConvI2L_eAXRegI__immL_32bits_rule,
  /*  196 */  _AndL__ConvI2L_nadxRegI__immL_32bits_rule,
  /*  197 */  _RShiftI__LoadI_memory__immI_1_rule,
  /*  198 */  _RShiftI__LoadI_memory__immI8_rule,
  /*  199 */  _LShiftI_xRegI_immI_24_rule,
  /*  200 */  _LShiftI_xRegI_immI_16_rule,
  /*  201 */  _AndI__LoadI_memory__rRegI_rule,
  /*  202 */  _AndI_rRegI__LoadI_memory__rule,
  /*  203 */  _AndI__LoadI_memory__immI_rule,
  /*  204 */  _XorI_rRegI_immI_M1_rule,
  /*  205 */  _SubI_immI_0__LoadI_memory__rule,
  /*  206 */  _AddI_rRegI_immI_M1_rule,
  /*  207 */  _CastP2X_eRegP__rule,
  /*  208 */  _OrI__LoadI_memory__rRegI_rule,
  /*  209 */  _OrI_rRegI__LoadI_memory__rule,
  /*  210 */  _OrI__LoadI_memory__immI_rule,
  /*  211 */  _LShiftI_rRegI_immI_1_rule,
  /*  212 */  _URShiftI_rRegI_immI_M1_rule,
  /*  213 */  _LShiftI_rRegI_immI8_rule,
  /*  214 */  _URShiftI_rRegI_immI8_rule,
  /*  215 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  216 */  _SubI_immI_0_eCXRegI_rule,
  /*  217 */  _URShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  218 */  _SubI_immI_32_eCXRegI_rule,
  /*  219 */  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  220 */  _URShiftI_rRegI_immI_1_rule,
  /*  221 */  _LShiftI_rRegI_immI_M1_rule,
  /*  222 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  223 */  _LShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  224 */  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  225 */  _XorI__LoadI_memory__rRegI_rule,
  /*  226 */  _XorI_rRegI__LoadI_memory__rule,
  /*  227 */  _XorI__LoadI_memory__immI_rule,
  /*  228 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  229 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  230 */  _SubI_rRegI_rRegI_rule,
  /*  231 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  232 */  _LoadL_load_long_memory__rule,
  /*  233 */  _XorL_eRegL_immL_M1_rule,
  /*  234 */  _SubL_immL0_eRegL_rule,
  /*  235 */  _SubL_immL0__LoadL_memory__rule,
  /*  236 */  _AddL_eRegL_immL_M1_rule,
  /*  237 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  238 */  _LoadD_memory__rule,
  /*  239 */  _SubD_regDPR_regDPR_rule,
  /*  240 */  _AddD_regDPR_regDPR_rule,
  /*  241 */  _AddD__LoadD_memory__regDPR_rule,
  /*  242 */  _RoundDouble__AddD__LoadD_memory__regDPR__rule,
  /*  243 */  _AddD_regDPR__LoadD_memory__rule,
  /*  244 */  _RoundDouble__AddD_regDPR__LoadD_memory___rule,
  /*  245 */  _AddD_regDPR_immDPR_rule,
  /*  246 */  _MulD_regDPR_regDPR_rule,
  /*  247 */  _DivD_regDPR_regDPR_rule,
  /*  248 */  _LoadF_memory__rule,
  /*  249 */  _MulF__LoadF_memory__regFPR_rule,
  /*  250 */  _MulF_regFPR__LoadF_memory__rule,
  /*  251 */  _MulF_regFPR_regFPR_rule,
  /*  252 */  _SubF_regFPR_regFPR_rule,
  /*  253 */  _ConvI2L_rRegI__rule,
  /*  254 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  255 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  256 */  _Binary_eDIRegP_eSIRegP_rule,
  /*  257 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  258 */  _Binary_eSIRegP_immI_rule,
  /*  259 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  260 */  _AndI_rRegI_immI_rule,
  /*  261 */  _AndI_rRegI_memory_rule,
  /*  262 */  _LoadP_memory__rule,
  /*  263 */  _Binary_nadxRegI_eAXRegI_rule,
  /*  264 */  _PartialSubtypeCheck_eSIRegP_eAXRegP_rule,
  /*  265 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  266 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /*  267 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  268 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  269 */  _Binary_cmpOpU_commute_flagsReg_ulong_LEGT_rule,
  /*  270 */  _Binary_vec_vec_rule,
  /*  271 */  _Binary_vec_rRegI_rule,
  /*  272 */  _Binary_vec_regF_rule,
  /*  273 */  _LoadVector_memory__rule,
  /*  274 */  _Binary_cmpOp_vcmppd_immI8_rule,
  /*  275 */  _Binary_legVec_legVec_rule,
  /*  276 */  _LShiftCntV_immI8__rule,
  /*  277 */  _RShiftCntV_immI8__rule,
  /*  278 */  _Binary_vec__LoadVector_memory__rule,
  /*  279 */  _MulAddVS2VI_vec_vec_rule,
  /*  280 */  _Binary_vec_immU8_rule,
  /*  281 */  _Binary__LoadVector_memory__immU8_rule,
  // last internally defined operand
  /*  282 */  stackSlotI_rule,
  /*  283 */  stackSlotP_rule,
  /*  284 */  stackSlotL_rule,
  /*  285 */  legRegF_rule,
  /*  286 */  regF_rule,
  /*  287 */  legRegD_rule,
  /*  288 */  regD_rule,
  /*  289 */  eRegP_rule,
  /*  290 */  eRegP_rule,
  /*  291 */  eRegP_rule,
  /*  292 */  eRegP_rule,
  /*  293 */  eRegP_rule,
  /*  294 */  rRegI_rule,
  /*  295 */  rRegI_rule,
  /*  296 */  eRegP_rule,
  /*  297 */  eRegL_rule,
  /*  298 */  eRegL_rule,
  /*  299 */  regFPR_rule,
  /*  300 */  regFPR_rule,
  /*  301 */  regFPR_rule,
  /*  302 */  regF_rule,
  /*  303 */  regF_rule,
  /*  304 */  regDPR_rule,
  /*  305 */  regDPR_rule,
  /*  306 */  regDPR_rule,
  /*  307 */  regD_rule,
  /*  308 */  regD_rule,
  /*  309 */  rRegI_rule,
  /*  310 */  eRegL_rule,
  /*  311 */  eRegP_rule,
  /*  312 */  regFPR_rule,
  /*  313 */  regDPR_rule,
  /*  314 */  vlRegD_rule,
  /*  315 */  regD_rule,
  /*  316 */  vlRegF_rule,
  /*  317 */  regF_rule,
  /*  318 */  eADXRegL_low_only_rule,
  /*  319 */  legVec_rule,
  /*  320 */  vec_rule,
  /*  321 */  Universe_rule,
  /*  322 */  eAXRegP_rule,
  /*  323 */  rRegI_rule,
  /*  324 */  eRegP_rule,
  /*  325 */  eRegP_rule,
  /*  326 */  rRegI_rule,
  /*  327 */  eRegL_rule,
  /*  328 */  eFlagsReg_rule,
  /*  329 */  eFlagsReg_rule,
  /*  330 */  eFlagsReg_rule,
  /*  331 */  0,
  /*  332 */  0,
  /*  333 */  eFlagsReg_rule,
  /*  334 */  eFlagsReg_rule,
  /*  335 */  eFlagsReg_rule,
  /*  336 */  eFlagsReg_rule,
  /*  337 */  eFlagsReg_rule,
  /*  338 */  eFlagsReg_rule,
  /*  339 */  eFlagsReg_rule,
  /*  340 */  eFlagsRegU_rule,
  /*  341 */  eFlagsRegUCF_rule,
  /*  342 */  eFlagsRegU_rule,
  /*  343 */  eFlagsRegU_rule,
  /*  344 */  eFlagsRegUCF_rule,
  /*  345 */  eFlagsRegU_rule,
  /*  346 */  eFlagsRegUCF_rule,
  /*  347 */  regDPR1_rule,
  /*  348 */  regDPR1_rule,
  /*  349 */  eFlagsRegU_rule,
  /*  350 */  eFlagsRegUCF_rule,
  /*  351 */  eFlagsRegU_rule,
  /*  352 */  eFlagsRegU_rule,
  /*  353 */  eFlagsRegUCF_rule,
  /*  354 */  eFlagsRegU_rule,
  /*  355 */  eFlagsRegUCF_rule,
  /*  356 */  regFPR1_rule,
  /*  357 */  regFPR1_rule,
  /*  358 */  rRegI_rule,
  /*  359 */  rRegI_rule,
  /*  360 */  regFPR_rule,
  /*  361 */  regF_rule,
  /*  362 */  regF_rule,
  /*  363 */  eRegL_rule,
  /*  364 */  regDPR_rule,
  /*  365 */  regD_rule,
  /*  366 */  regD_rule,
  /*  367 */  eFlagsReg_rule,
  /*  368 */  eFlagsReg_rule,
  /*  369 */  eFlagsReg_rule,
  /*  370 */  eFlagsReg_rule,
  /*  371 */  eFlagsReg_rule,
  /*  372 */  eFlagsReg_rule,
  /*  373 */  eFlagsRegU_rule,
  /*  374 */  eFlagsRegU_rule,
  /*  375 */  eFlagsRegU_rule,
  /*  376 */  eFlagsRegU_rule,
  /*  377 */  eFlagsRegU_rule,
  /*  378 */  eFlagsRegU_rule,
  /*  379 */  eFlagsRegU_rule,
  /*  380 */  eFlagsRegU_rule,
  /*  381 */  eFlagsReg_rule,
  /*  382 */  eFlagsReg_rule,
  /*  383 */  eFlagsReg_rule,
  /*  384 */  flagsReg_long_LTGE_rule,
  /*  385 */  flagsReg_long_LTGE_rule,
  /*  386 */  flagsReg_ulong_LTGE_rule,
  /*  387 */  flagsReg_ulong_LTGE_rule,
  /*  388 */  flagsReg_long_EQNE_rule,
  /*  389 */  flagsReg_long_EQNE_rule,
  /*  390 */  flagsReg_ulong_EQNE_rule,
  /*  391 */  flagsReg_ulong_EQNE_rule,
  /*  392 */  flagsReg_long_LEGT_rule,
  /*  393 */  flagsReg_long_LEGT_rule,
  /*  394 */  flagsReg_ulong_LEGT_rule,
  /*  395 */  flagsReg_ulong_LEGT_rule,
  /*  396 */  eFlagsReg_rule,
  /*  397 */  eFlagsReg_rule,
  /*  398 */  eFlagsReg_rule,
  /*  399 */  regF_rule,
  /*  400 */  vlRegF_rule,
  /*  401 */  regD_rule,
  /*  402 */  vlRegD_rule,
  /*  403 */  regF_rule,
  /*  404 */  vlRegF_rule,
  /*  405 */  regD_rule,
  /*  406 */  vlRegD_rule,
  /*  407 */  vec_rule,
  /*  408 */  legVec_rule,
  /*  409 */  vec_rule,
  /*  410 */  vec_rule,
  /*  411 */  vec_rule,
  /*  412 */  vec_rule,
  /*  413 */  vec_rule,
  /*  414 */  vec_rule,
  /*  415 */  vec_rule,
  /*  416 */  vec_rule,
  /*  417 */  vec_rule,
  /*  418 */  vec_rule,
  /*  419 */  vec_rule,
  /*  420 */  vec_rule,
  /*  421 */  vec_rule,
  /*  422 */  vec_rule,
  /*  423 */  vec_rule,
  /*  424 */  vec_rule,
  /*  425 */  vec_rule,
  /*  426 */  vec_rule,
  /*  427 */  vec_rule,
  /*  428 */  vec_rule,
  /*  429 */  vec_rule,
  /*  430 */  vec_rule,
  /*  431 */  rRegI_rule,
  /*  432 */  eRegL_rule,
  /*  433 */  rRegI_rule,
  /*  434 */  rRegI_rule,
  /*  435 */  rRegI_rule,
  /*  436 */  rRegI_rule,
  /*  437 */  rRegI_rule,
  /*  438 */  rRegI_rule,
  /*  439 */  rRegI_rule,
  /*  440 */  rRegI_rule,
  /*  441 */  rRegI_rule,
  /*  442 */  rRegI_rule,
  /*  443 */  rRegI_rule,
  /*  444 */  rRegI_rule,
  /*  445 */  rRegI_rule,
  /*  446 */  rRegI_rule,
  /*  447 */  xRegI_rule,
  /*  448 */  eRegL_rule,
  /*  449 */  xRegI_rule,
  /*  450 */  eRegL_rule,
  /*  451 */  eRegL_rule,
  /*  452 */  rRegI_rule,
  /*  453 */  rRegI_rule,
  /*  454 */  eRegL_rule,
  /*  455 */  rRegI_rule,
  /*  456 */  rRegI_rule,
  /*  457 */  eRegL_rule,
  /*  458 */  eRegL_rule,
  /*  459 */  eRegL_rule,
  /*  460 */  rRegI_rule,
  /*  461 */  rRegI_rule,
  /*  462 */  rRegI_rule,
  /*  463 */  rRegI_rule,
  /*  464 */  rRegI_rule,
  /*  465 */  eRegL_rule,
  /*  466 */  eRegL_rule,
  /*  467 */  eRegL_rule,
  /*  468 */  eRegL_rule,
  /*  469 */  eRegL_rule,
  /*  470 */  eRegL_rule,
  /*  471 */  stackSlotL_rule,
  /*  472 */  stackSlotL_rule,
  /*  473 */  eRegL_rule,
  /*  474 */  rRegI_rule,
  /*  475 */  eRegP_rule,
  /*  476 */  eRegP_rule,
  /*  477 */  regDPR_rule,
  /*  478 */  regD_rule,
  /*  479 */  regD_rule,
  /*  480 */  regF_rule,
  /*  481 */  regFPR_rule,
  /*  482 */  Universe_rule,
  /*  483 */  Universe_rule,
  /*  484 */  Universe_rule,
  /*  485 */  Universe_rule,
  /*  486 */  Universe_rule,
  /*  487 */  Universe_rule,
  /*  488 */  Universe_rule,
  /*  489 */  Universe_rule,
  /*  490 */  Universe_rule,
  /*  491 */  Universe_rule,
  /*  492 */  Universe_rule,
  /*  493 */  Universe_rule,
  /*  494 */  Universe_rule,
  /*  495 */  Universe_rule,
  /*  496 */  Universe_rule,
  /*  497 */  Universe_rule,
  /*  498 */  Universe_rule,
  /*  499 */  Universe_rule,
  /*  500 */  Universe_rule,
  /*  501 */  Universe_rule,
  /*  502 */  Universe_rule,
  /*  503 */  Universe_rule,
  /*  504 */  Universe_rule,
  /*  505 */  Universe_rule,
  /*  506 */  Universe_rule,
  /*  507 */  Universe_rule,
  /*  508 */  Universe_rule,
  /*  509 */  Universe_rule,
  /*  510 */  Universe_rule,
  /*  511 */  Universe_rule,
  /*  512 */  Universe_rule,
  /*  513 */  Universe_rule,
  /*  514 */  Universe_rule,
  /*  515 */  Universe_rule,
  /*  516 */  Universe_rule,
  /*  517 */  Universe_rule,
  /*  518 */  rRegI_rule,
  /*  519 */  rRegI_rule,
  /*  520 */  rRegI_rule,
  /*  521 */  rRegI_rule,
  /*  522 */  rRegI_rule,
  /*  523 */  rRegI_rule,
  /*  524 */  rRegI_rule,
  /*  525 */  rRegI_rule,
  /*  526 */  eRegP_rule,
  /*  527 */  eRegP_rule,
  /*  528 */  eRegP_rule,
  /*  529 */  eRegP_rule,
  /*  530 */  regDPR1_rule,
  /*  531 */  regFPR1_rule,
  /*  532 */  regDPR_rule,
  /*  533 */  regFPR_rule,
  /*  534 */  regF_rule,
  /*  535 */  regD_rule,
  /*  536 */  regF_rule,
  /*  537 */  regF_rule,
  /*  538 */  regD_rule,
  /*  539 */  regD_rule,
  /*  540 */  eRegL_rule,
  /*  541 */  eRegL_rule,
  /*  542 */  eRegL_rule,
  /*  543 */  rRegI_rule,
  /*  544 */  rRegI_rule,
  /*  545 */  rRegI_rule,
  /*  546 */  rRegI_rule,
  /*  547 */  eRegP_rule,
  /*  548 */  rRegI_rule,
  /*  549 */  eRegP_rule,
  /*  550 */  eRegP_rule,
  /*  551 */  rRegI_rule,
  /*  552 */  rRegI_rule,
  /*  553 */  Universe_rule,
  /*  554 */  Universe_rule,
  /*  555 */  Universe_rule,
  /*  556 */  Universe_rule,
  /*  557 */  Universe_rule,
  /*  558 */  eRegP_rule,
  /*  559 */  rRegI_rule,
  /*  560 */  rRegI_rule,
  /*  561 */  rRegI_rule,
  /*  562 */  rRegI_rule,
  /*  563 */  rRegI_rule,
  /*  564 */  rRegI_rule,
  /*  565 */  rRegI_rule,
  /*  566 */  rRegI_rule,
  /*  567 */  rRegI_rule,
  /*  568 */  rRegI_rule,
  /*  569 */  eADXRegL_rule,
  /*  570 */  eAXRegP_rule,
  /*  571 */  eAXRegI_rule,
  /*  572 */  eAXRegI_rule,
  /*  573 */  eAXRegI_rule,
  /*  574 */  Universe_rule,
  /*  575 */  xRegI_rule,
  /*  576 */  Universe_rule,
  /*  577 */  rRegI_rule,
  /*  578 */  Universe_rule,
  /*  579 */  rRegI_rule,
  /*  580 */  xRegI_rule,
  /*  581 */  rRegI_rule,
  /*  582 */  rRegI_rule,
  /*  583 */  pRegP_rule,
  /*  584 */  rRegI_rule,
  /*  585 */  rRegI_rule,
  /*  586 */  rRegI_rule,
  /*  587 */  Universe_rule,
  /*  588 */  eRegP_rule,
  /*  589 */  rRegI_rule,
  /*  590 */  rRegI_rule,
  /*  591 */  rRegI_rule,
  /*  592 */  eDXRegI_rule,
  /*  593 */  eDXRegI_rule,
  /*  594 */  eDXRegI_rule,
  /*  595 */  eDXRegI_rule,
  /*  596 */  rRegI_rule,
  /*  597 */  rRegI_rule,
  /*  598 */  rRegI_rule,
  /*  599 */  rRegI_rule,
  /*  600 */  eADXRegL_rule,
  /*  601 */  eADXRegL_rule,
  /*  602 */  eADXRegL_rule,
  /*  603 */  eADXRegL_rule,
  /*  604 */  eADXRegL_rule,
  /*  605 */  eADXRegL_rule,
  /*  606 */  eADXRegL_rule,
  /*  607 */  eADXRegL_rule,
  /*  608 */  eADXRegL_rule,
  /*  609 */  eAXRegI_rule,
  /*  610 */  eADXRegL_rule,
  /*  611 */  Universe_rule,
  /*  612 */  eDXRegI_rule,
  /*  613 */  eADXRegL_rule,
  /*  614 */  eADXRegL_rule,
  /*  615 */  eADXRegL_rule,
  /*  616 */  rRegI_rule,
  /*  617 */  rRegI_rule,
  /*  618 */  rRegI_rule,
  /*  619 */  rRegI_rule,
  /*  620 */  Universe_rule,
  /*  621 */  rRegI_rule,
  /*  622 */  Universe_rule,
  /*  623 */  rRegI_rule,
  /*  624 */  rRegI_rule,
  /*  625 */  rRegI_rule,
  /*  626 */  rRegI_rule,
  /*  627 */  rRegI_rule,
  /*  628 */  rRegI_rule,
  /*  629 */  rRegI_rule,
  /*  630 */  rRegI_rule,
  /*  631 */  rRegI_rule,
  /*  632 */  rRegI_rule,
  /*  633 */  Universe_rule,
  /*  634 */  Universe_rule,
  /*  635 */  Universe_rule,
  /*  636 */  rRegI_rule,
  /*  637 */  rRegI_rule,
  /*  638 */  rRegI_rule,
  /*  639 */  rRegI_rule,
  /*  640 */  rRegI_rule,
  /*  641 */  rRegI_rule,
  /*  642 */  rRegI_rule,
  /*  643 */  rRegI_rule,
  /*  644 */  rRegI_rule,
  /*  645 */  rRegI_rule,
  /*  646 */  rRegI_rule,
  /*  647 */  rRegI_rule,
  /*  648 */  rRegI_rule,
  /*  649 */  rRegI_rule,
  /*  650 */  rRegI_rule,
  /*  651 */  rRegI_rule,
  /*  652 */  rRegI_rule,
  /*  653 */  rRegI_rule,
  /*  654 */  rRegI_rule,
  /*  655 */  rRegI_rule,
  /*  656 */  rRegI_rule,
  /*  657 */  rRegI_rule,
  /*  658 */  Universe_rule,
  /*  659 */  Universe_rule,
  /*  660 */  Universe_rule,
  /*  661 */  0,
  /*  662 */  0,
  /*  663 */  0,
  /*  664 */  rRegI_rule,
  /*  665 */  rRegI_rule,
  /*  666 */  rRegI_rule,
  /*  667 */  rRegI_rule,
  /*  668 */  ncxRegI_rule,
  /*  669 */  ncxRegI_rule,
  /*  670 */  ncxRegI_rule,
  /*  671 */  ncxRegI_rule,
  /*  672 */  0,
  /*  673 */  0,
  /*  674 */  0,
  /*  675 */  rRegI_rule,
  /*  676 */  rRegI_rule,
  /*  677 */  rRegI_rule,
  /*  678 */  rRegI_rule,
  /*  679 */  ncxRegI_rule,
  /*  680 */  ncxRegI_rule,
  /*  681 */  ncxRegI_rule,
  /*  682 */  ncxRegI_rule,
  /*  683 */  rRegI_rule,
  /*  684 */  rRegI_rule,
  /*  685 */  rRegI_rule,
  /*  686 */  rRegI_rule,
  /*  687 */  rRegI_rule,
  /*  688 */  Universe_rule,
  /*  689 */  Universe_rule,
  /*  690 */  Universe_rule,
  /*  691 */  0,
  /*  692 */  rRegI_rule,
  /*  693 */  0,
  /*  694 */  rRegI_rule,
  /*  695 */  eCXRegI_rule,
  /*  696 */  rRegI_rule,
  /*  697 */  rRegI_rule,
  /*  698 */  rRegI_rule,
  /*  699 */  rRegI_rule,
  /*  700 */  rRegI_rule,
  /*  701 */  rRegI_rule,
  /*  702 */  rRegI_rule,
  /*  703 */  rRegI_rule,
  /*  704 */  eRegL_rule,
  /*  705 */  eRegL_rule,
  /*  706 */  eRegL_rule,
  /*  707 */  eRegL_rule,
  /*  708 */  eRegL_rule,
  /*  709 */  eRegL_rule,
  /*  710 */  eRegL_rule,
  /*  711 */  eRegL_rule,
  /*  712 */  eRegL_rule,
  /*  713 */  eRegL_rule,
  /*  714 */  eRegL_rule,
  /*  715 */  eRegL_rule,
  /*  716 */  eRegL_rule,
  /*  717 */  eRegL_rule,
  /*  718 */  eRegL_rule,
  /*  719 */  eRegL_rule,
  /*  720 */  eRegL_rule,
  /*  721 */  eRegL_rule,
  /*  722 */  eRegL_rule,
  /*  723 */  eRegL_rule,
  /*  724 */  eRegL_rule,
  /*  725 */  eRegL_rule,
  /*  726 */  eRegL_rule,
  /*  727 */  eRegL_rule,
  /*  728 */  eRegL_rule,
  /*  729 */  eRegL_rule,
  /*  730 */  eRegL_rule,
  /*  731 */  eRegL_rule,
  /*  732 */  eRegL_rule,
  /*  733 */  eRegL_rule,
  /*  734 */  eRegL_rule,
  /*  735 */  eRegL_rule,
  /*  736 */  eRegL_rule,
  /*  737 */  eRegL_rule,
  /*  738 */  eRegL_rule,
  /*  739 */  eRegL_rule,
  /*  740 */  eRegL_rule,
  /*  741 */  eRegL_rule,
  /*  742 */  eRegL_rule,
  /*  743 */  eRegL_rule,
  /*  744 */  eRegL_rule,
  /*  745 */  eRegL_rule,
  /*  746 */  eRegL_rule,
  /*  747 */  eRegL_rule,
  /*  748 */  eRegL_rule,
  /*  749 */  eRegL_rule,
  /*  750 */  eRegL_rule,
  /*  751 */  eRegL_rule,
  /*  752 */  eRegL_rule,
  /*  753 */  rRegI_rule,
  /*  754 */  rRegI_rule,
  /*  755 */  xRegI_rule,
  /*  756 */  xRegI_rule,
  /*  757 */  regDPR_rule,
  /*  758 */  stackSlotD_rule,
  /*  759 */  regDPR_rule,
  /*  760 */  regDPR_rule,
  /*  761 */  stackSlotD_rule,
  /*  762 */  regDPR_rule,
  /*  763 */  regDPR_rule,
  /*  764 */  Universe_rule,
  /*  765 */  Universe_rule,
  /*  766 */  regDPR_rule,
  /*  767 */  regDPR_rule,
  /*  768 */  stackSlotD_rule,
  /*  769 */  regDPR_rule,
  /*  770 */  regDPR1_rule,
  /*  771 */  regDPR_rule,
  /*  772 */  regDPR_rule,
  /*  773 */  regDPR_rule,
  /*  774 */  regDPR_rule,
  /*  775 */  regDPR_rule,
  /*  776 */  regDPR_rule,
  /*  777 */  regDPR_rule,
  /*  778 */  regDPR_rule,
  /*  779 */  regDPR_rule,
  /*  780 */  regDPR1_rule,
  /*  781 */  stackSlotD_rule,
  /*  782 */  regDPR_rule,
  /*  783 */  regD_rule,
  /*  784 */  regDPR_rule,
  /*  785 */  regD_rule,
  /*  786 */  regDPR_rule,
  /*  787 */  rRegI_rule,
  /*  788 */  rRegI_rule,
  /*  789 */  xRegI_rule,
  /*  790 */  xRegI_rule,
  /*  791 */  stackSlotF_rule,
  /*  792 */  regFPR_rule,
  /*  793 */  stackSlotF_rule,
  /*  794 */  regFPR_rule,
  /*  795 */  stackSlotF_rule,
  /*  796 */  stackSlotF_rule,
  /*  797 */  regFPR_rule,
  /*  798 */  regFPR_rule,
  /*  799 */  stackSlotF_rule,
  /*  800 */  stackSlotF_rule,
  /*  801 */  stackSlotF_rule,
  /*  802 */  stackSlotF_rule,
  /*  803 */  stackSlotF_rule,
  /*  804 */  regFPR_rule,
  /*  805 */  stackSlotF_rule,
  /*  806 */  regFPR_rule,
  /*  807 */  stackSlotF_rule,
  /*  808 */  stackSlotF_rule,
  /*  809 */  regFPR_rule,
  /*  810 */  regFPR_rule,
  /*  811 */  stackSlotF_rule,
  /*  812 */  stackSlotF_rule,
  /*  813 */  regFPR_rule,
  /*  814 */  regFPR_rule,
  /*  815 */  regFPR_rule,
  /*  816 */  regFPR_rule,
  /*  817 */  regFPR_rule,
  /*  818 */  regFPR_rule,
  /*  819 */  regFPR_rule,
  /*  820 */  regFPR_rule,
  /*  821 */  regFPR_rule,
  /*  822 */  regFPR_rule,
  /*  823 */  stackSlotF_rule,
  /*  824 */  regFPR_rule,
  /*  825 */  stackSlotF_rule,
  /*  826 */  regFPR_rule,
  /*  827 */  regF_rule,
  /*  828 */  stackSlotF_rule,
  /*  829 */  stackSlotD_rule,
  /*  830 */  stackSlotF_rule,
  /*  831 */  regF_rule,
  /*  832 */  regF_rule,
  /*  833 */  regDPR_rule,
  /*  834 */  stackSlotD_rule,
  /*  835 */  regDPR_rule,
  /*  836 */  regD_rule,
  /*  837 */  eAXRegI_rule,
  /*  838 */  eAXRegI_rule,
  /*  839 */  eADXRegL_rule,
  /*  840 */  eADXRegL_rule,
  /*  841 */  eAXRegI_rule,
  /*  842 */  eAXRegI_rule,
  /*  843 */  eADXRegL_rule,
  /*  844 */  eADXRegL_rule,
  /*  845 */  regDPR_rule,
  /*  846 */  regD_rule,
  /*  847 */  regD_rule,
  /*  848 */  regD_rule,
  /*  849 */  regDPR_rule,
  /*  850 */  regFPR_rule,
  /*  851 */  stackSlotF_rule,
  /*  852 */  stackSlotF_rule,
  /*  853 */  regFPR_rule,
  /*  854 */  regFPR_rule,
  /*  855 */  regF_rule,
  /*  856 */  regF_rule,
  /*  857 */  eRegL_rule,
  /*  858 */  eRegL_rule,
  /*  859 */  eRegL_rule,
  /*  860 */  stackSlotD_rule,
  /*  861 */  regD_rule,
  /*  862 */  regF_rule,
  /*  863 */  stackSlotF_rule,
  /*  864 */  rRegI_rule,
  /*  865 */  stackSlotI_rule,
  /*  866 */  stackSlotI_rule,
  /*  867 */  stackSlotF_rule,
  /*  868 */  stackSlotL_rule,
  /*  869 */  stackSlotL_rule,
  /*  870 */  eRegL_rule,
  /*  871 */  stackSlotD_rule,
  /*  872 */  regD_rule,
  /*  873 */  Universe_rule,
  /*  874 */  Universe_rule,
  /*  875 */  eAXRegI_rule,
  /*  876 */  eAXRegI_rule,
  /*  877 */  eAXRegI_rule,
  /*  878 */  eAXRegI_rule,
  /*  879 */  eAXRegI_rule,
  /*  880 */  eBXRegI_rule,
  /*  881 */  eBXRegI_rule,
  /*  882 */  eBXRegI_rule,
  /*  883 */  eBXRegI_rule,
  /*  884 */  eBXRegI_rule,
  /*  885 */  eBXRegI_rule,
  /*  886 */  eBXRegI_rule,
  /*  887 */  eBXRegI_rule,
  /*  888 */  eAXRegI_rule,
  /*  889 */  eAXRegI_rule,
  /*  890 */  eAXRegI_rule,
  /*  891 */  eAXRegI_rule,
  /*  892 */  Universe_rule,
  /*  893 */  eAXRegI_rule,
  /*  894 */  rRegI_rule,
  /*  895 */  rRegI_rule,
  /*  896 */  eAXRegI_rule,
  /*  897 */  Universe_rule,
  /*  898 */  Universe_rule,
  /*  899 */  Universe_rule,
  /*  900 */  Universe_rule,
  /*  901 */  Universe_rule,
  /*  902 */  Universe_rule,
  /*  903 */  Universe_rule,
  /*  904 */  Universe_rule,
  /*  905 */  Universe_rule,
  /*  906 */  Universe_rule,
  /*  907 */  Universe_rule,
  /*  908 */  Universe_rule,
  /*  909 */  eDIRegP_rule,
  /*  910 */  Universe_rule,
  /*  911 */  Universe_rule,
  /*  912 */  Universe_rule,
  /*  913 */  Universe_rule,
  /*  914 */  Universe_rule,
  /*  915 */  Universe_rule,
  /*  916 */  Universe_rule,
  /*  917 */  Universe_rule,
  /*  918 */  eSIRegI_rule,
  /*  919 */  Universe_rule,
  /*  920 */  Universe_rule,
  /*  921 */  eRegL_rule,
  /*  922 */  eRegL_rule,
  /*  923 */  rRegI_rule,
  /*  924 */  rRegI_rule,
  /*  925 */  eRegP_rule,
  /*  926 */  regDPR_rule,
  /*  927 */  regD_rule,
  /*  928 */  regFPR_rule,
  /*  929 */  regF_rule,
  /*  930 */  Universe_rule,
  /*  931 */  Universe_rule,
  /*  932 */  eRegL_rule,
  /*  933 */  eRegL_rule,
  /*  934 */  rRegI_rule,
  /*  935 */  rRegI_rule,
  /*  936 */  eRegP_rule,
  /*  937 */  regDPR_rule,
  /*  938 */  regD_rule,
  /*  939 */  regFPR_rule,
  /*  940 */  regF_rule,
  /*  941 */  Universe_rule,
  /*  942 */  Universe_rule,
  /*  943 */  eRegL_rule,
  /*  944 */  eRegL_rule,
  /*  945 */  eRegL_rule,
  /*  946 */  eRegL_rule,
  /*  947 */  rRegI_rule,
  /*  948 */  rRegI_rule,
  /*  949 */  eRegP_rule,
  /*  950 */  regDPR_rule,
  /*  951 */  regD_rule,
  /*  952 */  regFPR_rule,
  /*  953 */  regF_rule,
  /*  954 */  Universe_rule,
  /*  955 */  Universe_rule,
  /*  956 */  Universe_rule,
  /*  957 */  Universe_rule,
  /*  958 */  Universe_rule,
  /*  959 */  Universe_rule,
  /*  960 */  Universe_rule,
  /*  961 */  Universe_rule,
  /*  962 */  eAXRegP_rule,
  /*  963 */  Universe_rule,
  /*  964 */  Universe_rule,
  /*  965 */  eRegP_rule,
  /*  966 */  Universe_rule,
  /*  967 */  rRegI_rule,
  /*  968 */  regF_rule,
  /*  969 */  regF_rule,
  /*  970 */  regF_rule,
  /*  971 */  regF_rule,
  /*  972 */  regF_rule,
  /*  973 */  regF_rule,
  /*  974 */  regF_rule,
  /*  975 */  regF_rule,
  /*  976 */  regD_rule,
  /*  977 */  regD_rule,
  /*  978 */  regD_rule,
  /*  979 */  regD_rule,
  /*  980 */  regD_rule,
  /*  981 */  regD_rule,
  /*  982 */  regD_rule,
  /*  983 */  regD_rule,
  /*  984 */  regF_rule,
  /*  985 */  regF_rule,
  /*  986 */  regF_rule,
  /*  987 */  regF_rule,
  /*  988 */  regF_rule,
  /*  989 */  regF_rule,
  /*  990 */  regD_rule,
  /*  991 */  regD_rule,
  /*  992 */  regD_rule,
  /*  993 */  regD_rule,
  /*  994 */  regD_rule,
  /*  995 */  regD_rule,
  /*  996 */  regF_rule,
  /*  997 */  regF_rule,
  /*  998 */  regF_rule,
  /*  999 */  regF_rule,
  /* 1000 */  regF_rule,
  /* 1001 */  regF_rule,
  /* 1002 */  regF_rule,
  /* 1003 */  regF_rule,
  /* 1004 */  regD_rule,
  /* 1005 */  regD_rule,
  /* 1006 */  regD_rule,
  /* 1007 */  regD_rule,
  /* 1008 */  regD_rule,
  /* 1009 */  regD_rule,
  /* 1010 */  regD_rule,
  /* 1011 */  regD_rule,
  /* 1012 */  regF_rule,
  /* 1013 */  regF_rule,
  /* 1014 */  regF_rule,
  /* 1015 */  regF_rule,
  /* 1016 */  regF_rule,
  /* 1017 */  regF_rule,
  /* 1018 */  regD_rule,
  /* 1019 */  regD_rule,
  /* 1020 */  regD_rule,
  /* 1021 */  regD_rule,
  /* 1022 */  regD_rule,
  /* 1023 */  regD_rule,
  /* 1024 */  regF_rule,
  /* 1025 */  regF_rule,
  /* 1026 */  regF_rule,
  /* 1027 */  regD_rule,
  /* 1028 */  regD_rule,
  /* 1029 */  regD_rule,
  /* 1030 */  vec_rule,
  /* 1031 */  legVec_rule,
  /* 1032 */  Universe_rule,
  /* 1033 */  regD_rule,
  /* 1034 */  regF_rule,
  /* 1035 */  vec_rule,
  /* 1036 */  Universe_rule,
  /* 1037 */  legVec_rule,
  /* 1038 */  vec_rule,
  /* 1039 */  Universe_rule,
  /* 1040 */  vec_rule,
  /* 1041 */  vec_rule,
  /* 1042 */  vec_rule,
  /* 1043 */  vec_rule,
  /* 1044 */  vec_rule,
  /* 1045 */  vec_rule,
  /* 1046 */  vec_rule,
  /* 1047 */  vec_rule,
  /* 1048 */  vec_rule,
  /* 1049 */  vec_rule,
  /* 1050 */  vec_rule,
  /* 1051 */  vec_rule,
  /* 1052 */  vec_rule,
  /* 1053 */  vec_rule,
  /* 1054 */  vec_rule,
  /* 1055 */  vec_rule,
  /* 1056 */  legVec_rule,
  /* 1057 */  vec_rule,
  /* 1058 */  vec_rule,
  /* 1059 */  vec_rule,
  /* 1060 */  vec_rule,
  /* 1061 */  vec_rule,
  /* 1062 */  vec_rule,
  /* 1063 */  vec_rule,
  /* 1064 */  vec_rule,
  /* 1065 */  vec_rule,
  /* 1066 */  vec_rule,
  /* 1067 */  vec_rule,
  /* 1068 */  vec_rule,
  /* 1069 */  vec_rule,
  /* 1070 */  vec_rule,
  /* 1071 */  vec_rule,
  /* 1072 */  rRegI_rule,
  /* 1073 */  rRegI_rule,
  /* 1074 */  rRegI_rule,
  /* 1075 */  rRegI_rule,
  /* 1076 */  rRegI_rule,
  /* 1077 */  rRegI_rule,
  /* 1078 */  rRegI_rule,
  /* 1079 */  regF_rule,
  /* 1080 */  regF_rule,
  /* 1081 */  regF_rule,
  /* 1082 */  regF_rule,
  /* 1083 */  regF_rule,
  /* 1084 */  regF_rule,
  /* 1085 */  regD_rule,
  /* 1086 */  regD_rule,
  /* 1087 */  regD_rule,
  /* 1088 */  regD_rule,
  /* 1089 */  regD_rule,
  /* 1090 */  regD_rule,
  /* 1091 */  rRegI_rule,
  /* 1092 */  rRegI_rule,
  /* 1093 */  rRegI_rule,
  /* 1094 */  rRegI_rule,
  /* 1095 */  rRegI_rule,
  /* 1096 */  rRegI_rule,
  /* 1097 */  rRegI_rule,
  /* 1098 */  rRegI_rule,
  /* 1099 */  rRegI_rule,
  /* 1100 */  legRegF_rule,
  /* 1101 */  legRegF_rule,
  /* 1102 */  legRegF_rule,
  /* 1103 */  legRegF_rule,
  /* 1104 */  legRegF_rule,
  /* 1105 */  legRegF_rule,
  /* 1106 */  legRegF_rule,
  /* 1107 */  legRegF_rule,
  /* 1108 */  legRegD_rule,
  /* 1109 */  legRegD_rule,
  /* 1110 */  legRegD_rule,
  /* 1111 */  legRegD_rule,
  /* 1112 */  legRegD_rule,
  /* 1113 */  legRegD_rule,
  /* 1114 */  legRegD_rule,
  /* 1115 */  legRegD_rule,
  /* 1116 */  vec_rule,
  /* 1117 */  vec_rule,
  /* 1118 */  vec_rule,
  /* 1119 */  vec_rule,
  /* 1120 */  vec_rule,
  /* 1121 */  vec_rule,
  /* 1122 */  vec_rule,
  /* 1123 */  vec_rule,
  /* 1124 */  vec_rule,
  /* 1125 */  vec_rule,
  /* 1126 */  vec_rule,
  /* 1127 */  vec_rule,
  /* 1128 */  vec_rule,
  /* 1129 */  vec_rule,
  /* 1130 */  vec_rule,
  /* 1131 */  vec_rule,
  /* 1132 */  vec_rule,
  /* 1133 */  vec_rule,
  /* 1134 */  vec_rule,
  /* 1135 */  vec_rule,
  /* 1136 */  vec_rule,
  /* 1137 */  vec_rule,
  /* 1138 */  vec_rule,
  /* 1139 */  vec_rule,
  /* 1140 */  vec_rule,
  /* 1141 */  vec_rule,
  /* 1142 */  vec_rule,
  /* 1143 */  vec_rule,
  /* 1144 */  vec_rule,
  /* 1145 */  vec_rule,
  /* 1146 */  vec_rule,
  /* 1147 */  vec_rule,
  /* 1148 */  vec_rule,
  /* 1149 */  vec_rule,
  /* 1150 */  vec_rule,
  /* 1151 */  vec_rule,
  /* 1152 */  vec_rule,
  /* 1153 */  vec_rule,
  /* 1154 */  vec_rule,
  /* 1155 */  vec_rule,
  /* 1156 */  vec_rule,
  /* 1157 */  vec_rule,
  /* 1158 */  vec_rule,
  /* 1159 */  vec_rule,
  /* 1160 */  vec_rule,
  /* 1161 */  vec_rule,
  /* 1162 */  vec_rule,
  /* 1163 */  vec_rule,
  /* 1164 */  vec_rule,
  /* 1165 */  vec_rule,
  /* 1166 */  vec_rule,
  /* 1167 */  vec_rule,
  /* 1168 */  vec_rule,
  /* 1169 */  vec_rule,
  /* 1170 */  vec_rule,
  /* 1171 */  vec_rule,
  /* 1172 */  vec_rule,
  /* 1173 */  vec_rule,
  /* 1174 */  vec_rule,
  /* 1175 */  vec_rule,
  /* 1176 */  vec_rule,
  /* 1177 */  vec_rule,
  /* 1178 */  vec_rule,
  /* 1179 */  vec_rule,
  /* 1180 */  vec_rule,
  /* 1181 */  vec_rule,
  /* 1182 */  vec_rule,
  /* 1183 */  vec_rule,
  /* 1184 */  legVec_rule,
  /* 1185 */  legVec_rule,
  /* 1186 */  vec_rule,
  /* 1187 */  vec_rule,
  /* 1188 */  vec_rule,
  /* 1189 */  vec_rule,
  /* 1190 */  vec_rule,
  /* 1191 */  vec_rule,
  /* 1192 */  vec_rule,
  /* 1193 */  vec_rule,
  /* 1194 */  vec_rule,
  /* 1195 */  vec_rule,
  /* 1196 */  vec_rule,
  /* 1197 */  vec_rule,
  /* 1198 */  legVec_rule,
  /* 1199 */  legVec_rule,
  /* 1200 */  vec_rule,
  /* 1201 */  vec_rule,
  /* 1202 */  legVec_rule,
  /* 1203 */  legVec_rule,
  /* 1204 */  vec_rule,
  /* 1205 */  vec_rule,
  /* 1206 */  vec_rule,
  /* 1207 */  vec_rule,
  /* 1208 */  vec_rule,
  /* 1209 */  vec_rule,
  /* 1210 */  vec_rule,
  /* 1211 */  vec_rule,
  /* 1212 */  vec_rule,
  /* 1213 */  vec_rule,
  /* 1214 */  vec_rule,
  /* 1215 */  vec_rule,
  /* 1216 */  vec_rule,
  /* 1217 */  vec_rule,
  /* 1218 */  vec_rule,
  /* 1219 */  vec_rule,
  /* 1220 */  vec_rule,
  /* 1221 */  vec_rule,
  /* 1222 */  vec_rule,
  /* 1223 */  vec_rule,
  /* 1224 */  vec_rule,
  /* 1225 */  vec_rule,
  /* 1226 */  vec_rule,
  /* 1227 */  vec_rule,
  /* 1228 */  vec_rule,
  /* 1229 */  vec_rule,
  /* 1230 */  vec_rule,
  /* 1231 */  vec_rule,
  /* 1232 */  vec_rule,
  /* 1233 */  vec_rule,
  /* 1234 */  vec_rule,
  /* 1235 */  vec_rule,
  /* 1236 */  vec_rule,
  /* 1237 */  vec_rule,
  /* 1238 */  vec_rule,
  /* 1239 */  vec_rule,
  /* 1240 */  vec_rule,
  /* 1241 */  vec_rule,
  /* 1242 */  vec_rule,
  /* 1243 */  vec_rule,
  /* 1244 */  vec_rule,
  /* 1245 */  vec_rule,
  /* 1246 */  vec_rule,
  /* 1247 */  vec_rule,
  /* 1248 */  vec_rule,
  /* 1249 */  vec_rule,
  /* 1250 */  vec_rule,
  /* 1251 */  vec_rule,
  /* 1252 */  vec_rule,
  /* 1253 */  vec_rule,
  /* 1254 */  vec_rule,
  /* 1255 */  vec_rule,
  /* 1256 */  vec_rule,
  /* 1257 */  vec_rule,
  /* 1258 */  vec_rule,
  /* 1259 */  vec_rule,
  /* 1260 */  vec_rule,
  /* 1261 */  vec_rule,
  /* 1262 */  vec_rule,
  /* 1263 */  vec_rule,
  /* 1264 */  vec_rule,
  /* 1265 */  vec_rule,
  /* 1266 */  vec_rule,
  /* 1267 */  vec_rule,
  /* 1268 */  vec_rule,
  /* 1269 */  vec_rule,
  /* 1270 */  vec_rule,
  /* 1271 */  vec_rule,
  /* 1272 */  vec_rule,
  /* 1273 */  vec_rule,
  /* 1274 */  vec_rule,
  /* 1275 */  vec_rule,
  /* 1276 */  vec_rule,
  /* 1277 */  vec_rule,
  /* 1278 */  vec_rule,
  /* 1279 */  vec_rule,
  /* 1280 */  vec_rule,
  /* 1281 */  vec_rule,
  /* 1282 */  vec_rule,
  /* 1283 */  vec_rule,
  /* 1284 */  legVec_rule,
  /* 1285 */  vec_rule,
  /* 1286 */  legVec_rule,
  /* 1287 */  vec_rule,
  /* 1288 */  rRegI_rule,
  /* 1289 */  rRegI_rule,
  /* 1290 */  rRegI_rule,
  /* 1291 */  rRegI_rule,
  /* 1292 */  legRegF_rule,
  /* 1293 */  legRegF_rule,
  /* 1294 */  legRegD_rule,
  /* 1295 */  legRegD_rule,
  /* 1296 */  vec_rule,
  /* 1297 */  legVec_rule,
  /* 1298 */  legVec_rule,
  /* 1299 */  vec_rule,
  /* 1300 */  vec_rule,
  /* 1301 */  vec_rule,
  /* 1302 */  vec_rule,
  /* 1303 */  vec_rule,
  /* 1304 */  vec_rule,
  /* 1305 */  vec_rule,
  /* 1306 */  vec_rule,
  /* 1307 */  vec_rule,
  /* 1308 */  vec_rule,
  /* 1309 */  vec_rule,
  /* 1310 */  vec_rule,
  /* 1311 */  vec_rule,
  /* 1312 */  vec_rule,
  /* 1313 */  vec_rule,
  /* 1314 */  vec_rule,
  /* 1315 */  vec_rule,
  /* 1316 */  vec_rule,
  /* 1317 */  vec_rule,
  /* 1318 */  vec_rule,
  /* 1319 */  vec_rule,
  /* 1320 */  vec_rule,
  /* 1321 */  vec_rule,
  /* 1322 */  vec_rule,
  /* 1323 */  vec_rule,
  /* 1324 */  vec_rule,
  /* 1325 */  vec_rule,
  /* 1326 */  vec_rule,
  /* 1327 */  vec_rule,
  /* 1328 */  vec_rule,
  /* 1329 */  vec_rule,
  /* 1330 */  vec_rule,
  /* 1331 */  vec_rule,
  /* 1332 */  vec_rule,
  /* 1333 */  vec_rule,
  /* 1334 */  vec_rule,
  /* 1335 */  vec_rule,
  /* 1336 */  vec_rule,
  /* 1337 */  vec_rule,
  /* 1338 */  vec_rule,
  /* 1339 */  vec_rule,
  /* 1340 */  vec_rule,
  /* 1341 */  vec_rule,
  /* 1342 */  vec_rule,
  /* 1343 */  vec_rule,
  /* 1344 */  rRegI_rule,
  /* 1345 */  rRegI_rule,
  /* 1346 */  eAXRegP_rule,
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  eRegP_rule,
  /*   99 */  eRegP_rule,
  /*  100 */  immP_rule,
  /*  101 */  _AddP_eRegP_rRegI_rule,
  /*  102 */  eRegP_rule,
  /*  103 */  eRegP_rule,
  /*  104 */  _AddP_eRegP__LShiftI_rRegI_immI2_rule,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  load_long_RegP_rule,
  /*  108 */  0,
  /*  109 */  0,
  /*  110 */  0,
  /*  111 */  0,
  /*  112 */  0,
  /*  113 */  0,
  /*  114 */  0,
  /*  115 */  0,
  /*  116 */  0,
  /*  117 */  0,
  /*  118 */  0,
  /*  119 */  0,
  /*  120 */  0,
  /*  121 */  0,
  /*  122 */  0,
  /*  123 */  0,
  /*  124 */  0,
  /*  125 */  0,
  /*  126 */  0,
  /*  127 */  0,
  /*  128 */  0,
  /*  129 */  0,
  /*  130 */  0,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  // last operand
  /*  134 */  0,
  /*  135 */  0,
  /*  136 */  0,
  // last operand class
  /*  137 */  eRegP_rule,
  /*  138 */  rRegI_rule,
  /*  139 */  eRegP_rule,
  /*  140 */  memory_rule,
  /*  141 */  memory_rule,
  /*  142 */  memory_rule,
  /*  143 */  memory_rule,
  /*  144 */  _LoadUB_memory__rule,
  /*  145 */  memory_rule,
  /*  146 */  _LoadS_memory__rule,
  /*  147 */  memory_rule,
  /*  148 */  _LoadUS_memory__rule,
  /*  149 */  _LoadUS_memory__rule,
  /*  150 */  _LoadUS_memory__rule,
  /*  151 */  _LoadI_memory__rule,
  /*  152 */  _LoadI_memory__rule,
  /*  153 */  _LoadI_memory__rule,
  /*  154 */  _LoadI_memory__rule,
  /*  155 */  _LoadI_memory__rule,
  /*  156 */  _LoadI_memory__rule,
  /*  157 */  eRegL_rule,
  /*  158 */  regDPR1_rule,
  /*  159 */  regFPR1_rule,
  /*  160 */  regDPR1_rule,
  /*  161 */  cmpOp_rule,
  /*  162 */  rRegI_rule,
  /*  163 */  cmpOpU_rule,
  /*  164 */  cmpOpUCF_rule,
  /*  165 */  rRegI_rule,
  /*  166 */  eRegP_rule,
  /*  167 */  cmpOp_fcmov_rule,
  /*  168 */  regDPR1_rule,
  /*  169 */  regFPR1_rule,
  /*  170 */  regDPR_rule,
  /*  171 */  regFPR_rule,
  /*  172 */  regF_rule,
  /*  173 */  regD_rule,
  /*  174 */  eRegL_rule,
  /*  175 */  _LoadI_memory__rule,
  /*  176 */  rRegI_rule,
  /*  177 */  _LoadI_memory__rule,
  /*  178 */  _LoadI_memory__rule,
  /*  179 */  _LoadI_memory__rule,
  /*  180 */  eAXRegP_rule,
  /*  181 */  eAXRegI_rule,
  /*  182 */  eADXRegL_rule,
  /*  183 */  eAXRegP_rule,
  /*  184 */  eAXRegI_rule,
  /*  185 */  _LoadI_memory__rule,
  /*  186 */  immI_0_rule,
  /*  187 */  nadxRegI_rule,
  /*  188 */  _ConvI2L_nadxRegI__rule,
  /*  189 */  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule,
  /*  190 */  eADXRegL_low_only_rule,
  /*  191 */  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule,
  /*  192 */  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule,
  /*  193 */  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule,
  /*  194 */  eAXRegI_rule,
  /*  195 */  _ConvI2L_eAXRegI__rule,
  /*  196 */  _ConvI2L_nadxRegI__rule,
  /*  197 */  _LoadI_memory__rule,
  /*  198 */  _LoadI_memory__rule,
  /*  199 */  xRegI_rule,
  /*  200 */  xRegI_rule,
  /*  201 */  _LoadI_memory__rule,
  /*  202 */  rRegI_rule,
  /*  203 */  _LoadI_memory__rule,
  /*  204 */  rRegI_rule,
  /*  205 */  immI_0_rule,
  /*  206 */  rRegI_rule,
  /*  207 */  eRegP_rule,
  /*  208 */  _LoadI_memory__rule,
  /*  209 */  rRegI_rule,
  /*  210 */  _LoadI_memory__rule,
  /*  211 */  rRegI_rule,
  /*  212 */  rRegI_rule,
  /*  213 */  rRegI_rule,
  /*  214 */  rRegI_rule,
  /*  215 */  ncxRegI_rule,
  /*  216 */  immI_0_rule,
  /*  217 */  ncxRegI_rule,
  /*  218 */  immI_32_rule,
  /*  219 */  ncxRegI_rule,
  /*  220 */  rRegI_rule,
  /*  221 */  rRegI_rule,
  /*  222 */  ncxRegI_rule,
  /*  223 */  ncxRegI_rule,
  /*  224 */  ncxRegI_rule,
  /*  225 */  _LoadI_memory__rule,
  /*  226 */  rRegI_rule,
  /*  227 */  _LoadI_memory__rule,
  /*  228 */  rRegI_rule,
  /*  229 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  230 */  rRegI_rule,
  /*  231 */  rRegI_rule,
  /*  232 */  load_long_memory_rule,
  /*  233 */  eRegL_rule,
  /*  234 */  immL0_rule,
  /*  235 */  immL0_rule,
  /*  236 */  eRegL_rule,
  /*  237 */  _LoadL_memory__rule,
  /*  238 */  memory_rule,
  /*  239 */  regDPR_rule,
  /*  240 */  regDPR_rule,
  /*  241 */  _LoadD_memory__rule,
  /*  242 */  _AddD__LoadD_memory__regDPR_rule,
  /*  243 */  regDPR_rule,
  /*  244 */  _AddD_regDPR__LoadD_memory__rule,
  /*  245 */  regDPR_rule,
  /*  246 */  regDPR_rule,
  /*  247 */  regDPR_rule,
  /*  248 */  memory_rule,
  /*  249 */  _LoadF_memory__rule,
  /*  250 */  regFPR_rule,
  /*  251 */  regFPR_rule,
  /*  252 */  regFPR_rule,
  /*  253 */  rRegI_rule,
  /*  254 */  eDIRegP_rule,
  /*  255 */  eSIRegP_rule,
  /*  256 */  eDIRegP_rule,
  /*  257 */  eDIRegP_rule,
  /*  258 */  eSIRegP_rule,
  /*  259 */  eSIRegP_rule,
  /*  260 */  rRegI_rule,
  /*  261 */  rRegI_rule,
  /*  262 */  memory_rule,
  /*  263 */  nadxRegI_rule,
  /*  264 */  eSIRegP_rule,
  /*  265 */  cmpOp_rule,
  /*  266 */  eRegL_rule,
  /*  267 */  cmpOp_rule,
  /*  268 */  cmpOp_commute_rule,
  /*  269 */  cmpOpU_commute_rule,
  /*  270 */  vec_rule,
  /*  271 */  vec_rule,
  /*  272 */  vec_rule,
  /*  273 */  memory_rule,
  /*  274 */  cmpOp_vcmppd_rule,
  /*  275 */  legVec_rule,
  /*  276 */  immI8_rule,
  /*  277 */  immI8_rule,
  /*  278 */  vec_rule,
  /*  279 */  vec_rule,
  /*  280 */  vec_rule,
  /*  281 */  _LoadVector_memory__rule,
  // last internally defined operand
  /*  282 */  rRegI_rule,
  /*  283 */  eRegP_rule,
  /*  284 */  eRegL_rule,
  /*  285 */  regF_rule,
  /*  286 */  legRegF_rule,
  /*  287 */  regD_rule,
  /*  288 */  legRegD_rule,
  /*  289 */  indOffset8_rule,
  /*  290 */  indOffset32_rule,
  /*  291 */  indIndexOffset_rule,
  /*  292 */  indIndexScale_rule,
  /*  293 */  indIndexScaleOffset_rule,
  /*  294 */  immI_rule,
  /*  295 */  immI_0_rule,
  /*  296 */  immP_rule,
  /*  297 */  immL_rule,
  /*  298 */  immL0_rule,
  /*  299 */  immFPR_rule,
  /*  300 */  immFPR0_rule,
  /*  301 */  immFPR1_rule,
  /*  302 */  immF_rule,
  /*  303 */  immF0_rule,
  /*  304 */  immDPR_rule,
  /*  305 */  immDPR0_rule,
  /*  306 */  immDPR1_rule,
  /*  307 */  immD_rule,
  /*  308 */  immD0_rule,
  /*  309 */  stackSlotI_rule,
  /*  310 */  stackSlotL_rule,
  /*  311 */  stackSlotP_rule,
  /*  312 */  stackSlotF_rule,
  /*  313 */  stackSlotD_rule,
  /*  314 */  regD_rule,
  /*  315 */  vlRegD_rule,
  /*  316 */  regF_rule,
  /*  317 */  vlRegF_rule,
  /*  318 */  immL32_rule,
  /*  319 */  vec_rule,
  /*  320 */  legVec_rule,
  /*  321 */  0,
  /*  322 */  eAXRegI_rule,
  /*  323 */  eRegP_rule,
  /*  324 */  eRegP_rule,
  /*  325 */  eRegP_rule,
  /*  326 */  rRegI_rule,
  /*  327 */  eRegL_rule,
  /*  328 */  memory_rule,
  /*  329 */  memory_rule,
  /*  330 */  memory_rule,
  /*  331 */  0,
  /*  332 */  0,
  /*  333 */  eAXRegI_rule,
  /*  334 */  eAXRegI_rule,
  /*  335 */  rRegI_rule,
  /*  336 */  rRegI_rule,
  /*  337 */  immI_0_rule,
  /*  338 */  eAXRegI_rule,
  /*  339 */  rRegI_rule,
  /*  340 */  regDPR_rule,
  /*  341 */  regDPR_rule,
  /*  342 */  regDPR_rule,
  /*  343 */  regD_rule,
  /*  344 */  regD_rule,
  /*  345 */  regD_rule,
  /*  346 */  regD_rule,
  /*  347 */  regDPR1_rule,
  /*  348 */  regDPR1_rule,
  /*  349 */  regFPR_rule,
  /*  350 */  regFPR_rule,
  /*  351 */  regFPR_rule,
  /*  352 */  regF_rule,
  /*  353 */  regF_rule,
  /*  354 */  regF_rule,
  /*  355 */  regF_rule,
  /*  356 */  regFPR1_rule,
  /*  357 */  regFPR1_rule,
  /*  358 */  stackSlotF_rule,
  /*  359 */  regF_rule,
  /*  360 */  stackSlotI_rule,
  /*  361 */  stackSlotI_rule,
  /*  362 */  rRegI_rule,
  /*  363 */  stackSlotD_rule,
  /*  364 */  stackSlotL_rule,
  /*  365 */  stackSlotL_rule,
  /*  366 */  stackSlotL_rule,
  /*  367 */  rRegI_rule,
  /*  368 */  rRegI_rule,
  /*  369 */  rRegI_rule,
  /*  370 */  rRegI_rule,
  /*  371 */  _AndI_rRegI_immI_rule,
  /*  372 */  _AndI_rRegI_memory_rule,
  /*  373 */  rRegI_rule,
  /*  374 */  rRegI_rule,
  /*  375 */  rRegI_rule,
  /*  376 */  rRegI_rule,
  /*  377 */  eRegP_rule,
  /*  378 */  eRegP_rule,
  /*  379 */  eRegP_rule,
  /*  380 */  eRegP_rule,
  /*  381 */  eRegP_rule,
  /*  382 */  _LoadP_memory__rule,
  /*  383 */  _PartialSubtypeCheck_eSIRegP_eAXRegP_rule,
  /*  384 */  eRegL_rule,
  /*  385 */  eRegL_rule,
  /*  386 */  eRegL_rule,
  /*  387 */  eRegL_rule,
  /*  388 */  eRegL_rule,
  /*  389 */  eRegL_rule,
  /*  390 */  eRegL_rule,
  /*  391 */  eRegL_rule,
  /*  392 */  eRegL_rule,
  /*  393 */  eRegL_rule,
  /*  394 */  eRegL_rule,
  /*  395 */  eRegL_rule,
  /*  396 */  eRegP_rule,
  /*  397 */  eRegP_rule,
  /*  398 */  eRegP_rule,
  /*  399 */  regF_rule,
  /*  400 */  vlRegF_rule,
  /*  401 */  regD_rule,
  /*  402 */  vlRegD_rule,
  /*  403 */  regF_rule,
  /*  404 */  vlRegF_rule,
  /*  405 */  regD_rule,
  /*  406 */  vlRegD_rule,
  /*  407 */  vec_rule,
  /*  408 */  vec_rule,
  /*  409 */  legVec_rule,
  /*  410 */  vec_rule,
  /*  411 */  _LoadVector_memory__rule,
  /*  412 */  vec_rule,
  /*  413 */  _LoadVector_memory__rule,
  /*  414 */  rRegI_rule,
  /*  415 */  rRegI_rule,
  /*  416 */  vec_rule,
  /*  417 */  vec_rule,
  /*  418 */  vec_rule,
  /*  419 */  vec_rule,
  /*  420 */  vec_rule,
  /*  421 */  vec_rule,
  /*  422 */  vec_rule,
  /*  423 */  vec_rule,
  /*  424 */  vec_rule,
  /*  425 */  vec_rule,
  /*  426 */  vec_rule,
  /*  427 */  vec_rule,
  /*  428 */  vec_rule,
  /*  429 */  vec_rule,
  /*  430 */  vec_rule,
  /*  431 */  rRegI_rule,
  /*  432 */  eRegL_rule,
  /*  433 */  rRegI_rule,
  /*  434 */  rRegI_rule,
  /*  435 */  rRegI_rule,
  /*  436 */  rRegI_rule,
  /*  437 */  eRegL_rule,
  /*  438 */  eRegL_rule,
  /*  439 */  rRegI_rule,
  /*  440 */  rRegI_rule,
  /*  441 */  eRegL_rule,
  /*  442 */  eRegL_rule,
  /*  443 */  rRegI_rule,
  /*  444 */  _LoadI_memory__rule,
  /*  445 */  eRegL_rule,
  /*  446 */  _LoadL_memory__rule,
  /*  447 */  memory_rule,
  /*  448 */  _LoadB_memory__rule,
  /*  449 */  memory_rule,
  /*  450 */  _LoadUB_memory__rule,
  /*  451 */  _AndI__LoadUB_memory__immI_rule,
  /*  452 */  memory_rule,
  /*  453 */  _LShiftI__LoadS_memory__immI_24_rule,
  /*  454 */  _LoadS_memory__rule,
  /*  455 */  memory_rule,
  /*  456 */  _LShiftI__LoadUS_memory__immI_24_rule,
  /*  457 */  _LoadUS_memory__rule,
  /*  458 */  _AndI__LoadUS_memory__immI_255_rule,
  /*  459 */  _AndI__LoadUS_memory__immI_rule,
  /*  460 */  memory_rule,
  /*  461 */  _LShiftI__LoadI_memory__immI_24_rule,
  /*  462 */  _LoadI_memory__rule,
  /*  463 */  _LShiftI__LoadI_memory__immI_16_rule,
  /*  464 */  _LoadI_memory__rule,
  /*  465 */  _LoadI_memory__rule,
  /*  466 */  _AndI__LoadI_memory__immI_255_rule,
  /*  467 */  _AndI__LoadI_memory__immI_65535_rule,
  /*  468 */  _AndI__LoadI_memory__immU31_rule,
  /*  469 */  _ConvI2L__LoadI_memory___rule,
  /*  470 */  load_long_memory_rule,
  /*  471 */  memory_rule,
  /*  472 */  memory_rule,
  /*  473 */  memory_rule,
  /*  474 */  memory_rule,
  /*  475 */  memory_rule,
  /*  476 */  memory_rule,
  /*  477 */  memory_rule,
  /*  478 */  memory_rule,
  /*  479 */  memory_rule,
  /*  480 */  memory_rule,
  /*  481 */  memory_rule,
  /*  482 */  memory_rule,
  /*  483 */  memory_rule,
  /*  484 */  memory_rule,
  /*  485 */  memory_rule,
  /*  486 */  memory_rule,
  /*  487 */  memory_rule,
  /*  488 */  memory_rule,
  /*  489 */  memory_rule,
  /*  490 */  long_memory_rule,
  /*  491 */  memory_rule,
  /*  492 */  memory_rule,
  /*  493 */  memory_rule,
  /*  494 */  memory_rule,
  /*  495 */  memory_rule,
  /*  496 */  memory_rule,
  /*  497 */  memory_rule,
  /*  498 */  memory_rule,
  /*  499 */  memory_rule,
  /*  500 */  memory_rule,
  /*  501 */  memory_rule,
  /*  502 */  memory_rule,
  /*  503 */  memory_rule,
  /*  504 */  memory_rule,
  /*  505 */  memory_rule,
  /*  506 */  memory_rule,
  /*  507 */  memory_rule,
  /*  508 */  memory_rule,
  /*  509 */  memory_rule,
  /*  510 */  0,
  /*  511 */  0,
  /*  512 */  0,
  /*  513 */  0,
  /*  514 */  0,
  /*  515 */  0,
  /*  516 */  0,
  /*  517 */  0,
  /*  518 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  519 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  520 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  521 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  522 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  523 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  524 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  525 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  526 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  527 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  528 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  529 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  530 */  _Binary_cmpOp_fcmov_eFlagsRegU_rule,
  /*  531 */  _Binary_cmpOp_fcmov_eFlagsRegU_rule,
  /*  532 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  533 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  534 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  535 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  536 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  537 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  538 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  539 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  540 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  541 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  542 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  543 */  rRegI_rule,
  /*  544 */  rRegI_rule,
  /*  545 */  rRegI_rule,
  /*  546 */  rRegI_rule,
  /*  547 */  eRegP_rule,
  /*  548 */  rRegI_rule,
  /*  549 */  eRegP_rule,
  /*  550 */  eRegP_rule,
  /*  551 */  rRegI_rule,
  /*  552 */  _LoadI_memory__rule,
  /*  553 */  memory_rule,
  /*  554 */  memory_rule,
  /*  555 */  memory_rule,
  /*  556 */  memory_rule,
  /*  557 */  memory_rule,
  /*  558 */  memory_rule,
  /*  559 */  eSIRegP_rule,
  /*  560 */  eSIRegP_rule,
  /*  561 */  pRegP_rule,
  /*  562 */  pRegP_rule,
  /*  563 */  pRegP_rule,
  /*  564 */  pRegP_rule,
  /*  565 */  pRegP_rule,
  /*  566 */  pRegP_rule,
  /*  567 */  pRegP_rule,
  /*  568 */  pRegP_rule,
  /*  569 */  eSIRegP_rule,
  /*  570 */  pRegP_rule,
  /*  571 */  pRegP_rule,
  /*  572 */  pRegP_rule,
  /*  573 */  pRegP_rule,
  /*  574 */  memory_rule,
  /*  575 */  memory_rule,
  /*  576 */  memory_rule,
  /*  577 */  memory_rule,
  /*  578 */  memory_rule,
  /*  579 */  memory_rule,
  /*  580 */  memory_rule,
  /*  581 */  memory_rule,
  /*  582 */  memory_rule,
  /*  583 */  memory_rule,
  /*  584 */  rRegI_rule,
  /*  585 */  rRegI_rule,
  /*  586 */  rRegI_rule,
  /*  587 */  memory_rule,
  /*  588 */  eRegP_rule,
  /*  589 */  immI_0_rule,
  /*  590 */  rRegI_rule,
  /*  591 */  rRegI_rule,
  /*  592 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_rule,
  /*  593 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_rule,
  /*  594 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63_rule,
  /*  595 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63_rule,
  /*  596 */  _LoadI_memory__rule,
  /*  597 */  rRegI_rule,
  /*  598 */  _LoadI_memory__rule,
  /*  599 */  _Binary_rRegI_rRegI_rule,
  /*  600 */  _ConvI2L_eAXRegI__rule,
  /*  601 */  _ConvI2L_nadxRegI__rule,
  /*  602 */  _AndL__ConvI2L_eAXRegI__immL_32bits_rule,
  /*  603 */  _AndL__ConvI2L_nadxRegI__immL_32bits_rule,
  /*  604 */  eADXRegL_rule,
  /*  605 */  eADXRegL_rule,
  /*  606 */  eADXRegL_rule,
  /*  607 */  eADXRegL_rule,
  /*  608 */  eADXRegL_rule,
  /*  609 */  eAXRegI_rule,
  /*  610 */  eRegL_rule,
  /*  611 */  eAXRegI_rule,
  /*  612 */  eAXRegI_rule,
  /*  613 */  eRegL_rule,
  /*  614 */  eADXRegL_rule,
  /*  615 */  eADXRegL_rule,
  /*  616 */  rRegI_rule,
  /*  617 */  rRegI_rule,
  /*  618 */  rRegI_rule,
  /*  619 */  rRegI_rule,
  /*  620 */  memory_rule,
  /*  621 */  rRegI_rule,
  /*  622 */  memory_rule,
  /*  623 */  rRegI_rule,
  /*  624 */  rRegI_rule,
  /*  625 */  rRegI_rule,
  /*  626 */  _LShiftI_xRegI_immI_24_rule,
  /*  627 */  _LShiftI_xRegI_immI_16_rule,
  /*  628 */  rRegI_rule,
  /*  629 */  rRegI_rule,
  /*  630 */  rRegI_rule,
  /*  631 */  rRegI_rule,
  /*  632 */  _LoadI_memory__rule,
  /*  633 */  memory_rule,
  /*  634 */  memory_rule,
  /*  635 */  memory_rule,
  /*  636 */  _XorI_rRegI_immI_M1_rule,
  /*  637 */  rRegI_rule,
  /*  638 */  _XorI_rRegI_immI_M1_rule,
  /*  639 */  _LoadI_memory__rule,
  /*  640 */  _SubI_immI_0_rRegI_rule,
  /*  641 */  rRegI_rule,
  /*  642 */  _SubI_immI_0__LoadI_memory__rule,
  /*  643 */  _LoadI_memory__rule,
  /*  644 */  _AddI_rRegI_immI_M1_rule,
  /*  645 */  rRegI_rule,
  /*  646 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  647 */  _LoadI_memory__rule,
  /*  648 */  _AddI_rRegI_immI_M1_rule,
  /*  649 */  rRegI_rule,
  /*  650 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  651 */  _LoadI_memory__rule,
  /*  652 */  rRegI_rule,
  /*  653 */  rRegI_rule,
  /*  654 */  _CastP2X_eRegP__rule,
  /*  655 */  rRegI_rule,
  /*  656 */  rRegI_rule,
  /*  657 */  _LoadI_memory__rule,
  /*  658 */  memory_rule,
  /*  659 */  memory_rule,
  /*  660 */  memory_rule,
  /*  661 */  0,
  /*  662 */  0,
  /*  663 */  0,
  /*  664 */  _LShiftI_rRegI_immI_1_rule,
  /*  665 */  _URShiftI_rRegI_immI_M1_rule,
  /*  666 */  _LShiftI_rRegI_immI8_rule,
  /*  667 */  _URShiftI_rRegI_immI8_rule,
  /*  668 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  669 */  _URShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  670 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  671 */  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  672 */  0,
  /*  673 */  0,
  /*  674 */  0,
  /*  675 */  _URShiftI_rRegI_immI_1_rule,
  /*  676 */  _LShiftI_rRegI_immI_M1_rule,
  /*  677 */  _URShiftI_rRegI_immI8_rule,
  /*  678 */  _LShiftI_rRegI_immI8_rule,
  /*  679 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  680 */  _LShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  681 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  682 */  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  683 */  rRegI_rule,
  /*  684 */  rRegI_rule,
  /*  685 */  rRegI_rule,
  /*  686 */  rRegI_rule,
  /*  687 */  _LoadI_memory__rule,
  /*  688 */  memory_rule,
  /*  689 */  memory_rule,
  /*  690 */  memory_rule,
  /*  691 */  0,
  /*  692 */  rRegI_rule,
  /*  693 */  0,
  /*  694 */  eRegP_rule,
  /*  695 */  ncxRegI_rule,
  /*  696 */  rRegI_rule,
  /*  697 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  698 */  _SubI_rRegI_rRegI_rule,
  /*  699 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  700 */  _SubI_rRegI_rRegI_rule,
  /*  701 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  702 */  rRegI_rule,
  /*  703 */  rRegI_rule,
  /*  704 */  eRegL_rule,
  /*  705 */  eRegL_rule,
  /*  706 */  eRegL_rule,
  /*  707 */  _LoadL_load_long_memory__rule,
  /*  708 */  eRegL_rule,
  /*  709 */  eRegL_rule,
  /*  710 */  eRegL_rule,
  /*  711 */  immL0_rule,
  /*  712 */  eRegL_rule,
  /*  713 */  eRegL_rule,
  /*  714 */  eRegL_rule,
  /*  715 */  _LoadL_load_long_memory__rule,
  /*  716 */  _XorL_eRegL_immL_M1_rule,
  /*  717 */  eRegL_rule,
  /*  718 */  _XorL_eRegL_immL_M1_rule,
  /*  719 */  _LoadL_memory__rule,
  /*  720 */  _SubL_immL0_eRegL_rule,
  /*  721 */  eRegL_rule,
  /*  722 */  _SubL_immL0__LoadL_memory__rule,
  /*  723 */  _LoadL_memory__rule,
  /*  724 */  _AddL_eRegL_immL_M1_rule,
  /*  725 */  eRegL_rule,
  /*  726 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  727 */  _LoadL_memory__rule,
  /*  728 */  _AddL_eRegL_immL_M1_rule,
  /*  729 */  eRegL_rule,
  /*  730 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  731 */  _LoadL_memory__rule,
  /*  732 */  eRegL_rule,
  /*  733 */  eRegL_rule,
  /*  734 */  eRegL_rule,
  /*  735 */  _LoadL_load_long_memory__rule,
  /*  736 */  eRegL_rule,
  /*  737 */  eRegL_rule,
  /*  738 */  eRegL_rule,
  /*  739 */  eRegL_rule,
  /*  740 */  _LoadL_load_long_memory__rule,
  /*  741 */  eRegL_rule,
  /*  742 */  eRegL_rule,
  /*  743 */  eRegL_rule,
  /*  744 */  eRegL_rule,
  /*  745 */  eRegL_rule,
  /*  746 */  eRegL_rule,
  /*  747 */  eRegL_rule,
  /*  748 */  eRegL_rule,
  /*  749 */  eRegL_rule,
  /*  750 */  eRegL_rule,
  /*  751 */  eRegL_rule,
  /*  752 */  eRegL_rule,
  /*  753 */  regDPR_rule,
  /*  754 */  regDPR_rule,
  /*  755 */  regD_rule,
  /*  756 */  regD_rule,
  /*  757 */  regDPR_rule,
  /*  758 */  _SubD_regDPR_regDPR_rule,
  /*  759 */  regDPR_rule,
  /*  760 */  regDPR_rule,
  /*  761 */  _AddD_regDPR_regDPR_rule,
  /*  762 */  regDPR_rule,
  /*  763 */  _LoadD_memory__rule,
  /*  764 */  memory_rule,
  /*  765 */  memory_rule,
  /*  766 */  regDPR_rule,
  /*  767 */  regDPR_rule,
  /*  768 */  _AddD_regDPR_immDPR_rule,
  /*  769 */  regDPR_rule,
  /*  770 */  regDPR1_rule,
  /*  771 */  regDPR_rule,
  /*  772 */  regDPR_rule,
  /*  773 */  _LoadD_memory__rule,
  /*  774 */  regDPR_rule,
  /*  775 */  _LoadD_memory__rule,
  /*  776 */  _MulD_regDPR_regDPR_rule,
  /*  777 */  regDPR_rule,
  /*  778 */  _MulD_regDPR_regDPR_rule,
  /*  779 */  regDPR_rule,
  /*  780 */  regDPR1_rule,
  /*  781 */  _DivD_regDPR_regDPR_rule,
  /*  782 */  regDPR_rule,
  /*  783 */  regD_rule,
  /*  784 */  regDPR_rule,
  /*  785 */  regD_rule,
  /*  786 */  regDPR_rule,
  /*  787 */  regFPR_rule,
  /*  788 */  regFPR_rule,
  /*  789 */  regF_rule,
  /*  790 */  regF_rule,
  /*  791 */  regFPR_rule,
  /*  792 */  regFPR_rule,
  /*  793 */  regFPR_rule,
  /*  794 */  regFPR_rule,
  /*  795 */  regFPR_rule,
  /*  796 */  _LoadF_memory__rule,
  /*  797 */  regFPR_rule,
  /*  798 */  _LoadF_memory__rule,
  /*  799 */  memory_rule,
  /*  800 */  memory_rule,
  /*  801 */  _LoadF_memory__rule,
  /*  802 */  memory_rule,
  /*  803 */  regFPR_rule,
  /*  804 */  regFPR_rule,
  /*  805 */  regFPR_rule,
  /*  806 */  regFPR_rule,
  /*  807 */  regFPR_rule,
  /*  808 */  _LoadF_memory__rule,
  /*  809 */  regFPR_rule,
  /*  810 */  _LoadF_memory__rule,
  /*  811 */  memory_rule,
  /*  812 */  regFPR_rule,
  /*  813 */  regFPR_rule,
  /*  814 */  _LoadF_memory__rule,
  /*  815 */  regFPR_rule,
  /*  816 */  _MulF__LoadF_memory__regFPR_rule,
  /*  817 */  regFPR_rule,
  /*  818 */  _MulF_regFPR__LoadF_memory__rule,
  /*  819 */  regFPR_rule,
  /*  820 */  _MulF_regFPR_regFPR_rule,
  /*  821 */  regFPR_rule,
  /*  822 */  _SubF_regFPR_regFPR_rule,
  /*  823 */  regFPR_rule,
  /*  824 */  regFPR_rule,
  /*  825 */  regFPR_rule,
  /*  826 */  regFPR_rule,
  /*  827 */  regF_rule,
  /*  828 */  regFPR_rule,
  /*  829 */  regDPR_rule,
  /*  830 */  regDPR_rule,
  /*  831 */  regDPR_rule,
  /*  832 */  regD_rule,
  /*  833 */  regFPR_rule,
  /*  834 */  regFPR_rule,
  /*  835 */  regF_rule,
  /*  836 */  regF_rule,
  /*  837 */  regDPR_rule,
  /*  838 */  regD_rule,
  /*  839 */  regDPR_rule,
  /*  840 */  regD_rule,
  /*  841 */  regFPR_rule,
  /*  842 */  regF_rule,
  /*  843 */  regFPR_rule,
  /*  844 */  regF_rule,
  /*  845 */  stackSlotI_rule,
  /*  846 */  rRegI_rule,
  /*  847 */  _LoadI_memory__rule,
  /*  848 */  rRegI_rule,
  /*  849 */  _LoadI_memory__rule,
  /*  850 */  stackSlotI_rule,
  /*  851 */  stackSlotI_rule,
  /*  852 */  _LoadI_memory__rule,
  /*  853 */  stackSlotI_rule,
  /*  854 */  _LoadI_memory__rule,
  /*  855 */  rRegI_rule,
  /*  856 */  rRegI_rule,
  /*  857 */  rRegI_rule,
  /*  858 */  _ConvI2L_rRegI__rule,
  /*  859 */  eRegL_rule,
  /*  860 */  eRegL_rule,
  /*  861 */  eRegL_rule,
  /*  862 */  eRegL_rule,
  /*  863 */  eRegL_rule,
  /*  864 */  eRegL_rule,
  /*  865 */  regFPR_rule,
  /*  866 */  regF_rule,
  /*  867 */  rRegI_rule,
  /*  868 */  regDPR_rule,
  /*  869 */  regD_rule,
  /*  870 */  regD_rule,
  /*  871 */  eRegL_rule,
  /*  872 */  eRegL_rule,
  /*  873 */  eCXRegI_rule,
  /*  874 */  eCXRegI_rule,
  /*  875 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  876 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  877 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  878 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  879 */  _Binary_eDIRegP_eSIRegP_rule,
  /*  880 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  881 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  882 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  883 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  884 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  885 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  886 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  887 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  888 */  eDIRegP_rule,
  /*  889 */  eDIRegP_rule,
  /*  890 */  eSIRegP_rule,
  /*  891 */  eSIRegP_rule,
  /*  892 */  eSIRegP_rule,
  /*  893 */  eSIRegP_rule,
  /*  894 */  rRegI_rule,
  /*  895 */  rRegI_rule,
  /*  896 */  _Binary_nadxRegI_eAXRegI_rule,
  /*  897 */  rRegI_rule,
  /*  898 */  0,
  /*  899 */  cmpOp_rule,
  /*  900 */  cmpOp_rule,
  /*  901 */  cmpOpU_rule,
  /*  902 */  cmpOpUCF_rule,
  /*  903 */  cmpOp_rule,
  /*  904 */  cmpOpU_rule,
  /*  905 */  cmpOpUCF_rule,
  /*  906 */  cmpOpU_rule,
  /*  907 */  cmpOpUCF_rule,
  /*  908 */  cmpOpUCF2_rule,
  /*  909 */  eSIRegP_rule,
  /*  910 */  0,
  /*  911 */  cmpOp_rule,
  /*  912 */  cmpOp_rule,
  /*  913 */  cmpOpU_rule,
  /*  914 */  cmpOpUCF_rule,
  /*  915 */  cmpOpU_rule,
  /*  916 */  cmpOpUCF_rule,
  /*  917 */  cmpOpUCF2_rule,
  /*  918 */  eRegL_rule,
  /*  919 */  cmpOp_rule,
  /*  920 */  cmpOpU_rule,
  /*  921 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  922 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  923 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  924 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  925 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  926 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  927 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  928 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  929 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  930 */  cmpOp_rule,
  /*  931 */  cmpOpU_rule,
  /*  932 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  933 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  934 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  935 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  936 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  937 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  938 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  939 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  940 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  941 */  cmpOp_commute_rule,
  /*  942 */  cmpOpU_commute_rule,
  /*  943 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  944 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  945 */  _Binary_cmpOpU_commute_flagsReg_ulong_LEGT_rule,
  /*  946 */  _Binary_cmpOpU_commute_flagsReg_ulong_LEGT_rule,
  /*  947 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  948 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  949 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  950 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  951 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  952 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  953 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  954 */  0,
  /*  955 */  0,
  /*  956 */  0,
  /*  957 */  0,
  /*  958 */  0,
  /*  959 */  0,
  /*  960 */  eRegP_no_EBP_rule,
  /*  961 */  eRegP_no_EBP_rule,
  /*  962 */  0,
  /*  963 */  0,
  /*  964 */  eRegP_no_EBP_rule,
  /*  965 */  0,
  /*  966 */  0,
  /*  967 */  rRegI_rule,
  /*  968 */  regF_rule,
  /*  969 */  regF_rule,
  /*  970 */  _LoadF_memory__rule,
  /*  971 */  regF_rule,
  /*  972 */  regF_rule,
  /*  973 */  regF_rule,
  /*  974 */  _LoadF_memory__rule,
  /*  975 */  regF_rule,
  /*  976 */  regD_rule,
  /*  977 */  regD_rule,
  /*  978 */  _LoadD_memory__rule,
  /*  979 */  regD_rule,
  /*  980 */  regD_rule,
  /*  981 */  regD_rule,
  /*  982 */  _LoadD_memory__rule,
  /*  983 */  regD_rule,
  /*  984 */  regF_rule,
  /*  985 */  regF_rule,
  /*  986 */  regF_rule,
  /*  987 */  regF_rule,
  /*  988 */  regF_rule,
  /*  989 */  regF_rule,
  /*  990 */  regD_rule,
  /*  991 */  regD_rule,
  /*  992 */  regD_rule,
  /*  993 */  regD_rule,
  /*  994 */  regD_rule,
  /*  995 */  regD_rule,
  /*  996 */  regF_rule,
  /*  997 */  regF_rule,
  /*  998 */  _LoadF_memory__rule,
  /*  999 */  regF_rule,
  /* 1000 */  regF_rule,
  /* 1001 */  regF_rule,
  /* 1002 */  _LoadF_memory__rule,
  /* 1003 */  regF_rule,
  /* 1004 */  regD_rule,
  /* 1005 */  regD_rule,
  /* 1006 */  _LoadD_memory__rule,
  /* 1007 */  regD_rule,
  /* 1008 */  regD_rule,
  /* 1009 */  regD_rule,
  /* 1010 */  _LoadD_memory__rule,
  /* 1011 */  regD_rule,
  /* 1012 */  regF_rule,
  /* 1013 */  regF_rule,
  /* 1014 */  regF_rule,
  /* 1015 */  regF_rule,
  /* 1016 */  regF_rule,
  /* 1017 */  regF_rule,
  /* 1018 */  regD_rule,
  /* 1019 */  regD_rule,
  /* 1020 */  regD_rule,
  /* 1021 */  regD_rule,
  /* 1022 */  regD_rule,
  /* 1023 */  regD_rule,
  /* 1024 */  regF_rule,
  /* 1025 */  _LoadF_memory__rule,
  /* 1026 */  immF_rule,
  /* 1027 */  regD_rule,
  /* 1028 */  _LoadD_memory__rule,
  /* 1029 */  immD_rule,
  /* 1030 */  vec_rule,
  /* 1031 */  vec_rule,
  /* 1032 */  0,
  /* 1033 */  regD_rule,
  /* 1034 */  regF_rule,
  /* 1035 */  memory_rule,
  /* 1036 */  memory_rule,
  /* 1037 */  memory_rule,
  /* 1038 */  memory_rule,
  /* 1039 */  memory_rule,
  /* 1040 */  rRegI_rule,
  /* 1041 */  _LoadB_memory__rule,
  /* 1042 */  immI_rule,
  /* 1043 */  immI_0_rule,
  /* 1044 */  rRegI_rule,
  /* 1045 */  _LoadS_memory__rule,
  /* 1046 */  immI_rule,
  /* 1047 */  immI_0_rule,
  /* 1048 */  rRegI_rule,
  /* 1049 */  _LoadI_memory__rule,
  /* 1050 */  immI_rule,
  /* 1051 */  immI_0_rule,
  /* 1052 */  immI_M1_rule,
  /* 1053 */  immI_M1_rule,
  /* 1054 */  immI_M1_rule,
  /* 1055 */  eRegL_rule,
  /* 1056 */  eRegL_rule,
  /* 1057 */  _LoadL_memory__rule,
  /* 1058 */  immL_rule,
  /* 1059 */  immL0_rule,
  /* 1060 */  immL_M1_rule,
  /* 1061 */  vlRegF_rule,
  /* 1062 */  _LoadF_memory__rule,
  /* 1063 */  immF0_rule,
  /* 1064 */  vlRegD_rule,
  /* 1065 */  _LoadD_memory__rule,
  /* 1066 */  immD0_rule,
  /* 1067 */  _Binary_vec_rRegI_rule,
  /* 1068 */  _Binary_vec_rRegI_rule,
  /* 1069 */  _Binary_vec_rRegI_rule,
  /* 1070 */  _Binary_vec_regF_rule,
  /* 1071 */  _Binary_vec_regF_rule,
  /* 1072 */  rRegI_rule,
  /* 1073 */  rRegI_rule,
  /* 1074 */  rRegI_rule,
  /* 1075 */  rRegI_rule,
  /* 1076 */  rRegI_rule,
  /* 1077 */  rRegI_rule,
  /* 1078 */  rRegI_rule,
  /* 1079 */  regF_rule,
  /* 1080 */  regF_rule,
  /* 1081 */  regF_rule,
  /* 1082 */  regF_rule,
  /* 1083 */  regF_rule,
  /* 1084 */  regF_rule,
  /* 1085 */  regD_rule,
  /* 1086 */  regD_rule,
  /* 1087 */  regD_rule,
  /* 1088 */  regD_rule,
  /* 1089 */  regD_rule,
  /* 1090 */  regD_rule,
  /* 1091 */  rRegI_rule,
  /* 1092 */  rRegI_rule,
  /* 1093 */  rRegI_rule,
  /* 1094 */  rRegI_rule,
  /* 1095 */  rRegI_rule,
  /* 1096 */  rRegI_rule,
  /* 1097 */  rRegI_rule,
  /* 1098 */  rRegI_rule,
  /* 1099 */  rRegI_rule,
  /* 1100 */  immF_rule,
  /* 1101 */  immF_rule,
  /* 1102 */  immF_rule,
  /* 1103 */  immF_rule,
  /* 1104 */  legRegF_rule,
  /* 1105 */  legRegF_rule,
  /* 1106 */  legRegF_rule,
  /* 1107 */  legRegF_rule,
  /* 1108 */  immD_rule,
  /* 1109 */  immD_rule,
  /* 1110 */  immD_rule,
  /* 1111 */  immD_rule,
  /* 1112 */  legRegD_rule,
  /* 1113 */  legRegD_rule,
  /* 1114 */  legRegD_rule,
  /* 1115 */  legRegD_rule,
  /* 1116 */  vec_rule,
  /* 1117 */  vec_rule,
  /* 1118 */  vec_rule,
  /* 1119 */  _LoadVector_memory__rule,
  /* 1120 */  vec_rule,
  /* 1121 */  vec_rule,
  /* 1122 */  vec_rule,
  /* 1123 */  _LoadVector_memory__rule,
  /* 1124 */  vec_rule,
  /* 1125 */  vec_rule,
  /* 1126 */  vec_rule,
  /* 1127 */  _LoadVector_memory__rule,
  /* 1128 */  vec_rule,
  /* 1129 */  vec_rule,
  /* 1130 */  vec_rule,
  /* 1131 */  _LoadVector_memory__rule,
  /* 1132 */  vec_rule,
  /* 1133 */  vec_rule,
  /* 1134 */  vec_rule,
  /* 1135 */  _LoadVector_memory__rule,
  /* 1136 */  vec_rule,
  /* 1137 */  vec_rule,
  /* 1138 */  vec_rule,
  /* 1139 */  _LoadVector_memory__rule,
  /* 1140 */  vec_rule,
  /* 1141 */  vec_rule,
  /* 1142 */  vec_rule,
  /* 1143 */  vec_rule,
  /* 1144 */  vec_rule,
  /* 1145 */  vec_rule,
  /* 1146 */  vec_rule,
  /* 1147 */  vec_rule,
  /* 1148 */  vec_rule,
  /* 1149 */  vec_rule,
  /* 1150 */  vec_rule,
  /* 1151 */  vec_rule,
  /* 1152 */  vec_rule,
  /* 1153 */  vec_rule,
  /* 1154 */  vec_rule,
  /* 1155 */  vec_rule,
  /* 1156 */  vec_rule,
  /* 1157 */  vec_rule,
  /* 1158 */  vec_rule,
  /* 1159 */  vec_rule,
  /* 1160 */  vec_rule,
  /* 1161 */  vec_rule,
  /* 1162 */  vec_rule,
  /* 1163 */  vec_rule,
  /* 1164 */  vec_rule,
  /* 1165 */  vec_rule,
  /* 1166 */  _LoadVector_memory__rule,
  /* 1167 */  vec_rule,
  /* 1168 */  vec_rule,
  /* 1169 */  vec_rule,
  /* 1170 */  _LoadVector_memory__rule,
  /* 1171 */  vec_rule,
  /* 1172 */  vec_rule,
  /* 1173 */  _LoadVector_memory__rule,
  /* 1174 */  vec_rule,
  /* 1175 */  vec_rule,
  /* 1176 */  vec_rule,
  /* 1177 */  vec_rule,
  /* 1178 */  vec_rule,
  /* 1179 */  _LoadVector_memory__rule,
  /* 1180 */  vec_rule,
  /* 1181 */  vec_rule,
  /* 1182 */  vec_rule,
  /* 1183 */  _LoadVector_memory__rule,
  /* 1184 */  _Binary_cmpOp_vcmppd_immI8_rule,
  /* 1185 */  _Binary_cmpOp_vcmppd_immI8_rule,
  /* 1186 */  vec_rule,
  /* 1187 */  vec_rule,
  /* 1188 */  vec_rule,
  /* 1189 */  vec_rule,
  /* 1190 */  vec_rule,
  /* 1191 */  vec_rule,
  /* 1192 */  vec_rule,
  /* 1193 */  vec_rule,
  /* 1194 */  vec_rule,
  /* 1195 */  vec_rule,
  /* 1196 */  vec_rule,
  /* 1197 */  vec_rule,
  /* 1198 */  legVec_rule,
  /* 1199 */  legVec_rule,
  /* 1200 */  vec_rule,
  /* 1201 */  vec_rule,
  /* 1202 */  legVec_rule,
  /* 1203 */  legVec_rule,
  /* 1204 */  vec_rule,
  /* 1205 */  vec_rule,
  /* 1206 */  vec_rule,
  /* 1207 */  vec_rule,
  /* 1208 */  vec_rule,
  /* 1209 */  vec_rule,
  /* 1210 */  vec_rule,
  /* 1211 */  vec_rule,
  /* 1212 */  vec_rule,
  /* 1213 */  vec_rule,
  /* 1214 */  vec_rule,
  /* 1215 */  vec_rule,
  /* 1216 */  vec_rule,
  /* 1217 */  vec_rule,
  /* 1218 */  vec_rule,
  /* 1219 */  vec_rule,
  /* 1220 */  vec_rule,
  /* 1221 */  vec_rule,
  /* 1222 */  vec_rule,
  /* 1223 */  vec_rule,
  /* 1224 */  vec_rule,
  /* 1225 */  vec_rule,
  /* 1226 */  vec_rule,
  /* 1227 */  vec_rule,
  /* 1228 */  vec_rule,
  /* 1229 */  vec_rule,
  /* 1230 */  vec_rule,
  /* 1231 */  vec_rule,
  /* 1232 */  vec_rule,
  /* 1233 */  vec_rule,
  /* 1234 */  vec_rule,
  /* 1235 */  vec_rule,
  /* 1236 */  vec_rule,
  /* 1237 */  vec_rule,
  /* 1238 */  vec_rule,
  /* 1239 */  vec_rule,
  /* 1240 */  vec_rule,
  /* 1241 */  vec_rule,
  /* 1242 */  vec_rule,
  /* 1243 */  vec_rule,
  /* 1244 */  vec_rule,
  /* 1245 */  vec_rule,
  /* 1246 */  vec_rule,
  /* 1247 */  vec_rule,
  /* 1248 */  vec_rule,
  /* 1249 */  vec_rule,
  /* 1250 */  vec_rule,
  /* 1251 */  vec_rule,
  /* 1252 */  vec_rule,
  /* 1253 */  vec_rule,
  /* 1254 */  vec_rule,
  /* 1255 */  vec_rule,
  /* 1256 */  vec_rule,
  /* 1257 */  vec_rule,
  /* 1258 */  vec_rule,
  /* 1259 */  vec_rule,
  /* 1260 */  vec_rule,
  /* 1261 */  vec_rule,
  /* 1262 */  vec_rule,
  /* 1263 */  vec_rule,
  /* 1264 */  vec_rule,
  /* 1265 */  vec_rule,
  /* 1266 */  vec_rule,
  /* 1267 */  vec_rule,
  /* 1268 */  vec_rule,
  /* 1269 */  vec_rule,
  /* 1270 */  _LoadVector_memory__rule,
  /* 1271 */  vec_rule,
  /* 1272 */  vec_rule,
  /* 1273 */  vec_rule,
  /* 1274 */  _LoadVector_memory__rule,
  /* 1275 */  vec_rule,
  /* 1276 */  vec_rule,
  /* 1277 */  vec_rule,
  /* 1278 */  _LoadVector_memory__rule,
  /* 1279 */  vec_rule,
  /* 1280 */  vec_rule,
  /* 1281 */  vec_rule,
  /* 1282 */  vec_rule,
  /* 1283 */  vec_rule,
  /* 1284 */  _Binary_legVec_legVec_rule,
  /* 1285 */  _Binary_vec_vec_rule,
  /* 1286 */  _Binary_legVec_legVec_rule,
  /* 1287 */  _Binary_vec_vec_rule,
  /* 1288 */  legVec_rule,
  /* 1289 */  legVec_rule,
  /* 1290 */  legVec_rule,
  /* 1291 */  legVec_rule,
  /* 1292 */  legVec_rule,
  /* 1293 */  legVec_rule,
  /* 1294 */  legVec_rule,
  /* 1295 */  legVec_rule,
  /* 1296 */  _Binary_vec_vec_rule,
  /* 1297 */  _Binary_legVec_legVec_rule,
  /* 1298 */  _Binary_legVec_legVec_rule,
  /* 1299 */  _Binary_vec_vec_rule,
  /* 1300 */  vec_rule,
  /* 1301 */  vec_rule,
  /* 1302 */  vec_rule,
  /* 1303 */  vec_rule,
  /* 1304 */  vec_rule,
  /* 1305 */  vec_rule,
  /* 1306 */  vec_rule,
  /* 1307 */  vec_rule,
  /* 1308 */  vec_rule,
  /* 1309 */  vec_rule,
  /* 1310 */  vec_rule,
  /* 1311 */  vec_rule,
  /* 1312 */  vec_rule,
  /* 1313 */  vec_rule,
  /* 1314 */  vec_rule,
  /* 1315 */  vec_rule,
  /* 1316 */  vec_rule,
  /* 1317 */  immI_0_rule,
  /* 1318 */  vec_rule,
  /* 1319 */  vec_rule,
  /* 1320 */  vec_rule,
  /* 1321 */  vec_rule,
  /* 1322 */  vec_rule,
  /* 1323 */  vec_rule,
  /* 1324 */  vec_rule,
  /* 1325 */  vec_rule,
  /* 1326 */  vec_rule,
  /* 1327 */  vec_rule,
  /* 1328 */  vec_rule,
  /* 1329 */  vec_rule,
  /* 1330 */  vec_rule,
  /* 1331 */  vec_rule,
  /* 1332 */  vec_rule,
  /* 1333 */  vec_rule,
  /* 1334 */  vec_rule,
  /* 1335 */  vec_rule,
  /* 1336 */  _MulAddVS2VI_vec_vec_rule,
  /* 1337 */  vec_rule,
  /* 1338 */  _Binary_vec_vec_rule,
  /* 1339 */  _Binary_vec_vec_rule,
  /* 1340 */  vec_rule,
  /* 1341 */  vec_rule,
  /* 1342 */  vec_rule,
  /* 1343 */  vec_rule,
  /* 1344 */  memory_rule,
  /* 1345 */  memory_rule,
  /* 1346 */  memory_rule,
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  immI8_rule,
  /*   99 */  immI_rule,
  /*  100 */  rRegI_rule,
  /*  101 */  immI_rule,
  /*  102 */  rRegI_rule,
  /*  103 */  _LShiftI_rRegI_immI2_rule,
  /*  104 */  immI_rule,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  immI_rule,
  /*  108 */  0,
  /*  109 */  0,
  /*  110 */  0,
  /*  111 */  0,
  /*  112 */  0,
  /*  113 */  0,
  /*  114 */  0,
  /*  115 */  0,
  /*  116 */  0,
  /*  117 */  0,
  /*  118 */  0,
  /*  119 */  0,
  /*  120 */  0,
  /*  121 */  0,
  /*  122 */  0,
  /*  123 */  0,
  /*  124 */  0,
  /*  125 */  0,
  /*  126 */  0,
  /*  127 */  0,
  /*  128 */  0,
  /*  129 */  0,
  /*  130 */  0,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  // last operand
  /*  134 */  0,
  /*  135 */  0,
  /*  136 */  0,
  // last operand class
  /*  137 */  rRegI_rule,
  /*  138 */  immI2_rule,
  /*  139 */  _LShiftI_rRegI_immI2_rule,
  /*  140 */  0,
  /*  141 */  0,
  /*  142 */  0,
  /*  143 */  0,
  /*  144 */  immI_rule,
  /*  145 */  0,
  /*  146 */  immI_24_rule,
  /*  147 */  0,
  /*  148 */  immI_24_rule,
  /*  149 */  immI_255_rule,
  /*  150 */  immI_rule,
  /*  151 */  immI_24_rule,
  /*  152 */  immI_16_rule,
  /*  153 */  immI_255_rule,
  /*  154 */  immI_65535_rule,
  /*  155 */  immU31_rule,
  /*  156 */  0,
  /*  157 */  0,
  /*  158 */  0,
  /*  159 */  0,
  /*  160 */  0,
  /*  161 */  eFlagsReg_rule,
  /*  162 */  rRegI_rule,
  /*  163 */  eFlagsRegU_rule,
  /*  164 */  eFlagsRegUCF_rule,
  /*  165 */  _LoadI_memory__rule,
  /*  166 */  eRegP_rule,
  /*  167 */  eFlagsRegU_rule,
  /*  168 */  regDPR_rule,
  /*  169 */  regFPR_rule,
  /*  170 */  regDPR_rule,
  /*  171 */  regFPR_rule,
  /*  172 */  regF_rule,
  /*  173 */  regD_rule,
  /*  174 */  eRegL_rule,
  /*  175 */  rRegI_rule,
  /*  176 */  _LoadI_memory__rule,
  /*  177 */  immI_rule,
  /*  178 */  immI_1_rule,
  /*  179 */  immI_M1_rule,
  /*  180 */  eRegP_rule,
  /*  181 */  rRegI_rule,
  /*  182 */  eBCXRegL_rule,
  /*  183 */  eCXRegP_rule,
  /*  184 */  eCXRegI_rule,
  /*  185 */  rRegI_rule,
  /*  186 */  rRegI_rule,
  /*  187 */  0,
  /*  188 */  eADXRegL_low_only_rule,
  /*  189 */  immI_32_rule,
  /*  190 */  _ConvI2L_nadxRegI__rule,
  /*  191 */  immI_32_rule,
  /*  192 */  immI_32_63_rule,
  /*  193 */  immI_32_63_rule,
  /*  194 */  0,
  /*  195 */  immL_32bits_rule,
  /*  196 */  immL_32bits_rule,
  /*  197 */  immI_1_rule,
  /*  198 */  immI8_rule,
  /*  199 */  immI_24_rule,
  /*  200 */  immI_16_rule,
  /*  201 */  rRegI_rule,
  /*  202 */  _LoadI_memory__rule,
  /*  203 */  immI_rule,
  /*  204 */  immI_M1_rule,
  /*  205 */  _LoadI_memory__rule,
  /*  206 */  immI_M1_rule,
  /*  207 */  0,
  /*  208 */  rRegI_rule,
  /*  209 */  _LoadI_memory__rule,
  /*  210 */  immI_rule,
  /*  211 */  immI_1_rule,
  /*  212 */  immI_M1_rule,
  /*  213 */  immI8_rule,
  /*  214 */  immI8_rule,
  /*  215 */  eCXRegI_rule,
  /*  216 */  eCXRegI_rule,
  /*  217 */  _SubI_immI_0_eCXRegI_rule,
  /*  218 */  eCXRegI_rule,
  /*  219 */  _SubI_immI_32_eCXRegI_rule,
  /*  220 */  immI_1_rule,
  /*  221 */  immI_M1_rule,
  /*  222 */  eCXRegI_rule,
  /*  223 */  _SubI_immI_0_eCXRegI_rule,
  /*  224 */  _SubI_immI_32_eCXRegI_rule,
  /*  225 */  rRegI_rule,
  /*  226 */  _LoadI_memory__rule,
  /*  227 */  immI_rule,
  /*  228 */  rRegI_rule,
  /*  229 */  rRegI_rule,
  /*  230 */  rRegI_rule,
  /*  231 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  232 */  0,
  /*  233 */  immL_M1_rule,
  /*  234 */  eRegL_rule,
  /*  235 */  _LoadL_memory__rule,
  /*  236 */  immL_M1_rule,
  /*  237 */  immL_M1_rule,
  /*  238 */  0,
  /*  239 */  regDPR_rule,
  /*  240 */  regDPR_rule,
  /*  241 */  regDPR_rule,
  /*  242 */  0,
  /*  243 */  _LoadD_memory__rule,
  /*  244 */  0,
  /*  245 */  immDPR_rule,
  /*  246 */  regDPR_rule,
  /*  247 */  regDPR_rule,
  /*  248 */  0,
  /*  249 */  regFPR_rule,
  /*  250 */  _LoadF_memory__rule,
  /*  251 */  regFPR_rule,
  /*  252 */  regFPR_rule,
  /*  253 */  0,
  /*  254 */  eCXRegI_rule,
  /*  255 */  eDXRegI_rule,
  /*  256 */  eSIRegP_rule,
  /*  257 */  eDXRegI_rule,
  /*  258 */  immI_rule,
  /*  259 */  eAXRegI_rule,
  /*  260 */  immI_rule,
  /*  261 */  memory_rule,
  /*  262 */  0,
  /*  263 */  eAXRegI_rule,
  /*  264 */  eAXRegP_rule,
  /*  265 */  flagsReg_long_LTGE_rule,
  /*  266 */  _LoadL_load_long_memory__rule,
  /*  267 */  flagsReg_long_EQNE_rule,
  /*  268 */  flagsReg_long_LEGT_rule,
  /*  269 */  flagsReg_ulong_LEGT_rule,
  /*  270 */  vec_rule,
  /*  271 */  rRegI_rule,
  /*  272 */  regF_rule,
  /*  273 */  0,
  /*  274 */  immI8_rule,
  /*  275 */  legVec_rule,
  /*  276 */  0,
  /*  277 */  0,
  /*  278 */  _LoadVector_memory__rule,
  /*  279 */  vec_rule,
  /*  280 */  immU8_rule,
  /*  281 */  immU8_rule,
  // last internally defined operand
  /*  282 */  0,
  /*  283 */  0,
  /*  284 */  0,
  /*  285 */  0,
  /*  286 */  0,
  /*  287 */  0,
  /*  288 */  0,
  /*  289 */  0,
  /*  290 */  0,
  /*  291 */  0,
  /*  292 */  0,
  /*  293 */  0,
  /*  294 */  0,
  /*  295 */  0,
  /*  296 */  0,
  /*  297 */  0,
  /*  298 */  0,
  /*  299 */  0,
  /*  300 */  0,
  /*  301 */  0,
  /*  302 */  0,
  /*  303 */  0,
  /*  304 */  0,
  /*  305 */  0,
  /*  306 */  0,
  /*  307 */  0,
  /*  308 */  0,
  /*  309 */  0,
  /*  310 */  0,
  /*  311 */  0,
  /*  312 */  0,
  /*  313 */  0,
  /*  314 */  0,
  /*  315 */  0,
  /*  316 */  0,
  /*  317 */  0,
  /*  318 */  0,
  /*  319 */  0,
  /*  320 */  0,
  /*  321 */  0,
  /*  322 */  0,
  /*  323 */  0,
  /*  324 */  0,
  /*  325 */  0,
  /*  326 */  0,
  /*  327 */  0,
  /*  328 */  _Binary_eAXRegP_eRegP_rule,
  /*  329 */  _Binary_eAXRegI_rRegI_rule,
  /*  330 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  331 */  0,
  /*  332 */  0,
  /*  333 */  rRegI_rule,
  /*  334 */  immI_rule,
  /*  335 */  rRegI_rule,
  /*  336 */  immI_rule,
  /*  337 */  eAXRegI_rule,
  /*  338 */  rRegI_rule,
  /*  339 */  immI_rule,
  /*  340 */  regDPR_rule,
  /*  341 */  regDPR_rule,
  /*  342 */  regDPR_rule,
  /*  343 */  regD_rule,
  /*  344 */  regD_rule,
  /*  345 */  _LoadD_memory__rule,
  /*  346 */  _LoadD_memory__rule,
  /*  347 */  0,
  /*  348 */  0,
  /*  349 */  regFPR_rule,
  /*  350 */  regFPR_rule,
  /*  351 */  regFPR_rule,
  /*  352 */  regF_rule,
  /*  353 */  regF_rule,
  /*  354 */  _LoadF_memory__rule,
  /*  355 */  _LoadF_memory__rule,
  /*  356 */  0,
  /*  357 */  0,
  /*  358 */  0,
  /*  359 */  0,
  /*  360 */  0,
  /*  361 */  0,
  /*  362 */  0,
  /*  363 */  0,
  /*  364 */  0,
  /*  365 */  0,
  /*  366 */  0,
  /*  367 */  rRegI_rule,
  /*  368 */  immI_rule,
  /*  369 */  _LoadI_memory__rule,
  /*  370 */  immI_0_rule,
  /*  371 */  immI_0_rule,
  /*  372 */  immI_0_rule,
  /*  373 */  rRegI_rule,
  /*  374 */  immI_rule,
  /*  375 */  _LoadI_memory__rule,
  /*  376 */  immI_0_rule,
  /*  377 */  eRegP_rule,
  /*  378 */  immP_rule,
  /*  379 */  _LoadP_memory__rule,
  /*  380 */  _LoadP_memory__rule,
  /*  381 */  immP0_rule,
  /*  382 */  immI_0_rule,
  /*  383 */  immP0_rule,
  /*  384 */  immL0_rule,
  /*  385 */  eRegL_rule,
  /*  386 */  immL0_rule,
  /*  387 */  eRegL_rule,
  /*  388 */  immL0_rule,
  /*  389 */  eRegL_rule,
  /*  390 */  immL0_rule,
  /*  391 */  eRegL_rule,
  /*  392 */  immL0_rule,
  /*  393 */  eRegL_rule,
  /*  394 */  immL0_rule,
  /*  395 */  eRegL_rule,
  /*  396 */  eBXRegP_rule,
  /*  397 */  eBXRegP_rule,
  /*  398 */  eAXRegP_rule,
  /*  399 */  0,
  /*  400 */  0,
  /*  401 */  0,
  /*  402 */  0,
  /*  403 */  0,
  /*  404 */  0,
  /*  405 */  0,
  /*  406 */  0,
  /*  407 */  0,
  /*  408 */  0,
  /*  409 */  0,
  /*  410 */  0,
  /*  411 */  0,
  /*  412 */  0,
  /*  413 */  0,
  /*  414 */  0,
  /*  415 */  0,
  /*  416 */  0,
  /*  417 */  0,
  /*  418 */  0,
  /*  419 */  0,
  /*  420 */  0,
  /*  421 */  0,
  /*  422 */  0,
  /*  423 */  0,
  /*  424 */  0,
  /*  425 */  0,
  /*  426 */  0,
  /*  427 */  0,
  /*  428 */  0,
  /*  429 */  0,
  /*  430 */  0,
  /*  431 */  0,
  /*  432 */  0,
  /*  433 */  0,
  /*  434 */  0,
  /*  435 */  0,
  /*  436 */  0,
  /*  437 */  0,
  /*  438 */  0,
  /*  439 */  0,
  /*  440 */  0,
  /*  441 */  0,
  /*  442 */  0,
  /*  443 */  0,
  /*  444 */  0,
  /*  445 */  0,
  /*  446 */  0,
  /*  447 */  0,
  /*  448 */  0,
  /*  449 */  0,
  /*  450 */  0,
  /*  451 */  0,
  /*  452 */  0,
  /*  453 */  immI_24_rule,
  /*  454 */  0,
  /*  455 */  0,
  /*  456 */  immI_24_rule,
  /*  457 */  0,
  /*  458 */  0,
  /*  459 */  0,
  /*  460 */  0,
  /*  461 */  immI_24_rule,
  /*  462 */  immI_255_rule,
  /*  463 */  immI_16_rule,
  /*  464 */  immI_65535_rule,
  /*  465 */  0,
  /*  466 */  0,
  /*  467 */  0,
  /*  468 */  0,
  /*  469 */  immL_32bits_rule,
  /*  470 */  0,
  /*  471 */  0,
  /*  472 */  0,
  /*  473 */  0,
  /*  474 */  0,
  /*  475 */  0,
  /*  476 */  0,
  /*  477 */  0,
  /*  478 */  0,
  /*  479 */  0,
  /*  480 */  0,
  /*  481 */  0,
  /*  482 */  0,
  /*  483 */  0,
  /*  484 */  0,
  /*  485 */  0,
  /*  486 */  0,
  /*  487 */  xRegI_rule,
  /*  488 */  rRegI_rule,
  /*  489 */  rRegI_rule,
  /*  490 */  eRegL_rule,
  /*  491 */  _ConvL2I_eRegL__rule,
  /*  492 */  stackSlotL_rule,
  /*  493 */  stackSlotL_rule,
  /*  494 */  eRegL_rule,
  /*  495 */  anyRegP_rule,
  /*  496 */  immI_rule,
  /*  497 */  immI16_rule,
  /*  498 */  immP_rule,
  /*  499 */  immI8_rule,
  /*  500 */  immI8_rule,
  /*  501 */  regDPR1_rule,
  /*  502 */  _RoundDouble_regDPR1__rule,
  /*  503 */  regD_rule,
  /*  504 */  regF_rule,
  /*  505 */  regFPR1_rule,
  /*  506 */  _RoundFloat_regFPR1__rule,
  /*  507 */  _ConvD2F_regDPR1__rule,
  /*  508 */  immFPR_rule,
  /*  509 */  immF_rule,
  /*  510 */  0,
  /*  511 */  0,
  /*  512 */  0,
  /*  513 */  0,
  /*  514 */  0,
  /*  515 */  0,
  /*  516 */  0,
  /*  517 */  0,
  /*  518 */  _Binary_rRegI_rRegI_rule,
  /*  519 */  _Binary_rRegI_rRegI_rule,
  /*  520 */  _Binary_rRegI_rRegI_rule,
  /*  521 */  _Binary_rRegI_rRegI_rule,
  /*  522 */  _Binary_rRegI_rRegI_rule,
  /*  523 */  _Binary_rRegI__LoadI_memory__rule,
  /*  524 */  _Binary_rRegI__LoadI_memory__rule,
  /*  525 */  _Binary_rRegI__LoadI_memory__rule,
  /*  526 */  _Binary_eRegP_eRegP_rule,
  /*  527 */  _Binary_eRegP_eRegP_rule,
  /*  528 */  _Binary_eRegP_eRegP_rule,
  /*  529 */  _Binary_eRegP_eRegP_rule,
  /*  530 */  _Binary_regDPR1_regDPR_rule,
  /*  531 */  _Binary_regFPR1_regFPR_rule,
  /*  532 */  _Binary_regDPR_regDPR_rule,
  /*  533 */  _Binary_regFPR_regFPR_rule,
  /*  534 */  _Binary_regF_regF_rule,
  /*  535 */  _Binary_regD_regD_rule,
  /*  536 */  _Binary_regF_regF_rule,
  /*  537 */  _Binary_regF_regF_rule,
  /*  538 */  _Binary_regD_regD_rule,
  /*  539 */  _Binary_regD_regD_rule,
  /*  540 */  _Binary_eRegL_eRegL_rule,
  /*  541 */  _Binary_eRegL_eRegL_rule,
  /*  542 */  _Binary_eRegL_eRegL_rule,
  /*  543 */  rRegI_rule,
  /*  544 */  immI_rule,
  /*  545 */  immI_1_rule,
  /*  546 */  immI_rule,
  /*  547 */  immI_rule,
  /*  548 */  immI_M1_rule,
  /*  549 */  rRegI_rule,
  /*  550 */  immI_rule,
  /*  551 */  _LoadI_memory__rule,
  /*  552 */  rRegI_rule,
  /*  553 */  _AddI__LoadI_memory__rRegI_rule,
  /*  554 */  _AddI_rRegI__LoadI_memory__rule,
  /*  555 */  _AddI__LoadI_memory__immI_rule,
  /*  556 */  _AddI__LoadI_memory__immI_1_rule,
  /*  557 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  558 */  0,
  /*  559 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  560 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  561 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  562 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  563 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  564 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  565 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  566 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  567 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  568 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  569 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  570 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  571 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  572 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  573 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  574 */  immI_rule,
  /*  575 */  xRegI_rule,
  /*  576 */  immI_rule,
  /*  577 */  rRegI_rule,
  /*  578 */  immI_rule,
  /*  579 */  rRegI_rule,
  /*  580 */  xRegI_rule,
  /*  581 */  rRegI_rule,
  /*  582 */  rRegI_rule,
  /*  583 */  pRegP_rule,
  /*  584 */  rRegI_rule,
  /*  585 */  immI_rule,
  /*  586 */  _LoadI_memory__rule,
  /*  587 */  _SubI__LoadI_memory__rRegI_rule,
  /*  588 */  _SubI_immI_0_rRegI_rule,
  /*  589 */  rRegI_rule,
  /*  590 */  rRegI_rule,
  /*  591 */  immI_rule,
  /*  592 */  0,
  /*  593 */  0,
  /*  594 */  0,
  /*  595 */  0,
  /*  596 */  immI_rule,
  /*  597 */  _LoadI_memory__rule,
  /*  598 */  rRegI_rule,
  /*  599 */  _Binary_rRegI_rRegI_rule,
  /*  600 */  _ConvI2L_nadxRegI__rule,
  /*  601 */  _ConvI2L_eAXRegI__rule,
  /*  602 */  _AndL__ConvI2L_nadxRegI__immL_32bits_rule,
  /*  603 */  _AndL__ConvI2L_eAXRegI__immL_32bits_rule,
  /*  604 */  eRegL_rule,
  /*  605 */  eRegL_rule,
  /*  606 */  eRegL_rule,
  /*  607 */  eRegL_rule,
  /*  608 */  immL_127_rule,
  /*  609 */  eCXRegI_rule,
  /*  610 */  eRegL_rule,
  /*  611 */  eCXRegI_rule,
  /*  612 */  eCXRegI_rule,
  /*  613 */  eRegL_rule,
  /*  614 */  immL32_rule,
  /*  615 */  immL32_rule,
  /*  616 */  immI_1_rule,
  /*  617 */  immI8_rule,
  /*  618 */  eCXRegI_rule,
  /*  619 */  immI_1_rule,
  /*  620 */  _RShiftI__LoadI_memory__immI_1_rule,
  /*  621 */  immI8_rule,
  /*  622 */  _RShiftI__LoadI_memory__immI8_rule,
  /*  623 */  eCXRegI_rule,
  /*  624 */  immI_1_rule,
  /*  625 */  immI8_rule,
  /*  626 */  immI_24_rule,
  /*  627 */  immI_16_rule,
  /*  628 */  eCXRegI_rule,
  /*  629 */  rRegI_rule,
  /*  630 */  immI_rule,
  /*  631 */  _LoadI_memory__rule,
  /*  632 */  rRegI_rule,
  /*  633 */  _AndI__LoadI_memory__rRegI_rule,
  /*  634 */  _AndI_rRegI__LoadI_memory__rule,
  /*  635 */  _AndI__LoadI_memory__immI_rule,
  /*  636 */  rRegI_rule,
  /*  637 */  _XorI_rRegI_immI_M1_rule,
  /*  638 */  _LoadI_memory__rule,
  /*  639 */  _XorI_rRegI_immI_M1_rule,
  /*  640 */  rRegI_rule,
  /*  641 */  _SubI_immI_0_rRegI_rule,
  /*  642 */  _LoadI_memory__rule,
  /*  643 */  _SubI_immI_0__LoadI_memory__rule,
  /*  644 */  rRegI_rule,
  /*  645 */  _AddI_rRegI_immI_M1_rule,
  /*  646 */  _LoadI_memory__rule,
  /*  647 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  648 */  rRegI_rule,
  /*  649 */  _AddI_rRegI_immI_M1_rule,
  /*  650 */  _LoadI_memory__rule,
  /*  651 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  652 */  rRegI_rule,
  /*  653 */  _CastP2X_eRegP__rule,
  /*  654 */  rRegI_rule,
  /*  655 */  immI_rule,
  /*  656 */  _LoadI_memory__rule,
  /*  657 */  rRegI_rule,
  /*  658 */  _OrI__LoadI_memory__rRegI_rule,
  /*  659 */  _OrI_rRegI__LoadI_memory__rule,
  /*  660 */  _OrI__LoadI_memory__immI_rule,
  /*  661 */  0,
  /*  662 */  0,
  /*  663 */  0,
  /*  664 */  _URShiftI_rRegI_immI_M1_rule,
  /*  665 */  _LShiftI_rRegI_immI_1_rule,
  /*  666 */  _URShiftI_rRegI_immI8_rule,
  /*  667 */  _LShiftI_rRegI_immI8_rule,
  /*  668 */  _URShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  669 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  670 */  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  671 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  672 */  0,
  /*  673 */  0,
  /*  674 */  0,
  /*  675 */  _LShiftI_rRegI_immI_M1_rule,
  /*  676 */  _URShiftI_rRegI_immI_1_rule,
  /*  677 */  _LShiftI_rRegI_immI8_rule,
  /*  678 */  _URShiftI_rRegI_immI8_rule,
  /*  679 */  _LShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  680 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  681 */  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  682 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  683 */  rRegI_rule,
  /*  684 */  immI_M1_rule,
  /*  685 */  immI_rule,
  /*  686 */  _LoadI_memory__rule,
  /*  687 */  rRegI_rule,
  /*  688 */  _XorI__LoadI_memory__rRegI_rule,
  /*  689 */  _XorI_rRegI__LoadI_memory__rule,
  /*  690 */  _XorI__LoadI_memory__immI_rule,
  /*  691 */  0,
  /*  692 */  0,
  /*  693 */  0,
  /*  694 */  0,
  /*  695 */  ncxRegI_rule,
  /*  696 */  immI_0_rule,
  /*  697 */  _SubI_rRegI_rRegI_rule,
  /*  698 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  699 */  _SubI_rRegI_rRegI_rule,
  /*  700 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  701 */  rRegI_rule,
  /*  702 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  703 */  0,
  /*  704 */  eRegL_rule,
  /*  705 */  immL_rule,
  /*  706 */  _LoadL_load_long_memory__rule,
  /*  707 */  eRegL_rule,
  /*  708 */  eRegL_rule,
  /*  709 */  immL_rule,
  /*  710 */  _LoadL_load_long_memory__rule,
  /*  711 */  eRegL_rule,
  /*  712 */  eRegL_rule,
  /*  713 */  immL_rule,
  /*  714 */  _LoadL_load_long_memory__rule,
  /*  715 */  eRegL_rule,
  /*  716 */  eRegL_rule,
  /*  717 */  _XorL_eRegL_immL_M1_rule,
  /*  718 */  _LoadL_memory__rule,
  /*  719 */  _XorL_eRegL_immL_M1_rule,
  /*  720 */  eRegL_rule,
  /*  721 */  _SubL_immL0_eRegL_rule,
  /*  722 */  _LoadL_memory__rule,
  /*  723 */  _SubL_immL0__LoadL_memory__rule,
  /*  724 */  eRegL_rule,
  /*  725 */  _AddL_eRegL_immL_M1_rule,
  /*  726 */  _LoadL_memory__rule,
  /*  727 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  728 */  eRegL_rule,
  /*  729 */  _AddL_eRegL_immL_M1_rule,
  /*  730 */  _LoadL_memory__rule,
  /*  731 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  732 */  eRegL_rule,
  /*  733 */  immL_rule,
  /*  734 */  _LoadL_load_long_memory__rule,
  /*  735 */  eRegL_rule,
  /*  736 */  eRegL_rule,
  /*  737 */  immL_M1_rule,
  /*  738 */  immL_rule,
  /*  739 */  _LoadL_load_long_memory__rule,
  /*  740 */  eRegL_rule,
  /*  741 */  immI_1_rule,
  /*  742 */  immI_2_rule,
  /*  743 */  immI_3_rule,
  /*  744 */  immI_1_31_rule,
  /*  745 */  immI_32_63_rule,
  /*  746 */  eCXRegI_rule,
  /*  747 */  immI_1_31_rule,
  /*  748 */  immI_32_63_rule,
  /*  749 */  eCXRegI_rule,
  /*  750 */  immI_1_31_rule,
  /*  751 */  immI_32_63_rule,
  /*  752 */  eCXRegI_rule,
  /*  753 */  immDPR0_rule,
  /*  754 */  regDPR_rule,
  /*  755 */  regD_rule,
  /*  756 */  _LoadD_memory__rule,
  /*  757 */  regDPR_rule,
  /*  758 */  0,
  /*  759 */  _LoadD_memory__rule,
  /*  760 */  regDPR_rule,
  /*  761 */  0,
  /*  762 */  _LoadD_memory__rule,
  /*  763 */  regDPR_rule,
  /*  764 */  _RoundDouble__AddD__LoadD_memory__regDPR__rule,
  /*  765 */  _RoundDouble__AddD_regDPR__LoadD_memory___rule,
  /*  766 */  immDPR1_rule,
  /*  767 */  immDPR_rule,
  /*  768 */  0,
  /*  769 */  regDPR_rule,
  /*  770 */  regnotDPR1_rule,
  /*  771 */  immDPR_rule,
  /*  772 */  _LoadD_memory__rule,
  /*  773 */  regDPR_rule,
  /*  774 */  _LoadD_memory__rule,
  /*  775 */  regDPR_rule,
  /*  776 */  regDPR_rule,
  /*  777 */  _MulD_regDPR_regDPR_rule,
  /*  778 */  regDPR_rule,
  /*  779 */  regDPR_rule,
  /*  780 */  regnotDPR1_rule,
  /*  781 */  0,
  /*  782 */  regDPR_rule,
  /*  783 */  regD_rule,
  /*  784 */  regDPR_rule,
  /*  785 */  regD_rule,
  /*  786 */  0,
  /*  787 */  immFPR0_rule,
  /*  788 */  regFPR_rule,
  /*  789 */  regF_rule,
  /*  790 */  _LoadF_memory__rule,
  /*  791 */  regFPR_rule,
  /*  792 */  regFPR_rule,
  /*  793 */  regFPR_rule,
  /*  794 */  regFPR_rule,
  /*  795 */  _LoadF_memory__rule,
  /*  796 */  regFPR_rule,
  /*  797 */  _LoadF_memory__rule,
  /*  798 */  regFPR_rule,
  /*  799 */  regFPR_rule,
  /*  800 */  _LoadF_memory__rule,
  /*  801 */  memory_rule,
  /*  802 */  memory_rule,
  /*  803 */  immFPR_rule,
  /*  804 */  immFPR_rule,
  /*  805 */  regFPR_rule,
  /*  806 */  regFPR_rule,
  /*  807 */  _LoadF_memory__rule,
  /*  808 */  regFPR_rule,
  /*  809 */  _LoadF_memory__rule,
  /*  810 */  regFPR_rule,
  /*  811 */  memory_rule,
  /*  812 */  immFPR_rule,
  /*  813 */  immFPR_rule,
  /*  814 */  regFPR_rule,
  /*  815 */  _LoadF_memory__rule,
  /*  816 */  regFPR_rule,
  /*  817 */  _MulF__LoadF_memory__regFPR_rule,
  /*  818 */  regFPR_rule,
  /*  819 */  _MulF_regFPR__LoadF_memory__rule,
  /*  820 */  regFPR_rule,
  /*  821 */  _MulF_regFPR_regFPR_rule,
  /*  822 */  regFPR_rule,
  /*  823 */  regFPR_rule,
  /*  824 */  regFPR_rule,
  /*  825 */  regFPR_rule,
  /*  826 */  regFPR_rule,
  /*  827 */  regF_rule,
  /*  828 */  0,
  /*  829 */  0,
  /*  830 */  0,
  /*  831 */  0,
  /*  832 */  0,
  /*  833 */  0,
  /*  834 */  0,
  /*  835 */  0,
  /*  836 */  0,
  /*  837 */  0,
  /*  838 */  0,
  /*  839 */  0,
  /*  840 */  0,
  /*  841 */  0,
  /*  842 */  0,
  /*  843 */  0,
  /*  844 */  0,
  /*  845 */  0,
  /*  846 */  0,
  /*  847 */  0,
  /*  848 */  0,
  /*  849 */  0,
  /*  850 */  0,
  /*  851 */  0,
  /*  852 */  0,
  /*  853 */  0,
  /*  854 */  0,
  /*  855 */  0,
  /*  856 */  0,
  /*  857 */  0,
  /*  858 */  immL_32bits_rule,
  /*  859 */  immL_32bits_rule,
  /*  860 */  0,
  /*  861 */  0,
  /*  862 */  0,
  /*  863 */  0,
  /*  864 */  0,
  /*  865 */  0,
  /*  866 */  0,
  /*  867 */  0,
  /*  868 */  0,
  /*  869 */  0,
  /*  870 */  0,
  /*  871 */  0,
  /*  872 */  0,
  /*  873 */  eDIRegP_rule,
  /*  874 */  eDIRegP_rule,
  /*  875 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  876 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  877 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  878 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  879 */  eCXRegI_rule,
  /*  880 */  _Binary_eSIRegP_immI_rule,
  /*  881 */  _Binary_eSIRegP_immI_rule,
  /*  882 */  _Binary_eSIRegP_immI_rule,
  /*  883 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  884 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  885 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  886 */  eAXRegI_rule,
  /*  887 */  eAXRegI_rule,
  /*  888 */  eSIRegP_rule,
  /*  889 */  eSIRegP_rule,
  /*  890 */  eCXRegI_rule,
  /*  891 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  892 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  893 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  894 */  rRegI_rule,
  /*  895 */  rRegI_rule,
  /*  896 */  immI_rule,
  /*  897 */  0,
  /*  898 */  0,
  /*  899 */  eFlagsReg_rule,
  /*  900 */  eFlagsReg_rule,
  /*  901 */  eFlagsRegU_rule,
  /*  902 */  eFlagsRegUCF_rule,
  /*  903 */  eFlagsReg_rule,
  /*  904 */  eFlagsRegU_rule,
  /*  905 */  eFlagsRegUCF_rule,
  /*  906 */  eFlagsRegU_rule,
  /*  907 */  eFlagsRegUCF_rule,
  /*  908 */  eFlagsRegUCF_rule,
  /*  909 */  eAXRegP_rule,
  /*  910 */  0,
  /*  911 */  eFlagsReg_rule,
  /*  912 */  eFlagsReg_rule,
  /*  913 */  eFlagsRegU_rule,
  /*  914 */  eFlagsRegUCF_rule,
  /*  915 */  eFlagsRegU_rule,
  /*  916 */  eFlagsRegUCF_rule,
  /*  917 */  eFlagsRegUCF_rule,
  /*  918 */  eRegL_rule,
  /*  919 */  flagsReg_long_LTGE_rule,
  /*  920 */  flagsReg_ulong_LTGE_rule,
  /*  921 */  _Binary_eRegL_eRegL_rule,
  /*  922 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /*  923 */  _Binary_rRegI_rRegI_rule,
  /*  924 */  _Binary_rRegI__LoadI_memory__rule,
  /*  925 */  _Binary_eRegP_eRegP_rule,
  /*  926 */  _Binary_regDPR_regDPR_rule,
  /*  927 */  _Binary_regD_regD_rule,
  /*  928 */  _Binary_regFPR_regFPR_rule,
  /*  929 */  _Binary_regF_regF_rule,
  /*  930 */  flagsReg_long_EQNE_rule,
  /*  931 */  flagsReg_ulong_EQNE_rule,
  /*  932 */  _Binary_eRegL_eRegL_rule,
  /*  933 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /*  934 */  _Binary_rRegI_rRegI_rule,
  /*  935 */  _Binary_rRegI__LoadI_memory__rule,
  /*  936 */  _Binary_eRegP_eRegP_rule,
  /*  937 */  _Binary_regDPR_regDPR_rule,
  /*  938 */  _Binary_regD_regD_rule,
  /*  939 */  _Binary_regFPR_regFPR_rule,
  /*  940 */  _Binary_regF_regF_rule,
  /*  941 */  flagsReg_long_LEGT_rule,
  /*  942 */  flagsReg_ulong_LEGT_rule,
  /*  943 */  _Binary_eRegL_eRegL_rule,
  /*  944 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /*  945 */  _Binary_eRegL_eRegL_rule,
  /*  946 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /*  947 */  _Binary_rRegI_rRegI_rule,
  /*  948 */  _Binary_rRegI__LoadI_memory__rule,
  /*  949 */  _Binary_eRegP_eRegP_rule,
  /*  950 */  _Binary_regDPR_regDPR_rule,
  /*  951 */  _Binary_regD_regD_rule,
  /*  952 */  _Binary_regFPR_regFPR_rule,
  /*  953 */  _Binary_regF_regF_rule,
  /*  954 */  0,
  /*  955 */  0,
  /*  956 */  0,
  /*  957 */  0,
  /*  958 */  0,
  /*  959 */  0,
  /*  960 */  eBXRegP_rule,
  /*  961 */  eAXRegP_rule,
  /*  962 */  0,
  /*  963 */  0,
  /*  964 */  0,
  /*  965 */  0,
  /*  966 */  0,
  /*  967 */  0,
  /*  968 */  regF_rule,
  /*  969 */  _LoadF_memory__rule,
  /*  970 */  regF_rule,
  /*  971 */  immF_rule,
  /*  972 */  regF_rule,
  /*  973 */  _LoadF_memory__rule,
  /*  974 */  regF_rule,
  /*  975 */  immF_rule,
  /*  976 */  regD_rule,
  /*  977 */  _LoadD_memory__rule,
  /*  978 */  regD_rule,
  /*  979 */  immD_rule,
  /*  980 */  regD_rule,
  /*  981 */  _LoadD_memory__rule,
  /*  982 */  regD_rule,
  /*  983 */  immD_rule,
  /*  984 */  regF_rule,
  /*  985 */  _LoadF_memory__rule,
  /*  986 */  immF_rule,
  /*  987 */  regF_rule,
  /*  988 */  _LoadF_memory__rule,
  /*  989 */  immF_rule,
  /*  990 */  regD_rule,
  /*  991 */  _LoadD_memory__rule,
  /*  992 */  immD_rule,
  /*  993 */  regD_rule,
  /*  994 */  _LoadD_memory__rule,
  /*  995 */  immD_rule,
  /*  996 */  regF_rule,
  /*  997 */  _LoadF_memory__rule,
  /*  998 */  regF_rule,
  /*  999 */  immF_rule,
  /* 1000 */  regF_rule,
  /* 1001 */  _LoadF_memory__rule,
  /* 1002 */  regF_rule,
  /* 1003 */  immF_rule,
  /* 1004 */  regD_rule,
  /* 1005 */  _LoadD_memory__rule,
  /* 1006 */  regD_rule,
  /* 1007 */  immD_rule,
  /* 1008 */  regD_rule,
  /* 1009 */  _LoadD_memory__rule,
  /* 1010 */  regD_rule,
  /* 1011 */  immD_rule,
  /* 1012 */  regF_rule,
  /* 1013 */  _LoadF_memory__rule,
  /* 1014 */  immF_rule,
  /* 1015 */  regF_rule,
  /* 1016 */  _LoadF_memory__rule,
  /* 1017 */  immF_rule,
  /* 1018 */  regD_rule,
  /* 1019 */  _LoadD_memory__rule,
  /* 1020 */  immD_rule,
  /* 1021 */  regD_rule,
  /* 1022 */  _LoadD_memory__rule,
  /* 1023 */  immD_rule,
  /* 1024 */  0,
  /* 1025 */  0,
  /* 1026 */  0,
  /* 1027 */  0,
  /* 1028 */  0,
  /* 1029 */  0,
  /* 1030 */  0,
  /* 1031 */  0,
  /* 1032 */  0,
  /* 1033 */  _Binary_regD_regD_rule,
  /* 1034 */  _Binary_regF_regF_rule,
  /* 1035 */  0,
  /* 1036 */  vec_rule,
  /* 1037 */  legVec_rule,
  /* 1038 */  vec_rule,
  /* 1039 */  _Binary_vec_vec_rule,
  /* 1040 */  0,
  /* 1041 */  0,
  /* 1042 */  0,
  /* 1043 */  0,
  /* 1044 */  0,
  /* 1045 */  0,
  /* 1046 */  0,
  /* 1047 */  0,
  /* 1048 */  0,
  /* 1049 */  0,
  /* 1050 */  0,
  /* 1051 */  0,
  /* 1052 */  0,
  /* 1053 */  0,
  /* 1054 */  0,
  /* 1055 */  0,
  /* 1056 */  0,
  /* 1057 */  0,
  /* 1058 */  0,
  /* 1059 */  0,
  /* 1060 */  0,
  /* 1061 */  0,
  /* 1062 */  0,
  /* 1063 */  0,
  /* 1064 */  0,
  /* 1065 */  0,
  /* 1066 */  0,
  /* 1067 */  immU8_rule,
  /* 1068 */  immU8_rule,
  /* 1069 */  immU8_rule,
  /* 1070 */  immU8_rule,
  /* 1071 */  immU8_rule,
  /* 1072 */  legVec_rule,
  /* 1073 */  legVec_rule,
  /* 1074 */  legVec_rule,
  /* 1075 */  legVec_rule,
  /* 1076 */  legVec_rule,
  /* 1077 */  legVec_rule,
  /* 1078 */  legVec_rule,
  /* 1079 */  vec_rule,
  /* 1080 */  vec_rule,
  /* 1081 */  vec_rule,
  /* 1082 */  vec_rule,
  /* 1083 */  legVec_rule,
  /* 1084 */  legVec_rule,
  /* 1085 */  vec_rule,
  /* 1086 */  vec_rule,
  /* 1087 */  vec_rule,
  /* 1088 */  vec_rule,
  /* 1089 */  legVec_rule,
  /* 1090 */  legVec_rule,
  /* 1091 */  legVec_rule,
  /* 1092 */  legVec_rule,
  /* 1093 */  legVec_rule,
  /* 1094 */  legVec_rule,
  /* 1095 */  legVec_rule,
  /* 1096 */  legVec_rule,
  /* 1097 */  legVec_rule,
  /* 1098 */  vec_rule,
  /* 1099 */  legVec_rule,
  /* 1100 */  legVec_rule,
  /* 1101 */  legVec_rule,
  /* 1102 */  legVec_rule,
  /* 1103 */  legVec_rule,
  /* 1104 */  legVec_rule,
  /* 1105 */  legVec_rule,
  /* 1106 */  legVec_rule,
  /* 1107 */  legVec_rule,
  /* 1108 */  legVec_rule,
  /* 1109 */  legVec_rule,
  /* 1110 */  legVec_rule,
  /* 1111 */  legVec_rule,
  /* 1112 */  legVec_rule,
  /* 1113 */  legVec_rule,
  /* 1114 */  legVec_rule,
  /* 1115 */  legVec_rule,
  /* 1116 */  vec_rule,
  /* 1117 */  vec_rule,
  /* 1118 */  _LoadVector_memory__rule,
  /* 1119 */  vec_rule,
  /* 1120 */  vec_rule,
  /* 1121 */  vec_rule,
  /* 1122 */  _LoadVector_memory__rule,
  /* 1123 */  vec_rule,
  /* 1124 */  vec_rule,
  /* 1125 */  vec_rule,
  /* 1126 */  _LoadVector_memory__rule,
  /* 1127 */  vec_rule,
  /* 1128 */  vec_rule,
  /* 1129 */  vec_rule,
  /* 1130 */  _LoadVector_memory__rule,
  /* 1131 */  vec_rule,
  /* 1132 */  vec_rule,
  /* 1133 */  vec_rule,
  /* 1134 */  _LoadVector_memory__rule,
  /* 1135 */  vec_rule,
  /* 1136 */  vec_rule,
  /* 1137 */  vec_rule,
  /* 1138 */  _LoadVector_memory__rule,
  /* 1139 */  vec_rule,
  /* 1140 */  vec_rule,
  /* 1141 */  vec_rule,
  /* 1142 */  _LoadVector_memory__rule,
  /* 1143 */  vec_rule,
  /* 1144 */  vec_rule,
  /* 1145 */  _LoadVector_memory__rule,
  /* 1146 */  vec_rule,
  /* 1147 */  vec_rule,
  /* 1148 */  _LoadVector_memory__rule,
  /* 1149 */  vec_rule,
  /* 1150 */  vec_rule,
  /* 1151 */  _LoadVector_memory__rule,
  /* 1152 */  vec_rule,
  /* 1153 */  vec_rule,
  /* 1154 */  _LoadVector_memory__rule,
  /* 1155 */  vec_rule,
  /* 1156 */  vec_rule,
  /* 1157 */  _LoadVector_memory__rule,
  /* 1158 */  vec_rule,
  /* 1159 */  vec_rule,
  /* 1160 */  vec_rule,
  /* 1161 */  vec_rule,
  /* 1162 */  vec_rule,
  /* 1163 */  vec_rule,
  /* 1164 */  vec_rule,
  /* 1165 */  _LoadVector_memory__rule,
  /* 1166 */  vec_rule,
  /* 1167 */  vec_rule,
  /* 1168 */  vec_rule,
  /* 1169 */  _LoadVector_memory__rule,
  /* 1170 */  vec_rule,
  /* 1171 */  vec_rule,
  /* 1172 */  _LoadVector_memory__rule,
  /* 1173 */  vec_rule,
  /* 1174 */  vec_rule,
  /* 1175 */  vec_rule,
  /* 1176 */  vec_rule,
  /* 1177 */  vec_rule,
  /* 1178 */  _LoadVector_memory__rule,
  /* 1179 */  vec_rule,
  /* 1180 */  vec_rule,
  /* 1181 */  vec_rule,
  /* 1182 */  _LoadVector_memory__rule,
  /* 1183 */  vec_rule,
  /* 1184 */  _Binary_legVec_legVec_rule,
  /* 1185 */  _Binary_legVec_legVec_rule,
  /* 1186 */  vec_rule,
  /* 1187 */  vec_rule,
  /* 1188 */  _LoadVector_memory__rule,
  /* 1189 */  vec_rule,
  /* 1190 */  vec_rule,
  /* 1191 */  _LoadVector_memory__rule,
  /* 1192 */  vec_rule,
  /* 1193 */  vec_rule,
  /* 1194 */  vec_rule,
  /* 1195 */  vec_rule,
  /* 1196 */  vec_rule,
  /* 1197 */  vec_rule,
  /* 1198 */  legVec_rule,
  /* 1199 */  legVec_rule,
  /* 1200 */  vec_rule,
  /* 1201 */  vec_rule,
  /* 1202 */  legVec_rule,
  /* 1203 */  legVec_rule,
  /* 1204 */  vec_rule,
  /* 1205 */  vec_rule,
  /* 1206 */  vec_rule,
  /* 1207 */  vec_rule,
  /* 1208 */  vec_rule,
  /* 1209 */  vec_rule,
  /* 1210 */  vec_rule,
  /* 1211 */  vec_rule,
  /* 1212 */  vec_rule,
  /* 1213 */  vec_rule,
  /* 1214 */  vec_rule,
  /* 1215 */  vec_rule,
  /* 1216 */  vec_rule,
  /* 1217 */  vec_rule,
  /* 1218 */  vec_rule,
  /* 1219 */  vec_rule,
  /* 1220 */  vec_rule,
  /* 1221 */  vec_rule,
  /* 1222 */  vec_rule,
  /* 1223 */  vec_rule,
  /* 1224 */  vec_rule,
  /* 1225 */  vec_rule,
  /* 1226 */  vec_rule,
  /* 1227 */  _LShiftCntV_immI8__rule,
  /* 1228 */  _RShiftCntV_immI8__rule,
  /* 1229 */  _RShiftCntV_immI8__rule,
  /* 1230 */  vec_rule,
  /* 1231 */  vec_rule,
  /* 1232 */  _LShiftCntV_immI8__rule,
  /* 1233 */  _RShiftCntV_immI8__rule,
  /* 1234 */  vec_rule,
  /* 1235 */  vec_rule,
  /* 1236 */  vec_rule,
  /* 1237 */  vec_rule,
  /* 1238 */  vec_rule,
  /* 1239 */  vec_rule,
  /* 1240 */  vec_rule,
  /* 1241 */  vec_rule,
  /* 1242 */  vec_rule,
  /* 1243 */  vec_rule,
  /* 1244 */  vec_rule,
  /* 1245 */  vec_rule,
  /* 1246 */  vec_rule,
  /* 1247 */  vec_rule,
  /* 1248 */  vec_rule,
  /* 1249 */  vec_rule,
  /* 1250 */  vec_rule,
  /* 1251 */  vec_rule,
  /* 1252 */  vec_rule,
  /* 1253 */  vec_rule,
  /* 1254 */  vec_rule,
  /* 1255 */  vec_rule,
  /* 1256 */  vec_rule,
  /* 1257 */  vec_rule,
  /* 1258 */  vec_rule,
  /* 1259 */  vec_rule,
  /* 1260 */  vec_rule,
  /* 1261 */  vec_rule,
  /* 1262 */  vec_rule,
  /* 1263 */  vec_rule,
  /* 1264 */  vec_rule,
  /* 1265 */  vec_rule,
  /* 1266 */  vec_rule,
  /* 1267 */  vec_rule,
  /* 1268 */  vec_rule,
  /* 1269 */  _LoadVector_memory__rule,
  /* 1270 */  vec_rule,
  /* 1271 */  vec_rule,
  /* 1272 */  vec_rule,
  /* 1273 */  _LoadVector_memory__rule,
  /* 1274 */  vec_rule,
  /* 1275 */  vec_rule,
  /* 1276 */  vec_rule,
  /* 1277 */  _LoadVector_memory__rule,
  /* 1278 */  vec_rule,
  /* 1279 */  0,
  /* 1280 */  0,
  /* 1281 */  0,
  /* 1282 */  0,
  /* 1283 */  0,
  /* 1284 */  immI8_rule,
  /* 1285 */  immI8_rule,
  /* 1286 */  immI8_rule,
  /* 1287 */  immI8_rule,
  /* 1288 */  immU8_rule,
  /* 1289 */  immU8_rule,
  /* 1290 */  immI_rule,
  /* 1291 */  immI_rule,
  /* 1292 */  immU8_rule,
  /* 1293 */  immU8_rule,
  /* 1294 */  immU8_rule,
  /* 1295 */  immU8_rule,
  /* 1296 */  vec_rule,
  /* 1297 */  legVec_rule,
  /* 1298 */  legVec_rule,
  /* 1299 */  vec_rule,
  /* 1300 */  0,
  /* 1301 */  0,
  /* 1302 */  0,
  /* 1303 */  0,
  /* 1304 */  0,
  /* 1305 */  0,
  /* 1306 */  0,
  /* 1307 */  immI_1_rule,
  /* 1308 */  immI_2_rule,
  /* 1309 */  immI_2_rule,
  /* 1310 */  immI_2_rule,
  /* 1311 */  immI_4_rule,
  /* 1312 */  immI_4_rule,
  /* 1313 */  immI_4_rule,
  /* 1314 */  immI_8_rule,
  /* 1315 */  immI_8_rule,
  /* 1316 */  immI_8_rule,
  /* 1317 */  0,
  /* 1318 */  vec_rule,
  /* 1319 */  vec_rule,
  /* 1320 */  vec_rule,
  /* 1321 */  0,
  /* 1322 */  vec_rule,
  /* 1323 */  vec_rule,
  /* 1324 */  0,
  /* 1325 */  vec_rule,
  /* 1326 */  vec_rule,
  /* 1327 */  0,
  /* 1328 */  vec_rule,
  /* 1329 */  vec_rule,
  /* 1330 */  _Binary_vec_vec_rule,
  /* 1331 */  _Binary_vec__LoadVector_memory__rule,
  /* 1332 */  _Binary_vec_vec_rule,
  /* 1333 */  _Binary_vec__LoadVector_memory__rule,
  /* 1334 */  vec_rule,
  /* 1335 */  vec_rule,
  /* 1336 */  vec_rule,
  /* 1337 */  _MulAddVS2VI_vec_vec_rule,
  /* 1338 */  _Binary_vec_immU8_rule,
  /* 1339 */  _Binary__LoadVector_memory__immU8_rule,
  /* 1340 */  immI8_rule,
  /* 1341 */  immI8_rule,
  /* 1342 */  vec_rule,
  /* 1343 */  vec_rule,
  /* 1344 */  _Binary_eAXRegP_eRegP_rule,
  /* 1345 */  _Binary_eAXRegP_eRegP_rule,
  /* 1346 */  _Binary_eAXRegP_eRegP_rule,
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  /*    0 */  "UNIVERSE",
  /*    1 */  "LABEL",
  /*    2 */  "SREGI",
  /*    3 */  "SREGP",
  /*    4 */  "SREGF",
  /*    5 */  "SREGD",
  /*    6 */  "SREGL",
  /*    7 */  "METHOD",
  /*    8 */  "IMMI",
  /*    9 */  "IMMI_0",
  /*   10 */  "IMMI_1",
  /*   11 */  "IMMI_M1",
  /*   12 */  "IMMI2",
  /*   13 */  "IMMI8",
  /*   14 */  "IMMU8",
  /*   15 */  "IMMI16",
  /*   16 */  "IMMU31",
  /*   17 */  "IMMI_32",
  /*   18 */  "IMMI_1_31",
  /*   19 */  "IMMI_32_63",
  /*   20 */  "IMMI_2",
  /*   21 */  "IMMI_3",
  /*   22 */  "IMMI_4",
  /*   23 */  "IMMI_8",
  /*   24 */  "IMMP",
  /*   25 */  "IMMP0",
  /*   26 */  "IMML",
  /*   27 */  "IMML0",
  /*   28 */  "IMML_M1",
  /*   29 */  "IMML_127",
  /*   30 */  "IMML_32BITS",
  /*   31 */  "IMML32",
  /*   32 */  "IMMDPR0",
  /*   33 */  "IMMDPR1",
  /*   34 */  "IMMDPR",
  /*   35 */  "IMMD",
  /*   36 */  "IMMD0",
  /*   37 */  "IMMFPR0",
  /*   38 */  "IMMFPR1",
  /*   39 */  "IMMFPR",
  /*   40 */  "IMMF",
  /*   41 */  "IMMF0",
  /*   42 */  "IMMI_16",
  /*   43 */  "IMMI_24",
  /*   44 */  "IMMI_255",
  /*   45 */  "IMMI_65535",
  /*   46 */  "RREGI",
  /*   47 */  "XREGI",
  /*   48 */  "EAXREGI",
  /*   49 */  "EBXREGI",
  /*   50 */  "ECXREGI",
  /*   51 */  "EDXREGI",
  /*   52 */  "EDIREGI",
  /*   53 */  "NAXREGI",
  /*   54 */  "NADXREGI",
  /*   55 */  "NCXREGI",
  /*   56 */  "ESIREGI",
  /*   57 */  "ANYREGP",
  /*   58 */  "EREGP",
  /*   59 */  "RREGP",
  /*   60 */  "EREGP_NO_EBP",
  /*   61 */  "NAXREGP",
  /*   62 */  "NABXREGP",
  /*   63 */  "PREGP",
  /*   64 */  "EAXREGP",
  /*   65 */  "EBXREGP",
  /*   66 */  "ECXREGP",
  /*   67 */  "EDXREGP",
  /*   68 */  "ESIREGP",
  /*   69 */  "EDIREGP",
  /*   70 */  "EREGL",
  /*   71 */  "EADXREGL",
  /*   72 */  "EBCXREGL",
  /*   73 */  "EADXREGL_LOW_ONLY",
  /*   74 */  "RFLAGSREG",
  /*   75 */  "EFLAGSREG",
  /*   76 */  "EFLAGSREGU",
  /*   77 */  "EFLAGSREGUCF",
  /*   78 */  "FLAGSREG_LONG_LTGE",
  /*   79 */  "FLAGSREG_LONG_EQNE",
  /*   80 */  "FLAGSREG_LONG_LEGT",
  /*   81 */  "FLAGSREG_ULONG_LTGE",
  /*   82 */  "FLAGSREG_ULONG_EQNE",
  /*   83 */  "FLAGSREG_ULONG_LEGT",
  /*   84 */  "REGDPR",
  /*   85 */  "REGDPR1",
  /*   86 */  "REGDPR2",
  /*   87 */  "REGNOTDPR1",
  /*   88 */  "REGFPR",
  /*   89 */  "REGFPR1",
  /*   90 */  "REGF",
  /*   91 */  "LEGREGF",
  /*   92 */  "VLREGF",
  /*   93 */  "REGD",
  /*   94 */  "LEGREGD",
  /*   95 */  "VLREGD",
  /*   96 */  "DIRECT",
  /*   97 */  "INDIRECT",
  /*   98 */  "INDOFFSET8",
  /*   99 */  "INDOFFSET32",
  /*  100 */  "INDOFFSET32X",
  /*  101 */  "INDINDEXOFFSET",
  /*  102 */  "INDINDEX",
  /*  103 */  "INDINDEXSCALE",
  /*  104 */  "INDINDEXSCALEOFFSET",
  /*  105 */  "LOAD_LONG_REGP",
  /*  106 */  "LOAD_LONG_INDIRECT",
  /*  107 */  "LOAD_LONG_INDOFFSET32",
  /*  108 */  "STACKSLOTP",
  /*  109 */  "STACKSLOTI",
  /*  110 */  "STACKSLOTF",
  /*  111 */  "STACKSLOTD",
  /*  112 */  "STACKSLOTL",
  /*  113 */  "CMPOP",
  /*  114 */  "CMPOPU",
  /*  115 */  "CMPOPUCF",
  /*  116 */  "CMPOPUCF2",
  /*  117 */  "CMPOP_FCMOV",
  /*  118 */  "CMPOP_COMMUTE",
  /*  119 */  "CMPOPU_COMMUTE",
  /*  120 */  "RXMM0",
  /*  121 */  "VEC",
  /*  122 */  "LEGVEC",
  /*  123 */  "VECS",
  /*  124 */  "LEGVECS",
  /*  125 */  "VECD",
  /*  126 */  "LEGVECD",
  /*  127 */  "VECX",
  /*  128 */  "LEGVECX",
  /*  129 */  "VECY",
  /*  130 */  "LEGVECY",
  /*  131 */  "VECZ",
  /*  132 */  "LEGVECZ",
  /*  133 */  "CMPOP_VCMPPD",
  // last operand
  /*  134 */  "LOAD_LONG_MEMORY",
  /*  135 */  "MEMORY",
  /*  136 */  "LONG_MEMORY",
  // last operand class
  /*  137 */  "_AddP_eRegP_rRegI",
  /*  138 */  "_LShiftI_rRegI_immI2",
  /*  139 */  "_AddP_eRegP__LShiftI_rRegI_immI2",
  /*  140 */  "_LoadI_memory_",
  /*  141 */  "_LoadL_memory_",
  /*  142 */  "_LoadB_memory_",
  /*  143 */  "_LoadUB_memory_",
  /*  144 */  "_AndI__LoadUB_memory__immI",
  /*  145 */  "_LoadS_memory_",
  /*  146 */  "_LShiftI__LoadS_memory__immI_24",
  /*  147 */  "_LoadUS_memory_",
  /*  148 */  "_LShiftI__LoadUS_memory__immI_24",
  /*  149 */  "_AndI__LoadUS_memory__immI_255",
  /*  150 */  "_AndI__LoadUS_memory__immI",
  /*  151 */  "_LShiftI__LoadI_memory__immI_24",
  /*  152 */  "_LShiftI__LoadI_memory__immI_16",
  /*  153 */  "_AndI__LoadI_memory__immI_255",
  /*  154 */  "_AndI__LoadI_memory__immI_65535",
  /*  155 */  "_AndI__LoadI_memory__immU31",
  /*  156 */  "_ConvI2L__LoadI_memory__",
  /*  157 */  "_ConvL2I_eRegL_",
  /*  158 */  "_RoundDouble_regDPR1_",
  /*  159 */  "_RoundFloat_regFPR1_",
  /*  160 */  "_ConvD2F_regDPR1_",
  /*  161 */  "_Binary_cmpOp_eFlagsReg",
  /*  162 */  "_Binary_rRegI_rRegI",
  /*  163 */  "_Binary_cmpOpU_eFlagsRegU",
  /*  164 */  "_Binary_cmpOpUCF_eFlagsRegUCF",
  /*  165 */  "_Binary_rRegI__LoadI_memory_",
  /*  166 */  "_Binary_eRegP_eRegP",
  /*  167 */  "_Binary_cmpOp_fcmov_eFlagsRegU",
  /*  168 */  "_Binary_regDPR1_regDPR",
  /*  169 */  "_Binary_regFPR1_regFPR",
  /*  170 */  "_Binary_regDPR_regDPR",
  /*  171 */  "_Binary_regFPR_regFPR",
  /*  172 */  "_Binary_regF_regF",
  /*  173 */  "_Binary_regD_regD",
  /*  174 */  "_Binary_eRegL_eRegL",
  /*  175 */  "_AddI__LoadI_memory__rRegI",
  /*  176 */  "_AddI_rRegI__LoadI_memory_",
  /*  177 */  "_AddI__LoadI_memory__immI",
  /*  178 */  "_AddI__LoadI_memory__immI_1",
  /*  179 */  "_AddI__LoadI_memory__immI_M1",
  /*  180 */  "_Binary_eAXRegP_eRegP",
  /*  181 */  "_Binary_eAXRegI_rRegI",
  /*  182 */  "_Binary_eADXRegL_eBCXRegL",
  /*  183 */  "_Binary_eAXRegP_eCXRegP",
  /*  184 */  "_Binary_eAXRegI_eCXRegI",
  /*  185 */  "_SubI__LoadI_memory__rRegI",
  /*  186 */  "_SubI_immI_0_rRegI",
  /*  187 */  "_ConvI2L_nadxRegI_",
  /*  188 */  "_MulL__ConvI2L_nadxRegI__eADXRegL_low_only",
  /*  189 */  "_RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32",
  /*  190 */  "_MulL_eADXRegL_low_only__ConvI2L_nadxRegI_",
  /*  191 */  "_RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32",
  /*  192 */  "_RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63",
  /*  193 */  "_RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63",
  /*  194 */  "_ConvI2L_eAXRegI_",
  /*  195 */  "_AndL__ConvI2L_eAXRegI__immL_32bits",
  /*  196 */  "_AndL__ConvI2L_nadxRegI__immL_32bits",
  /*  197 */  "_RShiftI__LoadI_memory__immI_1",
  /*  198 */  "_RShiftI__LoadI_memory__immI8",
  /*  199 */  "_LShiftI_xRegI_immI_24",
  /*  200 */  "_LShiftI_xRegI_immI_16",
  /*  201 */  "_AndI__LoadI_memory__rRegI",
  /*  202 */  "_AndI_rRegI__LoadI_memory_",
  /*  203 */  "_AndI__LoadI_memory__immI",
  /*  204 */  "_XorI_rRegI_immI_M1",
  /*  205 */  "_SubI_immI_0__LoadI_memory_",
  /*  206 */  "_AddI_rRegI_immI_M1",
  /*  207 */  "_CastP2X_eRegP_",
  /*  208 */  "_OrI__LoadI_memory__rRegI",
  /*  209 */  "_OrI_rRegI__LoadI_memory_",
  /*  210 */  "_OrI__LoadI_memory__immI",
  /*  211 */  "_LShiftI_rRegI_immI_1",
  /*  212 */  "_URShiftI_rRegI_immI_M1",
  /*  213 */  "_LShiftI_rRegI_immI8",
  /*  214 */  "_URShiftI_rRegI_immI8",
  /*  215 */  "_LShiftI_ncxRegI_eCXRegI",
  /*  216 */  "_SubI_immI_0_eCXRegI",
  /*  217 */  "_URShiftI_ncxRegI__SubI_immI_0_eCXRegI",
  /*  218 */  "_SubI_immI_32_eCXRegI",
  /*  219 */  "_URShiftI_ncxRegI__SubI_immI_32_eCXRegI",
  /*  220 */  "_URShiftI_rRegI_immI_1",
  /*  221 */  "_LShiftI_rRegI_immI_M1",
  /*  222 */  "_URShiftI_ncxRegI_eCXRegI",
  /*  223 */  "_LShiftI_ncxRegI__SubI_immI_0_eCXRegI",
  /*  224 */  "_LShiftI_ncxRegI__SubI_immI_32_eCXRegI",
  /*  225 */  "_XorI__LoadI_memory__rRegI",
  /*  226 */  "_XorI_rRegI__LoadI_memory_",
  /*  227 */  "_XorI__LoadI_memory__immI",
  /*  228 */  "_CmpLTMask_rRegI_rRegI",
  /*  229 */  "_AndI__CmpLTMask_rRegI_rRegI_rRegI",
  /*  230 */  "_SubI_rRegI_rRegI",
  /*  231 */  "_AndI_rRegI__CmpLTMask_rRegI_rRegI",
  /*  232 */  "_LoadL_load_long_memory_",
  /*  233 */  "_XorL_eRegL_immL_M1",
  /*  234 */  "_SubL_immL0_eRegL",
  /*  235 */  "_SubL_immL0__LoadL_memory_",
  /*  236 */  "_AddL_eRegL_immL_M1",
  /*  237 */  "_AddL__LoadL_memory__immL_M1",
  /*  238 */  "_LoadD_memory_",
  /*  239 */  "_SubD_regDPR_regDPR",
  /*  240 */  "_AddD_regDPR_regDPR",
  /*  241 */  "_AddD__LoadD_memory__regDPR",
  /*  242 */  "_RoundDouble__AddD__LoadD_memory__regDPR_",
  /*  243 */  "_AddD_regDPR__LoadD_memory_",
  /*  244 */  "_RoundDouble__AddD_regDPR__LoadD_memory__",
  /*  245 */  "_AddD_regDPR_immDPR",
  /*  246 */  "_MulD_regDPR_regDPR",
  /*  247 */  "_DivD_regDPR_regDPR",
  /*  248 */  "_LoadF_memory_",
  /*  249 */  "_MulF__LoadF_memory__regFPR",
  /*  250 */  "_MulF_regFPR__LoadF_memory_",
  /*  251 */  "_MulF_regFPR_regFPR",
  /*  252 */  "_SubF_regFPR_regFPR",
  /*  253 */  "_ConvI2L_rRegI_",
  /*  254 */  "_Binary_eDIRegP_eCXRegI",
  /*  255 */  "_Binary_eSIRegP_eDXRegI",
  /*  256 */  "_Binary_eDIRegP_eSIRegP",
  /*  257 */  "_Binary_eDIRegP_eDXRegI",
  /*  258 */  "_Binary_eSIRegP_immI",
  /*  259 */  "_Binary_eSIRegP_eAXRegI",
  /*  260 */  "_AndI_rRegI_immI",
  /*  261 */  "_AndI_rRegI_memory",
  /*  262 */  "_LoadP_memory_",
  /*  263 */  "_Binary_nadxRegI_eAXRegI",
  /*  264 */  "_PartialSubtypeCheck_eSIRegP_eAXRegP",
  /*  265 */  "_Binary_cmpOp_flagsReg_long_LTGE",
  /*  266 */  "_Binary_eRegL__LoadL_load_long_memory_",
  /*  267 */  "_Binary_cmpOp_flagsReg_long_EQNE",
  /*  268 */  "_Binary_cmpOp_commute_flagsReg_long_LEGT",
  /*  269 */  "_Binary_cmpOpU_commute_flagsReg_ulong_LEGT",
  /*  270 */  "_Binary_vec_vec",
  /*  271 */  "_Binary_vec_rRegI",
  /*  272 */  "_Binary_vec_regF",
  /*  273 */  "_LoadVector_memory_",
  /*  274 */  "_Binary_cmpOp_vcmppd_immI8",
  /*  275 */  "_Binary_legVec_legVec",
  /*  276 */  "_LShiftCntV_immI8_",
  /*  277 */  "_RShiftCntV_immI8_",
  /*  278 */  "_Binary_vec__LoadVector_memory_",
  /*  279 */  "_MulAddVS2VI_vec_vec",
  /*  280 */  "_Binary_vec_immU8",
  /*  281 */  "_Binary__LoadVector_memory__immU8",
  // last internally defined operand
  /*  282 */  "storeSSI",
  /*  283 */  "storeSSP",
  /*  284 */  "storeSSL",
  /*  285 */  "MoveF2LEG",
  /*  286 */  "MoveLEG2F",
  /*  287 */  "MoveD2LEG",
  /*  288 */  "MoveLEG2D",
  /*  289 */  "leaP8",
  /*  290 */  "leaP32",
  /*  291 */  "leaPIdxOff",
  /*  292 */  "leaPIdxScale",
  /*  293 */  "leaPIdxScaleOff",
  /*  294 */  "loadConI",
  /*  295 */  "loadConI0",
  /*  296 */  "loadConP",
  /*  297 */  "loadConL",
  /*  298 */  "loadConL0",
  /*  299 */  "loadConFPR",
  /*  300 */  "loadConFPR0",
  /*  301 */  "loadConFPR1",
  /*  302 */  "loadConF",
  /*  303 */  "loadConF0",
  /*  304 */  "loadConDPR",
  /*  305 */  "loadConDPR0",
  /*  306 */  "loadConDPR1",
  /*  307 */  "loadConD",
  /*  308 */  "loadConD0",
  /*  309 */  "loadSSI",
  /*  310 */  "loadSSL",
  /*  311 */  "loadSSP",
  /*  312 */  "loadSSF",
  /*  313 */  "loadSSD",
  /*  314 */  "MoveD2VL",
  /*  315 */  "MoveVL2D",
  /*  316 */  "MoveF2VL",
  /*  317 */  "MoveVL2F",
  /*  318 */  "loadConL_low_only",
  /*  319 */  "MoveVec2Leg",
  /*  320 */  "MoveLeg2Vec",
  /*  321 */  "membar_volatile",
  /*  322 */  "castX2P",
  /*  323 */  "castP2X",
  /*  324 */  "checkCastPP",
  /*  325 */  "castPP",
  /*  326 */  "castII",
  /*  327 */  "castLL",
  /*  328 */  "storePConditional",
  /*  329 */  "storeIConditional",
  /*  330 */  "storeLConditional",
  /*  331 */  "movI_nocopy",
  /*  332 */  "movP_nocopy",
  /*  333 */  "overflowAddI_eReg",
  /*  334 */  "overflowAddI_rReg_imm",
  /*  335 */  "overflowSubI_rReg",
  /*  336 */  "overflowSubI_rReg_imm",
  /*  337 */  "overflowNegI_rReg",
  /*  338 */  "overflowMulI_rReg",
  /*  339 */  "overflowMulI_rReg_imm",
  /*  340 */  "cmpDPR_cc_P6",
  /*  341 */  "cmpDPR_cc_P6CF",
  /*  342 */  "cmpDPR_cc",
  /*  343 */  "cmpD_cc",
  /*  344 */  "cmpD_ccCF",
  /*  345 */  "cmpD_ccmem",
  /*  346 */  "cmpD_ccmemCF",
  /*  347 */  "absDPR_reg",
  /*  348 */  "negDPR_reg",
  /*  349 */  "cmpFPR_cc_P6",
  /*  350 */  "cmpFPR_cc_P6CF",
  /*  351 */  "cmpFPR_cc",
  /*  352 */  "cmpF_cc",
  /*  353 */  "cmpF_ccCF",
  /*  354 */  "cmpF_ccmem",
  /*  355 */  "cmpF_ccmemCF",
  /*  356 */  "absFPR_reg",
  /*  357 */  "negFPR_reg",
  /*  358 */  "MoveF2I_stack_reg",
  /*  359 */  "MoveF2I_reg_reg_sse",
  /*  360 */  "MoveI2FPR_stack_reg",
  /*  361 */  "MoveI2F_stack_reg_sse",
  /*  362 */  "MoveI2F_reg_reg_sse",
  /*  363 */  "MoveD2L_stack_reg",
  /*  364 */  "MoveL2DPR_stack_reg",
  /*  365 */  "MoveL2D_stack_reg_sse",
  /*  366 */  "MoveL2D_stack_reg_sse_partial",
  /*  367 */  "compI_eReg",
  /*  368 */  "compI_eReg_imm",
  /*  369 */  "compI_eReg_mem",
  /*  370 */  "testI_reg",
  /*  371 */  "testI_reg_imm",
  /*  372 */  "testI_reg_mem",
  /*  373 */  "compU_eReg",
  /*  374 */  "compU_eReg_imm",
  /*  375 */  "compU_eReg_mem",
  /*  376 */  "testU_reg",
  /*  377 */  "compP_eReg",
  /*  378 */  "compP_eReg_imm",
  /*  379 */  "compP_eReg_mem",
  /*  380 */  "compP_mem_eReg",
  /*  381 */  "testP_reg",
  /*  382 */  "testP_Reg_mem",
  /*  383 */  "partialSubtypeCheck_vs_Zero",
  /*  384 */  "cmpL_zero_flags_LTGE",
  /*  385 */  "cmpL_reg_flags_LTGE",
  /*  386 */  "cmpUL_zero_flags_LTGE",
  /*  387 */  "cmpUL_reg_flags_LTGE",
  /*  388 */  "cmpL_zero_flags_EQNE",
  /*  389 */  "cmpL_reg_flags_EQNE",
  /*  390 */  "cmpUL_zero_flags_EQNE",
  /*  391 */  "cmpUL_reg_flags_EQNE",
  /*  392 */  "cmpL_zero_flags_LEGT",
  /*  393 */  "cmpL_reg_flags_LEGT",
  /*  394 */  "cmpUL_zero_flags_LEGT",
  /*  395 */  "cmpUL_reg_flags_LEGT",
  /*  396 */  "cmpFastLockRTM",
  /*  397 */  "cmpFastLock",
  /*  398 */  "cmpFastUnlock",
  /*  399 */  "absF_reg",
  /*  400 */  "absF_reg_reg",
  /*  401 */  "absD_reg",
  /*  402 */  "absD_reg_reg",
  /*  403 */  "negF_reg",
  /*  404 */  "negF_reg_reg",
  /*  405 */  "negD_reg",
  /*  406 */  "negD_reg_reg",
  /*  407 */  "reinterpret",
  /*  408 */  "vreinterpret_expand",
  /*  409 */  "reinterpret_shrink",
  /*  410 */  "vsqrtF_reg",
  /*  411 */  "vsqrtF_mem",
  /*  412 */  "vsqrtD_reg",
  /*  413 */  "vsqrtD_mem",
  /*  414 */  "vshiftcnt",
  /*  415 */  "vshiftcnt_0",
  /*  416 */  "vcastBtoX",
  /*  417 */  "vcastStoX_evex",
  /*  418 */  "vcastItoX_evex",
  /*  419 */  "vcastLtoX_evex",
  /*  420 */  "vcastFtoD_reg",
  /*  421 */  "vcastDtoF_reg",
  /*  422 */  "vabsB_reg",
  /*  423 */  "vabsS_reg",
  /*  424 */  "vabsI_reg",
  /*  425 */  "vabsL_reg",
  /*  426 */  "loadShuffleB",
  /*  427 */  "loadShuffleS_evex",
  /*  428 */  "loadShuffleI_avx",
  /*  429 */  "loadShuffleL_evex",
  /*  430 */  "vpopcountI",
  /*  431 */  "bytes_reverse_int",
  /*  432 */  "bytes_reverse_long",
  /*  433 */  "bytes_reverse_unsigned_short",
  /*  434 */  "bytes_reverse_short",
  /*  435 */  "countLeadingZerosI",
  /*  436 */  "countLeadingZerosI_bsr",
  /*  437 */  "countLeadingZerosL",
  /*  438 */  "countLeadingZerosL_bsr",
  /*  439 */  "countTrailingZerosI",
  /*  440 */  "countTrailingZerosI_bsf",
  /*  441 */  "countTrailingZerosL",
  /*  442 */  "countTrailingZerosL_bsf",
  /*  443 */  "popCountI",
  /*  444 */  "popCountI_mem",
  /*  445 */  "popCountL",
  /*  446 */  "popCountL_mem",
  /*  447 */  "loadB",
  /*  448 */  "loadB2L",
  /*  449 */  "loadUB",
  /*  450 */  "loadUB2L",
  /*  451 */  "loadUB2L_immI",
  /*  452 */  "loadS",
  /*  453 */  "loadS2B",
  /*  454 */  "loadS2L",
  /*  455 */  "loadUS",
  /*  456 */  "loadUS2B",
  /*  457 */  "loadUS2L",
  /*  458 */  "loadUS2L_immI_255",
  /*  459 */  "loadUS2L_immI",
  /*  460 */  "loadI",
  /*  461 */  "loadI2B",
  /*  462 */  "loadI2UB",
  /*  463 */  "loadI2S",
  /*  464 */  "loadI2US",
  /*  465 */  "loadI2L",
  /*  466 */  "loadI2L_immI_255",
  /*  467 */  "loadI2L_immI_65535",
  /*  468 */  "loadI2L_immU31",
  /*  469 */  "loadUI2L",
  /*  470 */  "loadL",
  /*  471 */  "loadL_volatile",
  /*  472 */  "loadLX_volatile",
  /*  473 */  "loadLX_reg_volatile",
  /*  474 */  "loadRange",
  /*  475 */  "loadP",
  /*  476 */  "loadKlass",
  /*  477 */  "loadDPR",
  /*  478 */  "loadD",
  /*  479 */  "loadD_partial",
  /*  480 */  "loadF",
  /*  481 */  "loadFPR",
  /*  482 */  "prefetchAlloc0",
  /*  483 */  "prefetchAlloc",
  /*  484 */  "prefetchAllocNTA",
  /*  485 */  "prefetchAllocT0",
  /*  486 */  "prefetchAllocT2",
  /*  487 */  "storeB",
  /*  488 */  "storeC",
  /*  489 */  "storeI",
  /*  490 */  "storeL",
  /*  491 */  "storeL2I",
  /*  492 */  "storeL_volatile",
  /*  493 */  "storeLX_volatile",
  /*  494 */  "storeLX_reg_volatile",
  /*  495 */  "storeP",
  /*  496 */  "storeImmI",
  /*  497 */  "storeImmI16",
  /*  498 */  "storeImmP",
  /*  499 */  "storeImmB",
  /*  500 */  "storeImmCM",
  /*  501 */  "storeDPR",
  /*  502 */  "storeDPR_rounded",
  /*  503 */  "storeD",
  /*  504 */  "storeF",
  /*  505 */  "storeFPR",
  /*  506 */  "storeFPR_rounded",
  /*  507 */  "storeFPR_Drounded",
  /*  508 */  "storeFPR_imm",
  /*  509 */  "storeF_imm",
  /*  510 */  "membar_acquire",
  /*  511 */  "membar_acquire_0",
  /*  512 */  "membar_acquire_lock",
  /*  513 */  "membar_release",
  /*  514 */  "membar_release_0",
  /*  515 */  "membar_release_lock",
  /*  516 */  "unnecessary_membar_volatile",
  /*  517 */  "membar_storestore",
  /*  518 */  "jmovI_reg",
  /*  519 */  "jmovI_regU",
  /*  520 */  "cmovI_reg",
  /*  521 */  "cmovI_regU",
  /*  522 */  "cmovI_regUCF",
  /*  523 */  "cmovI_mem",
  /*  524 */  "cmovI_memU",
  /*  525 */  "cmovI_memUCF",
  /*  526 */  "cmovP_reg",
  /*  527 */  "cmovP_reg_nonP6",
  /*  528 */  "cmovP_regU",
  /*  529 */  "cmovP_regUCF",
  /*  530 */  "fcmovDPR_regU",
  /*  531 */  "fcmovFPR_regU",
  /*  532 */  "fcmovDPR_regS",
  /*  533 */  "fcmovFPR_regS",
  /*  534 */  "fcmovF_regS",
  /*  535 */  "fcmovD_regS",
  /*  536 */  "fcmovF_regU",
  /*  537 */  "fcmovF_regUCF",
  /*  538 */  "fcmovD_regU",
  /*  539 */  "fcmovD_regUCF",
  /*  540 */  "cmovL_reg",
  /*  541 */  "cmovL_regU",
  /*  542 */  "cmovL_regUCF",
  /*  543 */  "addI_eReg",
  /*  544 */  "addI_eReg_imm",
  /*  545 */  "incI_eReg",
  /*  546 */  "leaI_eReg_immI",
  /*  547 */  "leaP_eReg_immI",
  /*  548 */  "decI_eReg",
  /*  549 */  "addP_eReg",
  /*  550 */  "addP_eReg_imm",
  /*  551 */  "addI_eReg_mem",
  /*  552 */  "addI_eReg_mem_0",
  /*  553 */  "addI_mem_eReg",
  /*  554 */  "addI_mem_eReg_0",
  /*  555 */  "addI_mem_imm",
  /*  556 */  "incI_mem",
  /*  557 */  "decI_mem",
  /*  558 */  "loadPLocked",
  /*  559 */  "compareAndSwapL",
  /*  560 */  "compareAndSwapL_0",
  /*  561 */  "compareAndSwapP",
  /*  562 */  "compareAndSwapP_0",
  /*  563 */  "compareAndSwapB",
  /*  564 */  "compareAndSwapB_0",
  /*  565 */  "compareAndSwapS",
  /*  566 */  "compareAndSwapS_0",
  /*  567 */  "compareAndSwapI",
  /*  568 */  "compareAndSwapI_0",
  /*  569 */  "compareAndExchangeL",
  /*  570 */  "compareAndExchangeP",
  /*  571 */  "compareAndExchangeB",
  /*  572 */  "compareAndExchangeS",
  /*  573 */  "compareAndExchangeI",
  /*  574 */  "xaddB_no_res",
  /*  575 */  "xaddB",
  /*  576 */  "xaddS_no_res",
  /*  577 */  "xaddS",
  /*  578 */  "xaddI_no_res",
  /*  579 */  "xaddI",
  /*  580 */  "xchgB",
  /*  581 */  "xchgS",
  /*  582 */  "xchgI",
  /*  583 */  "xchgP",
  /*  584 */  "subI_eReg",
  /*  585 */  "subI_eReg_imm",
  /*  586 */  "subI_eReg_mem",
  /*  587 */  "subI_mem_eReg",
  /*  588 */  "subP_eReg",
  /*  589 */  "negI_eReg",
  /*  590 */  "mulI_eReg",
  /*  591 */  "mulI_eReg_imm",
  /*  592 */  "mulI_imm_high",
  /*  593 */  "mulI_imm_high_0",
  /*  594 */  "mulI_imm_RShift_high",
  /*  595 */  "mulI_imm_RShift_high_0",
  /*  596 */  "mulI_mem_imm",
  /*  597 */  "mulI",
  /*  598 */  "mulI_0",
  /*  599 */  "mulAddS2I_rReg",
  /*  600 */  "mulI2L",
  /*  601 */  "mulI2L_0",
  /*  602 */  "mulIS_eReg",
  /*  603 */  "mulIS_eReg_0",
  /*  604 */  "mulL_eReg",
  /*  605 */  "mulL_eReg_lhi0",
  /*  606 */  "mulL_eReg_rhi0",
  /*  607 */  "mulL_eReg_hi0",
  /*  608 */  "mulL_eReg_con",
  /*  609 */  "divI_eReg",
  /*  610 */  "divL_eReg",
  /*  611 */  "divModI_eReg_divmod",
  /*  612 */  "modI_eReg",
  /*  613 */  "modL_eReg",
  /*  614 */  "divL_eReg_imm32",
  /*  615 */  "modL_eReg_imm32",
  /*  616 */  "shlI_eReg_1",
  /*  617 */  "salI_eReg_imm",
  /*  618 */  "salI_eReg_CL",
  /*  619 */  "sarI_eReg_1",
  /*  620 */  "sarI_mem_1",
  /*  621 */  "sarI_eReg_imm",
  /*  622 */  "sarI_mem_imm",
  /*  623 */  "sarI_eReg_CL",
  /*  624 */  "shrI_eReg_1",
  /*  625 */  "shrI_eReg_imm",
  /*  626 */  "i2b",
  /*  627 */  "i2s",
  /*  628 */  "shrI_eReg_CL",
  /*  629 */  "andI_eReg",
  /*  630 */  "andI_eReg_imm",
  /*  631 */  "andI_eReg_mem",
  /*  632 */  "andI_eReg_mem_0",
  /*  633 */  "andI_mem_eReg",
  /*  634 */  "andI_mem_eReg_0",
  /*  635 */  "andI_mem_imm",
  /*  636 */  "andnI_rReg_rReg_rReg",
  /*  637 */  "andnI_rReg_rReg_rReg_0",
  /*  638 */  "andnI_rReg_rReg_mem",
  /*  639 */  "andnI_rReg_rReg_mem_0",
  /*  640 */  "blsiI_rReg_rReg",
  /*  641 */  "blsiI_rReg_rReg_0",
  /*  642 */  "blsiI_rReg_mem",
  /*  643 */  "blsiI_rReg_mem_0",
  /*  644 */  "blsmskI_rReg_rReg",
  /*  645 */  "blsmskI_rReg_rReg_0",
  /*  646 */  "blsmskI_rReg_mem",
  /*  647 */  "blsmskI_rReg_mem_0",
  /*  648 */  "blsrI_rReg_rReg",
  /*  649 */  "blsrI_rReg_rReg_0",
  /*  650 */  "blsrI_rReg_mem",
  /*  651 */  "blsrI_rReg_mem_0",
  /*  652 */  "orI_eReg",
  /*  653 */  "orI_eReg_castP2X",
  /*  654 */  "orI_eReg_castP2X_0",
  /*  655 */  "orI_eReg_imm",
  /*  656 */  "orI_eReg_mem",
  /*  657 */  "orI_eReg_mem_0",
  /*  658 */  "orI_mem_eReg",
  /*  659 */  "orI_mem_eReg_0",
  /*  660 */  "orI_mem_imm",
  /*  661 */  "rolI_eReg_imm1",
  /*  662 */  "rolI_eReg_imm8",
  /*  663 */  "rolI_eReg_CL",
  /*  664 */  "rolI_eReg_i1",
  /*  665 */  "rolI_eReg_i1_0",
  /*  666 */  "rolI_eReg_i8",
  /*  667 */  "rolI_eReg_i8_0",
  /*  668 */  "rolI_eReg_Var_C0",
  /*  669 */  "rolI_eReg_Var_C0_0",
  /*  670 */  "rolI_eReg_Var_C32",
  /*  671 */  "rolI_eReg_Var_C32_0",
  /*  672 */  "rorI_eReg_imm1",
  /*  673 */  "rorI_eReg_imm8",
  /*  674 */  "rorI_eReg_CL",
  /*  675 */  "rorI_eReg_i1",
  /*  676 */  "rorI_eReg_i1_0",
  /*  677 */  "rorI_eReg_i8",
  /*  678 */  "rorI_eReg_i8_0",
  /*  679 */  "rorI_eReg_Var_C0",
  /*  680 */  "rorI_eReg_Var_C0_0",
  /*  681 */  "rorI_eReg_Var_C32",
  /*  682 */  "rorI_eReg_Var_C32_0",
  /*  683 */  "xorI_eReg",
  /*  684 */  "xorI_eReg_im1",
  /*  685 */  "xorI_eReg_imm",
  /*  686 */  "xorI_eReg_mem",
  /*  687 */  "xorI_eReg_mem_0",
  /*  688 */  "xorI_mem_eReg",
  /*  689 */  "xorI_mem_eReg_0",
  /*  690 */  "xorI_mem_imm",
  /*  691 */  "ci2b",
  /*  692 */  "convI2B",
  /*  693 */  "cp2b",
  /*  694 */  "convP2B",
  /*  695 */  "cmpLTMask",
  /*  696 */  "cmpLTMask0",
  /*  697 */  "cadd_cmpLTMask",
  /*  698 */  "cadd_cmpLTMask_1",
  /*  699 */  "cadd_cmpLTMask_0",
  /*  700 */  "cadd_cmpLTMask_2",
  /*  701 */  "and_cmpLTMask",
  /*  702 */  "and_cmpLTMask_0",
  /*  703 */  "absI_rReg",
  /*  704 */  "addL_eReg",
  /*  705 */  "addL_eReg_imm",
  /*  706 */  "addL_eReg_mem",
  /*  707 */  "addL_eReg_mem_0",
  /*  708 */  "subL_eReg",
  /*  709 */  "subL_eReg_imm",
  /*  710 */  "subL_eReg_mem",
  /*  711 */  "negL_eReg",
  /*  712 */  "andL_eReg",
  /*  713 */  "andL_eReg_imm",
  /*  714 */  "andL_eReg_mem",
  /*  715 */  "andL_eReg_mem_0",
  /*  716 */  "andnL_eReg_eReg_eReg",
  /*  717 */  "andnL_eReg_eReg_eReg_0",
  /*  718 */  "andnL_eReg_eReg_mem",
  /*  719 */  "andnL_eReg_eReg_mem_0",
  /*  720 */  "blsiL_eReg_eReg",
  /*  721 */  "blsiL_eReg_eReg_0",
  /*  722 */  "blsiL_eReg_mem",
  /*  723 */  "blsiL_eReg_mem_0",
  /*  724 */  "blsmskL_eReg_eReg",
  /*  725 */  "blsmskL_eReg_eReg_0",
  /*  726 */  "blsmskL_eReg_mem",
  /*  727 */  "blsmskL_eReg_mem_0",
  /*  728 */  "blsrL_eReg_eReg",
  /*  729 */  "blsrL_eReg_eReg_0",
  /*  730 */  "blsrL_eReg_mem",
  /*  731 */  "blsrL_eReg_mem_0",
  /*  732 */  "orl_eReg",
  /*  733 */  "orl_eReg_imm",
  /*  734 */  "orl_eReg_mem",
  /*  735 */  "orl_eReg_mem_0",
  /*  736 */  "xorl_eReg",
  /*  737 */  "xorl_eReg_im1",
  /*  738 */  "xorl_eReg_imm",
  /*  739 */  "xorl_eReg_mem",
  /*  740 */  "xorl_eReg_mem_0",
  /*  741 */  "shlL_eReg_1",
  /*  742 */  "shlL_eReg_2",
  /*  743 */  "shlL_eReg_3",
  /*  744 */  "shlL_eReg_1_31",
  /*  745 */  "shlL_eReg_32_63",
  /*  746 */  "salL_eReg_CL",
  /*  747 */  "shrL_eReg_1_31",
  /*  748 */  "shrL_eReg_32_63",
  /*  749 */  "shrL_eReg_CL",
  /*  750 */  "sarL_eReg_1_31",
  /*  751 */  "sarL_eReg_32_63",
  /*  752 */  "sarL_eReg_CL",
  /*  753 */  "cmpDPR_0",
  /*  754 */  "cmpDPR_reg",
  /*  755 */  "cmpD_reg",
  /*  756 */  "cmpD_regmem",
  /*  757 */  "subDPR_reg",
  /*  758 */  "subDPR_reg_round",
  /*  759 */  "subDPR_reg_mem",
  /*  760 */  "addDPR_reg",
  /*  761 */  "addDPR_reg_round",
  /*  762 */  "addDPR_reg_mem",
  /*  763 */  "addDPR_reg_mem_0",
  /*  764 */  "addDPR_mem_reg",
  /*  765 */  "addDPR_mem_reg_0",
  /*  766 */  "addDPR_reg_imm1",
  /*  767 */  "addDPR_reg_imm",
  /*  768 */  "addDPR_reg_imm_round",
  /*  769 */  "mulDPR_reg",
  /*  770 */  "strictfp_mulDPR_reg",
  /*  771 */  "mulDPR_reg_imm",
  /*  772 */  "mulDPR_reg_mem",
  /*  773 */  "mulDPR_reg_mem_0",
  /*  774 */  "mulDPR_reg_mem_cisc",
  /*  775 */  "mulDPR_reg_mem_cisc_0",
  /*  776 */  "addDPR_mulDPR_reg",
  /*  777 */  "addDPR_mulDPR_reg_0",
  /*  778 */  "subDPR_mulDPR_reg",
  /*  779 */  "divDPR_reg",
  /*  780 */  "strictfp_divDPR_reg",
  /*  781 */  "divDPR_reg_round",
  /*  782 */  "modDPR_reg",
  /*  783 */  "modD_reg",
  /*  784 */  "atanDPR_reg",
  /*  785 */  "atanD_reg",
  /*  786 */  "sqrtDPR_reg",
  /*  787 */  "cmpFPR_0",
  /*  788 */  "cmpFPR_reg",
  /*  789 */  "cmpF_reg",
  /*  790 */  "cmpF_regmem",
  /*  791 */  "subFPR24_reg",
  /*  792 */  "subFPR_reg",
  /*  793 */  "addFPR24_reg",
  /*  794 */  "addFPR_reg",
  /*  795 */  "addFPR24_reg_mem",
  /*  796 */  "addFPR24_reg_mem_0",
  /*  797 */  "addFPR_reg_mem",
  /*  798 */  "addFPR_reg_mem_0",
  /*  799 */  "addFPR24_mem_reg",
  /*  800 */  "addFPR24_mem_cisc",
  /*  801 */  "addFPR24_mem_cisc_0",
  /*  802 */  "addFPR24_mem_mem",
  /*  803 */  "addFPR24_reg_imm",
  /*  804 */  "addFPR_reg_imm",
  /*  805 */  "mulFPR24_reg",
  /*  806 */  "mulFPR_reg",
  /*  807 */  "mulFPR24_reg_mem",
  /*  808 */  "mulFPR24_reg_mem_0",
  /*  809 */  "mulFPR_reg_mem",
  /*  810 */  "mulFPR_reg_mem_0",
  /*  811 */  "mulFPR24_mem_mem",
  /*  812 */  "mulFPR24_reg_imm",
  /*  813 */  "mulFPR_reg_imm",
  /*  814 */  "mulFPR_reg_load1",
  /*  815 */  "mulFPR_reg_load1_0",
  /*  816 */  "addFPR_mulFPR_reg_load1",
  /*  817 */  "addFPR_mulFPR_reg_load1_1",
  /*  818 */  "addFPR_mulFPR_reg_load1_0",
  /*  819 */  "addFPR_mulFPR_reg_load1_2",
  /*  820 */  "addFPR_mulFPR_reg",
  /*  821 */  "addFPR_mulFPR_reg_0",
  /*  822 */  "subFPR_divFPR_reg",
  /*  823 */  "divFPR24_reg",
  /*  824 */  "divFPR_reg",
  /*  825 */  "modFPR24_reg",
  /*  826 */  "modFPR_reg",
  /*  827 */  "modF_reg",
  /*  828 */  "roundFloat_mem_reg",
  /*  829 */  "roundDouble_mem_reg",
  /*  830 */  "convDPR2FPR_reg",
  /*  831 */  "convDPR2F_reg",
  /*  832 */  "convD2F_reg",
  /*  833 */  "convFPR2DPR_reg_reg",
  /*  834 */  "convFPR2D_reg",
  /*  835 */  "convF2DPR_reg",
  /*  836 */  "convF2D_reg",
  /*  837 */  "convDPR2I_reg_reg",
  /*  838 */  "convD2I_reg_reg",
  /*  839 */  "convDPR2L_reg_reg",
  /*  840 */  "convD2L_reg_reg",
  /*  841 */  "convFPR2I_reg_reg",
  /*  842 */  "convF2I_reg",
  /*  843 */  "convFPR2L_reg_reg",
  /*  844 */  "convF2L_reg_reg",
  /*  845 */  "convI2DPR_reg",
  /*  846 */  "convI2D_reg",
  /*  847 */  "convI2D_mem",
  /*  848 */  "convXI2D_reg",
  /*  849 */  "convI2DPR_mem",
  /*  850 */  "conv24I2FPR_reg",
  /*  851 */  "convI2FPR_SSF",
  /*  852 */  "convI2FPR_SSF_mem",
  /*  853 */  "convI2FPR_reg",
  /*  854 */  "convI2FPR_mem",
  /*  855 */  "convI2F_reg",
  /*  856 */  "convXI2F_reg",
  /*  857 */  "convI2L_reg",
  /*  858 */  "convI2L_reg_zex",
  /*  859 */  "zerox_long",
  /*  860 */  "convL2DPR_reg",
  /*  861 */  "convL2D_reg",
  /*  862 */  "convL2F_reg",
  /*  863 */  "convL2FPR_reg",
  /*  864 */  "convL2I_reg",
  /*  865 */  "MoveFPR2I_reg_stack",
  /*  866 */  "MoveF2I_reg_stack_sse",
  /*  867 */  "MoveI2F_reg_stack",
  /*  868 */  "MoveDPR2L_reg_stack",
  /*  869 */  "MoveD2L_reg_stack_sse",
  /*  870 */  "MoveD2L_reg_reg_sse",
  /*  871 */  "MoveL2D_reg_stack",
  /*  872 */  "MoveL2D_reg_reg_sse",
  /*  873 */  "rep_stos",
  /*  874 */  "rep_stos_large",
  /*  875 */  "string_compareL",
  /*  876 */  "string_compareU",
  /*  877 */  "string_compareLU",
  /*  878 */  "string_compareUL",
  /*  879 */  "string_equals",
  /*  880 */  "string_indexof_conL",
  /*  881 */  "string_indexof_conU",
  /*  882 */  "string_indexof_conUL",
  /*  883 */  "string_indexofL",
  /*  884 */  "string_indexofU",
  /*  885 */  "string_indexofUL",
  /*  886 */  "string_indexof_char",
  /*  887 */  "stringL_indexof_char",
  /*  888 */  "array_equalsB",
  /*  889 */  "array_equalsC",
  /*  890 */  "has_negatives",
  /*  891 */  "string_compress",
  /*  892 */  "string_inflate",
  /*  893 */  "encode_iso_array",
  /*  894 */  "minI_eReg",
  /*  895 */  "maxI_eReg",
  /*  896 */  "loopLimit_eReg",
  /*  897 */  "jumpXtnd",
  /*  898 */  "jmpDir",
  /*  899 */  "jmpCon",
  /*  900 */  "jmpLoopEnd",
  /*  901 */  "jmpLoopEndU",
  /*  902 */  "jmpLoopEndUCF",
  /*  903 */  "jmpLoopEnd_and_restoreMask",
  /*  904 */  "jmpLoopEndU_and_restoreMask",
  /*  905 */  "jmpLoopEndUCF_and_restoreMask",
  /*  906 */  "jmpConU",
  /*  907 */  "jmpConUCF",
  /*  908 */  "jmpConUCF2",
  /*  909 */  "partialSubtypeCheck",
  /*  910 */  "jmpDir_short",
  /*  911 */  "jmpCon_short",
  /*  912 */  "jmpLoopEnd_short",
  /*  913 */  "jmpLoopEndU_short",
  /*  914 */  "jmpLoopEndUCF_short",
  /*  915 */  "jmpConU_short",
  /*  916 */  "jmpConUCF_short",
  /*  917 */  "jmpConUCF2_short",
  /*  918 */  "cmpL3_reg_reg",
  /*  919 */  "cmpL_LTGE",
  /*  920 */  "cmpUL_LTGE",
  /*  921 */  "cmovLL_reg_LTGE",
  /*  922 */  "cmovLL_mem_LTGE",
  /*  923 */  "cmovII_reg_LTGE",
  /*  924 */  "cmovII_mem_LTGE",
  /*  925 */  "cmovPP_reg_LTGE",
  /*  926 */  "cmovDDPR_reg_LTGE",
  /*  927 */  "cmovDD_reg_LTGE",
  /*  928 */  "cmovFFPR_reg_LTGE",
  /*  929 */  "cmovFF_reg_LTGE",
  /*  930 */  "cmpL_EQNE",
  /*  931 */  "cmpUL_EQNE",
  /*  932 */  "cmovLL_reg_EQNE",
  /*  933 */  "cmovLL_mem_EQNE",
  /*  934 */  "cmovII_reg_EQNE",
  /*  935 */  "cmovII_mem_EQNE",
  /*  936 */  "cmovPP_reg_EQNE",
  /*  937 */  "cmovDDPR_reg_EQNE",
  /*  938 */  "cmovDD_reg_EQNE",
  /*  939 */  "cmovFFPR_reg_EQNE",
  /*  940 */  "cmovFF_reg_EQNE",
  /*  941 */  "cmpL_LEGT",
  /*  942 */  "cmpUL_LEGT",
  /*  943 */  "cmovLL_reg_LEGT",
  /*  944 */  "cmovLL_mem_LEGT",
  /*  945 */  "cmovLL_reg_LEGT_U",
  /*  946 */  "cmovLL_mem_LEGT_U",
  /*  947 */  "cmovII_reg_LEGT",
  /*  948 */  "cmovII_mem_LEGT",
  /*  949 */  "cmovPP_reg_LEGT",
  /*  950 */  "cmovDDPR_reg_LEGT",
  /*  951 */  "cmovDD_reg_LEGT",
  /*  952 */  "cmovFFPR_reg_LEGT",
  /*  953 */  "cmovFF_reg_LEGT",
  /*  954 */  "CallStaticJavaDirect",
  /*  955 */  "CallDynamicJavaDirect",
  /*  956 */  "CallRuntimeDirect",
  /*  957 */  "CallLeafDirect",
  /*  958 */  "CallLeafNoFPDirect",
  /*  959 */  "Ret",
  /*  960 */  "TailCalljmpInd",
  /*  961 */  "tailjmpInd",
  /*  962 */  "CreateException",
  /*  963 */  "RethrowException",
  /*  964 */  "safePoint_poll_tls",
  /*  965 */  "tlsLoadP",
  /*  966 */  "ShouldNotReachHere",
  /*  967 */  "setMask",
  /*  968 */  "addF_reg",
  /*  969 */  "addF_mem",
  /*  970 */  "addF_mem_0",
  /*  971 */  "addF_imm",
  /*  972 */  "addF_reg_reg",
  /*  973 */  "addF_reg_mem",
  /*  974 */  "addF_reg_mem_0",
  /*  975 */  "addF_reg_imm",
  /*  976 */  "addD_reg",
  /*  977 */  "addD_mem",
  /*  978 */  "addD_mem_0",
  /*  979 */  "addD_imm",
  /*  980 */  "addD_reg_reg",
  /*  981 */  "addD_reg_mem",
  /*  982 */  "addD_reg_mem_0",
  /*  983 */  "addD_reg_imm",
  /*  984 */  "subF_reg",
  /*  985 */  "subF_mem",
  /*  986 */  "subF_imm",
  /*  987 */  "subF_reg_reg",
  /*  988 */  "subF_reg_mem",
  /*  989 */  "subF_reg_imm",
  /*  990 */  "subD_reg",
  /*  991 */  "subD_mem",
  /*  992 */  "subD_imm",
  /*  993 */  "subD_reg_reg",
  /*  994 */  "subD_reg_mem",
  /*  995 */  "subD_reg_imm",
  /*  996 */  "mulF_reg",
  /*  997 */  "mulF_mem",
  /*  998 */  "mulF_mem_0",
  /*  999 */  "mulF_imm",
  /* 1000 */  "mulF_reg_reg",
  /* 1001 */  "mulF_reg_mem",
  /* 1002 */  "mulF_reg_mem_0",
  /* 1003 */  "mulF_reg_imm",
  /* 1004 */  "mulD_reg",
  /* 1005 */  "mulD_mem",
  /* 1006 */  "mulD_mem_0",
  /* 1007 */  "mulD_imm",
  /* 1008 */  "mulD_reg_reg",
  /* 1009 */  "mulD_reg_mem",
  /* 1010 */  "mulD_reg_mem_0",
  /* 1011 */  "mulD_reg_imm",
  /* 1012 */  "divF_reg",
  /* 1013 */  "divF_mem",
  /* 1014 */  "divF_imm",
  /* 1015 */  "divF_reg_reg",
  /* 1016 */  "divF_reg_mem",
  /* 1017 */  "divF_reg_imm",
  /* 1018 */  "divD_reg",
  /* 1019 */  "divD_mem",
  /* 1020 */  "divD_imm",
  /* 1021 */  "divD_reg_reg",
  /* 1022 */  "divD_reg_mem",
  /* 1023 */  "divD_reg_imm",
  /* 1024 */  "sqrtF_reg",
  /* 1025 */  "sqrtF_mem",
  /* 1026 */  "sqrtF_imm",
  /* 1027 */  "sqrtD_reg",
  /* 1028 */  "sqrtD_mem",
  /* 1029 */  "sqrtD_imm",
  /* 1030 */  "reinterpret_expand",
  /* 1031 */  "vreinterpret_expand4",
  /* 1032 */  "onspinwait",
  /* 1033 */  "fmaD_reg",
  /* 1034 */  "fmaF_reg",
  /* 1035 */  "loadV",
  /* 1036 */  "storeV",
  /* 1037 */  "gather",
  /* 1038 */  "evgather",
  /* 1039 */  "scatter",
  /* 1040 */  "ReplB_reg",
  /* 1041 */  "ReplB_mem",
  /* 1042 */  "ReplB_imm",
  /* 1043 */  "ReplB_zero",
  /* 1044 */  "ReplS_reg",
  /* 1045 */  "ReplS_mem",
  /* 1046 */  "ReplS_imm",
  /* 1047 */  "ReplS_zero",
  /* 1048 */  "ReplI_reg",
  /* 1049 */  "ReplI_mem",
  /* 1050 */  "ReplI_imm",
  /* 1051 */  "ReplI_zero",
  /* 1052 */  "ReplI_M1",
  /* 1053 */  "ReplI_M1_0",
  /* 1054 */  "ReplI_M1_1",
  /* 1055 */  "ReplL_reg",
  /* 1056 */  "ReplL_reg_leg",
  /* 1057 */  "ReplL_mem",
  /* 1058 */  "ReplL_imm",
  /* 1059 */  "ReplL_zero",
  /* 1060 */  "ReplL_M1",
  /* 1061 */  "ReplF_reg",
  /* 1062 */  "ReplF_mem",
  /* 1063 */  "ReplF_zero",
  /* 1064 */  "ReplD_reg",
  /* 1065 */  "ReplD_mem",
  /* 1066 */  "ReplD_zero",
  /* 1067 */  "insert",
  /* 1068 */  "insert32",
  /* 1069 */  "insert64",
  /* 1070 */  "insertF",
  /* 1071 */  "vinsertF",
  /* 1072 */  "reductionI",
  /* 1073 */  "reductionI_0",
  /* 1074 */  "reductionI_1",
  /* 1075 */  "reductionI_2",
  /* 1076 */  "reductionI_3",
  /* 1077 */  "reductionI_4",
  /* 1078 */  "reductionI_5",
  /* 1079 */  "reductionF128",
  /* 1080 */  "reductionF128_0",
  /* 1081 */  "reduction8F",
  /* 1082 */  "reduction8F_0",
  /* 1083 */  "reduction16F",
  /* 1084 */  "reduction16F_0",
  /* 1085 */  "reduction2D",
  /* 1086 */  "reduction2D_0",
  /* 1087 */  "reduction4D",
  /* 1088 */  "reduction4D_0",
  /* 1089 */  "reduction8D",
  /* 1090 */  "reduction8D_0",
  /* 1091 */  "reductionS",
  /* 1092 */  "reductionS_0",
  /* 1093 */  "reductionS_1",
  /* 1094 */  "reductionS_2",
  /* 1095 */  "reductionS_3",
  /* 1096 */  "reductionS_4",
  /* 1097 */  "reductionS_5",
  /* 1098 */  "mul_reductionB",
  /* 1099 */  "mul_reduction64B",
  /* 1100 */  "minmax_reduction2F",
  /* 1101 */  "minmax_reduction2F_0",
  /* 1102 */  "minmax_reductionF",
  /* 1103 */  "minmax_reductionF_0",
  /* 1104 */  "minmax_reduction2F_av",
  /* 1105 */  "minmax_reduction2F_av_0",
  /* 1106 */  "minmax_reductionF_av",
  /* 1107 */  "minmax_reductionF_av_0",
  /* 1108 */  "minmax_reduction2D",
  /* 1109 */  "minmax_reduction2D_0",
  /* 1110 */  "minmax_reductionD",
  /* 1111 */  "minmax_reductionD_0",
  /* 1112 */  "minmax_reduction2D_av",
  /* 1113 */  "minmax_reduction2D_av_0",
  /* 1114 */  "minmax_reductionD_av",
  /* 1115 */  "minmax_reductionD_av_0",
  /* 1116 */  "vaddB",
  /* 1117 */  "vaddB_reg",
  /* 1118 */  "vaddB_mem",
  /* 1119 */  "vaddB_mem_0",
  /* 1120 */  "vaddS",
  /* 1121 */  "vaddS_reg",
  /* 1122 */  "vaddS_mem",
  /* 1123 */  "vaddS_mem_0",
  /* 1124 */  "vaddI",
  /* 1125 */  "vaddI_reg",
  /* 1126 */  "vaddI_mem",
  /* 1127 */  "vaddI_mem_0",
  /* 1128 */  "vaddL",
  /* 1129 */  "vaddL_reg",
  /* 1130 */  "vaddL_mem",
  /* 1131 */  "vaddL_mem_0",
  /* 1132 */  "vaddF",
  /* 1133 */  "vaddF_reg",
  /* 1134 */  "vaddF_mem",
  /* 1135 */  "vaddF_mem_0",
  /* 1136 */  "vaddD",
  /* 1137 */  "vaddD_reg",
  /* 1138 */  "vaddD_mem",
  /* 1139 */  "vaddD_mem_0",
  /* 1140 */  "vsubB",
  /* 1141 */  "vsubB_reg",
  /* 1142 */  "vsubB_mem",
  /* 1143 */  "vsubS",
  /* 1144 */  "vsubS_reg",
  /* 1145 */  "vsubS_mem",
  /* 1146 */  "vsubI",
  /* 1147 */  "vsubI_reg",
  /* 1148 */  "vsubI_mem",
  /* 1149 */  "vsubL",
  /* 1150 */  "vsubL_reg",
  /* 1151 */  "vsubL_mem",
  /* 1152 */  "vsubF",
  /* 1153 */  "vsubF_reg",
  /* 1154 */  "vsubF_mem",
  /* 1155 */  "vsubD",
  /* 1156 */  "vsubD_reg",
  /* 1157 */  "vsubD_mem",
  /* 1158 */  "mulB_reg",
  /* 1159 */  "mul16B_reg",
  /* 1160 */  "vmul16B_reg_avx",
  /* 1161 */  "vmul32B_reg_avx",
  /* 1162 */  "vmul64B_reg_avx",
  /* 1163 */  "vmulS",
  /* 1164 */  "vmulS_reg",
  /* 1165 */  "vmulS_mem",
  /* 1166 */  "vmulS_mem_0",
  /* 1167 */  "vmulI",
  /* 1168 */  "vmulI_reg",
  /* 1169 */  "vmulI_mem",
  /* 1170 */  "vmulI_mem_0",
  /* 1171 */  "vmulL_reg",
  /* 1172 */  "vmulL_mem",
  /* 1173 */  "vmulL_mem_0",
  /* 1174 */  "mul2L_reg",
  /* 1175 */  "vmul4L_reg_avx",
  /* 1176 */  "vmulF",
  /* 1177 */  "vmulF_reg",
  /* 1178 */  "vmulF_mem",
  /* 1179 */  "vmulF_mem_0",
  /* 1180 */  "vmulD",
  /* 1181 */  "vmulD_reg",
  /* 1182 */  "vmulD_mem",
  /* 1183 */  "vmulD_mem_0",
  /* 1184 */  "vcmov8F_reg",
  /* 1185 */  "vcmov4D_reg",
  /* 1186 */  "vdivF",
  /* 1187 */  "vdivF_reg",
  /* 1188 */  "vdivF_mem",
  /* 1189 */  "vdivD",
  /* 1190 */  "vdivD_reg",
  /* 1191 */  "vdivD_mem",
  /* 1192 */  "minmax_reg_sse",
  /* 1193 */  "minmax_reg_sse_0",
  /* 1194 */  "vminmax_reg",
  /* 1195 */  "vminmax_reg_0",
  /* 1196 */  "minmaxL_reg_sse",
  /* 1197 */  "minmaxL_reg_sse_0",
  /* 1198 */  "vminmaxL_reg_avx",
  /* 1199 */  "vminmaxL_reg_avx_0",
  /* 1200 */  "vminmaxL_reg_evex",
  /* 1201 */  "vminmaxL_reg_evex_0",
  /* 1202 */  "minmaxFP_reg",
  /* 1203 */  "minmaxFP_reg_0",
  /* 1204 */  "evminmaxFP_reg_eavx",
  /* 1205 */  "evminmaxFP_reg_eavx_0",
  /* 1206 */  "vshiftB",
  /* 1207 */  "vshiftB_0",
  /* 1208 */  "vshiftB_1",
  /* 1209 */  "vshift16B",
  /* 1210 */  "vshift16B_0",
  /* 1211 */  "vshift16B_1",
  /* 1212 */  "vshift16B_avx",
  /* 1213 */  "vshift16B_avx_0",
  /* 1214 */  "vshift16B_avx_1",
  /* 1215 */  "vshift32B_avx",
  /* 1216 */  "vshift32B_avx_0",
  /* 1217 */  "vshift32B_avx_1",
  /* 1218 */  "vshift64B_avx",
  /* 1219 */  "vshift64B_avx_0",
  /* 1220 */  "vshift64B_avx_1",
  /* 1221 */  "vshiftS",
  /* 1222 */  "vshiftS_0",
  /* 1223 */  "vshiftS_1",
  /* 1224 */  "vshiftI",
  /* 1225 */  "vshiftI_0",
  /* 1226 */  "vshiftI_1",
  /* 1227 */  "vshiftI_imm",
  /* 1228 */  "vshiftI_imm_0",
  /* 1229 */  "vshiftI_imm_1",
  /* 1230 */  "vshiftL",
  /* 1231 */  "vshiftL_0",
  /* 1232 */  "vshiftL_imm",
  /* 1233 */  "vshiftL_imm_0",
  /* 1234 */  "vshiftL_arith_reg",
  /* 1235 */  "vshiftL_arith_reg_evex",
  /* 1236 */  "vshift8B_var_nobw",
  /* 1237 */  "vshift8B_var_nobw_0",
  /* 1238 */  "vshift8B_var_nobw_1",
  /* 1239 */  "vshift16B_var_nobw",
  /* 1240 */  "vshift16B_var_nobw_0",
  /* 1241 */  "vshift16B_var_nobw_1",
  /* 1242 */  "vshift32B_var_nobw",
  /* 1243 */  "vshift32B_var_nobw_0",
  /* 1244 */  "vshift32B_var_nobw_1",
  /* 1245 */  "vshiftB_var_evex_bw",
  /* 1246 */  "vshiftB_var_evex_bw_0",
  /* 1247 */  "vshiftB_var_evex_bw_1",
  /* 1248 */  "vshift64B_var_evex_bw",
  /* 1249 */  "vshift64B_var_evex_bw_0",
  /* 1250 */  "vshift64B_var_evex_bw_1",
  /* 1251 */  "vshift8S_var_nobw",
  /* 1252 */  "vshift8S_var_nobw_0",
  /* 1253 */  "vshift8S_var_nobw_1",
  /* 1254 */  "vshift16S_var_nobw",
  /* 1255 */  "vshift16S_var_nobw_0",
  /* 1256 */  "vshift16S_var_nobw_1",
  /* 1257 */  "vshift16S_var_evex_bw",
  /* 1258 */  "vshift16S_var_evex_bw_0",
  /* 1259 */  "vshift16S_var_evex_bw_1",
  /* 1260 */  "vshiftI_var",
  /* 1261 */  "vshiftI_var_0",
  /* 1262 */  "vshiftI_var_1",
  /* 1263 */  "vshiftL_var",
  /* 1264 */  "vshiftL_var_0",
  /* 1265 */  "vshiftL_arith_var",
  /* 1266 */  "vshiftL_arith_var_evex",
  /* 1267 */  "vand",
  /* 1268 */  "vand_reg",
  /* 1269 */  "vand_mem",
  /* 1270 */  "vand_mem_0",
  /* 1271 */  "vor",
  /* 1272 */  "vor_reg",
  /* 1273 */  "vor_mem",
  /* 1274 */  "vor_mem_0",
  /* 1275 */  "vxor",
  /* 1276 */  "vxor_reg",
  /* 1277 */  "vxor_mem",
  /* 1278 */  "vxor_mem_0",
  /* 1279 */  "castStoX",
  /* 1280 */  "vcastStoX",
  /* 1281 */  "castItoX",
  /* 1282 */  "vcastItoX",
  /* 1283 */  "vcastLtoBS",
  /* 1284 */  "vcmpFD",
  /* 1285 */  "evcmpFD",
  /* 1286 */  "vcmp",
  /* 1287 */  "evcmp",
  /* 1288 */  "extractI",
  /* 1289 */  "extractI_0",
  /* 1290 */  "vextractI",
  /* 1291 */  "vextractI_0",
  /* 1292 */  "extractF",
  /* 1293 */  "vextractF",
  /* 1294 */  "extractD",
  /* 1295 */  "vextractD",
  /* 1296 */  "blendvp",
  /* 1297 */  "vblendvpI",
  /* 1298 */  "vblendvpFD",
  /* 1299 */  "evblendvp64",
  /* 1300 */  "vabsnegF",
  /* 1301 */  "vabsnegF_0",
  /* 1302 */  "vabsneg4F",
  /* 1303 */  "vabsneg4F_0",
  /* 1304 */  "vabsnegD",
  /* 1305 */  "vabsnegD_0",
  /* 1306 */  "loadMask",
  /* 1307 */  "storeMask1B",
  /* 1308 */  "storeMask2B",
  /* 1309 */  "vstoreMask2B",
  /* 1310 */  "vstoreMask2B_evex",
  /* 1311 */  "storeMask4B",
  /* 1312 */  "vstoreMask4B",
  /* 1313 */  "vstoreMask4B_evex",
  /* 1314 */  "storeMask8B",
  /* 1315 */  "storeMask8B_avx",
  /* 1316 */  "vstoreMask8B_evex",
  /* 1317 */  "loadIotaIndices",
  /* 1318 */  "rearrangeB",
  /* 1319 */  "rearrangeB_avx",
  /* 1320 */  "rearrangeB_evex",
  /* 1321 */  "loadShuffleS",
  /* 1322 */  "rearrangeS",
  /* 1323 */  "rearrangeS_evex",
  /* 1324 */  "loadShuffleI",
  /* 1325 */  "rearrangeI",
  /* 1326 */  "rearrangeI_avx",
  /* 1327 */  "loadShuffleL",
  /* 1328 */  "rearrangeL",
  /* 1329 */  "rearrangeL_evex",
  /* 1330 */  "vfmaF_reg",
  /* 1331 */  "vfmaF_mem",
  /* 1332 */  "vfmaD_reg",
  /* 1333 */  "vfmaD_mem",
  /* 1334 */  "vmuladdS2I_reg_sse",
  /* 1335 */  "vmuladdS2I_reg_avx",
  /* 1336 */  "vmuladdaddS2I_reg",
  /* 1337 */  "vmuladdaddS2I_reg_0",
  /* 1338 */  "vpternlog",
  /* 1339 */  "vpternlog_mem",
  /* 1340 */  "vprotate_immI8",
  /* 1341 */  "vprotate_immI8_0",
  /* 1342 */  "vprorate",
  /* 1343 */  "vprorate_0",
  /* 1344 */  "compareAndSwapP_shenandoah",
  /* 1345 */  "compareAndSwapP_shenandoah_0",
  /* 1346 */  "compareAndExchangeP_shenandoah",
  // last instruction
  "invalid rule name" // no trailing comma
};

const        bool  swallowed[] = {
  /*    0 */  false,
  /*    1 */  false,
  /*    2 */  false,
  /*    3 */  false,
  /*    4 */  false,
  /*    5 */  false,
  /*    6 */  false,
  /*    7 */  false,
  /*    8 */  true,
  /*    9 */  true,
  /*   10 */  true,
  /*   11 */  true,
  /*   12 */  true,
  /*   13 */  true,
  /*   14 */  true,
  /*   15 */  true,
  /*   16 */  true,
  /*   17 */  true,
  /*   18 */  true,
  /*   19 */  true,
  /*   20 */  true,
  /*   21 */  true,
  /*   22 */  true,
  /*   23 */  true,
  /*   24 */  true,
  /*   25 */  true,
  /*   26 */  true,
  /*   27 */  true,
  /*   28 */  true,
  /*   29 */  true,
  /*   30 */  true,
  /*   31 */  true,
  /*   32 */  true,
  /*   33 */  true,
  /*   34 */  true,
  /*   35 */  true,
  /*   36 */  true,
  /*   37 */  true,
  /*   38 */  true,
  /*   39 */  true,
  /*   40 */  true,
  /*   41 */  true,
  /*   42 */  true,
  /*   43 */  true,
  /*   44 */  true,
  /*   45 */  true,
  /*   46 */  false,
  /*   47 */  false,
  /*   48 */  false,
  /*   49 */  false,
  /*   50 */  false,
  /*   51 */  false,
  /*   52 */  false,
  /*   53 */  false,
  /*   54 */  false,
  /*   55 */  false,
  /*   56 */  false,
  /*   57 */  false,
  /*   58 */  false,
  /*   59 */  false,
  /*   60 */  false,
  /*   61 */  false,
  /*   62 */  false,
  /*   63 */  false,
  /*   64 */  false,
  /*   65 */  false,
  /*   66 */  false,
  /*   67 */  false,
  /*   68 */  false,
  /*   69 */  false,
  /*   70 */  false,
  /*   71 */  false,
  /*   72 */  false,
  /*   73 */  false,
  /*   74 */  false,
  /*   75 */  false,
  /*   76 */  false,
  /*   77 */  false,
  /*   78 */  false,
  /*   79 */  false,
  /*   80 */  false,
  /*   81 */  false,
  /*   82 */  false,
  /*   83 */  false,
  /*   84 */  false,
  /*   85 */  false,
  /*   86 */  false,
  /*   87 */  false,
  /*   88 */  false,
  /*   89 */  false,
  /*   90 */  false,
  /*   91 */  false,
  /*   92 */  false,
  /*   93 */  false,
  /*   94 */  false,
  /*   95 */  false,
  /*   96 */  true,
  /*   97 */  false,
  /*   98 */  false,
  /*   99 */  false,
  /*  100 */  false,
  /*  101 */  false,
  /*  102 */  false,
  /*  103 */  false,
  /*  104 */  false,
  /*  105 */  false,
  /*  106 */  false,
  /*  107 */  false,
  /*  108 */  false,
  /*  109 */  false,
  /*  110 */  false,
  /*  111 */  false,
  /*  112 */  false,
  /*  113 */  true,
  /*  114 */  true,
  /*  115 */  true,
  /*  116 */  true,
  /*  117 */  true,
  /*  118 */  true,
  /*  119 */  true,
  /*  120 */  false,
  /*  121 */  false,
  /*  122 */  false,
  /*  123 */  false,
  /*  124 */  false,
  /*  125 */  false,
  /*  126 */  false,
  /*  127 */  false,
  /*  128 */  false,
  /*  129 */  false,
  /*  130 */  false,
  /*  131 */  false,
  /*  132 */  false,
  /*  133 */  true,
  // last operand
  /*  134 */  false,
  /*  135 */  false,
  /*  136 */  false,
  // last operand class
  /*  137 */  false,
  /*  138 */  false,
  /*  139 */  false,
  /*  140 */  false,
  /*  141 */  false,
  /*  142 */  false,
  /*  143 */  false,
  /*  144 */  false,
  /*  145 */  false,
  /*  146 */  false,
  /*  147 */  false,
  /*  148 */  false,
  /*  149 */  false,
  /*  150 */  false,
  /*  151 */  false,
  /*  152 */  false,
  /*  153 */  false,
  /*  154 */  false,
  /*  155 */  false,
  /*  156 */  false,
  /*  157 */  false,
  /*  158 */  false,
  /*  159 */  false,
  /*  160 */  false,
  /*  161 */  false,
  /*  162 */  false,
  /*  163 */  false,
  /*  164 */  false,
  /*  165 */  false,
  /*  166 */  false,
  /*  167 */  false,
  /*  168 */  false,
  /*  169 */  false,
  /*  170 */  false,
  /*  171 */  false,
  /*  172 */  false,
  /*  173 */  false,
  /*  174 */  false,
  /*  175 */  false,
  /*  176 */  false,
  /*  177 */  false,
  /*  178 */  false,
  /*  179 */  false,
  /*  180 */  false,
  /*  181 */  false,
  /*  182 */  false,
  /*  183 */  false,
  /*  184 */  false,
  /*  185 */  false,
  /*  186 */  false,
  /*  187 */  false,
  /*  188 */  false,
  /*  189 */  false,
  /*  190 */  false,
  /*  191 */  false,
  /*  192 */  false,
  /*  193 */  false,
  /*  194 */  false,
  /*  195 */  false,
  /*  196 */  false,
  /*  197 */  false,
  /*  198 */  false,
  /*  199 */  false,
  /*  200 */  false,
  /*  201 */  false,
  /*  202 */  false,
  /*  203 */  false,
  /*  204 */  false,
  /*  205 */  false,
  /*  206 */  false,
  /*  207 */  false,
  /*  208 */  false,
  /*  209 */  false,
  /*  210 */  false,
  /*  211 */  false,
  /*  212 */  false,
  /*  213 */  false,
  /*  214 */  false,
  /*  215 */  false,
  /*  216 */  false,
  /*  217 */  false,
  /*  218 */  false,
  /*  219 */  false,
  /*  220 */  false,
  /*  221 */  false,
  /*  222 */  false,
  /*  223 */  false,
  /*  224 */  false,
  /*  225 */  false,
  /*  226 */  false,
  /*  227 */  false,
  /*  228 */  false,
  /*  229 */  false,
  /*  230 */  false,
  /*  231 */  false,
  /*  232 */  false,
  /*  233 */  false,
  /*  234 */  false,
  /*  235 */  false,
  /*  236 */  false,
  /*  237 */  false,
  /*  238 */  false,
  /*  239 */  false,
  /*  240 */  false,
  /*  241 */  false,
  /*  242 */  false,
  /*  243 */  false,
  /*  244 */  false,
  /*  245 */  false,
  /*  246 */  false,
  /*  247 */  false,
  /*  248 */  false,
  /*  249 */  false,
  /*  250 */  false,
  /*  251 */  false,
  /*  252 */  false,
  /*  253 */  false,
  /*  254 */  false,
  /*  255 */  false,
  /*  256 */  false,
  /*  257 */  false,
  /*  258 */  false,
  /*  259 */  false,
  /*  260 */  false,
  /*  261 */  false,
  /*  262 */  false,
  /*  263 */  false,
  /*  264 */  false,
  /*  265 */  false,
  /*  266 */  false,
  /*  267 */  false,
  /*  268 */  false,
  /*  269 */  false,
  /*  270 */  false,
  /*  271 */  false,
  /*  272 */  false,
  /*  273 */  false,
  /*  274 */  false,
  /*  275 */  false,
  /*  276 */  false,
  /*  277 */  false,
  /*  278 */  false,
  /*  279 */  false,
  /*  280 */  false,
  /*  281 */  false,
  // last internally defined operand
  /*  282 */  false,
  /*  283 */  false,
  /*  284 */  false,
  /*  285 */  false,
  /*  286 */  false,
  /*  287 */  false,
  /*  288 */  false,
  /*  289 */  false,
  /*  290 */  false,
  /*  291 */  false,
  /*  292 */  false,
  /*  293 */  false,
  /*  294 */  false,
  /*  295 */  false,
  /*  296 */  false,
  /*  297 */  false,
  /*  298 */  false,
  /*  299 */  false,
  /*  300 */  false,
  /*  301 */  false,
  /*  302 */  false,
  /*  303 */  false,
  /*  304 */  false,
  /*  305 */  false,
  /*  306 */  false,
  /*  307 */  false,
  /*  308 */  false,
  /*  309 */  false,
  /*  310 */  false,
  /*  311 */  false,
  /*  312 */  false,
  /*  313 */  false,
  /*  314 */  false,
  /*  315 */  false,
  /*  316 */  false,
  /*  317 */  false,
  /*  318 */  false,
  /*  319 */  false,
  /*  320 */  false,
  /*  321 */  false,
  /*  322 */  false,
  /*  323 */  false,
  /*  324 */  false,
  /*  325 */  false,
  /*  326 */  false,
  /*  327 */  false,
  /*  328 */  false,
  /*  329 */  false,
  /*  330 */  false,
  /*  331 */  false,
  /*  332 */  false,
  /*  333 */  false,
  /*  334 */  false,
  /*  335 */  false,
  /*  336 */  false,
  /*  337 */  false,
  /*  338 */  false,
  /*  339 */  false,
  /*  340 */  false,
  /*  341 */  false,
  /*  342 */  false,
  /*  343 */  false,
  /*  344 */  false,
  /*  345 */  false,
  /*  346 */  false,
  /*  347 */  false,
  /*  348 */  false,
  /*  349 */  false,
  /*  350 */  false,
  /*  351 */  false,
  /*  352 */  false,
  /*  353 */  false,
  /*  354 */  false,
  /*  355 */  false,
  /*  356 */  false,
  /*  357 */  false,
  /*  358 */  false,
  /*  359 */  false,
  /*  360 */  false,
  /*  361 */  false,
  /*  362 */  false,
  /*  363 */  false,
  /*  364 */  false,
  /*  365 */  false,
  /*  366 */  false,
  /*  367 */  false,
  /*  368 */  false,
  /*  369 */  false,
  /*  370 */  false,
  /*  371 */  false,
  /*  372 */  false,
  /*  373 */  false,
  /*  374 */  false,
  /*  375 */  false,
  /*  376 */  false,
  /*  377 */  false,
  /*  378 */  false,
  /*  379 */  false,
  /*  380 */  false,
  /*  381 */  false,
  /*  382 */  false,
  /*  383 */  false,
  /*  384 */  false,
  /*  385 */  false,
  /*  386 */  false,
  /*  387 */  false,
  /*  388 */  false,
  /*  389 */  false,
  /*  390 */  false,
  /*  391 */  false,
  /*  392 */  false,
  /*  393 */  false,
  /*  394 */  false,
  /*  395 */  false,
  /*  396 */  false,
  /*  397 */  false,
  /*  398 */  false,
  /*  399 */  false,
  /*  400 */  false,
  /*  401 */  false,
  /*  402 */  false,
  /*  403 */  false,
  /*  404 */  false,
  /*  405 */  false,
  /*  406 */  false,
  /*  407 */  false,
  /*  408 */  false,
  /*  409 */  false,
  /*  410 */  false,
  /*  411 */  false,
  /*  412 */  false,
  /*  413 */  false,
  /*  414 */  false,
  /*  415 */  false,
  /*  416 */  false,
  /*  417 */  false,
  /*  418 */  false,
  /*  419 */  false,
  /*  420 */  false,
  /*  421 */  false,
  /*  422 */  false,
  /*  423 */  false,
  /*  424 */  false,
  /*  425 */  false,
  /*  426 */  false,
  /*  427 */  false,
  /*  428 */  false,
  /*  429 */  false,
  /*  430 */  false,
  /*  431 */  false,
  /*  432 */  false,
  /*  433 */  false,
  /*  434 */  false,
  /*  435 */  false,
  /*  436 */  false,
  /*  437 */  false,
  /*  438 */  false,
  /*  439 */  false,
  /*  440 */  false,
  /*  441 */  false,
  /*  442 */  false,
  /*  443 */  false,
  /*  444 */  false,
  /*  445 */  false,
  /*  446 */  false,
  /*  447 */  false,
  /*  448 */  false,
  /*  449 */  false,
  /*  450 */  false,
  /*  451 */  false,
  /*  452 */  false,
  /*  453 */  false,
  /*  454 */  false,
  /*  455 */  false,
  /*  456 */  false,
  /*  457 */  false,
  /*  458 */  false,
  /*  459 */  false,
  /*  460 */  false,
  /*  461 */  false,
  /*  462 */  false,
  /*  463 */  false,
  /*  464 */  false,
  /*  465 */  false,
  /*  466 */  false,
  /*  467 */  false,
  /*  468 */  false,
  /*  469 */  false,
  /*  470 */  false,
  /*  471 */  false,
  /*  472 */  false,
  /*  473 */  false,
  /*  474 */  false,
  /*  475 */  false,
  /*  476 */  false,
  /*  477 */  false,
  /*  478 */  false,
  /*  479 */  false,
  /*  480 */  false,
  /*  481 */  false,
  /*  482 */  false,
  /*  483 */  false,
  /*  484 */  false,
  /*  485 */  false,
  /*  486 */  false,
  /*  487 */  false,
  /*  488 */  false,
  /*  489 */  false,
  /*  490 */  false,
  /*  491 */  false,
  /*  492 */  false,
  /*  493 */  false,
  /*  494 */  false,
  /*  495 */  false,
  /*  496 */  false,
  /*  497 */  false,
  /*  498 */  false,
  /*  499 */  false,
  /*  500 */  false,
  /*  501 */  false,
  /*  502 */  false,
  /*  503 */  false,
  /*  504 */  false,
  /*  505 */  false,
  /*  506 */  false,
  /*  507 */  false,
  /*  508 */  false,
  /*  509 */  false,
  /*  510 */  false,
  /*  511 */  false,
  /*  512 */  false,
  /*  513 */  false,
  /*  514 */  false,
  /*  515 */  false,
  /*  516 */  false,
  /*  517 */  false,
  /*  518 */  false,
  /*  519 */  false,
  /*  520 */  false,
  /*  521 */  false,
  /*  522 */  false,
  /*  523 */  false,
  /*  524 */  false,
  /*  525 */  false,
  /*  526 */  false,
  /*  527 */  false,
  /*  528 */  false,
  /*  529 */  false,
  /*  530 */  false,
  /*  531 */  false,
  /*  532 */  false,
  /*  533 */  false,
  /*  534 */  false,
  /*  535 */  false,
  /*  536 */  false,
  /*  537 */  false,
  /*  538 */  false,
  /*  539 */  false,
  /*  540 */  false,
  /*  541 */  false,
  /*  542 */  false,
  /*  543 */  false,
  /*  544 */  false,
  /*  545 */  false,
  /*  546 */  false,
  /*  547 */  false,
  /*  548 */  false,
  /*  549 */  false,
  /*  550 */  false,
  /*  551 */  false,
  /*  552 */  false,
  /*  553 */  false,
  /*  554 */  false,
  /*  555 */  false,
  /*  556 */  false,
  /*  557 */  false,
  /*  558 */  false,
  /*  559 */  false,
  /*  560 */  false,
  /*  561 */  false,
  /*  562 */  false,
  /*  563 */  false,
  /*  564 */  false,
  /*  565 */  false,
  /*  566 */  false,
  /*  567 */  false,
  /*  568 */  false,
  /*  569 */  false,
  /*  570 */  false,
  /*  571 */  false,
  /*  572 */  false,
  /*  573 */  false,
  /*  574 */  false,
  /*  575 */  false,
  /*  576 */  false,
  /*  577 */  false,
  /*  578 */  false,
  /*  579 */  false,
  /*  580 */  false,
  /*  581 */  false,
  /*  582 */  false,
  /*  583 */  false,
  /*  584 */  false,
  /*  585 */  false,
  /*  586 */  false,
  /*  587 */  false,
  /*  588 */  false,
  /*  589 */  false,
  /*  590 */  false,
  /*  591 */  false,
  /*  592 */  false,
  /*  593 */  false,
  /*  594 */  false,
  /*  595 */  false,
  /*  596 */  false,
  /*  597 */  false,
  /*  598 */  false,
  /*  599 */  false,
  /*  600 */  false,
  /*  601 */  false,
  /*  602 */  false,
  /*  603 */  false,
  /*  604 */  false,
  /*  605 */  false,
  /*  606 */  false,
  /*  607 */  false,
  /*  608 */  false,
  /*  609 */  false,
  /*  610 */  false,
  /*  611 */  false,
  /*  612 */  false,
  /*  613 */  false,
  /*  614 */  false,
  /*  615 */  false,
  /*  616 */  false,
  /*  617 */  false,
  /*  618 */  false,
  /*  619 */  false,
  /*  620 */  false,
  /*  621 */  false,
  /*  622 */  false,
  /*  623 */  false,
  /*  624 */  false,
  /*  625 */  false,
  /*  626 */  false,
  /*  627 */  false,
  /*  628 */  false,
  /*  629 */  false,
  /*  630 */  false,
  /*  631 */  false,
  /*  632 */  false,
  /*  633 */  false,
  /*  634 */  false,
  /*  635 */  false,
  /*  636 */  false,
  /*  637 */  false,
  /*  638 */  false,
  /*  639 */  false,
  /*  640 */  false,
  /*  641 */  false,
  /*  642 */  false,
  /*  643 */  false,
  /*  644 */  false,
  /*  645 */  false,
  /*  646 */  false,
  /*  647 */  false,
  /*  648 */  false,
  /*  649 */  false,
  /*  650 */  false,
  /*  651 */  false,
  /*  652 */  false,
  /*  653 */  false,
  /*  654 */  false,
  /*  655 */  false,
  /*  656 */  false,
  /*  657 */  false,
  /*  658 */  false,
  /*  659 */  false,
  /*  660 */  false,
  /*  661 */  false,
  /*  662 */  false,
  /*  663 */  false,
  /*  664 */  false,
  /*  665 */  false,
  /*  666 */  false,
  /*  667 */  false,
  /*  668 */  false,
  /*  669 */  false,
  /*  670 */  false,
  /*  671 */  false,
  /*  672 */  false,
  /*  673 */  false,
  /*  674 */  false,
  /*  675 */  false,
  /*  676 */  false,
  /*  677 */  false,
  /*  678 */  false,
  /*  679 */  false,
  /*  680 */  false,
  /*  681 */  false,
  /*  682 */  false,
  /*  683 */  false,
  /*  684 */  false,
  /*  685 */  false,
  /*  686 */  false,
  /*  687 */  false,
  /*  688 */  false,
  /*  689 */  false,
  /*  690 */  false,
  /*  691 */  false,
  /*  692 */  false,
  /*  693 */  false,
  /*  694 */  false,
  /*  695 */  false,
  /*  696 */  false,
  /*  697 */  false,
  /*  698 */  false,
  /*  699 */  false,
  /*  700 */  false,
  /*  701 */  false,
  /*  702 */  false,
  /*  703 */  false,
  /*  704 */  false,
  /*  705 */  false,
  /*  706 */  false,
  /*  707 */  false,
  /*  708 */  false,
  /*  709 */  false,
  /*  710 */  false,
  /*  711 */  false,
  /*  712 */  false,
  /*  713 */  false,
  /*  714 */  false,
  /*  715 */  false,
  /*  716 */  false,
  /*  717 */  false,
  /*  718 */  false,
  /*  719 */  false,
  /*  720 */  false,
  /*  721 */  false,
  /*  722 */  false,
  /*  723 */  false,
  /*  724 */  false,
  /*  725 */  false,
  /*  726 */  false,
  /*  727 */  false,
  /*  728 */  false,
  /*  729 */  false,
  /*  730 */  false,
  /*  731 */  false,
  /*  732 */  false,
  /*  733 */  false,
  /*  734 */  false,
  /*  735 */  false,
  /*  736 */  false,
  /*  737 */  false,
  /*  738 */  false,
  /*  739 */  false,
  /*  740 */  false,
  /*  741 */  false,
  /*  742 */  false,
  /*  743 */  false,
  /*  744 */  false,
  /*  745 */  false,
  /*  746 */  false,
  /*  747 */  false,
  /*  748 */  false,
  /*  749 */  false,
  /*  750 */  false,
  /*  751 */  false,
  /*  752 */  false,
  /*  753 */  false,
  /*  754 */  false,
  /*  755 */  false,
  /*  756 */  false,
  /*  757 */  false,
  /*  758 */  false,
  /*  759 */  false,
  /*  760 */  false,
  /*  761 */  false,
  /*  762 */  false,
  /*  763 */  false,
  /*  764 */  false,
  /*  765 */  false,
  /*  766 */  false,
  /*  767 */  false,
  /*  768 */  false,
  /*  769 */  false,
  /*  770 */  false,
  /*  771 */  false,
  /*  772 */  false,
  /*  773 */  false,
  /*  774 */  false,
  /*  775 */  false,
  /*  776 */  false,
  /*  777 */  false,
  /*  778 */  false,
  /*  779 */  false,
  /*  780 */  false,
  /*  781 */  false,
  /*  782 */  false,
  /*  783 */  false,
  /*  784 */  false,
  /*  785 */  false,
  /*  786 */  false,
  /*  787 */  false,
  /*  788 */  false,
  /*  789 */  false,
  /*  790 */  false,
  /*  791 */  false,
  /*  792 */  false,
  /*  793 */  false,
  /*  794 */  false,
  /*  795 */  false,
  /*  796 */  false,
  /*  797 */  false,
  /*  798 */  false,
  /*  799 */  false,
  /*  800 */  false,
  /*  801 */  false,
  /*  802 */  false,
  /*  803 */  false,
  /*  804 */  false,
  /*  805 */  false,
  /*  806 */  false,
  /*  807 */  false,
  /*  808 */  false,
  /*  809 */  false,
  /*  810 */  false,
  /*  811 */  false,
  /*  812 */  false,
  /*  813 */  false,
  /*  814 */  false,
  /*  815 */  false,
  /*  816 */  false,
  /*  817 */  false,
  /*  818 */  false,
  /*  819 */  false,
  /*  820 */  false,
  /*  821 */  false,
  /*  822 */  false,
  /*  823 */  false,
  /*  824 */  false,
  /*  825 */  false,
  /*  826 */  false,
  /*  827 */  false,
  /*  828 */  false,
  /*  829 */  false,
  /*  830 */  false,
  /*  831 */  false,
  /*  832 */  false,
  /*  833 */  false,
  /*  834 */  false,
  /*  835 */  false,
  /*  836 */  false,
  /*  837 */  false,
  /*  838 */  false,
  /*  839 */  false,
  /*  840 */  false,
  /*  841 */  false,
  /*  842 */  false,
  /*  843 */  false,
  /*  844 */  false,
  /*  845 */  false,
  /*  846 */  false,
  /*  847 */  false,
  /*  848 */  false,
  /*  849 */  false,
  /*  850 */  false,
  /*  851 */  false,
  /*  852 */  false,
  /*  853 */  false,
  /*  854 */  false,
  /*  855 */  false,
  /*  856 */  false,
  /*  857 */  false,
  /*  858 */  false,
  /*  859 */  false,
  /*  860 */  false,
  /*  861 */  false,
  /*  862 */  false,
  /*  863 */  false,
  /*  864 */  false,
  /*  865 */  false,
  /*  866 */  false,
  /*  867 */  false,
  /*  868 */  false,
  /*  869 */  false,
  /*  870 */  false,
  /*  871 */  false,
  /*  872 */  false,
  /*  873 */  false,
  /*  874 */  false,
  /*  875 */  false,
  /*  876 */  false,
  /*  877 */  false,
  /*  878 */  false,
  /*  879 */  false,
  /*  880 */  false,
  /*  881 */  false,
  /*  882 */  false,
  /*  883 */  false,
  /*  884 */  false,
  /*  885 */  false,
  /*  886 */  false,
  /*  887 */  false,
  /*  888 */  false,
  /*  889 */  false,
  /*  890 */  false,
  /*  891 */  false,
  /*  892 */  false,
  /*  893 */  false,
  /*  894 */  false,
  /*  895 */  false,
  /*  896 */  false,
  /*  897 */  false,
  /*  898 */  false,
  /*  899 */  false,
  /*  900 */  false,
  /*  901 */  false,
  /*  902 */  false,
  /*  903 */  false,
  /*  904 */  false,
  /*  905 */  false,
  /*  906 */  false,
  /*  907 */  false,
  /*  908 */  false,
  /*  909 */  false,
  /*  910 */  false,
  /*  911 */  false,
  /*  912 */  false,
  /*  913 */  false,
  /*  914 */  false,
  /*  915 */  false,
  /*  916 */  false,
  /*  917 */  false,
  /*  918 */  false,
  /*  919 */  false,
  /*  920 */  false,
  /*  921 */  false,
  /*  922 */  false,
  /*  923 */  false,
  /*  924 */  false,
  /*  925 */  false,
  /*  926 */  false,
  /*  927 */  false,
  /*  928 */  false,
  /*  929 */  false,
  /*  930 */  false,
  /*  931 */  false,
  /*  932 */  false,
  /*  933 */  false,
  /*  934 */  false,
  /*  935 */  false,
  /*  936 */  false,
  /*  937 */  false,
  /*  938 */  false,
  /*  939 */  false,
  /*  940 */  false,
  /*  941 */  false,
  /*  942 */  false,
  /*  943 */  false,
  /*  944 */  false,
  /*  945 */  false,
  /*  946 */  false,
  /*  947 */  false,
  /*  948 */  false,
  /*  949 */  false,
  /*  950 */  false,
  /*  951 */  false,
  /*  952 */  false,
  /*  953 */  false,
  /*  954 */  false,
  /*  955 */  false,
  /*  956 */  false,
  /*  957 */  false,
  /*  958 */  false,
  /*  959 */  false,
  /*  960 */  false,
  /*  961 */  false,
  /*  962 */  false,
  /*  963 */  false,
  /*  964 */  false,
  /*  965 */  false,
  /*  966 */  false,
  /*  967 */  false,
  /*  968 */  false,
  /*  969 */  false,
  /*  970 */  false,
  /*  971 */  false,
  /*  972 */  false,
  /*  973 */  false,
  /*  974 */  false,
  /*  975 */  false,
  /*  976 */  false,
  /*  977 */  false,
  /*  978 */  false,
  /*  979 */  false,
  /*  980 */  false,
  /*  981 */  false,
  /*  982 */  false,
  /*  983 */  false,
  /*  984 */  false,
  /*  985 */  false,
  /*  986 */  false,
  /*  987 */  false,
  /*  988 */  false,
  /*  989 */  false,
  /*  990 */  false,
  /*  991 */  false,
  /*  992 */  false,
  /*  993 */  false,
  /*  994 */  false,
  /*  995 */  false,
  /*  996 */  false,
  /*  997 */  false,
  /*  998 */  false,
  /*  999 */  false,
  /* 1000 */  false,
  /* 1001 */  false,
  /* 1002 */  false,
  /* 1003 */  false,
  /* 1004 */  false,
  /* 1005 */  false,
  /* 1006 */  false,
  /* 1007 */  false,
  /* 1008 */  false,
  /* 1009 */  false,
  /* 1010 */  false,
  /* 1011 */  false,
  /* 1012 */  false,
  /* 1013 */  false,
  /* 1014 */  false,
  /* 1015 */  false,
  /* 1016 */  false,
  /* 1017 */  false,
  /* 1018 */  false,
  /* 1019 */  false,
  /* 1020 */  false,
  /* 1021 */  false,
  /* 1022 */  false,
  /* 1023 */  false,
  /* 1024 */  false,
  /* 1025 */  false,
  /* 1026 */  false,
  /* 1027 */  false,
  /* 1028 */  false,
  /* 1029 */  false,
  /* 1030 */  false,
  /* 1031 */  false,
  /* 1032 */  false,
  /* 1033 */  false,
  /* 1034 */  false,
  /* 1035 */  false,
  /* 1036 */  false,
  /* 1037 */  false,
  /* 1038 */  false,
  /* 1039 */  false,
  /* 1040 */  false,
  /* 1041 */  false,
  /* 1042 */  false,
  /* 1043 */  false,
  /* 1044 */  false,
  /* 1045 */  false,
  /* 1046 */  false,
  /* 1047 */  false,
  /* 1048 */  false,
  /* 1049 */  false,
  /* 1050 */  false,
  /* 1051 */  false,
  /* 1052 */  false,
  /* 1053 */  false,
  /* 1054 */  false,
  /* 1055 */  false,
  /* 1056 */  false,
  /* 1057 */  false,
  /* 1058 */  false,
  /* 1059 */  false,
  /* 1060 */  false,
  /* 1061 */  false,
  /* 1062 */  false,
  /* 1063 */  false,
  /* 1064 */  false,
  /* 1065 */  false,
  /* 1066 */  false,
  /* 1067 */  false,
  /* 1068 */  false,
  /* 1069 */  false,
  /* 1070 */  false,
  /* 1071 */  false,
  /* 1072 */  false,
  /* 1073 */  false,
  /* 1074 */  false,
  /* 1075 */  false,
  /* 1076 */  false,
  /* 1077 */  false,
  /* 1078 */  false,
  /* 1079 */  false,
  /* 1080 */  false,
  /* 1081 */  false,
  /* 1082 */  false,
  /* 1083 */  false,
  /* 1084 */  false,
  /* 1085 */  false,
  /* 1086 */  false,
  /* 1087 */  false,
  /* 1088 */  false,
  /* 1089 */  false,
  /* 1090 */  false,
  /* 1091 */  false,
  /* 1092 */  false,
  /* 1093 */  false,
  /* 1094 */  false,
  /* 1095 */  false,
  /* 1096 */  false,
  /* 1097 */  false,
  /* 1098 */  false,
  /* 1099 */  false,
  /* 1100 */  false,
  /* 1101 */  false,
  /* 1102 */  false,
  /* 1103 */  false,
  /* 1104 */  false,
  /* 1105 */  false,
  /* 1106 */  false,
  /* 1107 */  false,
  /* 1108 */  false,
  /* 1109 */  false,
  /* 1110 */  false,
  /* 1111 */  false,
  /* 1112 */  false,
  /* 1113 */  false,
  /* 1114 */  false,
  /* 1115 */  false,
  /* 1116 */  false,
  /* 1117 */  false,
  /* 1118 */  false,
  /* 1119 */  false,
  /* 1120 */  false,
  /* 1121 */  false,
  /* 1122 */  false,
  /* 1123 */  false,
  /* 1124 */  false,
  /* 1125 */  false,
  /* 1126 */  false,
  /* 1127 */  false,
  /* 1128 */  false,
  /* 1129 */  false,
  /* 1130 */  false,
  /* 1131 */  false,
  /* 1132 */  false,
  /* 1133 */  false,
  /* 1134 */  false,
  /* 1135 */  false,
  /* 1136 */  false,
  /* 1137 */  false,
  /* 1138 */  false,
  /* 1139 */  false,
  /* 1140 */  false,
  /* 1141 */  false,
  /* 1142 */  false,
  /* 1143 */  false,
  /* 1144 */  false,
  /* 1145 */  false,
  /* 1146 */  false,
  /* 1147 */  false,
  /* 1148 */  false,
  /* 1149 */  false,
  /* 1150 */  false,
  /* 1151 */  false,
  /* 1152 */  false,
  /* 1153 */  false,
  /* 1154 */  false,
  /* 1155 */  false,
  /* 1156 */  false,
  /* 1157 */  false,
  /* 1158 */  false,
  /* 1159 */  false,
  /* 1160 */  false,
  /* 1161 */  false,
  /* 1162 */  false,
  /* 1163 */  false,
  /* 1164 */  false,
  /* 1165 */  false,
  /* 1166 */  false,
  /* 1167 */  false,
  /* 1168 */  false,
  /* 1169 */  false,
  /* 1170 */  false,
  /* 1171 */  false,
  /* 1172 */  false,
  /* 1173 */  false,
  /* 1174 */  false,
  /* 1175 */  false,
  /* 1176 */  false,
  /* 1177 */  false,
  /* 1178 */  false,
  /* 1179 */  false,
  /* 1180 */  false,
  /* 1181 */  false,
  /* 1182 */  false,
  /* 1183 */  false,
  /* 1184 */  false,
  /* 1185 */  false,
  /* 1186 */  false,
  /* 1187 */  false,
  /* 1188 */  false,
  /* 1189 */  false,
  /* 1190 */  false,
  /* 1191 */  false,
  /* 1192 */  false,
  /* 1193 */  false,
  /* 1194 */  false,
  /* 1195 */  false,
  /* 1196 */  false,
  /* 1197 */  false,
  /* 1198 */  false,
  /* 1199 */  false,
  /* 1200 */  false,
  /* 1201 */  false,
  /* 1202 */  false,
  /* 1203 */  false,
  /* 1204 */  false,
  /* 1205 */  false,
  /* 1206 */  false,
  /* 1207 */  false,
  /* 1208 */  false,
  /* 1209 */  false,
  /* 1210 */  false,
  /* 1211 */  false,
  /* 1212 */  false,
  /* 1213 */  false,
  /* 1214 */  false,
  /* 1215 */  false,
  /* 1216 */  false,
  /* 1217 */  false,
  /* 1218 */  false,
  /* 1219 */  false,
  /* 1220 */  false,
  /* 1221 */  false,
  /* 1222 */  false,
  /* 1223 */  false,
  /* 1224 */  false,
  /* 1225 */  false,
  /* 1226 */  false,
  /* 1227 */  false,
  /* 1228 */  false,
  /* 1229 */  false,
  /* 1230 */  false,
  /* 1231 */  false,
  /* 1232 */  false,
  /* 1233 */  false,
  /* 1234 */  false,
  /* 1235 */  false,
  /* 1236 */  false,
  /* 1237 */  false,
  /* 1238 */  false,
  /* 1239 */  false,
  /* 1240 */  false,
  /* 1241 */  false,
  /* 1242 */  false,
  /* 1243 */  false,
  /* 1244 */  false,
  /* 1245 */  false,
  /* 1246 */  false,
  /* 1247 */  false,
  /* 1248 */  false,
  /* 1249 */  false,
  /* 1250 */  false,
  /* 1251 */  false,
  /* 1252 */  false,
  /* 1253 */  false,
  /* 1254 */  false,
  /* 1255 */  false,
  /* 1256 */  false,
  /* 1257 */  false,
  /* 1258 */  false,
  /* 1259 */  false,
  /* 1260 */  false,
  /* 1261 */  false,
  /* 1262 */  false,
  /* 1263 */  false,
  /* 1264 */  false,
  /* 1265 */  false,
  /* 1266 */  false,
  /* 1267 */  false,
  /* 1268 */  false,
  /* 1269 */  false,
  /* 1270 */  false,
  /* 1271 */  false,
  /* 1272 */  false,
  /* 1273 */  false,
  /* 1274 */  false,
  /* 1275 */  false,
  /* 1276 */  false,
  /* 1277 */  false,
  /* 1278 */  false,
  /* 1279 */  false,
  /* 1280 */  false,
  /* 1281 */  false,
  /* 1282 */  false,
  /* 1283 */  false,
  /* 1284 */  false,
  /* 1285 */  false,
  /* 1286 */  false,
  /* 1287 */  false,
  /* 1288 */  false,
  /* 1289 */  false,
  /* 1290 */  false,
  /* 1291 */  false,
  /* 1292 */  false,
  /* 1293 */  false,
  /* 1294 */  false,
  /* 1295 */  false,
  /* 1296 */  false,
  /* 1297 */  false,
  /* 1298 */  false,
  /* 1299 */  false,
  /* 1300 */  false,
  /* 1301 */  false,
  /* 1302 */  false,
  /* 1303 */  false,
  /* 1304 */  false,
  /* 1305 */  false,
  /* 1306 */  false,
  /* 1307 */  false,
  /* 1308 */  false,
  /* 1309 */  false,
  /* 1310 */  false,
  /* 1311 */  false,
  /* 1312 */  false,
  /* 1313 */  false,
  /* 1314 */  false,
  /* 1315 */  false,
  /* 1316 */  false,
  /* 1317 */  false,
  /* 1318 */  false,
  /* 1319 */  false,
  /* 1320 */  false,
  /* 1321 */  false,
  /* 1322 */  false,
  /* 1323 */  false,
  /* 1324 */  false,
  /* 1325 */  false,
  /* 1326 */  false,
  /* 1327 */  false,
  /* 1328 */  false,
  /* 1329 */  false,
  /* 1330 */  false,
  /* 1331 */  false,
  /* 1332 */  false,
  /* 1333 */  false,
  /* 1334 */  false,
  /* 1335 */  false,
  /* 1336 */  false,
  /* 1337 */  false,
  /* 1338 */  false,
  /* 1339 */  false,
  /* 1340 */  false,
  /* 1341 */  false,
  /* 1342 */  false,
  /* 1343 */  false,
  /* 1344 */  false,
  /* 1345 */  false,
  /* 1346 */  false,
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // VecA: 8
  0, // VecS: 9
  0, // VecD: 10
  0, // VecX: 11
  0, // VecY: 12
  0, // VecZ: 13
  0, // RegVMask: 14
  0, // RegFlags: 15
  0, // _last_machine_leaf: 16
  0, // AbsD: 17
  0, // AbsF: 18
  0, // AbsI: 19
  0, // AbsL: 20
  0, // AddD: 21
  0, // AddF: 22
  0, // AddI: 23
  0, // AddL: 24
  0, // AddP: 25
  0, // Allocate: 26
  0, // AllocateArray: 27
  0, // AndI: 28
  0, // AndL: 29
  0, // ArrayCopy: 30
  0, // AryEq: 31
  0, // AtanD: 32
  1, // Binary: 33
  1, // Bool: 34
  0, // BoxLock: 35
  0, // ReverseBytesI: 36
  0, // ReverseBytesL: 37
  0, // ReverseBytesUS: 38
  0, // ReverseBytesS: 39
  0, // CProj: 40
  0, // CacheWB: 41
  0, // CacheWBPreSync: 42
  0, // CacheWBPostSync: 43
  0, // CallDynamicJava: 44
  0, // CallJava: 45
  0, // CallLeaf: 46
  0, // CallLeafNoFP: 47
  0, // CallRuntime: 48
  0, // CallNative: 49
  0, // CallStaticJava: 50
  0, // CastII: 51
  0, // CastLL: 52
  0, // CastX2P: 53
  0, // CastP2X: 54
  0, // CastPP: 55
  0, // Catch: 56
  0, // CatchProj: 57
  0, // CheckCastPP: 58
  0, // ClearArray: 59
  0, // ConstraintCast: 60
  0, // CMoveD: 61
  0, // CMoveVD: 62
  0, // CMoveF: 63
  0, // CMoveVF: 64
  0, // CMoveI: 65
  0, // CMoveL: 66
  0, // CMoveP: 67
  0, // CMoveN: 68
  1, // CmpN: 69
  1, // CmpD: 70
  0, // CmpD3: 71
  1, // CmpF: 72
  0, // CmpF3: 73
  1, // CmpI: 74
  1, // CmpL: 75
  0, // CmpL3: 76
  0, // CmpLTMask: 77
  1, // CmpP: 78
  1, // CmpU: 79
  1, // CmpUL: 80
  0, // CompareAndSwapB: 81
  0, // CompareAndSwapS: 82
  0, // CompareAndSwapI: 83
  0, // CompareAndSwapL: 84
  0, // CompareAndSwapP: 85
  0, // CompareAndSwapN: 86
  0, // WeakCompareAndSwapB: 87
  0, // WeakCompareAndSwapS: 88
  0, // WeakCompareAndSwapI: 89
  0, // WeakCompareAndSwapL: 90
  0, // WeakCompareAndSwapP: 91
  0, // WeakCompareAndSwapN: 92
  0, // CompareAndExchangeB: 93
  0, // CompareAndExchangeS: 94
  0, // CompareAndExchangeI: 95
  0, // CompareAndExchangeL: 96
  0, // CompareAndExchangeP: 97
  0, // CompareAndExchangeN: 98
  0, // GetAndAddB: 99
  0, // GetAndAddS: 100
  0, // GetAndAddI: 101
  0, // GetAndAddL: 102
  0, // GetAndSetB: 103
  0, // GetAndSetS: 104
  0, // GetAndSetI: 105
  0, // GetAndSetL: 106
  0, // GetAndSetP: 107
  0, // GetAndSetN: 108
  0, // Con: 109
  0, // ConN: 110
  0, // ConNKlass: 111
  0, // ConD: 112
  0, // ConF: 113
  0, // ConI: 114
  0, // ConL: 115
  0, // ConP: 116
  0, // Conv2B: 117
  0, // ConvD2F: 118
  0, // ConvD2I: 119
  0, // ConvD2L: 120
  0, // ConvF2D: 121
  0, // ConvF2I: 122
  0, // ConvF2L: 123
  0, // ConvI2D: 124
  0, // ConvI2F: 125
  0, // ConvI2L: 126
  0, // ConvL2D: 127
  0, // ConvL2F: 128
  0, // ConvL2I: 129
  0, // CountedLoop: 130
  0, // CountedLoopEnd: 131
  0, // OuterStripMinedLoop: 132
  0, // OuterStripMinedLoopEnd: 133
  0, // LongCountedLoop: 134
  0, // LongCountedLoopEnd: 135
  0, // CountLeadingZerosI: 136
  0, // CountLeadingZerosL: 137
  0, // CountTrailingZerosI: 138
  0, // CountTrailingZerosL: 139
  0, // CreateEx: 140
  0, // DecodeN: 141
  0, // DecodeNKlass: 142
  0, // DivD: 143
  0, // DivF: 144
  0, // DivI: 145
  0, // DivL: 146
  0, // DivMod: 147
  0, // DivModI: 148
  0, // DivModL: 149
  0, // EncodeISOArray: 150
  0, // EncodeP: 151
  0, // EncodePKlass: 152
  1, // FastLock: 153
  1, // FastUnlock: 154
  0, // FmaD: 155
  0, // FmaF: 156
  0, // Goto: 157
  0, // Halt: 158
  0, // HasNegatives: 159
  0, // If: 160
  0, // RangeCheck: 161
  0, // IfFalse: 162
  0, // IfTrue: 163
  0, // Initialize: 164
  0, // JProj: 165
  0, // Jump: 166
  0, // JumpProj: 167
  0, // LShiftI: 168
  0, // LShiftL: 169
  0, // LoadB: 170
  0, // LoadUB: 171
  0, // LoadUS: 172
  0, // LoadD: 173
  0, // LoadD_unaligned: 174
  0, // LoadF: 175
  0, // LoadI: 176
  0, // LoadKlass: 177
  0, // LoadNKlass: 178
  0, // LoadL: 179
  0, // LoadL_unaligned: 180
  0, // LoadPLocked: 181
  0, // LoadP: 182
  0, // LoadN: 183
  0, // LoadRange: 184
  0, // LoadS: 185
  0, // Lock: 186
  0, // Loop: 187
  0, // LoopLimit: 188
  0, // Mach: 189
  0, // MachProj: 190
  0, // MulAddS2I: 191
  0, // MaxI: 192
  0, // MaxL: 193
  0, // MaxD: 194
  0, // MaxF: 195
  0, // MemBarAcquire: 196
  0, // LoadFence: 197
  0, // SetVectMaskI: 198
  0, // MemBarAcquireLock: 199
  0, // MemBarCPUOrder: 200
  0, // MemBarRelease: 201
  0, // StoreFence: 202
  0, // MemBarReleaseLock: 203
  0, // MemBarVolatile: 204
  0, // MemBarStoreStore: 205
  0, // MergeMem: 206
  0, // MinI: 207
  0, // MinL: 208
  0, // MinF: 209
  0, // MinD: 210
  0, // ModD: 211
  0, // ModF: 212
  0, // ModI: 213
  0, // ModL: 214
  0, // MoveI2F: 215
  0, // MoveF2I: 216
  0, // MoveL2D: 217
  0, // MoveD2L: 218
  0, // MulD: 219
  0, // MulF: 220
  0, // MulHiL: 221
  0, // MulI: 222
  0, // MulL: 223
  0, // Multi: 224
  0, // NegI: 225
  0, // NegL: 226
  0, // NegD: 227
  0, // NegF: 228
  0, // NeverBranch: 229
  0, // OnSpinWait: 230
  0, // Opaque1: 231
  0, // OpaqueLoopInit: 232
  0, // OpaqueLoopStride: 233
  0, // Opaque2: 234
  0, // Opaque3: 235
  0, // Opaque4: 236
  0, // ProfileBoolean: 237
  0, // OrI: 238
  0, // OrL: 239
  1, // OverflowAddI: 240
  1, // OverflowSubI: 241
  1, // OverflowMulI: 242
  1, // OverflowAddL: 243
  1, // OverflowSubL: 244
  1, // OverflowMulL: 245
  0, // PCTable: 246
  0, // Parm: 247
  0, // PartialSubtypeCheck: 248
  0, // SubTypeCheck: 249
  0, // Phi: 250
  0, // PopCountI: 251
  0, // PopCountL: 252
  0, // PopCountVI: 253
  0, // PrefetchAllocation: 254
  0, // Proj: 255
  0, // RShiftI: 256
  0, // RShiftL: 257
  0, // Region: 258
  0, // Rethrow: 259
  0, // Return: 260
  0, // Root: 261
  0, // RoundDouble: 262
  0, // RoundDoubleMode: 263
  0, // RoundDoubleModeV: 264
  0, // RoundFloat: 265
  0, // RotateLeft: 266
  0, // RotateLeftV: 267
  0, // RotateRight: 268
  0, // RotateRightV: 269
  0, // SafePoint: 270
  0, // SafePointScalarObject: 271
  0, // ShenandoahCompareAndExchangeP: 272
  0, // ShenandoahCompareAndExchangeN: 273
  0, // ShenandoahCompareAndSwapN: 274
  0, // ShenandoahCompareAndSwapP: 275
  0, // ShenandoahWeakCompareAndSwapN: 276
  0, // ShenandoahWeakCompareAndSwapP: 277
  0, // ShenandoahIUBarrier: 278
  0, // ShenandoahLoadReferenceBarrier: 279
  0, // SCMemProj: 280
  0, // CopySignD: 281
  0, // CopySignF: 282
  0, // SignumD: 283
  0, // SignumF: 284
  0, // SqrtD: 285
  0, // SqrtF: 286
  0, // Start: 287
  0, // StartOSR: 288
  0, // StoreB: 289
  0, // StoreC: 290
  0, // StoreCM: 291
  0, // StorePConditional: 292
  0, // StoreIConditional: 293
  0, // StoreLConditional: 294
  0, // StoreD: 295
  0, // StoreF: 296
  0, // StoreI: 297
  0, // StoreL: 298
  0, // StoreP: 299
  0, // StoreN: 300
  0, // StoreNKlass: 301
  0, // StrComp: 302
  0, // StrCompressedCopy: 303
  0, // StrEquals: 304
  0, // StrIndexOf: 305
  0, // StrIndexOfChar: 306
  0, // StrInflatedCopy: 307
  0, // SubD: 308
  0, // SubF: 309
  0, // SubI: 310
  0, // SubL: 311
  0, // TailCall: 312
  0, // TailJump: 313
  0, // MacroLogicV: 314
  0, // ThreadLocal: 315
  0, // Unlock: 316
  0, // URShiftB: 317
  0, // URShiftS: 318
  0, // URShiftI: 319
  0, // URShiftL: 320
  0, // XorI: 321
  0, // XorL: 322
  0, // Vector: 323
  0, // AddVB: 324
  0, // AddVS: 325
  0, // AddVI: 326
  0, // AddReductionVI: 327
  0, // AddVL: 328
  0, // AddReductionVL: 329
  0, // AddVF: 330
  0, // AddReductionVF: 331
  0, // AddVD: 332
  0, // AddReductionVD: 333
  0, // SubVB: 334
  0, // SubVS: 335
  0, // SubVI: 336
  0, // SubVL: 337
  0, // SubVF: 338
  0, // SubVD: 339
  0, // MulVB: 340
  0, // MulVS: 341
  0, // MulVI: 342
  0, // MulReductionVI: 343
  0, // MulVL: 344
  0, // MulReductionVL: 345
  0, // MulVF: 346
  0, // MulReductionVF: 347
  0, // MulVD: 348
  0, // MulReductionVD: 349
  0, // MulAddVS2VI: 350
  0, // FmaVD: 351
  0, // FmaVF: 352
  0, // DivVF: 353
  0, // DivVD: 354
  0, // AbsVB: 355
  0, // AbsVS: 356
  0, // AbsVI: 357
  0, // AbsVL: 358
  0, // AbsVF: 359
  0, // AbsVD: 360
  0, // NegVI: 361
  0, // NegVF: 362
  0, // NegVD: 363
  0, // SqrtVD: 364
  0, // SqrtVF: 365
  0, // LShiftCntV: 366
  0, // RShiftCntV: 367
  0, // LShiftVB: 368
  0, // LShiftVS: 369
  0, // LShiftVI: 370
  0, // LShiftVL: 371
  0, // RShiftVB: 372
  0, // RShiftVS: 373
  0, // RShiftVI: 374
  0, // RShiftVL: 375
  0, // URShiftVB: 376
  0, // URShiftVS: 377
  0, // URShiftVI: 378
  0, // URShiftVL: 379
  0, // AndV: 380
  0, // AndReductionV: 381
  0, // OrV: 382
  0, // OrReductionV: 383
  0, // XorV: 384
  0, // XorReductionV: 385
  0, // MinV: 386
  0, // MaxV: 387
  0, // MinReductionV: 388
  0, // MaxReductionV: 389
  0, // LoadVector: 390
  0, // LoadVectorGather: 391
  0, // StoreVector: 392
  0, // StoreVectorScatter: 393
  0, // LoadVectorMasked: 394
  0, // StoreVectorMasked: 395
  0, // VectorMaskGen: 396
  0, // Pack: 397
  0, // PackB: 398
  0, // PackS: 399
  0, // PackI: 400
  0, // PackL: 401
  0, // PackF: 402
  0, // PackD: 403
  0, // Pack2L: 404
  0, // Pack2D: 405
  0, // ReplicateB: 406
  0, // ReplicateS: 407
  0, // ReplicateI: 408
  0, // ReplicateL: 409
  0, // ReplicateF: 410
  0, // ReplicateD: 411
  0, // Extract: 412
  0, // ExtractB: 413
  0, // ExtractUB: 414
  0, // ExtractC: 415
  0, // ExtractS: 416
  0, // ExtractI: 417
  0, // ExtractL: 418
  0, // ExtractF: 419
  0, // ExtractD: 420
  0, // Digit: 421
  0, // LowerCase: 422
  0, // UpperCase: 423
  0, // Whitespace: 424
  0, // VectorBox: 425
  0, // VectorBoxAllocate: 426
  0, // VectorUnbox: 427
  0, // VectorMaskWrapper: 428
  0, // VectorMaskCmp: 429
  0, // VectorTest: 430
  0, // VectorBlend: 431
  0, // VectorRearrange: 432
  0, // VectorLoadMask: 433
  0, // VectorLoadShuffle: 434
  0, // VectorLoadConst: 435
  0, // VectorStoreMask: 436
  0, // VectorReinterpret: 437
  0, // VectorCast: 438
  0, // VectorCastB2X: 439
  0, // VectorCastS2X: 440
  0, // VectorCastI2X: 441
  0, // VectorCastL2X: 442
  0, // VectorCastF2X: 443
  0, // VectorCastD2X: 444
  0 // no trailing comma // VectorInsert: 445
};
//  The following instructions can cisc-spill
//  popCountI can cisc-spill operand 1 to popCountI_mem
//  popCountL can cisc-spill operand 1 to popCountL_mem
//  cmovI_reg can cisc-spill operand 4 to cmovI_mem
//  cmovI_regU can cisc-spill operand 4 to cmovI_memU
//  cmovI_regUCF can cisc-spill operand 4 to cmovI_memUCF
//  addI_eReg can cisc-spill operand 2 to addI_eReg_mem
//  subI_eReg can cisc-spill operand 2 to subI_eReg_mem
//  mulI_eReg can cisc-spill operand 2 to mulI
//  mulI_eReg_imm can cisc-spill operand 1 to mulI_mem_imm
//  i2b can cisc-spill operand 1 to loadI2B
//  i2s can cisc-spill operand 1 to loadI2S
//  andI_eReg can cisc-spill operand 2 to andI_eReg_mem
//  andnI_rReg_rReg_rReg can cisc-spill operand 3 to andnI_rReg_rReg_mem
//  andnI_rReg_rReg_rReg_0 can cisc-spill operand 1 to andnI_rReg_rReg_mem_0
//  orI_eReg can cisc-spill operand 2 to orI_eReg_mem
//  xorI_eReg can cisc-spill operand 2 to xorI_eReg_mem
//  addL_eReg can cisc-spill operand 2 to addL_eReg_mem
//  subL_eReg can cisc-spill operand 2 to subL_eReg_mem
//  andL_eReg can cisc-spill operand 2 to andL_eReg_mem
//  andnL_eReg_eReg_eReg can cisc-spill operand 3 to andnL_eReg_eReg_mem
//  andnL_eReg_eReg_eReg_0 can cisc-spill operand 1 to andnL_eReg_eReg_mem_0
//  orl_eReg can cisc-spill operand 2 to orl_eReg_mem
//  xorl_eReg can cisc-spill operand 2 to xorl_eReg_mem
//  cmpD_cc can cisc-spill operand 2 to cmpD_ccmem
//  cmpD_ccCF can cisc-spill operand 2 to cmpD_ccmemCF
//  cmpD_reg can cisc-spill operand 2 to cmpD_regmem
//  subDPR_reg can cisc-spill operand 2 to subDPR_reg_mem
//  addDPR_reg can cisc-spill operand 2 to addDPR_reg_mem
//  mulDPR_reg can cisc-spill operand 2 to mulDPR_reg_mem
//  cmpF_cc can cisc-spill operand 2 to cmpF_ccmem
//  cmpF_ccCF can cisc-spill operand 2 to cmpF_ccmemCF
//  cmpF_reg can cisc-spill operand 2 to cmpF_regmem
//  addFPR24_reg can cisc-spill operand 2 to addFPR24_reg_mem
//  addFPR_reg can cisc-spill operand 2 to addFPR_reg_mem
//  addFPR24_reg_mem can cisc-spill operand 1 to addFPR24_mem_cisc
//  addFPR24_reg_mem_0 can cisc-spill operand 2 to addFPR24_mem_cisc_0
//  addFPR24_mem_reg can cisc-spill operand 2 to addFPR24_mem_cisc
//  mulFPR24_reg can cisc-spill operand 2 to mulFPR24_reg_mem
//  mulFPR_reg can cisc-spill operand 2 to mulFPR_reg_mem
//  addFPR_mulFPR_reg can cisc-spill operand 1 to addFPR_mulFPR_reg_load1
//  addFPR_mulFPR_reg_0 can cisc-spill operand 2 to addFPR_mulFPR_reg_load1_1
//  convI2L_reg can cisc-spill operand 1 to loadI2L
//  convI2L_reg_zex can cisc-spill operand 1 to loadUI2L
//  compI_eReg can cisc-spill operand 2 to compI_eReg_mem
//  compU_eReg can cisc-spill operand 2 to compU_eReg_mem
//  compP_eReg can cisc-spill operand 2 to compP_eReg_mem
//  cmovLL_reg_LTGE can cisc-spill operand 4 to cmovLL_mem_LTGE
//  cmovII_reg_LTGE can cisc-spill operand 4 to cmovII_mem_LTGE
//  cmovLL_reg_EQNE can cisc-spill operand 4 to cmovLL_mem_EQNE
//  cmovII_reg_EQNE can cisc-spill operand 4 to cmovII_mem_EQNE
//  cmovLL_reg_LEGT can cisc-spill operand 4 to cmovLL_mem_LEGT
//  cmovLL_reg_LEGT_U can cisc-spill operand 4 to cmovLL_mem_LEGT_U
//  cmovII_reg_LEGT can cisc-spill operand 4 to cmovII_mem_LEGT
//  addF_reg can cisc-spill operand 2 to addF_mem
//  addF_reg_reg can cisc-spill operand 2 to addF_reg_mem
//  addD_reg can cisc-spill operand 2 to addD_mem
//  addD_reg_reg can cisc-spill operand 2 to addD_reg_mem
//  subF_reg can cisc-spill operand 2 to subF_mem
//  subF_reg_reg can cisc-spill operand 2 to subF_reg_mem
//  subD_reg can cisc-spill operand 2 to subD_mem
//  subD_reg_reg can cisc-spill operand 2 to subD_reg_mem
//  mulF_reg can cisc-spill operand 2 to mulF_mem
//  mulF_reg_reg can cisc-spill operand 2 to mulF_reg_mem
//  mulD_reg can cisc-spill operand 2 to mulD_mem
//  mulD_reg_reg can cisc-spill operand 2 to mulD_reg_mem
//  divF_reg can cisc-spill operand 2 to divF_mem
//  divF_reg_reg can cisc-spill operand 2 to divF_reg_mem
//  divD_reg can cisc-spill operand 2 to divD_mem
//  divD_reg_reg can cisc-spill operand 2 to divD_reg_mem
//  sqrtF_reg can cisc-spill operand 1 to sqrtF_mem
//  sqrtD_reg can cisc-spill operand 1 to sqrtD_mem
//  ReplI_reg can cisc-spill operand 1 to ReplI_mem
//  ReplF_reg can cisc-spill operand 1 to ReplF_mem
//  ReplD_reg can cisc-spill operand 1 to ReplD_mem



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "ECX",
  "EBX",
  "EBP",
  "EDI",
  "EAX",
  "EDX",
  "ESI",
  "ESP",
  "FPR0L",
  "FPR0H",
  "FPR1L",
  "FPR1H",
  "FPR2L",
  "FPR2H",
  "FPR3L",
  "FPR3H",
  "FPR4L",
  "FPR4H",
  "FPR5L",
  "FPR5H",
  "FPR6L",
  "FPR6H",
  "FPR7L",
  "FPR7H",
  "FILL0",
  "FILL1",
  "FILL2",
  "FILL3",
  "FILL4",
  "FILL5",
  "FILL6",
  "FILL7",
  "XMM0",
  "XMM0b",
  "XMM0c",
  "XMM0d",
  "XMM0e",
  "XMM0f",
  "XMM0g",
  "XMM0h",
  "XMM0i",
  "XMM0j",
  "XMM0k",
  "XMM0l",
  "XMM0m",
  "XMM0n",
  "XMM0o",
  "XMM0p",
  "XMM1",
  "XMM1b",
  "XMM1c",
  "XMM1d",
  "XMM1e",
  "XMM1f",
  "XMM1g",
  "XMM1h",
  "XMM1i",
  "XMM1j",
  "XMM1k",
  "XMM1l",
  "XMM1m",
  "XMM1n",
  "XMM1o",
  "XMM1p",
  "XMM2",
  "XMM2b",
  "XMM2c",
  "XMM2d",
  "XMM2e",
  "XMM2f",
  "XMM2g",
  "XMM2h",
  "XMM2i",
  "XMM2j",
  "XMM2k",
  "XMM2l",
  "XMM2m",
  "XMM2n",
  "XMM2o",
  "XMM2p",
  "XMM3",
  "XMM3b",
  "XMM3c",
  "XMM3d",
  "XMM3e",
  "XMM3f",
  "XMM3g",
  "XMM3h",
  "XMM3i",
  "XMM3j",
  "XMM3k",
  "XMM3l",
  "XMM3m",
  "XMM3n",
  "XMM3o",
  "XMM3p",
  "XMM4",
  "XMM4b",
  "XMM4c",
  "XMM4d",
  "XMM4e",
  "XMM4f",
  "XMM4g",
  "XMM4h",
  "XMM4i",
  "XMM4j",
  "XMM4k",
  "XMM4l",
  "XMM4m",
  "XMM4n",
  "XMM4o",
  "XMM4p",
  "XMM5",
  "XMM5b",
  "XMM5c",
  "XMM5d",
  "XMM5e",
  "XMM5f",
  "XMM5g",
  "XMM5h",
  "XMM5i",
  "XMM5j",
  "XMM5k",
  "XMM5l",
  "XMM5m",
  "XMM5n",
  "XMM5o",
  "XMM5p",
  "XMM6",
  "XMM6b",
  "XMM6c",
  "XMM6d",
  "XMM6e",
  "XMM6f",
  "XMM6g",
  "XMM6h",
  "XMM6i",
  "XMM6j",
  "XMM6k",
  "XMM6l",
  "XMM6m",
  "XMM6n",
  "XMM6o",
  "XMM6p",
  "XMM7",
  "XMM7b",
  "XMM7c",
  "XMM7d",
  "XMM7e",
  "XMM7f",
  "XMM7g",
  "XMM7h",
  "XMM7i",
  "XMM7j",
  "XMM7k",
  "XMM7l",
  "XMM7m",
  "XMM7n",
  "XMM7o",
  "XMM7p",
  "RFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	rcx->as_VMReg(),
	rbx->as_VMReg(),
	rbp->as_VMReg(),
	rdi->as_VMReg(),
	rax->as_VMReg(),
	rdx->as_VMReg(),
	rsi->as_VMReg(),
	rsp->as_VMReg(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	as_FloatRegister(0)->as_VMReg(),
	as_FloatRegister(0)->as_VMReg()->next(),
	as_FloatRegister(1)->as_VMReg(),
	as_FloatRegister(1)->as_VMReg()->next(),
	as_FloatRegister(2)->as_VMReg(),
	as_FloatRegister(2)->as_VMReg()->next(),
	as_FloatRegister(3)->as_VMReg(),
	as_FloatRegister(3)->as_VMReg()->next(),
	as_FloatRegister(4)->as_VMReg(),
	as_FloatRegister(4)->as_VMReg()->next(),
	as_FloatRegister(5)->as_VMReg(),
	as_FloatRegister(5)->as_VMReg()->next(),
	as_FloatRegister(6)->as_VMReg(),
	as_FloatRegister(6)->as_VMReg()->next(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	xmm0->as_VMReg(),
	xmm0->as_VMReg()->next(1),
	xmm0->as_VMReg()->next(2),
	xmm0->as_VMReg()->next(3),
	xmm0->as_VMReg()->next(4),
	xmm0->as_VMReg()->next(5),
	xmm0->as_VMReg()->next(6),
	xmm0->as_VMReg()->next(7),
	xmm0->as_VMReg()->next(8),
	xmm0->as_VMReg()->next(9),
	xmm0->as_VMReg()->next(10),
	xmm0->as_VMReg()->next(11),
	xmm0->as_VMReg()->next(12),
	xmm0->as_VMReg()->next(13),
	xmm0->as_VMReg()->next(14),
	xmm0->as_VMReg()->next(15),
	xmm1->as_VMReg(),
	xmm1->as_VMReg()->next(1),
	xmm1->as_VMReg()->next(2),
	xmm1->as_VMReg()->next(3),
	xmm1->as_VMReg()->next(4),
	xmm1->as_VMReg()->next(5),
	xmm1->as_VMReg()->next(6),
	xmm1->as_VMReg()->next(7),
	xmm1->as_VMReg()->next(8),
	xmm1->as_VMReg()->next(9),
	xmm1->as_VMReg()->next(10),
	xmm1->as_VMReg()->next(11),
	xmm1->as_VMReg()->next(12),
	xmm1->as_VMReg()->next(13),
	xmm1->as_VMReg()->next(14),
	xmm1->as_VMReg()->next(15),
	xmm2->as_VMReg(),
	xmm2->as_VMReg()->next(1),
	xmm2->as_VMReg()->next(2),
	xmm2->as_VMReg()->next(3),
	xmm2->as_VMReg()->next(4),
	xmm2->as_VMReg()->next(5),
	xmm2->as_VMReg()->next(6),
	xmm2->as_VMReg()->next(7),
	xmm2->as_VMReg()->next(8),
	xmm2->as_VMReg()->next(9),
	xmm2->as_VMReg()->next(10),
	xmm2->as_VMReg()->next(11),
	xmm2->as_VMReg()->next(12),
	xmm2->as_VMReg()->next(13),
	xmm2->as_VMReg()->next(14),
	xmm2->as_VMReg()->next(15),
	xmm3->as_VMReg(),
	xmm3->as_VMReg()->next(1),
	xmm3->as_VMReg()->next(2),
	xmm3->as_VMReg()->next(3),
	xmm3->as_VMReg()->next(4),
	xmm3->as_VMReg()->next(5),
	xmm3->as_VMReg()->next(6),
	xmm3->as_VMReg()->next(7),
	xmm3->as_VMReg()->next(8),
	xmm3->as_VMReg()->next(9),
	xmm3->as_VMReg()->next(10),
	xmm3->as_VMReg()->next(11),
	xmm3->as_VMReg()->next(12),
	xmm3->as_VMReg()->next(13),
	xmm3->as_VMReg()->next(14),
	xmm3->as_VMReg()->next(15),
	xmm4->as_VMReg(),
	xmm4->as_VMReg()->next(1),
	xmm4->as_VMReg()->next(2),
	xmm4->as_VMReg()->next(3),
	xmm4->as_VMReg()->next(4),
	xmm4->as_VMReg()->next(5),
	xmm4->as_VMReg()->next(6),
	xmm4->as_VMReg()->next(7),
	xmm4->as_VMReg()->next(8),
	xmm4->as_VMReg()->next(9),
	xmm4->as_VMReg()->next(10),
	xmm4->as_VMReg()->next(11),
	xmm4->as_VMReg()->next(12),
	xmm4->as_VMReg()->next(13),
	xmm4->as_VMReg()->next(14),
	xmm4->as_VMReg()->next(15),
	xmm5->as_VMReg(),
	xmm5->as_VMReg()->next(1),
	xmm5->as_VMReg()->next(2),
	xmm5->as_VMReg()->next(3),
	xmm5->as_VMReg()->next(4),
	xmm5->as_VMReg()->next(5),
	xmm5->as_VMReg()->next(6),
	xmm5->as_VMReg()->next(7),
	xmm5->as_VMReg()->next(8),
	xmm5->as_VMReg()->next(9),
	xmm5->as_VMReg()->next(10),
	xmm5->as_VMReg()->next(11),
	xmm5->as_VMReg()->next(12),
	xmm5->as_VMReg()->next(13),
	xmm5->as_VMReg()->next(14),
	xmm5->as_VMReg()->next(15),
	xmm6->as_VMReg(),
	xmm6->as_VMReg()->next(1),
	xmm6->as_VMReg()->next(2),
	xmm6->as_VMReg()->next(3),
	xmm6->as_VMReg()->next(4),
	xmm6->as_VMReg()->next(5),
	xmm6->as_VMReg()->next(6),
	xmm6->as_VMReg()->next(7),
	xmm6->as_VMReg()->next(8),
	xmm6->as_VMReg()->next(9),
	xmm6->as_VMReg()->next(10),
	xmm6->as_VMReg()->next(11),
	xmm6->as_VMReg()->next(12),
	xmm6->as_VMReg()->next(13),
	xmm6->as_VMReg()->next(14),
	xmm6->as_VMReg()->next(15),
	xmm7->as_VMReg(),
	xmm7->as_VMReg()->next(1),
	xmm7->as_VMReg()->next(2),
	xmm7->as_VMReg()->next(3),
	xmm7->as_VMReg()->next(4),
	xmm7->as_VMReg()->next(5),
	xmm7->as_VMReg()->next(6),
	xmm7->as_VMReg()->next(7),
	xmm7->as_VMReg()->next(8),
	xmm7->as_VMReg()->next(9),
	xmm7->as_VMReg()->next(10),
	xmm7->as_VMReg()->next(11),
	xmm7->as_VMReg()->next(12),
	xmm7->as_VMReg()->next(13),
	xmm7->as_VMReg()->next(14),
	xmm7->as_VMReg()->next(15),
	VMRegImpl::Bad() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\x1',  // ECX
  (unsigned char)'\x3',  // EBX
  (unsigned char)'\x5',  // EBP
  (unsigned char)'\x7',  // EDI
  (unsigned char)'\x0',  // EAX
  (unsigned char)'\x2',  // EDX
  (unsigned char)'\x6',  // ESI
  (unsigned char)'\x4',  // ESP
  (unsigned char)'\x0',  // FPR0L
  (unsigned char)'\x0',  // FPR0H
  (unsigned char)'\x1',  // FPR1L
  (unsigned char)'\x1',  // FPR1H
  (unsigned char)'\x2',  // FPR2L
  (unsigned char)'\x2',  // FPR2H
  (unsigned char)'\x3',  // FPR3L
  (unsigned char)'\x3',  // FPR3H
  (unsigned char)'\x4',  // FPR4L
  (unsigned char)'\x4',  // FPR4H
  (unsigned char)'\x5',  // FPR5L
  (unsigned char)'\x5',  // FPR5H
  (unsigned char)'\x6',  // FPR6L
  (unsigned char)'\x6',  // FPR6H
  (unsigned char)'\x7',  // FPR7L
  (unsigned char)'\x7',  // FPR7H
  (unsigned char)'\x8',  // FILL0
  (unsigned char)'\x9',  // FILL1
  (unsigned char)'\xA',  // FILL2
  (unsigned char)'\xB',  // FILL3
  (unsigned char)'\xC',  // FILL4
  (unsigned char)'\xD',  // FILL5
  (unsigned char)'\xE',  // FILL6
  (unsigned char)'\xF',  // FILL7
  (unsigned char)'\x0',  // XMM0
  (unsigned char)'\x0',  // XMM0b
  (unsigned char)'\x0',  // XMM0c
  (unsigned char)'\x0',  // XMM0d
  (unsigned char)'\x0',  // XMM0e
  (unsigned char)'\x0',  // XMM0f
  (unsigned char)'\x0',  // XMM0g
  (unsigned char)'\x0',  // XMM0h
  (unsigned char)'\x0',  // XMM0i
  (unsigned char)'\x0',  // XMM0j
  (unsigned char)'\x0',  // XMM0k
  (unsigned char)'\x0',  // XMM0l
  (unsigned char)'\x0',  // XMM0m
  (unsigned char)'\x0',  // XMM0n
  (unsigned char)'\x0',  // XMM0o
  (unsigned char)'\x0',  // XMM0p
  (unsigned char)'\x1',  // XMM1
  (unsigned char)'\x1',  // XMM1b
  (unsigned char)'\x1',  // XMM1c
  (unsigned char)'\x1',  // XMM1d
  (unsigned char)'\x1',  // XMM1e
  (unsigned char)'\x1',  // XMM1f
  (unsigned char)'\x1',  // XMM1g
  (unsigned char)'\x1',  // XMM1h
  (unsigned char)'\x1',  // XMM1i
  (unsigned char)'\x1',  // XMM1j
  (unsigned char)'\x1',  // XMM1k
  (unsigned char)'\x1',  // XMM1l
  (unsigned char)'\x1',  // XMM1m
  (unsigned char)'\x1',  // XMM1n
  (unsigned char)'\x1',  // XMM1o
  (unsigned char)'\x1',  // XMM1p
  (unsigned char)'\x2',  // XMM2
  (unsigned char)'\x2',  // XMM2b
  (unsigned char)'\x2',  // XMM2c
  (unsigned char)'\x2',  // XMM2d
  (unsigned char)'\x2',  // XMM2e
  (unsigned char)'\x2',  // XMM2f
  (unsigned char)'\x2',  // XMM2g
  (unsigned char)'\x2',  // XMM2h
  (unsigned char)'\x2',  // XMM2i
  (unsigned char)'\x2',  // XMM2j
  (unsigned char)'\x2',  // XMM2k
  (unsigned char)'\x2',  // XMM2l
  (unsigned char)'\x2',  // XMM2m
  (unsigned char)'\x2',  // XMM2n
  (unsigned char)'\x2',  // XMM2o
  (unsigned char)'\x2',  // XMM2p
  (unsigned char)'\x3',  // XMM3
  (unsigned char)'\x3',  // XMM3b
  (unsigned char)'\x3',  // XMM3c
  (unsigned char)'\x3',  // XMM3d
  (unsigned char)'\x3',  // XMM3e
  (unsigned char)'\x3',  // XMM3f
  (unsigned char)'\x3',  // XMM3g
  (unsigned char)'\x3',  // XMM3h
  (unsigned char)'\x3',  // XMM3i
  (unsigned char)'\x3',  // XMM3j
  (unsigned char)'\x3',  // XMM3k
  (unsigned char)'\x3',  // XMM3l
  (unsigned char)'\x3',  // XMM3m
  (unsigned char)'\x3',  // XMM3n
  (unsigned char)'\x3',  // XMM3o
  (unsigned char)'\x3',  // XMM3p
  (unsigned char)'\x4',  // XMM4
  (unsigned char)'\x4',  // XMM4b
  (unsigned char)'\x4',  // XMM4c
  (unsigned char)'\x4',  // XMM4d
  (unsigned char)'\x4',  // XMM4e
  (unsigned char)'\x4',  // XMM4f
  (unsigned char)'\x4',  // XMM4g
  (unsigned char)'\x4',  // XMM4h
  (unsigned char)'\x4',  // XMM4i
  (unsigned char)'\x4',  // XMM4j
  (unsigned char)'\x4',  // XMM4k
  (unsigned char)'\x4',  // XMM4l
  (unsigned char)'\x4',  // XMM4m
  (unsigned char)'\x4',  // XMM4n
  (unsigned char)'\x4',  // XMM4o
  (unsigned char)'\x4',  // XMM4p
  (unsigned char)'\x5',  // XMM5
  (unsigned char)'\x5',  // XMM5b
  (unsigned char)'\x5',  // XMM5c
  (unsigned char)'\x5',  // XMM5d
  (unsigned char)'\x5',  // XMM5e
  (unsigned char)'\x5',  // XMM5f
  (unsigned char)'\x5',  // XMM5g
  (unsigned char)'\x5',  // XMM5h
  (unsigned char)'\x5',  // XMM5i
  (unsigned char)'\x5',  // XMM5j
  (unsigned char)'\x5',  // XMM5k
  (unsigned char)'\x5',  // XMM5l
  (unsigned char)'\x5',  // XMM5m
  (unsigned char)'\x5',  // XMM5n
  (unsigned char)'\x5',  // XMM5o
  (unsigned char)'\x5',  // XMM5p
  (unsigned char)'\x6',  // XMM6
  (unsigned char)'\x6',  // XMM6b
  (unsigned char)'\x6',  // XMM6c
  (unsigned char)'\x6',  // XMM6d
  (unsigned char)'\x6',  // XMM6e
  (unsigned char)'\x6',  // XMM6f
  (unsigned char)'\x6',  // XMM6g
  (unsigned char)'\x6',  // XMM6h
  (unsigned char)'\x6',  // XMM6i
  (unsigned char)'\x6',  // XMM6j
  (unsigned char)'\x6',  // XMM6k
  (unsigned char)'\x6',  // XMM6l
  (unsigned char)'\x6',  // XMM6m
  (unsigned char)'\x6',  // XMM6n
  (unsigned char)'\x6',  // XMM6o
  (unsigned char)'\x6',  // XMM6p
  (unsigned char)'\x7',  // XMM7
  (unsigned char)'\x7',  // XMM7b
  (unsigned char)'\x7',  // XMM7c
  (unsigned char)'\x7',  // XMM7d
  (unsigned char)'\x7',  // XMM7e
  (unsigned char)'\x7',  // XMM7f
  (unsigned char)'\x7',  // XMM7g
  (unsigned char)'\x7',  // XMM7h
  (unsigned char)'\x7',  // XMM7i
  (unsigned char)'\x7',  // XMM7j
  (unsigned char)'\x7',  // XMM7k
  (unsigned char)'\x7',  // XMM7l
  (unsigned char)'\x7',  // XMM7m
  (unsigned char)'\x7',  // XMM7n
  (unsigned char)'\x7',  // XMM7o
  (unsigned char)'\x7',  // XMM7p
  (unsigned char)'\x8' // no trailing comma  // RFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone() const {
  return  new labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone() const {
  return  new methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *rRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *xRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_X_REG_mask();
}

const RegMask *eAXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EAX_REG_mask();
}

const RegMask *eBXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBX_REG_mask();
}

const RegMask *eCXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ECX_REG_mask();
}

const RegMask *eDXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDX_REG_mask();
}

const RegMask *eDIRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDI_REG_mask();
}

const RegMask *naxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NAX_REG_mask();
}

const RegMask *nadxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NADX_REG_mask();
}

const RegMask *ncxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NCX_REG_mask();
}

const RegMask *eSIRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *anyRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *eRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *rRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *eRegP_no_EBPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_NO_EBP_mask();
}

const RegMask *naxRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NAX_REG_mask();
}

const RegMask *nabxRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NABX_REG_mask();
}

const RegMask *pRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &P_REG_mask();
}

const RegMask *eAXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EAX_REG_mask();
}

const RegMask *eBXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBX_REG_mask();
}

const RegMask *eCXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ECX_REG_mask();
}

const RegMask *eDXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDX_REG_mask();
}

const RegMask *eSIRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *eDIRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDI_REG_mask();
}

const RegMask *eRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_REG_mask();
}

const RegMask *eADXRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EADX_REG_mask();
}

const RegMask *eBCXRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBCX_REG_mask();
}

const RegMask *eADXRegL_low_onlyOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EADX_REG_mask();
}

const RegMask *rFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *eFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *eFlagsRegUOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *eFlagsRegUCFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_long_LTGEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_long_EQNEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_long_LEGTOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_ulong_LTGEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_ulong_EQNEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_ulong_LEGTOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *regDPROper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_REG_mask();
}

const RegMask *regDPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_REG0_mask();
}

const RegMask *regDPR2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_REG1_mask();
}

const RegMask *regnotDPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_NOTREG0_mask();
}

const RegMask *regFPROper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_FLT_REG_mask();
}

const RegMask *regFPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_FLT_REG0_mask();
}

const RegMask *regFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_LEGACY_mask();
}

const RegMask *legRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_LEGACY_mask();
}

const RegMask *vlRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_VL_mask();
}

const RegMask *regDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_LEGACY_mask();
}

const RegMask *legRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_LEGACY_mask();
}

const RegMask *vlRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_VL_mask();
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset32XOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexScaleOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexScaleOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *load_long_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *load_long_indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *load_long_indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *rxmm0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &XMM0_REG_mask();
}

const RegMask *vecOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RegMask::Empty;
}

const RegMask *legVecOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RegMask::Empty;
}

const RegMask *vecSOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORS_REG_VLBWDQ_mask();
}

const RegMask *legVecSOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORS_REG_LEGACY_mask();
}

const RegMask *vecDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORD_REG_VLBWDQ_mask();
}

const RegMask *legVecDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORD_REG_LEGACY_mask();
}

const RegMask *vecXOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORX_REG_VLBWDQ_mask();
}

const RegMask *legVecXOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORX_REG_LEGACY_mask();
}

const RegMask *vecYOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORY_REG_VLBWDQ_mask();
}

const RegMask *legVecYOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORY_REG_LEGACY_mask();
}

const RegMask *vecZOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORZ_REG_mask();
}

const RegMask *legVecZOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORZ_REG_LEGACY_mask();
}

//------------------Define members for classes derived from MachNode----------

void popCountINode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *popCountINode::cisc_version(int offset) {
  popCountI_memNode *node = new popCountI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void popCountLNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *popCountLNode::cisc_version(int offset) {
  popCountL_memNode *node = new popCountL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regNode::cisc_version(int offset) {
  cmovI_memNode *node = new cmovI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovI_regUNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regUNode::cisc_version(int offset) {
  cmovI_memUNode *node = new cmovI_memUNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovI_regUCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regUCFNode::cisc_version(int offset) {
  cmovI_memUCFNode *node = new cmovI_memUCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addI_eRegNode::cisc_version(int offset) {
  addI_eReg_memNode *node = new addI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *subI_eRegNode::cisc_version(int offset) {
  subI_eReg_memNode *node = new subI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulI_eRegNode::cisc_version(int offset) {
  mulINode *node = new mulINode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulI_eReg_immNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulI_eReg_immNode::cisc_version(int offset) {
  mulI_mem_immNode *node = new mulI_mem_immNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void i2bNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_X_REG_mask();
}

// Build CISC version of this instruction
MachNode *i2bNode::cisc_version(int offset) {
  loadI2BNode *node = new loadI2BNode();
  node->_num_opnds = 3;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void i2sNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_X_REG_mask();
}

// Build CISC version of this instruction
MachNode *i2sNode::cisc_version(int offset) {
  loadI2SNode *node = new loadI2SNode();
  node->_num_opnds = 3;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andI_eRegNode::cisc_version(int offset) {
  andI_eReg_memNode *node = new andI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnI_rReg_rReg_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnI_rReg_rReg_rRegNode::cisc_version(int offset) {
  andnI_rReg_rReg_memNode *node = new andnI_rReg_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnI_rReg_rReg_rReg_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnI_rReg_rReg_rReg_0Node::cisc_version(int offset) {
  andnI_rReg_rReg_mem_0Node *node = new andnI_rReg_rReg_mem_0Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void orI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *orI_eRegNode::cisc_version(int offset) {
  orI_eReg_memNode *node = new orI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void xorI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *xorI_eRegNode::cisc_version(int offset) {
  xorI_eReg_memNode *node = new xorI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *addL_eRegNode::cisc_version(int offset) {
  addL_eReg_memNode *node = new addL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *subL_eRegNode::cisc_version(int offset) {
  subL_eReg_memNode *node = new subL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *andL_eRegNode::cisc_version(int offset) {
  andL_eReg_memNode *node = new andL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnL_eReg_eReg_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnL_eReg_eReg_eRegNode::cisc_version(int offset) {
  andnL_eReg_eReg_memNode *node = new andnL_eReg_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnL_eReg_eReg_eReg_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnL_eReg_eReg_eReg_0Node::cisc_version(int offset) {
  andnL_eReg_eReg_mem_0Node *node = new andnL_eReg_eReg_mem_0Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void orl_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *orl_eRegNode::cisc_version(int offset) {
  orl_eReg_memNode *node = new orl_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void xorl_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *xorl_eRegNode::cisc_version(int offset) {
  xorl_eReg_memNode *node = new xorl_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpD_ccNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_ccNode::cisc_version(int offset) {
  cmpD_ccmemNode *node = new cmpD_ccmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpD_ccCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_ccCFNode::cisc_version(int offset) {
  cmpD_ccmemCFNode *node = new cmpD_ccmemCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_regNode::cisc_version(int offset) {
  cmpD_regmemNode *node = new cmpD_regmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subDPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_DBL_REG_mask();
}

// Build CISC version of this instruction
MachNode *subDPR_regNode::cisc_version(int offset) {
  subDPR_reg_memNode *node = new subDPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addDPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_DBL_REG_mask();
}

// Build CISC version of this instruction
MachNode *addDPR_regNode::cisc_version(int offset) {
  addDPR_reg_memNode *node = new addDPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulDPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_DBL_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulDPR_regNode::cisc_version(int offset) {
  mulDPR_reg_memNode *node = new mulDPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpF_ccNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_ccNode::cisc_version(int offset) {
  cmpF_ccmemNode *node = new cmpF_ccmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpF_ccCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_ccCFNode::cisc_version(int offset) {
  cmpF_ccmemCFNode *node = new cmpF_ccmemCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_regNode::cisc_version(int offset) {
  cmpF_regmemNode *node = new cmpF_regmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_regNode::cisc_version(int offset) {
  addFPR24_reg_memNode *node = new addFPR24_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR_regNode::cisc_version(int offset) {
  addFPR_reg_memNode *node = new addFPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_reg_memNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_reg_memNode::cisc_version(int offset) {
  addFPR24_mem_ciscNode *node = new addFPR24_mem_ciscNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_reg_mem_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_reg_mem_0Node::cisc_version(int offset) {
  addFPR24_mem_cisc_0Node *node = new addFPR24_mem_cisc_0Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_mem_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_mem_regNode::cisc_version(int offset) {
  addFPR24_mem_ciscNode *node = new addFPR24_mem_ciscNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulFPR24_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulFPR24_regNode::cisc_version(int offset) {
  mulFPR24_reg_memNode *node = new mulFPR24_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulFPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulFPR_regNode::cisc_version(int offset) {
  mulFPR_reg_memNode *node = new mulFPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR_mulFPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR_mulFPR_regNode::cisc_version(int offset) {
  addFPR_mulFPR_reg_load1Node *node = new addFPR_mulFPR_reg_load1Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR_mulFPR_reg_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR_mulFPR_reg_0Node::cisc_version(int offset) {
  addFPR_mulFPR_reg_load1_1Node *node = new addFPR_mulFPR_reg_load1_1Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void convI2L_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *convI2L_regNode::cisc_version(int offset) {
  loadI2LNode *node = new loadI2LNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void convI2L_reg_zexNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *convI2L_reg_zexNode::cisc_version(int offset) {
  loadUI2LNode *node = new loadUI2LNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void compI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compI_eRegNode::cisc_version(int offset) {
  compI_eReg_memNode *node = new compI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void compU_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compU_eRegNode::cisc_version(int offset) {
  compU_eReg_memNode *node = new compU_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void compP_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compP_eRegNode::cisc_version(int offset) {
  compP_eReg_memNode *node = new compP_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}

// Build short branch version of this instruction
MachNode *jmpDirNode::short_branch_version() {
  jmpDir_shortNode *node = new jmpDir_shortNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConNode::short_branch_version() {
  jmpCon_shortNode *node = new jmpCon_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndNode::short_branch_version() {
  jmpLoopEnd_shortNode *node = new jmpLoopEnd_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUNode::short_branch_version() {
  jmpLoopEndU_shortNode *node = new jmpLoopEndU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUCFNode::short_branch_version() {
  jmpLoopEndUCF_shortNode *node = new jmpLoopEndUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUNode::short_branch_version() {
  jmpConU_shortNode *node = new jmpConU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCFNode::short_branch_version() {
  jmpConUCF_shortNode *node = new jmpConUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCF2Node::short_branch_version() {
  jmpConUCF2_shortNode *node = new jmpConUCF2_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}


void cmovLL_reg_LTGENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LTGENode::cisc_version(int offset) {
  cmovLL_mem_LTGENode *node = new cmovLL_mem_LTGENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_LTGENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_LTGENode::cisc_version(int offset) {
  cmovII_mem_LTGENode *node = new cmovII_mem_LTGENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_EQNENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_EQNENode::cisc_version(int offset) {
  cmovLL_mem_EQNENode *node = new cmovLL_mem_EQNENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_EQNENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_EQNENode::cisc_version(int offset) {
  cmovII_mem_EQNENode *node = new cmovII_mem_EQNENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_LEGTNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LEGTNode::cisc_version(int offset) {
  cmovLL_mem_LEGTNode *node = new cmovLL_mem_LEGTNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_LEGT_UNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LEGT_UNode::cisc_version(int offset) {
  cmovLL_mem_LEGT_UNode *node = new cmovLL_mem_LEGT_UNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_LEGTNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_LEGTNode::cisc_version(int offset) {
  cmovII_mem_LEGTNode *node = new cmovII_mem_LEGTNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addF_regNode::cisc_version(int offset) {
  addF_memNode *node = new addF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addF_reg_regNode::cisc_version(int offset) {
  addF_reg_memNode *node = new addF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addD_regNode::cisc_version(int offset) {
  addD_memNode *node = new addD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addD_reg_regNode::cisc_version(int offset) {
  addD_reg_memNode *node = new addD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subF_regNode::cisc_version(int offset) {
  subF_memNode *node = new subF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subF_reg_regNode::cisc_version(int offset) {
  subF_reg_memNode *node = new subF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subD_regNode::cisc_version(int offset) {
  subD_memNode *node = new subD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subD_reg_regNode::cisc_version(int offset) {
  subD_reg_memNode *node = new subD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulF_regNode::cisc_version(int offset) {
  mulF_memNode *node = new mulF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulF_reg_regNode::cisc_version(int offset) {
  mulF_reg_memNode *node = new mulF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulD_regNode::cisc_version(int offset) {
  mulD_memNode *node = new mulD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulD_reg_regNode::cisc_version(int offset) {
  mulD_reg_memNode *node = new mulD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divF_regNode::cisc_version(int offset) {
  divF_memNode *node = new divF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divF_reg_regNode::cisc_version(int offset) {
  divF_reg_memNode *node = new divF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divD_regNode::cisc_version(int offset) {
  divD_memNode *node = new divD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divD_reg_regNode::cisc_version(int offset) {
  divD_reg_memNode *node = new divD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void sqrtF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *sqrtF_regNode::cisc_version(int offset) {
  sqrtF_memNode *node = new sqrtF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void sqrtD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *sqrtD_regNode::cisc_version(int offset) {
  sqrtD_memNode *node = new sqrtD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void ReplI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *ReplI_regNode::cisc_version(int offset) {
  ReplI_memNode *node = new ReplI_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void ReplF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_VL_mask();
}

// Build CISC version of this instruction
MachNode *ReplF_regNode::cisc_version(int offset) {
  ReplF_memNode *node = new ReplF_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void ReplD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_VL_mask();
}

// Build CISC version of this instruction
MachNode *ReplD_regNode::cisc_version(int offset) {
  ReplD_memNode *node = new ReplD_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode(MachNode* node) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone();
  }
}

void jmpDirNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpDirNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUCFNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCFNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEnd_and_restoreMaskNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEnd_and_restoreMaskNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndU_and_restoreMaskNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndU_and_restoreMaskNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUCF_and_restoreMaskNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCF_and_restoreMaskNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCFNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCFNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF2Node::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF2Node::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpDir_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpDir_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpCon_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpCon_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEnd_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEnd_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndU_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndU_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUCF_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCF_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConU_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConU_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF2_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF2_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_LTGENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_LTGENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_LTGENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_LTGENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_EQNENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_EQNENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_EQNENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_EQNENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_LEGTNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_LEGTNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_LEGTNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_LEGTNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int popCountI_memNode::reloc() const {
  return 1;
}

int popCountL_memNode::reloc() const {
  return 1;
}

int loadBNode::reloc() const {
  return 1;
}

int loadB2LNode::reloc() const {
  return 1;
}

int loadUBNode::reloc() const {
  return 1;
}

int loadUB2LNode::reloc() const {
  return 1;
}

int loadUB2L_immINode::reloc() const {
  return 1;
}

int loadSNode::reloc() const {
  return 1;
}

int loadS2BNode::reloc() const {
  return 1;
}

int loadS2LNode::reloc() const {
  return 1;
}

int loadUSNode::reloc() const {
  return 1;
}

int loadUS2BNode::reloc() const {
  return 1;
}

int loadUS2LNode::reloc() const {
  return 1;
}

int loadUS2L_immI_255Node::reloc() const {
  return 1;
}

int loadUS2L_immINode::reloc() const {
  return 1;
}

int loadINode::reloc() const {
  return 1;
}

int loadI2BNode::reloc() const {
  return 1;
}

int loadI2UBNode::reloc() const {
  return 1;
}

int loadI2SNode::reloc() const {
  return 1;
}

int loadI2USNode::reloc() const {
  return 1;
}

int loadI2LNode::reloc() const {
  return 1;
}

int loadI2L_immI_255Node::reloc() const {
  return 1;
}

int loadI2L_immI_65535Node::reloc() const {
  return 1;
}

int loadI2L_immU31Node::reloc() const {
  return 1;
}

int loadUI2LNode::reloc() const {
  return 1;
}

int loadLNode::reloc() const {
  return 1;
}

int loadL_volatileNode::reloc() const {
  return 1;
}

int loadLX_volatileNode::reloc() const {
  return 1;
}

int loadLX_reg_volatileNode::reloc() const {
  return 1;
}

int loadRangeNode::reloc() const {
  return 1;
}

int loadPNode::reloc() const {
  return 1;
}

int loadKlassNode::reloc() const {
  return 1;
}

int loadDPRNode::reloc() const {
  return 1;
}

int loadDNode::reloc() const {
  return 1;
}

int loadD_partialNode::reloc() const {
  return 1;
}

int loadFNode::reloc() const {
  return 1;
}

int loadFPRNode::reloc() const {
  return 1;
}

int loadConPNode::reloc() const {
  return 1;
}

int loadConFPRNode::reloc() const {
  return 2;
}

int loadConFPR0Node::reloc() const {
  return 2;
}

int loadConFPR1Node::reloc() const {
  return 2;
}

int loadConFNode::reloc() const {
  return 2;
}

int loadConF0Node::reloc() const {
  return 2;
}

int loadConDPRNode::reloc() const {
  return 2;
}

int loadConDPR0Node::reloc() const {
  return 2;
}

int loadConDPR1Node::reloc() const {
  return 2;
}

int loadConDNode::reloc() const {
  return 2;
}

int loadConD0Node::reloc() const {
  return 2;
}

int prefetchAlloc0Node::reloc() const {
  return 1;
}

int prefetchAllocNode::reloc() const {
  return 1;
}

int prefetchAllocNTANode::reloc() const {
  return 1;
}

int prefetchAllocT0Node::reloc() const {
  return 1;
}

int prefetchAllocT2Node::reloc() const {
  return 1;
}

int storeBNode::reloc() const {
  return 2;
}

int storeCNode::reloc() const {
  return 2;
}

int storeINode::reloc() const {
  return 2;
}

int storeLNode::reloc() const {
  return 2;
}

int storeL2INode::reloc() const {
  return 2;
}

int storeL_volatileNode::reloc() const {
  return 2;
}

int storeLX_volatileNode::reloc() const {
  return 2;
}

int storeLX_reg_volatileNode::reloc() const {
  return 2;
}

int storePNode::reloc() const {
  return 2;
}

int storeImmINode::reloc() const {
  return 2;
}

int storeImmI16Node::reloc() const {
  return 2;
}

int storeImmPNode::reloc() const {
  return 3;
}

int storeImmBNode::reloc() const {
  return 2;
}

int storeImmCMNode::reloc() const {
  return 2;
}

int storeDPRNode::reloc() const {
  return 2;
}

int storeDPR_roundedNode::reloc() const {
  return 2;
}

int storeDNode::reloc() const {
  return 2;
}

int storeFNode::reloc() const {
  return 2;
}

int storeFPRNode::reloc() const {
  return 2;
}

int storeFPR_roundedNode::reloc() const {
  return 2;
}

int storeFPR_DroundedNode::reloc() const {
  return 2;
}

int storeFPR_immNode::reloc() const {
  return 3;
}

int storeF_immNode::reloc() const {
  return 3;
}

int cmovI_memNode::reloc() const {
  return 1;
}

int cmovI_memUNode::reloc() const {
  return 1;
}

int cmovI_memUCFNode::reloc() const {
  return 1;
}

int addI_eReg_memNode::reloc() const {
  return 1;
}

int addI_eReg_mem_0Node::reloc() const {
  return 1;
}

int addI_mem_eRegNode::reloc() const {
  return 3;
}

int addI_mem_eReg_0Node::reloc() const {
  return 3;
}

int addI_mem_immNode::reloc() const {
  return 3;
}

int incI_memNode::reloc() const {
  return 3;
}

int decI_memNode::reloc() const {
  return 3;
}

int loadPLockedNode::reloc() const {
  return 1;
}

int storePConditionalNode::reloc() const {
  return 1;
}

int storeIConditionalNode::reloc() const {
  return 1;
}

int storeLConditionalNode::reloc() const {
  return 1;
}

int xaddB_no_resNode::reloc() const {
  return 1;
}

int xaddBNode::reloc() const {
  return 1;
}

int xaddS_no_resNode::reloc() const {
  return 1;
}

int xaddSNode::reloc() const {
  return 1;
}

int xaddI_no_resNode::reloc() const {
  return 1;
}

int xaddINode::reloc() const {
  return 1;
}

int xchgBNode::reloc() const {
  return 1;
}

int xchgSNode::reloc() const {
  return 1;
}

int xchgINode::reloc() const {
  return 1;
}

int xchgPNode::reloc() const {
  return 1;
}

int subI_eReg_memNode::reloc() const {
  return 1;
}

int subI_mem_eRegNode::reloc() const {
  return 3;
}

int mulI_mem_immNode::reloc() const {
  return 1;
}

int mulINode::reloc() const {
  return 1;
}

int mulI_0Node::reloc() const {
  return 1;
}

int sarI_mem_1Node::reloc() const {
  return 3;
}

int sarI_mem_immNode::reloc() const {
  return 3;
}

int andI_eReg_memNode::reloc() const {
  return 1;
}

int andI_eReg_mem_0Node::reloc() const {
  return 1;
}

int andI_mem_eRegNode::reloc() const {
  return 3;
}

int andI_mem_eReg_0Node::reloc() const {
  return 3;
}

int andI_mem_immNode::reloc() const {
  return 3;
}

int andnI_rReg_rReg_memNode::reloc() const {
  return 1;
}

int andnI_rReg_rReg_mem_0Node::reloc() const {
  return 1;
}

int blsiI_rReg_memNode::reloc() const {
  return 2;
}

int blsiI_rReg_mem_0Node::reloc() const {
  return 2;
}

int blsmskI_rReg_memNode::reloc() const {
  return 2;
}

int blsmskI_rReg_mem_0Node::reloc() const {
  return 2;
}

int blsrI_rReg_memNode::reloc() const {
  return 2;
}

int blsrI_rReg_mem_0Node::reloc() const {
  return 2;
}

int orI_eReg_memNode::reloc() const {
  return 1;
}

int orI_eReg_mem_0Node::reloc() const {
  return 1;
}

int orI_mem_eRegNode::reloc() const {
  return 3;
}

int orI_mem_eReg_0Node::reloc() const {
  return 3;
}

int orI_mem_immNode::reloc() const {
  return 3;
}

int xorI_eReg_memNode::reloc() const {
  return 1;
}

int xorI_eReg_mem_0Node::reloc() const {
  return 1;
}

int xorI_mem_eRegNode::reloc() const {
  return 3;
}

int xorI_mem_eReg_0Node::reloc() const {
  return 3;
}

int xorI_mem_immNode::reloc() const {
  return 3;
}

int addL_eReg_memNode::reloc() const {
  return 1;
}

int addL_eReg_mem_0Node::reloc() const {
  return 1;
}

int subL_eReg_memNode::reloc() const {
  return 1;
}

int andL_eReg_memNode::reloc() const {
  return 1;
}

int andL_eReg_mem_0Node::reloc() const {
  return 1;
}

int andnL_eReg_eReg_memNode::reloc() const {
  return 1;
}

int andnL_eReg_eReg_mem_0Node::reloc() const {
  return 1;
}

int blsiL_eReg_memNode::reloc() const {
  return 2;
}

int blsiL_eReg_mem_0Node::reloc() const {
  return 2;
}

int blsmskL_eReg_memNode::reloc() const {
  return 2;
}

int blsmskL_eReg_mem_0Node::reloc() const {
  return 2;
}

int blsrL_eReg_memNode::reloc() const {
  return 2;
}

int blsrL_eReg_mem_0Node::reloc() const {
  return 2;
}

int orl_eReg_memNode::reloc() const {
  return 1;
}

int orl_eReg_mem_0Node::reloc() const {
  return 1;
}

int xorl_eReg_memNode::reloc() const {
  return 1;
}

int xorl_eReg_mem_0Node::reloc() const {
  return 1;
}

int cmpDPR_0Node::reloc() const {
  return 1;
}

int cmpD_ccmemNode::reloc() const {
  return 1;
}

int cmpD_ccmemCFNode::reloc() const {
  return 1;
}

int cmpD_regmemNode::reloc() const {
  return 1;
}

int subDPR_reg_memNode::reloc() const {
  return 1;
}

int addDPR_reg_memNode::reloc() const {
  return 1;
}

int addDPR_reg_mem_0Node::reloc() const {
  return 1;
}

int addDPR_mem_regNode::reloc() const {
  return 3;
}

int addDPR_mem_reg_0Node::reloc() const {
  return 3;
}

int addDPR_reg_imm1Node::reloc() const {
  return 1;
}

int addDPR_reg_immNode::reloc() const {
  return 1;
}

int addDPR_reg_imm_roundNode::reloc() const {
  return 1;
}

int mulDPR_reg_immNode::reloc() const {
  return 1;
}

int mulDPR_reg_memNode::reloc() const {
  return 1;
}

int mulDPR_reg_mem_0Node::reloc() const {
  return 1;
}

int mulDPR_reg_mem_ciscNode::reloc() const {
  return 1;
}

int mulDPR_reg_mem_cisc_0Node::reloc() const {
  return 1;
}

int cmpFPR_0Node::reloc() const {
  return 1;
}

int cmpF_ccmemNode::reloc() const {
  return 1;
}

int cmpF_ccmemCFNode::reloc() const {
  return 1;
}

int cmpF_regmemNode::reloc() const {
  return 1;
}

int addFPR24_reg_memNode::reloc() const {
  return 1;
}

int addFPR24_reg_mem_0Node::reloc() const {
  return 1;
}

int addFPR_reg_memNode::reloc() const {
  return 1;
}

int addFPR_reg_mem_0Node::reloc() const {
  return 1;
}

int addFPR24_mem_regNode::reloc() const {
  return 1;
}

int addFPR24_mem_ciscNode::reloc() const {
  return 2;
}

int addFPR24_mem_cisc_0Node::reloc() const {
  return 2;
}

int addFPR24_mem_memNode::reloc() const {
  return 2;
}

int addFPR24_reg_immNode::reloc() const {
  return 1;
}

int addFPR_reg_immNode::reloc() const {
  return 1;
}

int mulFPR24_reg_memNode::reloc() const {
  return 1;
}

int mulFPR24_reg_mem_0Node::reloc() const {
  return 1;
}

int mulFPR_reg_memNode::reloc() const {
  return 1;
}

int mulFPR_reg_mem_0Node::reloc() const {
  return 1;
}

int mulFPR24_mem_memNode::reloc() const {
  return 2;
}

int mulFPR24_reg_immNode::reloc() const {
  return 1;
}

int mulFPR_reg_immNode::reloc() const {
  return 1;
}

int mulFPR_reg_load1Node::reloc() const {
  return 1;
}

int mulFPR_reg_load1_0Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1_1Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1_0Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1_2Node::reloc() const {
  return 1;
}

int convI2D_memNode::reloc() const {
  return 1;
}

int convI2DPR_memNode::reloc() const {
  return 1;
}

int convI2FPR_SSF_memNode::reloc() const {
  return 1;
}

int convI2FPR_memNode::reloc() const {
  return 1;
}

int compI_eReg_memNode::reloc() const {
  return 1;
}

int testI_reg_memNode::reloc() const {
  return 1;
}

int compU_eReg_memNode::reloc() const {
  return 1;
}

int compP_eReg_immNode::reloc() const {
  return 1;
}

int compP_eReg_memNode::reloc() const {
  return 1;
}

int compP_mem_eRegNode::reloc() const {
  return 1;
}

int testP_regNode::reloc() const {
  return 1;
}

int testP_Reg_memNode::reloc() const {
  return 1;
}

int partialSubtypeCheck_vs_ZeroNode::reloc() const {
  return 1;
}

int cmovLL_mem_LTGENode::reloc() const {
  return 1;
}

int cmovII_mem_LTGENode::reloc() const {
  return 1;
}

int cmovLL_mem_EQNENode::reloc() const {
  return 1;
}

int cmovII_mem_EQNENode::reloc() const {
  return 1;
}

int cmovLL_mem_LEGTNode::reloc() const {
  return 1;
}

int cmovLL_mem_LEGT_UNode::reloc() const {
  return 1;
}

int cmovII_mem_LEGTNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectNode::reloc() const {
  return 1;
}

int CallRuntimeDirectNode::reloc() const {
  return 1;
}

int CallLeafDirectNode::reloc() const {
  return 1;
}

int CallLeafNoFPDirectNode::reloc() const {
  return 1;
}

int RetNode::reloc() const {
  return 1;
}

int TailCalljmpIndNode::reloc() const {
  return 1;
}

int tailjmpIndNode::reloc() const {
  return 1;
}

int RethrowExceptionNode::reloc() const {
  return 1;
}

int safePoint_poll_tlsNode::reloc() const {
  return 1;
}

int addF_memNode::reloc() const {
  return 1;
}

int addF_mem_0Node::reloc() const {
  return 1;
}

int addF_immNode::reloc() const {
  return 1;
}

int addF_reg_memNode::reloc() const {
  return 1;
}

int addF_reg_mem_0Node::reloc() const {
  return 1;
}

int addF_reg_immNode::reloc() const {
  return 1;
}

int addD_memNode::reloc() const {
  return 1;
}

int addD_mem_0Node::reloc() const {
  return 1;
}

int addD_immNode::reloc() const {
  return 1;
}

int addD_reg_memNode::reloc() const {
  return 1;
}

int addD_reg_mem_0Node::reloc() const {
  return 1;
}

int addD_reg_immNode::reloc() const {
  return 1;
}

int subF_memNode::reloc() const {
  return 1;
}

int subF_immNode::reloc() const {
  return 1;
}

int subF_reg_memNode::reloc() const {
  return 1;
}

int subF_reg_immNode::reloc() const {
  return 1;
}

int subD_memNode::reloc() const {
  return 1;
}

int subD_immNode::reloc() const {
  return 1;
}

int subD_reg_memNode::reloc() const {
  return 1;
}

int subD_reg_immNode::reloc() const {
  return 1;
}

int mulF_memNode::reloc() const {
  return 1;
}

int mulF_mem_0Node::reloc() const {
  return 1;
}

int mulF_immNode::reloc() const {
  return 1;
}

int mulF_reg_memNode::reloc() const {
  return 1;
}

int mulF_reg_mem_0Node::reloc() const {
  return 1;
}

int mulF_reg_immNode::reloc() const {
  return 1;
}

int mulD_memNode::reloc() const {
  return 1;
}

int mulD_mem_0Node::reloc() const {
  return 1;
}

int mulD_immNode::reloc() const {
  return 1;
}

int mulD_reg_memNode::reloc() const {
  return 1;
}

int mulD_reg_mem_0Node::reloc() const {
  return 1;
}

int mulD_reg_immNode::reloc() const {
  return 1;
}

int divF_memNode::reloc() const {
  return 1;
}

int divF_immNode::reloc() const {
  return 1;
}

int divF_reg_memNode::reloc() const {
  return 1;
}

int divF_reg_immNode::reloc() const {
  return 1;
}

int divD_memNode::reloc() const {
  return 1;
}

int divD_immNode::reloc() const {
  return 1;
}

int divD_reg_memNode::reloc() const {
  return 1;
}

int divD_reg_immNode::reloc() const {
  return 1;
}

int sqrtF_memNode::reloc() const {
  return 1;
}

int sqrtF_immNode::reloc() const {
  return 1;
}

int sqrtD_memNode::reloc() const {
  return 1;
}

int sqrtD_immNode::reloc() const {
  return 1;
}

int loadVNode::reloc() const {
  return 1;
}

int storeVNode::reloc() const {
  return 2;
}

int gatherNode::reloc() const {
  return 1;
}

int evgatherNode::reloc() const {
  return 1;
}

int scatterNode::reloc() const {
  return 2;
}

int ReplB_memNode::reloc() const {
  return 1;
}

int ReplS_memNode::reloc() const {
  return 1;
}

int ReplI_memNode::reloc() const {
  return 1;
}

int ReplL_memNode::reloc() const {
  return 1;
}

int ReplF_memNode::reloc() const {
  return 1;
}

int ReplF_zeroNode::reloc() const {
  return 1;
}

int ReplD_memNode::reloc() const {
  return 1;
}

int ReplD_zeroNode::reloc() const {
  return 1;
}

int minmax_reduction2FNode::reloc() const {
  return 1;
}

int minmax_reduction2F_0Node::reloc() const {
  return 1;
}

int minmax_reductionFNode::reloc() const {
  return 1;
}

int minmax_reductionF_0Node::reloc() const {
  return 1;
}

int minmax_reduction2DNode::reloc() const {
  return 1;
}

int minmax_reduction2D_0Node::reloc() const {
  return 1;
}

int minmax_reductionDNode::reloc() const {
  return 1;
}

int minmax_reductionD_0Node::reloc() const {
  return 1;
}

int vaddB_memNode::reloc() const {
  return 1;
}

int vaddB_mem_0Node::reloc() const {
  return 1;
}

int vaddS_memNode::reloc() const {
  return 1;
}

int vaddS_mem_0Node::reloc() const {
  return 1;
}

int vaddI_memNode::reloc() const {
  return 1;
}

int vaddI_mem_0Node::reloc() const {
  return 1;
}

int vaddL_memNode::reloc() const {
  return 1;
}

int vaddL_mem_0Node::reloc() const {
  return 1;
}

int vaddF_memNode::reloc() const {
  return 1;
}

int vaddF_mem_0Node::reloc() const {
  return 1;
}

int vaddD_memNode::reloc() const {
  return 1;
}

int vaddD_mem_0Node::reloc() const {
  return 1;
}

int vsubB_memNode::reloc() const {
  return 1;
}

int vsubS_memNode::reloc() const {
  return 1;
}

int vsubI_memNode::reloc() const {
  return 1;
}

int vsubL_memNode::reloc() const {
  return 1;
}

int vsubF_memNode::reloc() const {
  return 1;
}

int vsubD_memNode::reloc() const {
  return 1;
}

int vmulS_memNode::reloc() const {
  return 1;
}

int vmulS_mem_0Node::reloc() const {
  return 1;
}

int vmulI_memNode::reloc() const {
  return 1;
}

int vmulI_mem_0Node::reloc() const {
  return 1;
}

int vmulL_memNode::reloc() const {
  return 1;
}

int vmulL_mem_0Node::reloc() const {
  return 1;
}

int vmulF_memNode::reloc() const {
  return 1;
}

int vmulF_mem_0Node::reloc() const {
  return 1;
}

int vmulD_memNode::reloc() const {
  return 1;
}

int vmulD_mem_0Node::reloc() const {
  return 1;
}

int vdivF_memNode::reloc() const {
  return 1;
}

int vdivD_memNode::reloc() const {
  return 1;
}

int vsqrtF_memNode::reloc() const {
  return 1;
}

int vsqrtD_memNode::reloc() const {
  return 1;
}

int vand_memNode::reloc() const {
  return 1;
}

int vand_mem_0Node::reloc() const {
  return 1;
}

int vor_memNode::reloc() const {
  return 1;
}

int vor_mem_0Node::reloc() const {
  return 1;
}

int vxor_memNode::reloc() const {
  return 1;
}

int vxor_mem_0Node::reloc() const {
  return 1;
}

int vfmaF_memNode::reloc() const {
  return 1;
}

int vfmaD_memNode::reloc() const {
  return 1;
}

int vpternlog_memNode::reloc() const {
  return 1;
}

int compareAndSwapP_shenandoahNode::reloc() const {
  return 1;
}

int compareAndSwapP_shenandoah_0Node::reloc() const {
  return 1;
}

int compareAndExchangeP_shenandoahNode::reloc() const {
  return 1;
}


void bytes_reverse_intNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 14150 "ad_x86.cpp"
  }
  {

#line 1671 "src/hotspot/cpu/x86/x86_32.ad"

    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 14158 "ad_x86.cpp"
  }
}

void bytes_reverse_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 1675 "src/hotspot/cpu/x86/x86_32.ad"

    int destlo = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int desthi = HIGH_FROM_LOW(destlo);
    // bswap lo
    emit_opcode(cbuf, 0x0F);
    emit_cc(cbuf, 0xC8, destlo);
    // bswap hi
    emit_opcode(cbuf, 0x0F);
    emit_cc(cbuf, 0xC8, desthi);
    // xchg lo and hi
    emit_opcode(cbuf, 0x87);
    emit_rm(cbuf, 0x3, destlo, desthi);
  
#line 14183 "ad_x86.cpp"
  }
}

void bytes_reverse_unsigned_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4986 "src/hotspot/cpu/x86/x86_32.ad"

    __ bswapl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ shrl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 16);
  
#line 14200 "ad_x86.cpp"
  }
}

void bytes_reverse_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4999 "src/hotspot/cpu/x86/x86_32.ad"

    __ bswapl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ sarl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 16);
  
#line 14217 "ad_x86.cpp"
  }
}

void countLeadingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5015 "src/hotspot/cpu/x86/x86_32.ad"

    __ lzcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 14233 "ad_x86.cpp"
  }
}

void countLeadingZerosI_bsrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5032 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label skip;
    __ bsrl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, skip);
    __ movl(Rdst, -1);
    __ bind(skip);
    __ negl(Rdst);
    __ addl(Rdst, BitsPerInt - 1);
  
#line 14257 "ad_x86.cpp"
  }
}

void countLeadingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5056 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label done;
    __ lzcntl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ jccb(Assembler::carryClear, done);
    __ lzcntl(Rdst, Rsrc);
    __ addl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 14281 "ad_x86.cpp"
  }
}

void countLeadingZerosL_bsrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5085 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label msw_is_zero;
    Label not_zero;
    __ bsrl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ jccb(Assembler::zero, msw_is_zero);
    __ addl(Rdst, BitsPerInt);
    __ jmpb(not_zero);
    __ bind(msw_is_zero);
    __ bsrl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, not_zero);
    __ movl(Rdst, -1);
    __ bind(not_zero);
    __ negl(Rdst);
    __ addl(Rdst, BitsPerLong - 1);
  
#line 14312 "ad_x86.cpp"
  }
}

void countTrailingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5111 "src/hotspot/cpu/x86/x86_32.ad"

    __ tzcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 14328 "ad_x86.cpp"
  }
}

void countTrailingZerosI_bsfNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5126 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Label done;
    __ bsfl(Rdst, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ jccb(Assembler::notZero, done);
    __ movl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 14349 "ad_x86.cpp"
  }
}

void countTrailingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5147 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label done;
    __ tzcntl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ tzcntl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ addl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 14373 "ad_x86.cpp"
  }
}

void countTrailingZerosL_bsfNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5173 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label msw_not_zero;
    Label done;
    __ bsfl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, done);
    __ bsfl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ jccb(Assembler::notZero, msw_not_zero);
    __ movl(Rdst, BitsPerInt);
    __ bind(msw_not_zero);
    __ addl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 14401 "ad_x86.cpp"
  }
}

void popCountINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5199 "src/hotspot/cpu/x86/x86_32.ad"

    __ popcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 14417 "ad_x86.cpp"
  }
}

void popCountI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5211 "src/hotspot/cpu/x86/x86_32.ad"

    __ popcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14433 "ad_x86.cpp"
  }
}

void popCountLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5226 "src/hotspot/cpu/x86/x86_32.ad"

    __ popcntl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ popcntl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
    __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 14453 "ad_x86.cpp"
  }
}

void popCountL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5243 "src/hotspot/cpu/x86/x86_32.ad"

    //__ popcntl($dst$$Register, $mem$$Address$$first);
    //__ popcntl($tmp$$Register, $mem$$Address$$second);
    __ popcntl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), relocInfo::none));
    __ popcntl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none));
    __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 14475 "ad_x86.cpp"
  }
}

void loadBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5263 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14491 "ad_x86.cpp"
  }
}

void loadB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5280 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(0)->as_Register(ra_,this)/* dst */); // This is always a different register.
    __ sarl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), 7); // 24+1 MSB are already signed extended.
  
#line 14509 "ad_x86.cpp"
  }
}

void loadUBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5296 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14525 "ad_x86.cpp"
  }
}

void loadUB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5312 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 14543 "ad_x86.cpp"
  }
}

void loadUB2L_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5329 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
    __ andl(Rdst, opnd_array(2)->constant()& right_n_bits(8));
  
#line 14563 "ad_x86.cpp"
  }
}

void loadSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5345 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14579 "ad_x86.cpp"
  }
}

void loadS2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5358 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14596 "ad_x86.cpp"
  }
}

void loadS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5374 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(0)->as_Register(ra_,this)/* dst */); // This is always a different register.
    __ sarl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), 15); // 16+1 MSB are already signed extended.
  
#line 14614 "ad_x86.cpp"
  }
}

void loadUSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5390 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14630 "ad_x86.cpp"
  }
}

void loadUS2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5403 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14647 "ad_x86.cpp"
  }
}

void loadUS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5418 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */));
  
#line 14664 "ad_x86.cpp"
  }
}

void loadUS2L_immI_255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5433 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 14683 "ad_x86.cpp"
  }
}

void loadUS2L_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5449 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzwl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
    __ andl(Rdst, opnd_array(2)->constant()& right_n_bits(16));
  
#line 14703 "ad_x86.cpp"
  }
}

void loadINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5465 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14719 "ad_x86.cpp"
  }
}

void loadI2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5478 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14736 "ad_x86.cpp"
  }
}

void loadI2UBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5490 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14753 "ad_x86.cpp"
  }
}

void loadI2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// sixteen
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5502 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14770 "ad_x86.cpp"
  }
}

void loadI2USNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5514 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14787 "ad_x86.cpp"
  }
}

void loadI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5530 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(0)->as_Register(ra_,this)/* dst */); // This is always a different register.
    __ sarl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), 31);
  
#line 14805 "ad_x86.cpp"
  }
}

void loadI2L_immI_255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5546 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 14824 "ad_x86.cpp"
  }
}

void loadI2L_immI_65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5561 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzwl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 14843 "ad_x86.cpp"
  }
}

void loadI2L_immU31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5577 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
    __ andl(Rdst, opnd_array(2)->constant());
  
#line 14863 "ad_x86.cpp"
  }
}

void loadUI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5595 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */));
  
#line 14881 "ad_x86.cpp"
  }
}

void loadLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5613 "src/hotspot/cpu/x86/x86_32.ad"

    Address Amemlo = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), relocInfo::none);
    Address Amemhi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);
    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Amemlo);
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), Amemhi);
  
#line 14900 "ad_x86.cpp"
  }
}

void loadL_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3031 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xDF);
    int rm_byte_opcode = 0x05;
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
    store_to_stackslot( cbuf, 0x0DF, 0x07, opnd_array(0)->disp(ra_,this,0) );
  
#line 14923 "ad_x86.cpp"
  }
}

void loadLX_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5644 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdbl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
  
#line 14941 "ad_x86.cpp"
  }
}

void loadLX_reg_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5660 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
    __ psrlq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, 32);
    __ movdl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
  
#line 14961 "ad_x86.cpp"
  }
}

void loadRangeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 14976 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14990 "ad_x86.cpp"
  }
}

void loadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 15005 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15019 "ad_x86.cpp"
  }
}

void loadKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 15034 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15048 "ad_x86.cpp"
  }
}

void MoveF2LEGNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5707 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 15064 "ad_x86.cpp"
  }
}

void MoveLEG2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5717 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 15080 "ad_x86.cpp"
  }
}

void MoveD2LEGNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5727 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 15096 "ad_x86.cpp"
  }
}

void MoveLEG2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5737 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 15112 "ad_x86.cpp"
  }
}

void loadDPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*primary()*/));
  
#line 15127 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 15141 "ad_x86.cpp"
  }
  {

#line 2345 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 15150 "ad_x86.cpp"
  }
}

void loadDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5763 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 15166 "ad_x86.cpp"
  }
}

void loadD_partialNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5774 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 15182 "ad_x86.cpp"
  }
}

void loadFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5787 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 15198 "ad_x86.cpp"
  }
}

void loadFPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 15213 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 15227 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 15236 "ad_x86.cpp"
  }
}

void leaP8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 15251 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15265 "ad_x86.cpp"
  }
}

void leaP32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 15280 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15294 "ad_x86.cpp"
  }
}

void leaPIdxOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 15309 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15323 "ad_x86.cpp"
  }
}

void leaPIdxScaleNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 15338 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15352 "ad_x86.cpp"
  }
}

void leaPIdxScaleOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 15367 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15381 "ad_x86.cpp"
  }
}

void loadConINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1883 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    emit_opcode(cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32(cbuf, opnd_array(1)->constant());
  
#line 15399 "ad_x86.cpp"
  }
}

void loadConI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 15414 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 15422 "ad_x86.cpp"
  }
}

void loadConPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1890 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    emit_opcode(cbuf, (0xB8 /*primary()*/) + opnd_array(0)->reg(ra_,this)/* dst */);
    
if ( opnd_array(1)->constant_reloc() != relocInfo::none ) {
  emit_d32_reloc(cbuf, opnd_array(1)->constant(), opnd_array(1)->constant_reloc(), 0);
} else {
  emit_d32(cbuf, opnd_array(1)->constant());
};
  
#line 15445 "ad_x86.cpp"
  }
}

void loadConLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1897 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this)/* dst */;
    int src_con = opnd_array(1)->constantL() & 0x0FFFFFFFFL;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  
#line 15471 "ad_x86.cpp"
  }
  {

#line 1912 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this)/* dst */ + 2;
    int src_con = ((julong)(opnd_array(1)->constantL())) >> 32;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  
#line 15490 "ad_x86.cpp"
  }
}

void loadConL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1941 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 15506 "ad_x86.cpp"
  }
  {

#line 1946 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 15515 "ad_x86.cpp"
  }
}

void loadConFPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5916 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 15532 "ad_x86.cpp"
  }
}

void loadConFPRNode::eval_constant(Compile* C) {
  {

#line 5917 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 15541 "ad_x86.cpp"
  }
}
void loadConFPR0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5929 "src/hotspot/cpu/x86/x86_32.ad"

    __ fldz();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 15557 "ad_x86.cpp"
  }
}

void loadConFPR1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5942 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld1();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 15574 "ad_x86.cpp"
  }
}

void loadConFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5954 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 15590 "ad_x86.cpp"
  }
}

void loadConFNode::eval_constant(Compile* C) {
  {

#line 5955 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 15599 "ad_x86.cpp"
  }
}
void loadConF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5965 "src/hotspot/cpu/x86/x86_32.ad"

    __ xorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 15614 "ad_x86.cpp"
  }
}

void loadConDPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5978 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 15631 "ad_x86.cpp"
  }
}

void loadConDPRNode::eval_constant(Compile* C) {
  {

#line 5979 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 15640 "ad_x86.cpp"
  }
}
void loadConDPR0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5992 "src/hotspot/cpu/x86/x86_32.ad"

    __ fldz();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 15656 "ad_x86.cpp"
  }
}

void loadConDPR1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6006 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld1();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 15673 "ad_x86.cpp"
  }
}

void loadConDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6018 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 15689 "ad_x86.cpp"
  }
}

void loadConDNode::eval_constant(Compile* C) {
  {

#line 6019 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 15698 "ad_x86.cpp"
  }
}
void loadConD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6029 "src/hotspot/cpu/x86/x86_32.ad"

    __ xorpd (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 15713 "ad_x86.cpp"
  }
}

void loadSSINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 15728 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15742 "ad_x86.cpp"
  }
}

void loadSSLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 15757 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15771 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*secondary()*/));
  
#line 15779 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 15793 "ad_x86.cpp"
  }
}

void loadSSPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 15808 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15822 "ad_x86.cpp"
  }
}

void loadSSFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 15837 "ad_x86.cpp"
  }
  {

#line 2154 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 15851 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 15860 "ad_x86.cpp"
  }
}

void loadSSDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*primary()*/));
  
#line 15875 "ad_x86.cpp"
  }
  {

#line 2154 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 15889 "ad_x86.cpp"
  }
  {

#line 2345 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 15898 "ad_x86.cpp"
  }
}

void prefetchAlloc0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  // User did not define which encode class to use.
}

uint prefetchAlloc0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void prefetchAllocNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6113 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetchw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 15927 "ad_x86.cpp"
  }
}

void prefetchAllocNTANode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6125 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetchnta(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 15943 "ad_x86.cpp"
  }
}

void prefetchAllocT0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6137 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetcht0(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 15959 "ad_x86.cpp"
  }
}

void prefetchAllocT2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6149 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetcht2(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 15975 "ad_x86.cpp"
  }
}

void storeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x88 /*primary()*/));
  
#line 15991 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16005 "ad_x86.cpp"
  }
}

void storeCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x66 /*secondary()*/));
  
#line 16021 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 16029 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16043 "ad_x86.cpp"
  }
}

void storeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 16059 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16073 "ad_x86.cpp"
  }
}

void storeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 16089 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16103 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*secondary()*/));
  
#line 16111 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 16125 "ad_x86.cpp"
  }
}

void storeL2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6208 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
  
#line 16142 "ad_x86.cpp"
  }
}

void storeL_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 16158 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = (EAX_enc);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16172 "ad_x86.cpp"
  }
  {

#line 3047 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0x0DF, 0x05, opnd_array(2)->disp(ra_,this,idx2) );
    cbuf.set_insts_mark();            // Mark start of FIST in case $mem has an oop
    emit_opcode(cbuf,0xDF);
    int rm_byte_opcode = 0x07;
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 16189 "ad_x86.cpp"
  }
}

void storeLX_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6239 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(rax, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, Address(rsp, opnd_array(2)->disp(ra_,this,idx2)));
    __ movdbl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 16209 "ad_x86.cpp"
  }
}

void storeLX_reg_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6257 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(rax, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdl(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp2 */, HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* src */));
    __ punpckldq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp2 */);
    __ movdbl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
  
#line 16232 "ad_x86.cpp"
  }
}

void storePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 16248 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16262 "ad_x86.cpp"
  }
}

void storeImmINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 16278 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 16292 "ad_x86.cpp"
  }
  {

#line 1963 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  
#line 16301 "ad_x86.cpp"
  }
}

void storeImmI16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1550 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0x66);
  
#line 16317 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 16325 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 16339 "ad_x86.cpp"
  }
  {

#line 1982 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    emit_d16(cbuf, opnd_array(2)->constant());
  
#line 16348 "ad_x86.cpp"
  }
}

void storeImmPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 16364 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 16378 "ad_x86.cpp"
  }
  {

#line 1963 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    
if ( opnd_array(2)->constant_reloc() != relocInfo::none ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), opnd_array(2)->constant_reloc(), 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
  
#line 16392 "ad_x86.cpp"
  }
}

void storeImmBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC6 /*primary()*/));
  
#line 16408 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 16422 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 16437 "ad_x86.cpp"
  }
}

void storeImmCMNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC6 /*primary()*/));
  
#line 16453 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 16467 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 16482 "ad_x86.cpp"
  }
}

void storeDPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2208 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xDD /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16513 "ad_x86.cpp"
  }
}

void storeDPR_roundedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2208 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xDD /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16544 "ad_x86.cpp"
  }
}

void storeDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6366 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 16561 "ad_x86.cpp"
  }
}

void MoveD2VLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6376 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 16577 "ad_x86.cpp"
  }
}

void MoveVL2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6386 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 16593 "ad_x86.cpp"
  }
}

void storeFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6399 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 16610 "ad_x86.cpp"
  }
}

void MoveF2VLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6409 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 16626 "ad_x86.cpp"
  }
}

void MoveVL2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6419 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 16642 "ad_x86.cpp"
  }
}

void storeFPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2208 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16673 "ad_x86.cpp"
  }
}

void storeFPR_roundedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2208 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16704 "ad_x86.cpp"
  }
}

void storeFPR_DroundedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2208 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16735 "ad_x86.cpp"
  }
}

void storeFPR_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 16751 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 16765 "ad_x86.cpp"
  }
  {

#line 1968 "src/hotspot/cpu/x86/x86_32.ad"

    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    int    jf_as_bits = jint_cast( jf );
    emit_d32(cbuf, jf_as_bits);
  
#line 16776 "ad_x86.cpp"
  }
}

void storeF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 16792 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 16806 "ad_x86.cpp"
  }
  {

#line 1975 "src/hotspot/cpu/x86/x86_32.ad"

    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    int    jf_as_bits = jint_cast( jf );
    emit_d32(cbuf, jf_as_bits);
  
#line 16817 "ad_x86.cpp"
  }
}

void storeSSINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0x89 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(0)->disp(ra_,this,0) );
  
#line 16832 "ad_x86.cpp"
  }
}

void storeSSPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0x89 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(0)->disp(ra_,this,0) );
  
#line 16847 "ad_x86.cpp"
  }
}

void storeSSLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 16862 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0);
    relocInfo::relocType disp_reloc = opnd_array(0)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16876 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*secondary()*/));
  
#line 16884 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */);  // Hi register of pair, computed from lo
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0) + 4;      // Offset is 4 further in memory
    assert( opnd_array(0)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 16898 "ad_x86.cpp"
  }
}

void membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquireNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_acquire_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquire_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_acquire_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquire_lockNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_releaseNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_release_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_release_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_release_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_release_lockNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6573 "src/hotspot/cpu/x86/x86_32.ad"

    __ membar(Assembler::StoreLoad);
  
#line 16992 "ad_x86.cpp"
  }
}

void unnecessary_membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint unnecessary_membar_volatileNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_storestoreNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_storestoreNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castX2PNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // User did not define which encode class to use.
}

void castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1929 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 17041 "ad_x86.cpp"
  }
}

void jmovI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6626 "src/hotspot/cpu/x86/x86_32.ad"

    Label Lskip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), Lskip);
    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* src */);
    __ bind(Lskip);
  
#line 17064 "ad_x86.cpp"
  }
}

void jmovI_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6643 "src/hotspot/cpu/x86/x86_32.ad"

    Label Lskip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), Lskip);
    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* src */);
    __ bind(Lskip);
  
#line 17087 "ad_x86.cpp"
  }
}

void cmovI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17106 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 17114 "ad_x86.cpp"
  }
}

void cmovI_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17133 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 17141 "ad_x86.cpp"
  }
}

void cmovI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17160 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17174 "ad_x86.cpp"
  }
}

void cmovI_memUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17193 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17207 "ad_x86.cpp"
  }
}

void cmovP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17226 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 17234 "ad_x86.cpp"
  }
}

void cmovP_reg_nonP6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1705 "src/hotspot/cpu/x86/x86_32.ad"

    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x2) );
  
#line 17254 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8b /*primary()*/));
  
#line 17262 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 17270 "ad_x86.cpp"
  }
}

void cmovP_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17289 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 17297 "ad_x86.cpp"
  }
}

void fcmovDPR_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1698 "src/hotspot/cpu/x86/x86_32.ad"

    int op = 0xDA00 + opnd_array(1)->ccode() + (opnd_array(4)->reg(ra_,this,idx4)/* src */-1);
    emit_d8(cbuf, op >> 8 );
    emit_d8(cbuf, op & 255);
  
#line 17317 "ad_x86.cpp"
  }
}

void fcmovFPR_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1698 "src/hotspot/cpu/x86/x86_32.ad"

    int op = 0xDA00 + opnd_array(1)->ccode() + (opnd_array(4)->reg(ra_,this,idx4)/* src */-1);
    emit_d8(cbuf, op >> 8 );
    emit_d8(cbuf, op & 255);
  
#line 17337 "ad_x86.cpp"
  }
}

void fcmovDPR_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1705 "src/hotspot/cpu/x86/x86_32.ad"

    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x4) );
  
#line 17357 "ad_x86.cpp"
  }
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(4)->reg(ra_,this,idx4)/* src */ );   // FLD ST(i-1)
  
#line 17366 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xdd /*primary()*/));
  
#line 17374 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(3)->reg(ra_,this,idx3)/* dst */ );
  
#line 17382 "ad_x86.cpp"
  }
}

void fcmovFPR_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1705 "src/hotspot/cpu/x86/x86_32.ad"

    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x4) );
  
#line 17402 "ad_x86.cpp"
  }
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(4)->reg(ra_,this,idx4)/* src */ );
  
#line 17411 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xdd /*primary()*/));
  
#line 17419 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(3)->reg(ra_,this,idx3)/* dst */ );
  
#line 17427 "ad_x86.cpp"
  }
}

void fcmovF_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6842 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 17450 "ad_x86.cpp"
  }
}

void fcmovD_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6860 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 17473 "ad_x86.cpp"
  }
}

void fcmovF_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6878 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 17496 "ad_x86.cpp"
  }
}

void fcmovD_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6905 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 17519 "ad_x86.cpp"
  }
}

void cmovL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17538 "ad_x86.cpp"
  }
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 17546 "ad_x86.cpp"
  }
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17555 "ad_x86.cpp"
  }
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 17563 "ad_x86.cpp"
  }
}

void cmovL_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17582 "ad_x86.cpp"
  }
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 17590 "ad_x86.cpp"
  }
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17599 "ad_x86.cpp"
  }
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 17607 "ad_x86.cpp"
  }
}

void addI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 17623 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 17631 "ad_x86.cpp"
  }
}

uint addI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void addI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1626 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 17660 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 17675 "ad_x86.cpp"
  }
}

void incI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1610 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x40 /*primary()*/) + opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 17691 "ad_x86.cpp"
  }
}

uint incI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 1, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 1);
}

void leaI_eReg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 17712 "ad_x86.cpp"
  }
  {

#line 2174 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base         = opnd_array(1)->reg(ra_,this,idx1)/* src0 */;      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    relocInfo::relocType disp_reloc = relocInfo::none;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17726 "ad_x86.cpp"
  }
}

void leaP_eReg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 17742 "ad_x86.cpp"
  }
  {

#line 2174 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base         = opnd_array(1)->reg(ra_,this,idx1)/* src0 */;      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    relocInfo::relocType disp_reloc = relocInfo::none;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17756 "ad_x86.cpp"
  }
}

void decI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1610 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x48 /*primary()*/) + opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 17772 "ad_x86.cpp"
  }
}

uint decI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 1, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 1);
}

void addP_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 17793 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 17801 "ad_x86.cpp"
  }
}

uint addP_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void addP_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1626 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 17830 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 17845 "ad_x86.cpp"
  }
}

void addI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 17861 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17875 "ad_x86.cpp"
  }
}

void addI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 17891 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17905 "ad_x86.cpp"
  }
}

void addI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 17921 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17935 "ad_x86.cpp"
  }
}

void addI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 17951 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17965 "ad_x86.cpp"
  }
}

void addI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1616 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 17987 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18001 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 18016 "ad_x86.cpp"
  }
}

void incI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 18032 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18046 "ad_x86.cpp"
  }
}

void decI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 18062 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x01);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18076 "ad_x86.cpp"
  }
}

void checkCastPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint checkCastPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void castIINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void castLLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void loadPLockedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 18128 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18142 "ad_x86.cpp"
  }
}

void storePConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// heap_top_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 1997 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xF0);         // [Lock]
  
#line 18159 "ad_x86.cpp"
  }
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 18167 "ad_x86.cpp"
  }
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xB1));
  
#line 18175 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18189 "ad_x86.cpp"
  }
}

void storeIConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 1997 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xF0);         // [Lock]
  
#line 18206 "ad_x86.cpp"
  }
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 18214 "ad_x86.cpp"
  }
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xB1));
  
#line 18222 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18236 "ad_x86.cpp"
  }
}

void storeLConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7177 "src/hotspot/cpu/x86/x86_32.ad"

    // Note: we need to swap rbx, and rcx before and after the
    //       cmpxchg8 instruction because the instruction uses
    //       rcx as the high order word of the new value to store but
    //       our register encoding uses rbx.
    __ xchgl(as_Register(EBX_enc), as_Register(ECX_enc));
    __ lock();
    __ cmpxchg8(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xchgl(as_Register(EBX_enc), as_Register(ECX_enc));
  
#line 18261 "ad_x86.cpp"
  }
}

void compareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2006 "src/hotspot/cpu/x86/x86_32.ad"


    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
    // [Lock]
    emit_opcode(cbuf,0xF0);
    // CMPXCHG8 [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xC7);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
  
#line 18290 "ad_x86.cpp"
  }
  {

#line 2055 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 18309 "ad_x86.cpp"
  }
}

void compareAndSwapL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2006 "src/hotspot/cpu/x86/x86_32.ad"


    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
    // [Lock]
    emit_opcode(cbuf,0xF0);
    // CMPXCHG8 [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xC7);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
  
#line 18338 "ad_x86.cpp"
  }
  {

#line 2055 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 18357 "ad_x86.cpp"
  }
}

void compareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18380 "ad_x86.cpp"
  }
  {

#line 2055 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 18399 "ad_x86.cpp"
  }
}

void compareAndSwapP_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18422 "ad_x86.cpp"
  }
  {

#line 2055 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 18441 "ad_x86.cpp"
  }
}

void compareAndSwapBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2032 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHGB [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB0);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18464 "ad_x86.cpp"
  }
  {

#line 2055 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 18483 "ad_x86.cpp"
  }
}

void compareAndSwapB_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2032 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHGB [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB0);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18506 "ad_x86.cpp"
  }
  {

#line 2055 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 18525 "ad_x86.cpp"
  }
}

void compareAndSwapSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2042 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // 16-bit mode
    emit_opcode(cbuf, 0x66);

    // CMPXCHGW [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18551 "ad_x86.cpp"
  }
  {

#line 2055 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 18570 "ad_x86.cpp"
  }
}

void compareAndSwapS_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2042 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // 16-bit mode
    emit_opcode(cbuf, 0x66);

    // CMPXCHGW [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18596 "ad_x86.cpp"
  }
  {

#line 2055 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 18615 "ad_x86.cpp"
  }
}

void compareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18638 "ad_x86.cpp"
  }
  {

#line 2055 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 18657 "ad_x86.cpp"
  }
}

void compareAndSwapI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18680 "ad_x86.cpp"
  }
  {

#line 2055 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 18699 "ad_x86.cpp"
  }
}

void compareAndExchangeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2006 "src/hotspot/cpu/x86/x86_32.ad"


    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
    // [Lock]
    emit_opcode(cbuf,0xF0);
    // CMPXCHG8 [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xC7);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
  
#line 18728 "ad_x86.cpp"
  }
}

void compareAndExchangePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18751 "ad_x86.cpp"
  }
}

void compareAndExchangeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2032 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHGB [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB0);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18774 "ad_x86.cpp"
  }
}

void compareAndExchangeSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2042 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // 16-bit mode
    emit_opcode(cbuf, 0x66);

    // CMPXCHGW [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18800 "ad_x86.cpp"
  }
}

void compareAndExchangeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 18823 "ad_x86.cpp"
  }
}

void xaddB_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7307 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ addb(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->constant());
  
#line 18841 "ad_x86.cpp"
  }
}

void xaddBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7319 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ xaddb(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 18859 "ad_x86.cpp"
  }
}

void xaddS_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7331 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ addw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->constant());
  
#line 18877 "ad_x86.cpp"
  }
}

void xaddSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7342 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ xaddw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 18895 "ad_x86.cpp"
  }
}

void xaddI_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7354 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ addl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->constant());
  
#line 18913 "ad_x86.cpp"
  }
}

void xaddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7365 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ xaddl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 18931 "ad_x86.cpp"
  }
}

void xchgBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7376 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgb(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 18948 "ad_x86.cpp"
  }
}

void xchgSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7385 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgw(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 18965 "ad_x86.cpp"
  }
}

void xchgINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7394 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgl(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 18982 "ad_x86.cpp"
  }
}

void xchgPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7403 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgl(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 18999 "ad_x86.cpp"
  }
}

void subI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 19015 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 19023 "ad_x86.cpp"
  }
}

uint subI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void subI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1626 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 19052 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 19067 "ad_x86.cpp"
  }
}

void subI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 19083 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19097 "ad_x86.cpp"
  }
}

void subI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x29 /*primary()*/));
  
#line 19113 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19127 "ad_x86.cpp"
  }
}

void subP_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 19144 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(3)->reg(ra_,this,idx3)/* src */);
  
#line 19152 "ad_x86.cpp"
  }
}

uint subP_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void negI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19173 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x03 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 19181 "ad_x86.cpp"
  }
}

uint negI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void mulI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 19202 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xAF /*primary()*/));
  
#line 19210 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 19218 "ad_x86.cpp"
  }
}

uint mulI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void mulI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {

#line 1616 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 19245 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 19253 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 19268 "ad_x86.cpp"
  }
}

void loadConL_low_onlyNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1897 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this)/* dst */;
    int src_con = opnd_array(1)->constantL() & 0x0FFFFFFFFL;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  
#line 19294 "ad_x86.cpp"
  }
}

void mulI_imm_highNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2696 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 19320 "ad_x86.cpp"
  }
}

void mulI_imm_high_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2696 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 19346 "ad_x86.cpp"
  }
}

void mulI_imm_RShift_highNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2696 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 19372 "ad_x86.cpp"
  }
}

void mulI_imm_RShift_high_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2696 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 19398 "ad_x86.cpp"
  }
}

void mulI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {

#line 1616 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 19420 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19434 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 19449 "ad_x86.cpp"
  }
}

void mulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 19465 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xAF /*primary()*/));
  
#line 19473 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19487 "ad_x86.cpp"
  }
}

void mulI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 19503 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xAF /*primary()*/));
  
#line 19511 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19525 "ad_x86.cpp"
  }
}

void mulI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2722 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: long = (long)int * (long)int
    // IMUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 19544 "ad_x86.cpp"
  }
}

void mulI2L_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2722 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: long = (long)int * (long)int
    // IMUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
  
#line 19563 "ad_x86.cpp"
  }
}

void mulIS_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 2729 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic Idea:  long = (int & 0xffffffffL) * (int & 0xffffffffL)
    // MUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 19583 "ad_x86.cpp"
  }
}

void mulIS_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  {

#line 2729 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic Idea:  long = (int & 0xffffffffL) * (int & 0xffffffffL)
    // MUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
  
#line 19603 "ad_x86.cpp"
  }
}

void mulL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2736 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: lo(result) = lo(x_lo * y_lo)
    //             hi(result) = hi(x_lo * y_lo) + lo(x_hi * y_lo) + lo(x_lo * y_hi)
    // MOV    $tmp,$src.lo
    encode_Copy( cbuf, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    // IMUL   $tmp,EDX
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0xAF );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // MOV    EDX,$src.hi
    encode_Copy( cbuf, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */) );
    // IMUL   EDX,EAX
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0xAF );
    emit_rm( cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    // ADD    $tmp,EDX
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // MUL   EDX:EAX,$src.lo
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    // ADD    EDX,ESI
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(3)->reg(ra_,this,idx3)/* tmp */ );
  
#line 19642 "ad_x86.cpp"
  }
}

void mulL_eReg_lhi0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7639 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* src */));
    __ imull(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, rax);
    __ mull(opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ addl(rdx, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 19663 "ad_x86.cpp"
  }
}

void mulL_eReg_rhi0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7660 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ imull(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, rdx);
    __ mull(opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ addl(rdx, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 19684 "ad_x86.cpp"
  }
}

void mulL_eReg_hi0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7678 "src/hotspot/cpu/x86/x86_32.ad"

    __ mull(opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
  
#line 19701 "ad_x86.cpp"
  }
}

void mulL_eReg_conNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2762 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: lo(result) = lo(src * y_lo)
    //             hi(result) = hi(src * y_lo) + lo(src * y_hi)
    // IMUL   $tmp,EDX,$src
    emit_opcode( cbuf, 0x6B );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    emit_d8( cbuf, (int)opnd_array(2)->constantL() );
    // MOV    EDX,$src
    emit_opcode(cbuf, 0xB8 + EDX_enc);
    emit_d32( cbuf, (int)opnd_array(2)->constantL() );
    // MUL   EDX:EAX,EDX
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, EDX_enc );
    // ADD    EDX,ESI
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, EDX_enc, opnd_array(3)->reg(ra_,this,idx3)/* tmp */ );
  
#line 19732 "ad_x86.cpp"
  }
}

uint mulL_eReg_conNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void divI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rax
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div
  {

#line 1568 "src/hotspot/cpu/x86/x86_32.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  
#line 19790 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19798 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ );
  
#line 19806 "ad_x86.cpp"
  }
}

uint divI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 26);
}

void divL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2780 "src/hotspot/cpu/x86/x86_32.ad"

    // PUSH src1.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // PUSH src1.lo
    emit_opcode(cbuf,               0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */  );
    // PUSH src2.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // PUSH src2.lo
    emit_opcode(cbuf,               0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */  );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (CAST_FROM_FN_PTR(address, SharedRuntime::ldiv) - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Restore stack
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 4*4);
  
#line 19842 "ad_x86.cpp"
  }
}

void divModI_eReg_divmodNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// div
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rdx
  {

#line 1568 "src/hotspot/cpu/x86/x86_32.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  
#line 19895 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19903 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ );
  
#line 19911 "ad_x86.cpp"
  }
}

uint divModI_eReg_divmodNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 26);
}

void modI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rax
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div
  {

#line 1568 "src/hotspot/cpu/x86/x86_32.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  
#line 19969 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19977 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ );
  
#line 19985 "ad_x86.cpp"
  }
}

uint modI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 26);
}

void modL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2799 "src/hotspot/cpu/x86/x86_32.ad"

    // PUSH src1.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // PUSH src1.lo
    emit_opcode(cbuf,               0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */  );
    // PUSH src2.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // PUSH src2.lo
    emit_opcode(cbuf,               0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */  );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (CAST_FROM_FN_PTR(address, SharedRuntime::lrem ) - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Restore stack
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 4*4);
  
#line 20021 "ad_x86.cpp"
  }
}

void divL_eReg_imm32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7809 "src/hotspot/cpu/x86/x86_32.ad"

    int con = (int)opnd_array(2)->constantL();
    assert(con != 0 && con != -1 && con != min_jint, "wrong divisor");
    int pcon = (con > 0) ? con : -con;
    Label Lfast, Lpos, Ldone;

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, pcon);
    __ xorl(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */,opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ cmpl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ jccb(Assembler::above, Lfast); // result fits into 32 bit

    __ movl(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* dst */); // save
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ movl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),0); // preserve flags
    __ jccb(Assembler::lessEqual, Lpos); // result is positive

    // Negative dividend.
    // convert value to positive to use unsigned division
    __ lneg(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ xchgl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    // revert result back to negative
    __ lneg(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ jmpb(Ldone);

    __ bind(Lpos);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */); // Use unsigned division
    __ xchgl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    // Fallthrow for final divide, tmp2 has 32 bit hi result

    __ bind(Lfast);
    // fast path: src is positive
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */); // Use unsigned division

    __ bind(Ldone);
    __ movl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    if (con < 0) {
      __ lneg(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */), opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    }
  
#line 20078 "ad_x86.cpp"
  }
}

void modL_eReg_imm32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7879 "src/hotspot/cpu/x86/x86_32.ad"

    int con = (int)opnd_array(2)->constantL();
    assert(con != 0 && con != -1 && con != min_jint, "wrong divisor");
    int pcon = (con > 0) ? con : -con;
    Label  Lfast, Lpos, Ldone;

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, pcon);
    __ cmpl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ jccb(Assembler::above, Lfast); // src is positive and result fits into 32 bit

    __ movl(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* dst */); // save
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ movl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),0); // preserve flags
    __ jccb(Assembler::lessEqual, Lpos); // result is positive

    // Negative dividend.
    // convert value to positive to use unsigned division
    __ lneg(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    // revert remainder back to negative
    __ negl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ jmpb(Ldone);

    __ bind(Lpos);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);

    __ bind(Lfast);
    // fast path: src is positive
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);

    __ bind(Ldone);
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ sarl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */), 31); // result sign

  
#line 20132 "ad_x86.cpp"
  }
}

void shlI_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20148 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20156 "ad_x86.cpp"
  }
}

uint shlI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void salI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1877 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 20179 "ad_x86.cpp"
  }
}

uint salI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void salI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20200 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20208 "ad_x86.cpp"
  }
}

uint salI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void sarI_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20229 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20237 "ad_x86.cpp"
  }
}

uint sarI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void sarI_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20258 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 20272 "ad_x86.cpp"
  }
}

void sarI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1877 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 20290 "ad_x86.cpp"
  }
}

uint sarI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void sarI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 20311 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 20325 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20340 "ad_x86.cpp"
  }
}

void sarI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20356 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20364 "ad_x86.cpp"
  }
}

uint sarI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void shrI_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20385 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20393 "ad_x86.cpp"
  }
}

uint shrI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void shrI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1877 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 20416 "ad_x86.cpp"
  }
}

uint shrI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void i2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8046 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 20438 "ad_x86.cpp"
  }
}

uint i2bNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void i2sNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// sixteen
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8059 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 20460 "ad_x86.cpp"
  }
}

uint i2sNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void shrI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20481 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20489 "ad_x86.cpp"
  }
}

uint shrI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void andI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 20510 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 20518 "ad_x86.cpp"
  }
}

uint andI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void andI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1626 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 20547 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20562 "ad_x86.cpp"
  }
}

void andI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 20578 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20592 "ad_x86.cpp"
  }
}

void andI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 20608 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20622 "ad_x86.cpp"
  }
}

void andI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 20638 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20652 "ad_x86.cpp"
  }
}

void andI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 20668 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20682 "ad_x86.cpp"
  }
}

void andI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1616 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 20704 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 20718 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20733 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8151 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* src2 */);
  
#line 20751 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8151 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */, opnd_array(1)->as_Register(ra_,this,idx1)/* src2 */);
  
#line 20769 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8165 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()));
  
#line 20787 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8165 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 20805 "ad_x86.cpp"
  }
}

void blsiI_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8178 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
  
#line 20822 "ad_x86.cpp"
  }
}

void blsiI_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8178 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 20839 "ad_x86.cpp"
  }
}

void blsiI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8192 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 20856 "ad_x86.cpp"
  }
}

void blsiI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8192 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 20873 "ad_x86.cpp"
  }
}

void blsmskI_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8206 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 20890 "ad_x86.cpp"
  }
}

void blsmskI_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8206 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 20907 "ad_x86.cpp"
  }
}

void blsmskI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8222 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 20924 "ad_x86.cpp"
  }
}

void blsmskI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8222 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 20941 "ad_x86.cpp"
  }
}

void blsrI_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8237 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 20958 "ad_x86.cpp"
  }
}

void blsrI_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8237 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 20975 "ad_x86.cpp"
  }
}

void blsrI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8253 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 20992 "ad_x86.cpp"
  }
}

void blsrI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8253 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 21009 "ad_x86.cpp"
  }
}

void orI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 21025 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 21033 "ad_x86.cpp"
  }
}

uint orI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void orI_eReg_castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 21054 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 21062 "ad_x86.cpp"
  }
}

uint orI_eReg_castP2XNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void orI_eReg_castP2X_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 21083 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 21091 "ad_x86.cpp"
  }
}

uint orI_eReg_castP2X_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void orI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1626 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 21120 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 21135 "ad_x86.cpp"
  }
}

void orI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 21151 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21165 "ad_x86.cpp"
  }
}

void orI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 21181 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21195 "ad_x86.cpp"
  }
}

void orI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 21211 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21225 "ad_x86.cpp"
  }
}

void orI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 21241 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21255 "ad_x86.cpp"
  }
}

void orI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1616 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 21277 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x1 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 21291 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 21306 "ad_x86.cpp"
  }
}

void rolI_eReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21322 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 21330 "ad_x86.cpp"
  }
}

void rolI_eReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1877 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 21348 "ad_x86.cpp"
  }
}

void rolI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21364 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 21372 "ad_x86.cpp"
  }
}

void rorI_eReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21388 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 21396 "ad_x86.cpp"
  }
}

void rorI_eReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1877 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 21414 "ad_x86.cpp"
  }
}

void rorI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21430 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 21438 "ad_x86.cpp"
  }
}

void xorI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 21454 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 21462 "ad_x86.cpp"
  }
}

uint xorI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void xorI_eReg_im1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8486 "src/hotspot/cpu/x86/x86_32.ad"

     __ notl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
  
#line 21484 "ad_x86.cpp"
  }
}

uint xorI_eReg_im1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void xorI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1626 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 21513 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 21528 "ad_x86.cpp"
  }
}

void xorI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 21544 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21558 "ad_x86.cpp"
  }
}

void xorI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 21574 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21588 "ad_x86.cpp"
  }
}

void xorI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 21604 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21618 "ad_x86.cpp"
  }
}

void xorI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 21634 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21648 "ad_x86.cpp"
  }
}

void xorI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1616 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 21670 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x6 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 21684 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 21699 "ad_x86.cpp"
  }
}

void movI_nocopyNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1929 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 21714 "ad_x86.cpp"
  }
}

void ci2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2227 "src/hotspot/cpu/x86/x86_32.ad"

    // NEG $dst
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0x03, opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 21732 "ad_x86.cpp"
  }
  {

#line 1558 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,(0x13));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 21741 "ad_x86.cpp"
  }
}

uint ci2bNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void movP_nocopyNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1929 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 21761 "ad_x86.cpp"
  }
}

void cp2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2227 "src/hotspot/cpu/x86/x86_32.ad"

    // NEG $dst
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0x03, opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 21779 "ad_x86.cpp"
  }
  {

#line 1558 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,(0x13));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 21788 "ad_x86.cpp"
  }
}

void cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8604 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Rd = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Label done;
    __ xorl(Rd, Rd);
    __ cmpl(Rp, Rq);
    __ setb(Assembler::less, Rd);
    __ negl(Rd);
  
#line 21812 "ad_x86.cpp"
  }
}

void cmpLTMask0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8624 "src/hotspot/cpu/x86/x86_32.ad"

  __ sarl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 31);
  
#line 21829 "ad_x86.cpp"
  }
}

void cadd_cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8639 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 21854 "ad_x86.cpp"
  }
}

void cadd_cmpLTMask_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8639 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 21879 "ad_x86.cpp"
  }
}

void cadd_cmpLTMask_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// y
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// p
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8639 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(2)->as_Register(ra_,this,idx2)/* p */;
    Register Rq = opnd_array(3)->as_Register(ra_,this,idx3)/* q */;
    Register Ry = opnd_array(1)->as_Register(ra_,this,idx1)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 21904 "ad_x86.cpp"
  }
}

void cadd_cmpLTMask_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8639 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 21929 "ad_x86.cpp"
  }
}

void and_cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8664 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ cmpl(Rp, Rq);
    __ jccb(Assembler::less, done);
    __ xorl(Ry, Ry);
    __ bind(done);
  
#line 21954 "ad_x86.cpp"
  }
}

void and_cmpLTMask_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// y
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// p
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8664 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(2)->as_Register(ra_,this,idx2)/* p */;
    Register Rq = opnd_array(3)->as_Register(ra_,this,idx3)/* q */;
    Register Ry = opnd_array(1)->as_Register(ra_,this,idx1)/* y */;
    Label done;
    __ cmpl(Rp, Rq);
    __ jccb(Assembler::less, done);
    __ xorl(Ry, Ry);
    __ bind(done);
  
#line 21979 "ad_x86.cpp"
  }
}

void overflowAddI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8691 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 21996 "ad_x86.cpp"
  }
}

void overflowAddI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8704 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 22013 "ad_x86.cpp"
  }
}

void overflowSubI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8715 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 22030 "ad_x86.cpp"
  }
}

void overflowSubI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8726 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 22047 "ad_x86.cpp"
  }
}

void overflowNegI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8738 "src/hotspot/cpu/x86/x86_32.ad"

    __ negl(opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 22064 "ad_x86.cpp"
  }
}

void overflowMulI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8750 "src/hotspot/cpu/x86/x86_32.ad"

    __ imull(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 22081 "ad_x86.cpp"
  }
}

void overflowMulI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8762 "src/hotspot/cpu/x86/x86_32.ad"

    __ imull(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 22099 "ad_x86.cpp"
  }
}

void absI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8779 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ sarl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, 31);
    __ movl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ xorl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ subl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 22121 "ad_x86.cpp"
  }
}

void addL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1941 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x03 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 22138 "ad_x86.cpp"
  }
  {

#line 1946 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x13 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 22147 "ad_x86.cpp"
  }
}

void addL_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1649 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 22170 "ad_x86.cpp"
  }
  {

#line 1660 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x02 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 22185 "ad_x86.cpp"
  }
}

void addL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 22201 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22215 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x13 /*secondary()*/));
  
#line 22223 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 22237 "ad_x86.cpp"
  }
}

void addL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 22253 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22267 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x13 /*secondary()*/));
  
#line 22275 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 22289 "ad_x86.cpp"
  }
}

void subL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1941 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x2B /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 22306 "ad_x86.cpp"
  }
  {

#line 1946 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x1B /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 22315 "ad_x86.cpp"
  }
}

void subL_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1649 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 22338 "ad_x86.cpp"
  }
  {

#line 1660 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x03 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 22353 "ad_x86.cpp"
  }
}

void subL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 22369 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22383 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x1B /*secondary()*/));
  
#line 22391 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 22405 "ad_x86.cpp"
  }
}

void negL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2864 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xF7);    // NEG hi
    emit_rm    (cbuf,0x3, 0x3, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */));
    emit_opcode(cbuf,0xF7);    // NEG lo
    emit_rm    (cbuf,0x3, 0x3,               opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
    emit_opcode(cbuf,0x83);    // SBB hi,0
    emit_rm    (cbuf,0x3, 0x3, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */));
    emit_d8    (cbuf,0 );
  
#line 22427 "ad_x86.cpp"
  }
}

void andL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1941 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x23 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 22444 "ad_x86.cpp"
  }
  {

#line 1946 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x23 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 22453 "ad_x86.cpp"
  }
}

void andL_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1649 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 22476 "ad_x86.cpp"
  }
  {

#line 1660 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 22491 "ad_x86.cpp"
  }
}

void andL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 22507 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22521 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*secondary()*/));
  
#line 22529 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 22543 "ad_x86.cpp"
  }
}

void andL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 22559 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22573 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*secondary()*/));
  
#line 22581 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 22595 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8914 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */;
    Register Rsrc2 = opnd_array(3)->as_Register(ra_,this,idx3)/* src2 */;
    __ andnl(Rdst, Rsrc1, Rsrc2);
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), HIGH_FROM_LOW(Rsrc2));
  
#line 22618 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8914 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */;
    Register Rsrc2 = opnd_array(1)->as_Register(ra_,this,idx1)/* src2 */;
    __ andnl(Rdst, Rsrc1, Rsrc2);
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), HIGH_FROM_LOW(Rsrc2));
  
#line 22641 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8934 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */;
    Address src2_hi = Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3)+ 4, relocInfo::none);

    __ andnl(Rdst, Rsrc1, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()));
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), src2_hi);
  
#line 22665 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8934 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */;
    Address src2_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ andnl(Rdst, Rsrc1, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), src2_hi);
  
#line 22689 "ad_x86.cpp"
  }
}

void blsiL_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8957 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(2)->as_Register(ra_,this,idx2)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Rsrc);
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 22714 "ad_x86.cpp"
  }
}

void blsiL_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8957 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Rsrc);
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 22739 "ad_x86.cpp"
  }
}

void blsiL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8983 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 22765 "ad_x86.cpp"
  }
}

void blsiL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8983 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 22791 "ad_x86.cpp"
  }
}

void blsmskL_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9010 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 22816 "ad_x86.cpp"
  }
}

void blsmskL_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9010 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 22841 "ad_x86.cpp"
  }
}

void blsmskL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9038 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 22867 "ad_x86.cpp"
  }
}

void blsmskL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9038 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 22893 "ad_x86.cpp"
  }
}

void blsrL_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9066 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ blsrl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 22918 "ad_x86.cpp"
  }
}

void blsrL_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9066 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ blsrl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 22943 "ad_x86.cpp"
  }
}

void blsrL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9094 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);
    __ movl(HIGH_FROM_LOW(Rdst), src_hi);
    __ blsrl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 22968 "ad_x86.cpp"
  }
}

void blsrL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9094 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);
    __ movl(HIGH_FROM_LOW(Rdst), src_hi);
    __ blsrl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 22993 "ad_x86.cpp"
  }
}

void orl_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1941 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0B /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 23010 "ad_x86.cpp"
  }
  {

#line 1946 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0B /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 23019 "ad_x86.cpp"
  }
}

void orl_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1649 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 23042 "ad_x86.cpp"
  }
  {

#line 1660 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 23057 "ad_x86.cpp"
  }
}

void orl_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 23073 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 23087 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*secondary()*/));
  
#line 23095 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 23109 "ad_x86.cpp"
  }
}

void orl_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 23125 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 23139 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*secondary()*/));
  
#line 23147 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 23161 "ad_x86.cpp"
  }
}

void xorl_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1941 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 23178 "ad_x86.cpp"
  }
  {

#line 1946 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 23187 "ad_x86.cpp"
  }
}

void xorl_eReg_im1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9158 "src/hotspot/cpu/x86/x86_32.ad"

     __ notl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
     __ notl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 23205 "ad_x86.cpp"
  }
}

void xorl_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1649 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 23228 "ad_x86.cpp"
  }
  {

#line 1660 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 23243 "ad_x86.cpp"
  }
}

void xorl_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 23259 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 23273 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*secondary()*/));
  
#line 23281 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 23295 "ad_x86.cpp"
  }
}

void xorl_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 23311 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 23325 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*secondary()*/));
  
#line 23333 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 23347 "ad_x86.cpp"
  }
}

void shlL_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9196 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 23365 "ad_x86.cpp"
  }
}

void shlL_eReg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9213 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 23385 "ad_x86.cpp"
  }
}

void shlL_eReg_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9234 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 23407 "ad_x86.cpp"
  }
}

void shlL_eReg_1_31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2094 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0xA4 /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xA4 /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  
#line 23432 "ad_x86.cpp"
  }
}

void shlL_eReg_32_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2120 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0x4 /*secondary()*/) == 0x5 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }

    emit_opcode( cbuf, 0x8B ); // Move r1,r2
    emit_rm(cbuf, 0x3, r1, r2);
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_opcode(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), r1);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_opcode(cbuf,0x33);  // XOR r2,r2
    emit_rm(cbuf, 0x3, r2, r2);
  
#line 23460 "ad_x86.cpp"
  }
}

void salL_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2257 "src/hotspot/cpu/x86/x86_32.ad"

    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x04);
    // MOV    $dst.hi,$dst.lo
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    // CLR    $dst.lo
    emit_opcode(cbuf, 0x33);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* dst */);
// small:
    // SHLD   $dst.hi,$dst.lo,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xA5);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    // SHL    $dst.lo,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x4, opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23496 "ad_x86.cpp"
  }
}

uint salL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 17, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 17);
}

void shrL_eReg_1_31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2094 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0xAC /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xAC /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  
#line 23526 "ad_x86.cpp"
  }
}

void shrL_eReg_32_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2120 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0x5 /*secondary()*/) == 0x5 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }

    emit_opcode( cbuf, 0x8B ); // Move r1,r2
    emit_rm(cbuf, 0x3, r1, r2);
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_opcode(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), r1);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_opcode(cbuf,0x33);  // XOR r2,r2
    emit_rm(cbuf, 0x3, r2, r2);
  
#line 23554 "ad_x86.cpp"
  }
}

void shrL_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2281 "src/hotspot/cpu/x86/x86_32.ad"

    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x04);
    // MOV    $dst.lo,$dst.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // CLR    $dst.hi
    emit_opcode(cbuf, 0x33);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
// small:
    // SHRD   $dst.lo,$dst.hi,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xAD);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    // SHR    $dst.hi,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x5, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
  
#line 23590 "ad_x86.cpp"
  }
}

uint shrL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 17, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 17);
}

void sarL_eReg_1_31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2094 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0xAC /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xAC /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  
#line 23620 "ad_x86.cpp"
  }
}

void sarL_eReg_32_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2107 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0x8B ); // Move
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_d8(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    emit_d8(cbuf,31);
  
#line 23645 "ad_x86.cpp"
  }
}

void sarL_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2305 "src/hotspot/cpu/x86/x86_32.ad"

    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);
    // MOV    $dst.lo,$dst.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // SAR    $dst.hi,31
    emit_opcode(cbuf, 0xC1);
    emit_rm(cbuf, 0x3, 7, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    emit_d8(cbuf, 0x1F );
// small:
    // SHRD   $dst.lo,$dst.hi,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xAD);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    // SAR    $dst.hi,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x7, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
  
#line 23682 "ad_x86.cpp"
  }
}

uint sarL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 18, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 18);
}

void cmpDPR_cc_P6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 23704 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 23712 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 23720 "ad_x86.cpp"
  }
  {

#line 2592 "src/hotspot/cpu/x86/x86_32.ad"

    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  
#line 23739 "ad_x86.cpp"
  }
}

void cmpDPR_cc_P6CFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 23756 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 23764 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 23772 "ad_x86.cpp"
  }
}

void cmpDPR_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 23789 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 23797 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 23805 "ad_x86.cpp"
  }
  {

#line 2569 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // test ax,0x0400
    emit_opcode( cbuf, 0x66 );   // operand-size prefix for 16-bit immediate
    emit_opcode( cbuf, 0xA9 );
    emit_d16   ( cbuf, 0x0400 );
    // // // This sequence works, but stalls for 12-16 cycles on PPro
    // // test rax,0x0400
    // emit_opcode( cbuf, 0xA9 );
    // emit_d32   ( cbuf, 0x00000400 );
    //
    // jz exit (no unordered comparison)
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x02 );
    // mov ah,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // sahf
    emit_opcode( cbuf, 0x9E);
  
#line 23832 "ad_x86.cpp"
  }
}

void cmpDPR_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 23849 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 23857 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE4 /*primary()*/));
  
#line 23865 "ad_x86.cpp"
  }
  {

#line 2339 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 23874 "ad_x86.cpp"
  }
  {

#line 2622 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 23907 "ad_x86.cpp"
  }
}

void cmpDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 23924 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 23932 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 23940 "ad_x86.cpp"
  }
  {

#line 2622 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 23973 "ad_x86.cpp"
  }
}

void cmpD_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9464 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp_fixup(_masm);
  
#line 23991 "ad_x86.cpp"
  }
}

void cmpD_ccCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9476 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 24008 "ad_x86.cpp"
  }
}

void cmpD_ccmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9493 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp_fixup(_masm);
  
#line 24026 "ad_x86.cpp"
  }
}

void cmpD_ccmemCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9505 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 24043 "ad_x86.cpp"
  }
}

void cmpD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9524 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 24061 "ad_x86.cpp"
  }
}

void cmpD_regmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9544 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 24079 "ad_x86.cpp"
  }
}

void subDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 24096 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24104 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24112 "ad_x86.cpp"
  }
}

void subDPR_reg_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );   // FLD ST(i-1)
  
#line 24129 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 24137 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 24145 "ad_x86.cpp"
  }
  {

#line 2396 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 24153 "ad_x86.cpp"
  }
}

void subDPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 24169 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24183 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24191 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24199 "ad_x86.cpp"
  }
}

void absDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 24214 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE1 /*primary()*/));
  
#line 24222 "ad_x86.cpp"
  }
}

void negDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 24237 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE0 /*primary()*/));
  
#line 24245 "ad_x86.cpp"
  }
}

void addDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 24262 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24270 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24278 "ad_x86.cpp"
  }
}

uint addDPR_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addDPR_reg_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );   // FLD ST(i-1)
  
#line 24300 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 24308 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 24316 "ad_x86.cpp"
  }
  {

#line 2396 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 24324 "ad_x86.cpp"
  }
}

void addDPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 24340 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24354 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24362 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24370 "ad_x86.cpp"
  }
}

void addDPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 24386 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24400 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24408 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 24416 "ad_x86.cpp"
  }
}

void addDPR_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 24432 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24446 "ad_x86.cpp"
  }
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8));
  
#line 24454 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 24462 "ad_x86.cpp"
  }
  {

#line 2070 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 24470 "ad_x86.cpp"
  }
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 24478 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x03);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24492 "ad_x86.cpp"
  }
}

void addDPR_mem_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 24508 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24522 "ad_x86.cpp"
  }
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8));
  
#line 24530 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 24538 "ad_x86.cpp"
  }
  {

#line 2070 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 24546 "ad_x86.cpp"
  }
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 24554 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x03);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24568 "ad_x86.cpp"
  }
}

void addDPR_reg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9678 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld1();
    __ faddp(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 24586 "ad_x86.cpp"
  }
}

void addDPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9691 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ faddp(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 24604 "ad_x86.cpp"
  }
}

void addDPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 9692 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 24613 "ad_x86.cpp"
  }
}
void addDPR_reg_imm_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9705 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fadd(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fstp_d(Address(rsp, opnd_array(0)->disp(ra_,this,0)));
  
#line 24631 "ad_x86.cpp"
  }
}

void addDPR_reg_imm_roundNode::eval_constant(Compile* C) {
  {

#line 9706 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 24640 "ad_x86.cpp"
  }
}
void mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 24656 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24664 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24672 "ad_x86.cpp"
  }
}

void strictfp_mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias1() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24692 "ad_x86.cpp"
  }
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 24701 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24709 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24717 "ad_x86.cpp"
  }
  {

#line 2363 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias2() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24729 "ad_x86.cpp"
  }
}

void mulDPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9758 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fmulp(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 24747 "ad_x86.cpp"
  }
}

void mulDPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 9759 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 24756 "ad_x86.cpp"
  }
}
void mulDPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 24771 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24785 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24793 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24801 "ad_x86.cpp"
  }
}

void mulDPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 24817 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24831 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24839 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 24847 "ad_x86.cpp"
  }
}

void mulDPR_reg_mem_ciscNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 24863 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24877 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 24886 "ad_x86.cpp"
  }
  {

#line 2345 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 24895 "ad_x86.cpp"
  }
}

void mulDPR_reg_mem_cisc_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 24911 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 24925 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 24934 "ad_x86.cpp"
  }
  {

#line 2345 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 24943 "ad_x86.cpp"
  }
}

void addDPR_mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src0 */ );
  
#line 24961 "ad_x86.cpp"
  }
  {

#line 2976 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 24972 "ad_x86.cpp"
  }
  {

#line 2990 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
  
#line 24982 "ad_x86.cpp"
  }
}

void addDPR_mulDPR_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src0
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src0 */ );
  
#line 25000 "ad_x86.cpp"
  }
  {

#line 2976 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 25011 "ad_x86.cpp"
  }
  {

#line 2990 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
  
#line 25021 "ad_x86.cpp"
  }
}

void subDPR_mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src0 */ );
  
#line 25039 "ad_x86.cpp"
  }
  {

#line 2976 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 25050 "ad_x86.cpp"
  }
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE));
  
#line 25058 "ad_x86.cpp"
  }
  {

#line 1610 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE0) + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
  
#line 25066 "ad_x86.cpp"
  }
}

void divDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 25083 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 25091 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25099 "ad_x86.cpp"
  }
}

void strictfp_divDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias1() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25119 "ad_x86.cpp"
  }
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 25128 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 25136 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25144 "ad_x86.cpp"
  }
  {

#line 2363 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias2() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25156 "ad_x86.cpp"
  }
}

void divDPR_reg_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25173 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 25181 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x6 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25189 "ad_x86.cpp"
  }
  {

#line 2396 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25197 "ad_x86.cpp"
  }
}

void modDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2445 "src/hotspot/cpu/x86/x86_32.ad"

    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  
#line 25227 "ad_x86.cpp"
  }
  {

#line 2547 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 25253 "ad_x86.cpp"
  }
  {

#line 2518 "src/hotspot/cpu/x86/x86_32.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  
#line 25275 "ad_x86.cpp"
  }
  {

#line 2345 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25284 "ad_x86.cpp"
  }
}

void modD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2463 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */);
    __ fld_d(Address(rsp, 0));
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src0 */);
    __ fld_d(Address(rsp, 0));
  
#line 25305 "ad_x86.cpp"
  }
  {

#line 2547 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 25331 "ad_x86.cpp"
  }
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 25342 "ad_x86.cpp"
  }
  {

#line 2339 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 25351 "ad_x86.cpp"
  }
}

void atanDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 25368 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 25376 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF3 /*secondary()*/));
  
#line 25384 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0xF3 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25392 "ad_x86.cpp"
  }
}

void atanD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2495 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
    __ fld_d(Address(rsp, 0));
  
#line 25411 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 25419 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF3 /*secondary()*/));
  
#line 25427 "ad_x86.cpp"
  }
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 25438 "ad_x86.cpp"
  }
}

void sqrtDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );   // FLD ST(i-1)
  
#line 25454 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 25462 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFA /*primary()*/));
  
#line 25470 "ad_x86.cpp"
  }
  {

#line 2345 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 25479 "ad_x86.cpp"
  }
}

void cmpFPR_cc_P6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25496 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 25504 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25512 "ad_x86.cpp"
  }
  {

#line 2592 "src/hotspot/cpu/x86/x86_32.ad"

    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  
#line 25531 "ad_x86.cpp"
  }
}

void cmpFPR_cc_P6CFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25548 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 25556 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25564 "ad_x86.cpp"
  }
}

void cmpFPR_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25581 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 25589 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25597 "ad_x86.cpp"
  }
  {

#line 2569 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // test ax,0x0400
    emit_opcode( cbuf, 0x66 );   // operand-size prefix for 16-bit immediate
    emit_opcode( cbuf, 0xA9 );
    emit_d16   ( cbuf, 0x0400 );
    // // // This sequence works, but stalls for 12-16 cycles on PPro
    // // test rax,0x0400
    // emit_opcode( cbuf, 0xA9 );
    // emit_d32   ( cbuf, 0x00000400 );
    //
    // jz exit (no unordered comparison)
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x02 );
    // mov ah,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // sahf
    emit_opcode( cbuf, 0x9E);
  
#line 25624 "ad_x86.cpp"
  }
}

void cmpFPR_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25641 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 25649 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE4 /*primary()*/));
  
#line 25657 "ad_x86.cpp"
  }
  {

#line 2339 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 25666 "ad_x86.cpp"
  }
  {

#line 2622 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 25699 "ad_x86.cpp"
  }
}

void cmpFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25716 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 25724 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25732 "ad_x86.cpp"
  }
  {

#line 2622 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 25765 "ad_x86.cpp"
  }
}

void cmpF_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10061 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp_fixup(_masm);
  
#line 25783 "ad_x86.cpp"
  }
}

void cmpF_ccCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10073 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 25800 "ad_x86.cpp"
  }
}

void cmpF_ccmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10090 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp_fixup(_masm);
  
#line 25818 "ad_x86.cpp"
  }
}

void cmpF_ccmemCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10102 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 25835 "ad_x86.cpp"
  }
}

void cmpF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10121 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 25853 "ad_x86.cpp"
  }
}

void cmpF_regmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10141 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 25871 "ad_x86.cpp"
  }
}

void subFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 25888 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25897 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25905 "ad_x86.cpp"
  }
}

void subFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 25922 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 25930 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25938 "ad_x86.cpp"
  }
}

void addFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25955 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 25964 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25972 "ad_x86.cpp"
  }
}

void addFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 25989 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 25997 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26005 "ad_x86.cpp"
  }
}

void absFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 26020 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE1 /*primary()*/));
  
#line 26028 "ad_x86.cpp"
  }
}

void negFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 26043 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE0 /*primary()*/));
  
#line 26051 "ad_x86.cpp"
  }
}

void addFPR24_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26067 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26081 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 26090 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26098 "ad_x86.cpp"
  }
}

void addFPR24_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26114 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26128 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
  
#line 26137 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26145 "ad_x86.cpp"
  }
}

void addFPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26161 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26175 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26183 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26191 "ad_x86.cpp"
  }
}

void addFPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26207 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26221 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26229 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 26237 "ad_x86.cpp"
  }
}

void addFPR24_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26253 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26267 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 26276 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26284 "ad_x86.cpp"
  }
}

void addFPR24_mem_ciscNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26300 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26314 "ad_x86.cpp"
  }
  {

#line 2070 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 26322 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 26330 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26344 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26352 "ad_x86.cpp"
  }
}

void addFPR24_mem_cisc_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26368 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26382 "ad_x86.cpp"
  }
  {

#line 2070 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 26390 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 26398 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26412 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26420 "ad_x86.cpp"
  }
}

void addFPR24_mem_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26436 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26450 "ad_x86.cpp"
  }
  {

#line 2070 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 26458 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 26466 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26480 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26488 "ad_x86.cpp"
  }
}

void addFPR24_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10299 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fadd_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_s(Address(rsp, opnd_array(0)->disp(ra_,this,0)));
  
#line 26507 "ad_x86.cpp"
  }
}

void addFPR24_reg_immNode::eval_constant(Compile* C) {
  {

#line 10301 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 26516 "ad_x86.cpp"
  }
}
void addFPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10314 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fadd_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 26534 "ad_x86.cpp"
  }
}

void addFPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 10316 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 26543 "ad_x86.cpp"
  }
}
void mulFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 26559 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 26568 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26576 "ad_x86.cpp"
  }
}

void mulFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 26593 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 26602 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26611 "ad_x86.cpp"
  }
}

void mulFPR24_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26627 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26641 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 26650 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26658 "ad_x86.cpp"
  }
}

void mulFPR24_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26674 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26688 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
  
#line 26697 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26705 "ad_x86.cpp"
  }
}

void mulFPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26721 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26735 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 26744 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26753 "ad_x86.cpp"
  }
}

void mulFPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26769 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26783 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
  
#line 26792 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26801 "ad_x86.cpp"
  }
}

void mulFPR24_mem_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26817 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26831 "ad_x86.cpp"
  }
  {

#line 2070 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 26839 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 26847 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x1 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26861 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26869 "ad_x86.cpp"
  }
}

void mulFPR24_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10405 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fmul_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_s(Address(rsp, opnd_array(0)->disp(ra_,this,0)));
  
#line 26888 "ad_x86.cpp"
  }
}

void mulFPR24_reg_immNode::eval_constant(Compile* C) {
  {

#line 10407 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 26897 "ad_x86.cpp"
  }
}
void mulFPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10421 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fmul_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 26915 "ad_x86.cpp"
  }
}

void mulFPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 10423 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 26924 "ad_x86.cpp"
  }
}
void mulFPR_reg_load1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26939 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26953 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 26962 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26971 "ad_x86.cpp"
  }
}

void mulFPR_reg_load1_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem1
  {

#line 1546 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26987 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27001 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 27010 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27019 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 27036 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27050 "ad_x86.cpp"
  }
  {

#line 2976 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 27061 "ad_x86.cpp"
  }
  {

#line 2983 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 27072 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27081 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 27098 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27112 "ad_x86.cpp"
  }
  {

#line 2976 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 27123 "ad_x86.cpp"
  }
  {

#line 2983 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 27134 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27143 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 27160 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27174 "ad_x86.cpp"
  }
  {

#line 2976 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
  
#line 27185 "ad_x86.cpp"
  }
  {

#line 2983 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 27196 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27205 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mem1
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 27222 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(3)->base(ra_,this,idx3);
    int index    = opnd_array(3)->index(ra_,this,idx3);
    int scale    = opnd_array(3)->scale();
    int displace = opnd_array(3)->disp(ra_,this,idx3);
    relocInfo::relocType disp_reloc = opnd_array(3)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27236 "ad_x86.cpp"
  }
  {

#line 2976 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 27247 "ad_x86.cpp"
  }
  {

#line 2983 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 27258 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27267 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src0 */ );
  
#line 27285 "ad_x86.cpp"
  }
  {

#line 2976 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 27296 "ad_x86.cpp"
  }
  {

#line 2990 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
  
#line 27306 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src0
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src0 */ );
  
#line 27324 "ad_x86.cpp"
  }
  {

#line 2976 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 27335 "ad_x86.cpp"
  }
  {

#line 2990 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
  
#line 27345 "ad_x86.cpp"
  }
}

void subFPR_divFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src2 */ );
  
#line 27363 "ad_x86.cpp"
  }
  {

#line 2996 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand has been loaded into fp ST (stack top)
      // FSUB   ST,$src1
      emit_opcode(cbuf, 0xD8);
      emit_opcode(cbuf, 0xE0 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);

      // FDIV
      emit_opcode(cbuf, 0xD8);
      emit_opcode(cbuf, 0xF0 + opnd_array(3)->reg(ra_,this,idx3)/* src3 */);
  
#line 27378 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27387 "ad_x86.cpp"
  }
}

void divFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 27404 "ad_x86.cpp"
  }
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x6 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 27413 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27421 "ad_x86.cpp"
  }
}

void divFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 27438 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 27446 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 27454 "ad_x86.cpp"
  }
}

void modFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2445 "src/hotspot/cpu/x86/x86_32.ad"

    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    if (opnd_array(2)->reg(ra_,this,idx2)/* src2 */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  
#line 27484 "ad_x86.cpp"
  }
  {

#line 2547 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 27510 "ad_x86.cpp"
  }
  {

#line 2518 "src/hotspot/cpu/x86/x86_32.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src2 */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  
#line 27532 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27540 "ad_x86.cpp"
  }
}

void modFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2445 "src/hotspot/cpu/x86/x86_32.ad"

    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  
#line 27570 "ad_x86.cpp"
  }
  {

#line 2547 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 27596 "ad_x86.cpp"
  }
  {

#line 2518 "src/hotspot/cpu/x86/x86_32.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  
#line 27618 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 27627 "ad_x86.cpp"
  }
}

void modF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2472 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */);
    __ fld_s(Address(rsp, 0));
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src0 */);
    __ fld_s(Address(rsp, 0));
  
#line 27648 "ad_x86.cpp"
  }
  {

#line 2547 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 27674 "ad_x86.cpp"
  }
  {

#line 2488 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_s(Address(rsp, 0));
    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, (0x4));
  
#line 27685 "ad_x86.cpp"
  }
  {

#line 2339 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 27694 "ad_x86.cpp"
  }
}

void roundFloat_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2411 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xD9, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_S  [ESP+dst]
  
#line 27715 "ad_x86.cpp"
  }
}

void roundDouble_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xDD, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_D  [ESP+dst]
  
#line 27736 "ad_x86.cpp"
  }
}

void convDPR2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10621 "src/hotspot/cpu/x86/x86_32.ad"

    __ subptr(rsp, 4);
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */!= FPR1L_enc) {
      __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */-1);
      __ fstp_s(Address(rsp, 0));
    } else {
      __ fst_s(Address(rsp, 0));
    }
    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 4);
  
#line 27760 "ad_x86.cpp"
  }
}

void convD2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10640 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsd2ss (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 27776 "ad_x86.cpp"
  }
}

void convFPR2DPR_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2433 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0xD0 - 1; // -1 since we skip FLD
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(src-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0xD8;
    }
    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, pop+opnd_array(0)->reg(ra_,this)/* dst */ );      // FST<P> ST(i)
  
#line 27798 "ad_x86.cpp"
  }
}

void convF2DPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10672 "src/hotspot/cpu/x86/x86_32.ad"

    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ addptr(rsp, 4);
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 27818 "ad_x86.cpp"
  }
}

void convF2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10686 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtss2sd (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 27834 "ad_x86.cpp"
  }
}

void convDPR2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );   // FLD ST(i-1)
  
#line 27850 "ad_x86.cpp"
  }
  {

#line 2891 "src/hotspot/cpu/x86/x86_32.ad"

    // Flip to round-to-zero mode.  We attempted to allow invalid-op
    // exceptions here, so that a NAN or other corner-case value will
    // thrown an exception (but normal values get converted at full speed).
    // However, I2C adapters and other float-stack manglers leave pending
    // invalid-op exceptions hanging.  We would have to clear them before
    // enabling them and that is more expensive than just testing for the
    // invalid value Intel stores down in the corner cases.
    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as an int, popping the FPU stack
    emit_opcode(cbuf,0xDB);            // FISTP [ESP]
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x3D);       // CMP EAX,imm
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2i_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  
#line 27897 "ad_x86.cpp"
  }
}

void convD2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10726 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ cvttsd2sil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ cmpl(opnd_array(0)->as_Register(ra_,this)/* dst */, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_d(Address(rsp, 0));
    __ addptr(rsp, 8);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::d2i_wrapper())));
    __ bind(fast);
  
#line 27922 "ad_x86.cpp"
  }
}

void convDPR2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2350 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );   // FLD ST(i-1)
  
#line 27938 "ad_x86.cpp"
  }
  {

#line 2934 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as a long, popping the FPU stack
    emit_opcode(cbuf,0xDF);            // FISTP [ESP]
    emit_opcode(cbuf,0x3C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x5A);       // POP EDX
    emit_opcode(cbuf,0x81);       // CMP EDX,imm
    emit_d8    (cbuf,0xFA);       // rdx
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07+4);     // Size of slow_call
    emit_opcode(cbuf,0x85);       // TEST EAX,EAX
    emit_opcode(cbuf,0xC0);       // 2/rax,/rax,
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2l_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  
#line 27984 "ad_x86.cpp"
  }
}

void convD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10786 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_d(Address(rsp, 0));
    __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_trunc()));
    __ fistp_d(Address(rsp, 0));
    // Restore the rounding mode, mask the exception
    if (Compile::current()->in_24_bit_fp_mode()) {
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    } else {
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Load the converted long, adjust CPU stack
    __ pop(rax);
    __ pop(rdx);
    __ cmpl(rdx, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ testl(rax, rax);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_d(Address(rsp, 0));
    __ addptr(rsp, 8);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::d2l_wrapper())));
    __ bind(fast);
  
#line 28024 "ad_x86.cpp"
  }
}

void convFPR2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 28040 "ad_x86.cpp"
  }
  {

#line 2891 "src/hotspot/cpu/x86/x86_32.ad"

    // Flip to round-to-zero mode.  We attempted to allow invalid-op
    // exceptions here, so that a NAN or other corner-case value will
    // thrown an exception (but normal values get converted at full speed).
    // However, I2C adapters and other float-stack manglers leave pending
    // invalid-op exceptions hanging.  We would have to clear them before
    // enabling them and that is more expensive than just testing for the
    // invalid value Intel stores down in the corner cases.
    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as an int, popping the FPU stack
    emit_opcode(cbuf,0xDB);            // FISTP [ESP]
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x3D);       // CMP EAX,imm
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2i_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  
#line 28087 "ad_x86.cpp"
  }
}

void convF2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10856 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ cvttss2sil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ cmpl(opnd_array(0)->as_Register(ra_,this)/* dst */, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ addptr(rsp, 4);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::d2i_wrapper())));
    __ bind(fast);
  
#line 28112 "ad_x86.cpp"
  }
}

void convFPR2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2405 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 28128 "ad_x86.cpp"
  }
  {

#line 2934 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as a long, popping the FPU stack
    emit_opcode(cbuf,0xDF);            // FISTP [ESP]
    emit_opcode(cbuf,0x3C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x5A);       // POP EDX
    emit_opcode(cbuf,0x81);       // CMP EDX,imm
    emit_d8    (cbuf,0xFA);       // rdx
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07+4);     // Size of slow_call
    emit_opcode(cbuf,0x85);       // TEST EAX,EAX
    emit_opcode(cbuf,0xC0);       // 2/rax,/rax,
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2l_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  
#line 28174 "ad_x86.cpp"
  }
}

void convF2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10917 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ subptr(rsp, 8);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_trunc()));
    __ fistp_d(Address(rsp, 0));
    // Restore the rounding mode, mask the exception
    if (Compile::current()->in_24_bit_fp_mode()) {
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    } else {
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Load the converted long, adjust CPU stack
    __ pop(rax);
    __ pop(rdx);
    __ cmpl(rdx, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ testl(rax, rax);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ addptr(rsp, 4);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::d2l_wrapper())));
    __ bind(fast);
  
#line 28214 "ad_x86.cpp"
  }
}

void convI2DPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2385 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 28229 "ad_x86.cpp"
  }
  {

#line 2345 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28238 "ad_x86.cpp"
  }
}

void convI2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10961 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsi2sdl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 28254 "ad_x86.cpp"
  }
}

void convI2D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10971 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsi2sdl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 28270 "ad_x86.cpp"
  }
}

void convXI2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10984 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 28287 "ad_x86.cpp"
  }
}

void convI2DPR_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDB /*primary()*/));
  
#line 28302 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28316 "ad_x86.cpp"
  }
  {

#line 2345 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28325 "ad_x86.cpp"
  }
}

void conv24I2FPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2385 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 28340 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28349 "ad_x86.cpp"
  }
}

void convI2FPR_SSFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2385 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 28364 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28372 "ad_x86.cpp"
  }
}

void convI2FPR_SSF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDB /*primary()*/));
  
#line 28387 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28401 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28409 "ad_x86.cpp"
  }
}

void convI2FPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2385 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 28424 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28433 "ad_x86.cpp"
  }
}

void convI2FPR_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDB /*primary()*/));
  
#line 28448 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28462 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28471 "ad_x86.cpp"
  }
}

void convI2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11069 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsi2ssl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 28487 "ad_x86.cpp"
  }
}

void convXI2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11082 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 28504 "ad_x86.cpp"
  }
}

void convI2L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2667 "src/hotspot/cpu/x86/x86_32.ad"

    // mov $dst.lo,$src
    int dst_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int src_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    encode_Copy( cbuf, dst_encoding  , src_encoding );
    // mov $dst.hi,$src
    encode_Copy( cbuf, HIGH_FROM_LOW(dst_encoding), src_encoding );
    // sar $dst.hi,31
    emit_opcode( cbuf, 0xC1 );
    emit_rm(cbuf, 0x3, 7, HIGH_FROM_LOW(dst_encoding) );
    emit_d8(cbuf, 0x1F );
  
#line 28528 "ad_x86.cpp"
  }
}

void convI2L_reg_zexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 1929 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 28544 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 28552 "ad_x86.cpp"
  }
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 28560 "ad_x86.cpp"
  }
}

void zerox_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 1929 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 28576 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 28584 "ad_x86.cpp"
  }
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 28592 "ad_x86.cpp"
  }
}

void convL2DPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2680 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
    // pop stack
    emit_opcode(cbuf, 0x83); // add  SP, #8
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 0x8);
  
#line 28619 "ad_x86.cpp"
  }
  {

#line 2396 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28627 "ad_x86.cpp"
  }
}

void convL2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2710 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
  
#line 28650 "ad_x86.cpp"
  }
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 28661 "ad_x86.cpp"
  }
}

void convL2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2710 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
  
#line 28684 "ad_x86.cpp"
  }
  {

#line 2488 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_s(Address(rsp, 0));
    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, (0x8));
  
#line 28695 "ad_x86.cpp"
  }
}

void convL2FPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2680 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
    // pop stack
    emit_opcode(cbuf, 0x83); // add  SP, #8
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 0x8);
  
#line 28722 "ad_x86.cpp"
  }
  {

#line 2390 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28730 "ad_x86.cpp"
  }
}

void convL2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1933 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 28745 "ad_x86.cpp"
  }
}

void MoveF2I_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11194 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 28761 "ad_x86.cpp"
  }
}

void MoveFPR2I_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2411 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xD9, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_S  [ESP+dst]
  
#line 28782 "ad_x86.cpp"
  }
}

void MoveF2I_reg_stack_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11218 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 28798 "ad_x86.cpp"
  }
}

void MoveF2I_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11230 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 28814 "ad_x86.cpp"
  }
}

void MoveI2F_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11242 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 28830 "ad_x86.cpp"
  }
}

void MoveI2FPR_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 28845 "ad_x86.cpp"
  }
  {

#line 2154 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 28859 "ad_x86.cpp"
  }
  {

#line 2400 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28868 "ad_x86.cpp"
  }
}

void MoveI2F_stack_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11270 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 28884 "ad_x86.cpp"
  }
}

void MoveI2F_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11283 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 28900 "ad_x86.cpp"
  }
}

void MoveD2L_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 28915 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 28929 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*secondary()*/));
  
#line 28937 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 28951 "ad_x86.cpp"
  }
}

void MoveDPR2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xDD, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_D  [ESP+dst]
  
#line 28972 "ad_x86.cpp"
  }
}

void MoveD2L_reg_stack_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11318 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 28988 "ad_x86.cpp"
  }
}

void MoveD2L_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11332 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ pshuflw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x4e);
    __ movdl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
  
#line 29007 "ad_x86.cpp"
  }
}

void MoveL2D_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 29022 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0);
    relocInfo::relocType disp_reloc = opnd_array(0)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 29036 "ad_x86.cpp"
  }
  {

#line 1541 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*secondary()*/));
  
#line 29044 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */);  // Hi register of pair, computed from lo
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0) + 4;      // Offset is 4 further in memory
    assert( opnd_array(0)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 29058 "ad_x86.cpp"
  }
}

void MoveL2DPR_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*primary()*/));
  
#line 29073 "ad_x86.cpp"
  }
  {

#line 2154 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 29087 "ad_x86.cpp"
  }
  {

#line 2345 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 29096 "ad_x86.cpp"
  }
}

void MoveL2D_stack_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11375 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 29112 "ad_x86.cpp"
  }
}

void MoveL2D_stack_reg_sse_partialNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11388 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 29128 "ad_x86.cpp"
  }
}

void MoveL2D_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11402 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
    __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 29148 "ad_x86.cpp"
  }
}

void rep_stosNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11465 "src/hotspot/cpu/x86/x86_32.ad"

    __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */, as_Register(EAX_enc),
                 opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, false);
  
#line 29167 "ad_x86.cpp"
  }
}

void rep_stos_largeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11514 "src/hotspot/cpu/x86/x86_32.ad"

    __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */, as_Register(EAX_enc),
                 opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, true);
  
#line 29186 "ad_x86.cpp"
  }
}

void string_compareLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11528 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::LL);
  
#line 29208 "ad_x86.cpp"
  }
}

void string_compareUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11543 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::UU);
  
#line 29230 "ad_x86.cpp"
  }
}

void string_compareLUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11558 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::LU);
  
#line 29252 "ad_x86.cpp"
  }
}

void string_compareULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11573 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */,
                      opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::UL);
  
#line 29274 "ad_x86.cpp"
  }
}

void string_equalsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11588 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(false, opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                     opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, false /* char */);
  
#line 29296 "ad_x86.cpp"
  }
}

void string_indexof_conLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11605 "src/hotspot/cpu/x86/x86_32.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    if (icnt2 >= 16) {
      // IndexOf for constant substrings with size >= 16 elements
      // which don't need to be loaded through stack.
      __ string_indexofC8(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                          opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                          icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                          opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::LL);
    } else {
      // Small strings are loaded through stack if they cross page boundary.
      __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                        icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                        opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::LL);
    }
  
#line 29330 "ad_x86.cpp"
  }
}

void string_indexof_conUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11633 "src/hotspot/cpu/x86/x86_32.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    if (icnt2 >= 8) {
      // IndexOf for constant substrings with size >= 8 elements
      // which don't need to be loaded through stack.
      __ string_indexofC8(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                          opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                          icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                          opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UU);
    } else {
      // Small strings are loaded through stack if they cross page boundary.
      __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                        icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                        opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UU);
    }
  
#line 29364 "ad_x86.cpp"
  }
}

void string_indexof_conULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11661 "src/hotspot/cpu/x86/x86_32.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    if (icnt2 >= 8) {
      // IndexOf for constant substrings with size >= 8 elements
      // which don't need to be loaded through stack.
      __ string_indexofC8(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                          opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                          icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                          opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UL);
    } else {
      // Small strings are loaded through stack if they cross page boundary.
      __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                        icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                        opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UL);
    }
  
#line 29398 "ad_x86.cpp"
  }
}

void string_indexofLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11688 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      (-1), opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::LL);
  
#line 29421 "ad_x86.cpp"
  }
}

void string_indexofUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11704 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      (-1), opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UU);
  
#line 29444 "ad_x86.cpp"
  }
}

void string_indexofULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11720 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      (-1), opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UL);
  
#line 29467 "ad_x86.cpp"
  }
}

void string_indexof_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vec1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vec3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11735 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof_char(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */, opnd_array(0)->as_Register(ra_,this)/* result */,
                           opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vec1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vec3 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp */);
  
#line 29490 "ad_x86.cpp"
  }
}

void stringL_indexof_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vec1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vec3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11748 "src/hotspot/cpu/x86/x86_32.ad"

    __ stringL_indexof_char(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */, opnd_array(0)->as_Register(ra_,this)/* result */,
                           opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vec1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vec3 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp */);
  
#line 29513 "ad_x86.cpp"
  }
}

void array_equalsBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11766 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(true, opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, false /* char */);
  
#line 29534 "ad_x86.cpp"
  }
}

void array_equalsCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11783 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(true, opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, true /* char */);
  
#line 29555 "ad_x86.cpp"
  }
}

void has_negativesNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// len
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11798 "src/hotspot/cpu/x86/x86_32.ad"

    __ has_negatives(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* len */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */);
  
#line 29576 "ad_x86.cpp"
  }
}

void string_compressNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11813 "src/hotspot/cpu/x86/x86_32.ad"

    __ char_array_compress(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                           opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */,
                           opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */);
  
#line 29600 "ad_x86.cpp"
  }
}

void string_inflateNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11828 "src/hotspot/cpu/x86/x86_32.ad"

    __ byte_array_inflate(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 29621 "ad_x86.cpp"
  }
}

void encode_iso_arrayNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11843 "src/hotspot/cpu/x86/x86_32.ad"

    __ encode_iso_array(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                        opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */,
                        opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */);
  
#line 29645 "ad_x86.cpp"
  }
}

void compI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 29661 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */, opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
  
#line 29669 "ad_x86.cpp"
  }
}

void compI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1626 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
  
#line 29693 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 29708 "ad_x86.cpp"
  }
}

void compI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 29724 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 29738 "ad_x86.cpp"
  }
}

void testI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 29754 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 29762 "ad_x86.cpp"
  }
}

void testI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 29779 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 29787 "ad_x86.cpp"
  }
  {

#line 1963 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  
#line 29796 "ad_x86.cpp"
  }
}

void testI_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 29813 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 29827 "ad_x86.cpp"
  }
}

void compU_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 29843 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */, opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
  
#line 29851 "ad_x86.cpp"
  }
}

void compU_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1626 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
  
#line 29875 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 29890 "ad_x86.cpp"
  }
}

void compU_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 29906 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 29920 "ad_x86.cpp"
  }
}

void testU_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 29936 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 29944 "ad_x86.cpp"
  }
}

void compP_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 29960 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */, opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
  
#line 29968 "ad_x86.cpp"
  }
}

void compP_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1626 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
  
#line 29992 "ad_x86.cpp"
  }
  {

#line 1638 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      
if ( opnd_array(2)->constant_reloc() != relocInfo::none ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), opnd_array(2)->constant_reloc(), 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
    }
  
#line 30012 "ad_x86.cpp"
  }
}

void compP_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 30028 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 30042 "ad_x86.cpp"
  }
}

void compP_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 30058 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 30072 "ad_x86.cpp"
  }
}

void testP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 30088 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 30096 "ad_x86.cpp"
  }
}

void testP_Reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 30112 "ad_x86.cpp"
  }
  {

#line 2164 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 30126 "ad_x86.cpp"
  }
  {

#line 1987 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d32(cbuf,(0xFFFFFFFF));
  
#line 30134 "ad_x86.cpp"
  }
}

void minI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2184 "src/hotspot/cpu/x86/x86_32.ad"

    // Compare dst,src
    emit_opcode(cbuf,0x3B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // jmp dst < src around move
    emit_opcode(cbuf,0x7C);
    emit_d8(cbuf,2);
    // move dst,src
    emit_opcode(cbuf,0x8B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 30158 "ad_x86.cpp"
  }
}

void maxI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2196 "src/hotspot/cpu/x86/x86_32.ad"

    // Compare dst,src
    emit_opcode(cbuf,0x3B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // jmp dst > src around move
    emit_opcode(cbuf,0x7F);
    emit_d8(cbuf,2);
    // move dst,src
    emit_opcode(cbuf,0x8B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 30182 "ad_x86.cpp"
  }
}

void loopLimit_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// init
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// limit
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// stride
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// limit_hi
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12127 "src/hotspot/cpu/x86/x86_32.ad"

    int strd = (int)opnd_array(3)->constant();
    assert(strd != 1 && strd != -1, "sanity");
    int m1 = (strd > 0) ? 1 : -1;
    // Convert limit to long (EAX:EDX)
    __ cdql();
    // Convert init to long (init:tmp)
    __ movl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* init */);
    __ sarl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, 31);
    // $limit - $init
    __ subl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, opnd_array(1)->as_Register(ra_,this,idx1)/* init */);
    __ sbbl(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    // + ($stride - 1)
    if (strd > 0) {
      __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, (strd - 1));
      __ adcl(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, 0);
      __ movl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, strd);
    } else {
      __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, (strd + 1));
      __ adcl(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, -1);
      __ lneg(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, opnd_array(2)->as_Register(ra_,this,idx2)/* limit */);
      __ movl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, -strd);
    }
    // signed devision: (EAX:EDX) / pos_stride
    __ idivl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    if (strd < 0) {
      // restore sign
      __ negl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    }
    // (EAX) * stride
    __ mull(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    // + init (ignore upper bits)
    __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, opnd_array(1)->as_Register(ra_,this,idx1)/* init */);
  
#line 30233 "ad_x86.cpp"
  }
}

void jumpXtndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  ra_->C->output()->constant_table().fill_jump_table(cbuf, (MachConstantNode*) this, _index2label);
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12171 "src/hotspot/cpu/x86/x86_32.ad"

    // Jump to Address(table_base + switch_reg)
    Address index(noreg, opnd_array(1)->as_Register(ra_,this,idx1)/* switch_val */, Address::times_1);
    __ jump(ArrayAddress(InternalAddress(__ code()->consts()->start() + constant_offset()), index));
  
#line 30252 "ad_x86.cpp"
  }
}

void jumpXtndNode::eval_constant(Compile* C) {
  _constant = C->output()->constant_table().add_jump_table(this);
  // User did not define an encode section.
}
void jmpDirNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12187 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(1)->label();
    __ jmp(*L, false); // Always long jump
  
#line 30273 "ad_x86.cpp"
  }
}

uint jmpDirNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 5, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 5);
}

void jmpConNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12202 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 30297 "ad_x86.cpp"
  }
}

uint jmpConNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12218 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 30321 "ad_x86.cpp"
  }
}

uint jmpLoopEndNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEndUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12234 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 30345 "ad_x86.cpp"
  }
}

uint jmpLoopEndUNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEndUCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12249 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 30369 "ad_x86.cpp"
  }
}

uint jmpLoopEndUCFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEnd_and_restoreMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12267 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
    __ restorevectmask();
  
#line 30394 "ad_x86.cpp"
  }
}

uint jmpLoopEnd_and_restoreMaskNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 10, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 10);
}

void jmpLoopEndU_and_restoreMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12285 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
    __ restorevectmask();
  
#line 30419 "ad_x86.cpp"
  }
}

uint jmpLoopEndU_and_restoreMaskNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 10, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 10);
}

void jmpLoopEndUCF_and_restoreMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12302 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
    __ restorevectmask();
  
#line 30444 "ad_x86.cpp"
  }
}

uint jmpLoopEndUCF_and_restoreMaskNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 10, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 10);
}

void jmpConUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12318 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 30468 "ad_x86.cpp"
  }
}

uint jmpConUNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpConUCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12332 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 30492 "ad_x86.cpp"
  }
}

uint jmpConUCFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpConUCF2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12354 "src/hotspot/cpu/x86/x86_32.ad"

    Label* l = opnd_array(3)->label();
    if (opnd_array(1)->ccode()== Assembler::notEqual) {
      __ jcc(Assembler::parity, *l, false);
      __ jcc(Assembler::notEqual, *l, false);
    } else if (opnd_array(1)->ccode()== Assembler::equal) {
      Label done;
      __ jccb(Assembler::parity, done);
      __ jcc(Assembler::equal, *l, false);
      __ bind(done);
    } else {
       ShouldNotReachHere();
    }
  
#line 30526 "ad_x86.cpp"
  }
}

void partialSubtypeCheckNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  {

#line 1711 "src/hotspot/cpu/x86/x86_32.ad"

    Register Redi = as_Register(EDI_enc); // result register
    Register Reax = as_Register(EAX_enc); // super class
    Register Recx = as_Register(ECX_enc); // killed
    Register Resi = as_Register(ESI_enc); // sub class
    Label miss;

    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(Resi, Reax, Recx, Redi,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
    if ((0x1 /*primary()*/)) {
      __ xorptr(Redi, Redi);
    }
    __ bind(miss);
  
#line 30555 "ad_x86.cpp"
  }
}

void partialSubtypeCheck_vs_ZeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 1711 "src/hotspot/cpu/x86/x86_32.ad"

    Register Redi = as_Register(EDI_enc); // result register
    Register Reax = as_Register(EAX_enc); // super class
    Register Recx = as_Register(ECX_enc); // killed
    Register Resi = as_Register(ESI_enc); // sub class
    Label miss;

    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(Resi, Reax, Recx, Redi,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
    if ((0x0 /*primary()*/)) {
      __ xorptr(Redi, Redi);
    }
    __ bind(miss);
  
#line 30585 "ad_x86.cpp"
  }
}

void jmpDir_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12433 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(1)->label();
    __ jmpb(*L);
  
#line 30602 "ad_x86.cpp"
  }
}

uint jmpDir_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpCon_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12449 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 30626 "ad_x86.cpp"
  }
}

uint jmpCon_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpLoopEnd_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12465 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 30650 "ad_x86.cpp"
  }
}

uint jmpLoopEnd_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpLoopEndU_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12481 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 30674 "ad_x86.cpp"
  }
}

uint jmpLoopEndU_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpLoopEndUCF_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12496 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 30698 "ad_x86.cpp"
  }
}

uint jmpLoopEndUCF_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConU_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12512 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 30722 "ad_x86.cpp"
  }
}

uint jmpConU_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConUCF_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12527 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 30746 "ad_x86.cpp"
  }
}

uint jmpConUCF_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConUCF2_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12551 "src/hotspot/cpu/x86/x86_32.ad"

    Label* l = opnd_array(3)->label();
    if (opnd_array(1)->ccode()== Assembler::notEqual) {
      __ jccb(Assembler::parity, *l);
      __ jccb(Assembler::notEqual, *l);
    } else if (opnd_array(1)->ccode()== Assembler::equal) {
      Label done;
      __ jccb(Assembler::parity, done);
      __ jccb(Assembler::equal, *l);
      __ bind(done);
    } else {
       ShouldNotReachHere();
    }
  
#line 30780 "ad_x86.cpp"
  }
}

uint jmpConUCF2_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpL3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12607 "src/hotspot/cpu/x86/x86_32.ad"

    Label p_one, m_one, done;
    __ xorptr(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ cmpl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */), HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* src2 */));
    __ jccb(Assembler::less,    m_one);
    __ jccb(Assembler::greater, p_one);
    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* src2 */);
    __ jccb(Assembler::below,   m_one);
    __ jccb(Assembler::equal,   done);
    __ bind(p_one);
    __ incrementl(opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ jmpb(done);
    __ bind(m_one);
    __ decrementl(opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ bind(done);
  
#line 30815 "ad_x86.cpp"
  }
}

void cmpL_zero_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 30831 "ad_x86.cpp"
  }
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 30839 "ad_x86.cpp"
  }
}

void cmpL_reg_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2839 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 30864 "ad_x86.cpp"
  }
}

void cmpUL_zero_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 30880 "ad_x86.cpp"
  }
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 30888 "ad_x86.cpp"
  }
}

void cmpUL_reg_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2839 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 30913 "ad_x86.cpp"
  }
}

void cmovLL_reg_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30932 "ad_x86.cpp"
  }
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 30940 "ad_x86.cpp"
  }
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30949 "ad_x86.cpp"
  }
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 30957 "ad_x86.cpp"
  }
}

void cmovLL_mem_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30976 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 30990 "ad_x86.cpp"
  }
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30999 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( opnd_array(4)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 31013 "ad_x86.cpp"
  }
}

void cmovII_reg_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31032 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 31040 "ad_x86.cpp"
  }
}

void cmovII_mem_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31059 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 31073 "ad_x86.cpp"
  }
}

void cmovPP_reg_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31092 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 31100 "ad_x86.cpp"
  }
}

void cmpL_zero_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2818 "src/hotspot/cpu/x86/x86_32.ad"

    // MOV   $tmp,$src.lo
    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
    // OR    $tmp,$src.hi
    emit_opcode(cbuf, 0x0B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 31122 "ad_x86.cpp"
  }
}

void cmpL_reg_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2827 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // JNE,s  skip
    emit_cc(cbuf, 0x70, 0x5);
    emit_d8(cbuf,2);
    // CMP    $src1.hi,$src2.hi
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 31146 "ad_x86.cpp"
  }
}

void cmpUL_zero_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2818 "src/hotspot/cpu/x86/x86_32.ad"

    // MOV   $tmp,$src.lo
    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
    // OR    $tmp,$src.hi
    emit_opcode(cbuf, 0x0B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 31168 "ad_x86.cpp"
  }
}

void cmpUL_reg_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2827 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // JNE,s  skip
    emit_cc(cbuf, 0x70, 0x5);
    emit_d8(cbuf,2);
    // CMP    $src1.hi,$src2.hi
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 31192 "ad_x86.cpp"
  }
}

void cmovLL_reg_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31211 "ad_x86.cpp"
  }
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 31219 "ad_x86.cpp"
  }
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31228 "ad_x86.cpp"
  }
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 31236 "ad_x86.cpp"
  }
}

void cmovLL_mem_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31255 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 31269 "ad_x86.cpp"
  }
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31278 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( opnd_array(4)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 31292 "ad_x86.cpp"
  }
}

void cmovII_reg_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31311 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 31319 "ad_x86.cpp"
  }
}

void cmovII_mem_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31338 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 31352 "ad_x86.cpp"
  }
}

void cmovPP_reg_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31371 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 31379 "ad_x86.cpp"
  }
}

void cmpL_zero_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2851 "src/hotspot/cpu/x86/x86_32.ad"

    // XOR    $tmp,$tmp
    emit_opcode(cbuf,0x33);  // XOR
    emit_rm(cbuf,0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    // CMP    $tmp,$src.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // SBB    $tmp,$src.hi
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */) );
  
#line 31404 "ad_x86.cpp"
  }
}

void cmpL_reg_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2839 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2)/* src2 */, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
  
#line 31429 "ad_x86.cpp"
  }
}

void cmpUL_zero_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2851 "src/hotspot/cpu/x86/x86_32.ad"

    // XOR    $tmp,$tmp
    emit_opcode(cbuf,0x33);  // XOR
    emit_rm(cbuf,0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    // CMP    $tmp,$src.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // SBB    $tmp,$src.hi
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */) );
  
#line 31454 "ad_x86.cpp"
  }
}

void cmpUL_reg_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2839 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2)/* src2 */, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
  
#line 31479 "ad_x86.cpp"
  }
}

void cmovLL_reg_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31498 "ad_x86.cpp"
  }
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 31506 "ad_x86.cpp"
  }
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31515 "ad_x86.cpp"
  }
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 31523 "ad_x86.cpp"
  }
}

void cmovLL_mem_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31542 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 31556 "ad_x86.cpp"
  }
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31565 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( opnd_array(4)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 31579 "ad_x86.cpp"
  }
}

void cmovLL_reg_LEGT_UNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31598 "ad_x86.cpp"
  }
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 31606 "ad_x86.cpp"
  }
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31615 "ad_x86.cpp"
  }
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 31623 "ad_x86.cpp"
  }
}

void cmovLL_mem_LEGT_UNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31642 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 31656 "ad_x86.cpp"
  }
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31665 "ad_x86.cpp"
  }
  {

#line 2084 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( opnd_array(4)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 31679 "ad_x86.cpp"
  }
}

void cmovII_reg_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31698 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 31706 "ad_x86.cpp"
  }
}

void cmovII_mem_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31725 "ad_x86.cpp"
  }
  {

#line 2074 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 31739 "ad_x86.cpp"
  }
}

void cmovPP_reg_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1693 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 31758 "ad_x86.cpp"
  }
  {

#line 1937 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 31766 "ad_x86.cpp"
  }
}

void CallStaticJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1786 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 31792 "ad_x86.cpp"
  }
  {

#line 1809 "src/hotspot/cpu/x86/x86_32.ad"

    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to determine
    // who we intended to call.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));

    if (!_method) {
      emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                     runtime_call_Relocation::spec(),
                     RELOC_IMM32);
    } else {
      int method_index = resolved_method_index(cbuf);
      RelocationHolder rspec = _optimized_virtual ? opt_virtual_call_Relocation::spec(method_index)
                                                  : static_call_Relocation::spec(method_index);
      emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                     rspec, RELOC_DISP32);
      // Emit stubs for static call.
      address stub = CompiledStaticCall::emit_to_interp_stub(cbuf);
      if (stub == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    }
  
#line 31821 "ad_x86.cpp"
  }
  {

#line 2361 "src/hotspot/cpu/x86/x86.ad"

    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      C2_MacroAssembler _masm(&cbuf);
      Label L;
      __ cmpptr(Address(rsp, framesize), (int32_t)0xbadb100d);
      __ jccb(Assembler::equal, L);
      // Die if stack mismatch
      __ int3();
      __ bind(L);
    }
  
#line 31839 "ad_x86.cpp"
  }
  {

#line 1801 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  
#line 31851 "ad_x86.cpp"
  }
}

void CallDynamicJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1786 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 31877 "ad_x86.cpp"
  }
  {

#line 1834 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ ic_call((address)opnd_array(1)->method(), resolved_method_index(cbuf));
  
#line 31886 "ad_x86.cpp"
  }
  {

#line 2361 "src/hotspot/cpu/x86/x86.ad"

    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      C2_MacroAssembler _masm(&cbuf);
      Label L;
      __ cmpptr(Address(rsp, framesize), (int32_t)0xbadb100d);
      __ jccb(Assembler::equal, L);
      // Die if stack mismatch
      __ int3();
      __ bind(L);
    }
  
#line 31904 "ad_x86.cpp"
  }
  {

#line 1801 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  
#line 31916 "ad_x86.cpp"
  }
}

void CallRuntimeDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1786 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 31942 "ad_x86.cpp"
  }
  {

#line 1728 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler masm(&cbuf);
    int start = masm.offset();
    if (UseSSE >= 2) {
      if (VerifyFPU) {
        masm.verify_FPU(0, "must be empty in SSE2+ mode");
      }
    } else {
      // External c_calling_convention expects the FPU stack to be 'clean'.
      // Compiled code leaves it dirty.  Do cleanup now.
      masm.empty_FPU_stack();
    }
    if (sizeof_FFree_Float_Stack_All == -1) {
      sizeof_FFree_Float_Stack_All = masm.offset() - start;
    } else {
      assert(masm.offset() - start == sizeof_FFree_Float_Stack_All, "wrong size");
    }
  
#line 31965 "ad_x86.cpp"
  }
  {

#line 1754 "src/hotspot/cpu/x86/x86_32.ad"

    // This is the instruction starting address for relocation info.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  
#line 32001 "ad_x86.cpp"
  }
  {

#line 1801 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  
#line 32013 "ad_x86.cpp"
  }
}

void CallLeafDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1786 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 32039 "ad_x86.cpp"
  }
  {

#line 1728 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler masm(&cbuf);
    int start = masm.offset();
    if (UseSSE >= 2) {
      if (VerifyFPU) {
        masm.verify_FPU(0, "must be empty in SSE2+ mode");
      }
    } else {
      // External c_calling_convention expects the FPU stack to be 'clean'.
      // Compiled code leaves it dirty.  Do cleanup now.
      masm.empty_FPU_stack();
    }
    if (sizeof_FFree_Float_Stack_All == -1) {
      sizeof_FFree_Float_Stack_All = masm.offset() - start;
    } else {
      assert(masm.offset() - start == sizeof_FFree_Float_Stack_All, "wrong size");
    }
  
#line 32062 "ad_x86.cpp"
  }
  {

#line 1754 "src/hotspot/cpu/x86/x86_32.ad"

    // This is the instruction starting address for relocation info.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  
#line 32098 "ad_x86.cpp"
  }
  {

#line 1747 "src/hotspot/cpu/x86/x86_32.ad"

    if( VerifyFPU ) {
      MacroAssembler masm(&cbuf);
      masm.verify_FPU( -3, "Returning from Runtime Leaf call");
    }
  
#line 32109 "ad_x86.cpp"
  }
  {

#line 1801 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  
#line 32121 "ad_x86.cpp"
  }
}

void CallLeafNoFPDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1786 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 32147 "ad_x86.cpp"
  }
  {

#line 1754 "src/hotspot/cpu/x86/x86_32.ad"

    // This is the instruction starting address for relocation info.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  
#line 32183 "ad_x86.cpp"
  }
}

void RetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC3 /*primary()*/));
  
#line 32198 "ad_x86.cpp"
  }
}

void TailCalljmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// method_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 32214 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* jump_target */ );
  
#line 32222 "ad_x86.cpp"
  }
}

void tailjmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// ex_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 2874 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0x5A);
  
#line 32238 "ad_x86.cpp"
  }
  {

#line 1536 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 32246 "ad_x86.cpp"
  }
  {

#line 1689 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* jump_target */ );
  
#line 32254 "ad_x86.cpp"
  }
}

void CreateExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void RethrowExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 2878 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0xE9);        // jmp    entry
    emit_d32_reloc(cbuf, (int)OptoRuntime::rethrow_stub() - ((int)cbuf.insts_end())-4,
                   runtime_call_Relocation::spec(), RELOC_IMM32 );
  
#line 32285 "ad_x86.cpp"
  }
}

void cmpFastLockRTMNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scr
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// cx1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// cx2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13310 "src/hotspot/cpu/x86/x86_32.ad"

    __ fast_lock(opnd_array(1)->as_Register(ra_,this,idx1)/* object */, opnd_array(2)->as_Register(ra_,this,idx2)/* box */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */,
                 opnd_array(4)->as_Register(ra_,this,idx4)/* scr */, opnd_array(5)->as_Register(ra_,this,idx5)/* cx1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* cx2 */,
                 _counters, _rtm_counters, _stack_rtm_counters,
                 ((Method*)(ra_->C->method()->constant_encoding()))->method_data(),
                 true, ra_->C->profile_rtm());
  
#line 32310 "ad_x86.cpp"
  }
}

void cmpFastLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13326 "src/hotspot/cpu/x86/x86_32.ad"

    __ fast_lock(opnd_array(1)->as_Register(ra_,this,idx1)/* object */, opnd_array(2)->as_Register(ra_,this,idx2)/* box */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */,
                 opnd_array(4)->as_Register(ra_,this,idx4)/* scr */, noreg, noreg, _counters, NULL, NULL, NULL, false, false);
  
#line 32330 "ad_x86.cpp"
  }
}

void cmpFastUnlockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13338 "src/hotspot/cpu/x86/x86_32.ad"

    __ fast_unlock(opnd_array(1)->as_Register(ra_,this,idx1)/* object */, opnd_array(2)->as_Register(ra_,this,idx2)/* box */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, ra_->C->use_rtm());
  
#line 32348 "ad_x86.cpp"
  }
}

void safePoint_poll_tlsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13356 "src/hotspot/cpu/x86/x86_32.ad"

    __ relocate(relocInfo::poll_type);
    address pre_pc = __ pc();
    __ testl(rax, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* poll */, 0));
    address post_pc = __ pc();
    guarantee(pre_pc[0] == 0x85, "must emit test-ax [reg]");
  
#line 32368 "ad_x86.cpp"
  }
}

uint safePoint_poll_tlsNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void tlsLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13376 "src/hotspot/cpu/x86/x86_32.ad"

    Register dstReg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ get_thread(dstReg);
  
#line 32390 "ad_x86.cpp"
  }
}

void ShouldNotReachHereNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2542 "src/hotspot/cpu/x86/x86.ad"

    if (is_reachable()) {
      __ stop(_halt_reason);
    }
  
#line 32408 "ad_x86.cpp"
  }
}

void setMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2557 "src/hotspot/cpu/x86/x86.ad"

    __ setvectmask(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 32425 "ad_x86.cpp"
  }
}

void addF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2571 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32442 "ad_x86.cpp"
  }
}

void addF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2583 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32459 "ad_x86.cpp"
  }
}

void addF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2583 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 32476 "ad_x86.cpp"
  }
}

void addF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2594 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32493 "ad_x86.cpp"
  }
}

void addF_immNode::eval_constant(Compile* C) {
  {

#line 2595 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32502 "ad_x86.cpp"
  }
}
void addF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2606 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32518 "ad_x86.cpp"
  }
}

void addF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2618 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32535 "ad_x86.cpp"
  }
}

void addF_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2618 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 32552 "ad_x86.cpp"
  }
}

void addF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2630 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32569 "ad_x86.cpp"
  }
}

void addF_reg_immNode::eval_constant(Compile* C) {
  {

#line 2631 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32578 "ad_x86.cpp"
  }
}
void addD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2642 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32594 "ad_x86.cpp"
  }
}

void addD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2654 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32611 "ad_x86.cpp"
  }
}

void addD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2654 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 32628 "ad_x86.cpp"
  }
}

void addD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2665 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32645 "ad_x86.cpp"
  }
}

void addD_immNode::eval_constant(Compile* C) {
  {

#line 2666 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32654 "ad_x86.cpp"
  }
}
void addD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2677 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32670 "ad_x86.cpp"
  }
}

void addD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2689 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32687 "ad_x86.cpp"
  }
}

void addD_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2689 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 32704 "ad_x86.cpp"
  }
}

void addD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2701 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32721 "ad_x86.cpp"
  }
}

void addD_reg_immNode::eval_constant(Compile* C) {
  {

#line 2702 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32730 "ad_x86.cpp"
  }
}
void subF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2713 "src/hotspot/cpu/x86/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32746 "ad_x86.cpp"
  }
}

void subF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2725 "src/hotspot/cpu/x86/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32763 "ad_x86.cpp"
  }
}

void subF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2736 "src/hotspot/cpu/x86/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32780 "ad_x86.cpp"
  }
}

void subF_immNode::eval_constant(Compile* C) {
  {

#line 2737 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32789 "ad_x86.cpp"
  }
}
void subF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2748 "src/hotspot/cpu/x86/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32805 "ad_x86.cpp"
  }
}

void subF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2760 "src/hotspot/cpu/x86/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32822 "ad_x86.cpp"
  }
}

void subF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2772 "src/hotspot/cpu/x86/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32839 "ad_x86.cpp"
  }
}

void subF_reg_immNode::eval_constant(Compile* C) {
  {

#line 2773 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32848 "ad_x86.cpp"
  }
}
void subD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2784 "src/hotspot/cpu/x86/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32864 "ad_x86.cpp"
  }
}

void subD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2796 "src/hotspot/cpu/x86/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32881 "ad_x86.cpp"
  }
}

void subD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2807 "src/hotspot/cpu/x86/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32898 "ad_x86.cpp"
  }
}

void subD_immNode::eval_constant(Compile* C) {
  {

#line 2808 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32907 "ad_x86.cpp"
  }
}
void subD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2819 "src/hotspot/cpu/x86/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32923 "ad_x86.cpp"
  }
}

void subD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2831 "src/hotspot/cpu/x86/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32940 "ad_x86.cpp"
  }
}

void subD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2843 "src/hotspot/cpu/x86/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32957 "ad_x86.cpp"
  }
}

void subD_reg_immNode::eval_constant(Compile* C) {
  {

#line 2844 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32966 "ad_x86.cpp"
  }
}
void mulF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2855 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32982 "ad_x86.cpp"
  }
}

void mulF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2867 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32999 "ad_x86.cpp"
  }
}

void mulF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2867 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 33016 "ad_x86.cpp"
  }
}

void mulF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2878 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33033 "ad_x86.cpp"
  }
}

void mulF_immNode::eval_constant(Compile* C) {
  {

#line 2879 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 33042 "ad_x86.cpp"
  }
}
void mulF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2890 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 33058 "ad_x86.cpp"
  }
}

void mulF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2902 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 33075 "ad_x86.cpp"
  }
}

void mulF_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2902 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 33092 "ad_x86.cpp"
  }
}

void mulF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2914 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33109 "ad_x86.cpp"
  }
}

void mulF_reg_immNode::eval_constant(Compile* C) {
  {

#line 2915 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 33118 "ad_x86.cpp"
  }
}
void mulD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2926 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33134 "ad_x86.cpp"
  }
}

void mulD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2938 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 33151 "ad_x86.cpp"
  }
}

void mulD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2938 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 33168 "ad_x86.cpp"
  }
}

void mulD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2949 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33185 "ad_x86.cpp"
  }
}

void mulD_immNode::eval_constant(Compile* C) {
  {

#line 2950 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 33194 "ad_x86.cpp"
  }
}
void mulD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2961 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 33210 "ad_x86.cpp"
  }
}

void mulD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2973 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 33227 "ad_x86.cpp"
  }
}

void mulD_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2973 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 33244 "ad_x86.cpp"
  }
}

void mulD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2985 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33261 "ad_x86.cpp"
  }
}

void mulD_reg_immNode::eval_constant(Compile* C) {
  {

#line 2986 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 33270 "ad_x86.cpp"
  }
}
void divF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2997 "src/hotspot/cpu/x86/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33286 "ad_x86.cpp"
  }
}

void divF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3009 "src/hotspot/cpu/x86/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 33303 "ad_x86.cpp"
  }
}

void divF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3020 "src/hotspot/cpu/x86/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33320 "ad_x86.cpp"
  }
}

void divF_immNode::eval_constant(Compile* C) {
  {

#line 3021 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 33329 "ad_x86.cpp"
  }
}
void divF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3032 "src/hotspot/cpu/x86/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 33345 "ad_x86.cpp"
  }
}

void divF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3044 "src/hotspot/cpu/x86/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 33362 "ad_x86.cpp"
  }
}

void divF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3056 "src/hotspot/cpu/x86/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33379 "ad_x86.cpp"
  }
}

void divF_reg_immNode::eval_constant(Compile* C) {
  {

#line 3057 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 33388 "ad_x86.cpp"
  }
}
void divD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3068 "src/hotspot/cpu/x86/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33404 "ad_x86.cpp"
  }
}

void divD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3080 "src/hotspot/cpu/x86/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 33421 "ad_x86.cpp"
  }
}

void divD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3091 "src/hotspot/cpu/x86/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33438 "ad_x86.cpp"
  }
}

void divD_immNode::eval_constant(Compile* C) {
  {

#line 3092 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 33447 "ad_x86.cpp"
  }
}
void divD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3103 "src/hotspot/cpu/x86/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 33463 "ad_x86.cpp"
  }
}

void divD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3115 "src/hotspot/cpu/x86/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 33480 "ad_x86.cpp"
  }
}

void divD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3127 "src/hotspot/cpu/x86/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33497 "ad_x86.cpp"
  }
}

void divD_reg_immNode::eval_constant(Compile* C) {
  {

#line 3128 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 33506 "ad_x86.cpp"
  }
}
void absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3138 "src/hotspot/cpu/x86/x86.ad"

    __ andps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(float_signmask()));
  
#line 33521 "ad_x86.cpp"
  }
}

void absF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3149 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_128bit;
    __ vandps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
              ExternalAddress(float_signmask()), vlen_enc);
  
#line 33539 "ad_x86.cpp"
  }
}

void absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3163 "src/hotspot/cpu/x86/x86.ad"

    __ andpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(double_signmask()));
  
#line 33555 "ad_x86.cpp"
  }
}

void absD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3175 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_128bit;
    __ vandpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
              ExternalAddress(double_signmask()), vlen_enc);
  
#line 33573 "ad_x86.cpp"
  }
}

void negF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3188 "src/hotspot/cpu/x86/x86.ad"

    __ xorps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(float_signflip()));
  
#line 33589 "ad_x86.cpp"
  }
}

void negF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3199 "src/hotspot/cpu/x86/x86.ad"

    __ vnegatess(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
                 ExternalAddress(float_signflip()));
  
#line 33606 "ad_x86.cpp"
  }
}

void negD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3212 "src/hotspot/cpu/x86/x86.ad"

    __ xorpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(double_signflip()));
  
#line 33622 "ad_x86.cpp"
  }
}

void negD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3224 "src/hotspot/cpu/x86/x86.ad"

    __ vnegatesd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
                 ExternalAddress(double_signflip()));
  
#line 33639 "ad_x86.cpp"
  }
}

void sqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3237 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 33655 "ad_x86.cpp"
  }
}

void sqrtF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3249 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 33671 "ad_x86.cpp"
  }
}

void sqrtF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3261 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33687 "ad_x86.cpp"
  }
}

void sqrtF_immNode::eval_constant(Compile* C) {
  {

#line 3262 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 33696 "ad_x86.cpp"
  }
}
void sqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3273 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 33711 "ad_x86.cpp"
  }
}

void sqrtD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3285 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 33727 "ad_x86.cpp"
  }
}

void sqrtD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3296 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33743 "ad_x86.cpp"
  }
}

void sqrtD_immNode::eval_constant(Compile* C) {
  {

#line 3297 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 33752 "ad_x86.cpp"
  }
}
void reinterpretNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3309 "src/hotspot/cpu/x86/x86.ad"

    // empty
  
#line 33767 "ad_x86.cpp"
  }
}

void reinterpret_expandNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3322 "src/hotspot/cpu/x86/x86.ad"

    assert(vector_length_in_bytes(this)       <= 16, "required");
    assert(vector_length_in_bytes(this, opnd_array(1)) <=  8, "required");

    int src_vlen_in_bytes = vector_length_in_bytes(this, opnd_array(1));
    if (src_vlen_in_bytes == 4) {
      __ movdqu(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, ExternalAddress(vector_32_bit_mask()), opnd_array(3)->as_Register(ra_,this,idx3)/* scratch */);
    } else {
      assert(src_vlen_in_bytes == 8, "");
      __ movdqu(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, ExternalAddress(vector_64_bit_mask()), opnd_array(3)->as_Register(ra_,this,idx3)/* scratch */);
    }
    __ pand(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 33795 "ad_x86.cpp"
  }
}

void vreinterpret_expand4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3346 "src/hotspot/cpu/x86/x86.ad"

    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_32_bit_mask()), 0, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
  
#line 33812 "ad_x86.cpp"
  }
}

void vreinterpret_expandNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3360 "src/hotspot/cpu/x86/x86.ad"

    switch (vector_length_in_bytes(this, opnd_array(1))) {
      case  8: __ movq   (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      case 16: __ movdqu (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      case 32: __ vmovdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      default: ShouldNotReachHere();
    }
  
#line 33833 "ad_x86.cpp"
  }
}

void reinterpret_shrinkNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3376 "src/hotspot/cpu/x86/x86.ad"

    switch (vector_length_in_bytes(this)) {
      case  4: __ movfltz(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      case  8: __ movq   (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      case 16: __ movdqu (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      case 32: __ vmovdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      default: ShouldNotReachHere();
    }
  
#line 33855 "ad_x86.cpp"
  }
}

void onspinwaitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3480 "src/hotspot/cpu/x86/x86.ad"

    __ pause();
  
#line 33871 "ad_x86.cpp"
  }
}

void fmaD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3492 "src/hotspot/cpu/x86/x86.ad"

    __ fmad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */);
  
#line 33889 "ad_x86.cpp"
  }
}

void fmaF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3504 "src/hotspot/cpu/x86/x86.ad"

    __ fmaf(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */);
  
#line 33907 "ad_x86.cpp"
  }
}

void MoveVec2LegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3516 "src/hotspot/cpu/x86/x86.ad"

    ShouldNotReachHere();
  
#line 33923 "ad_x86.cpp"
  }
}

void MoveLeg2VecNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3525 "src/hotspot/cpu/x86/x86.ad"

    ShouldNotReachHere();
  
#line 33939 "ad_x86.cpp"
  }
}

void loadVNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3538 "src/hotspot/cpu/x86/x86.ad"

    switch (vector_length_in_bytes(this)) {
      case  4: __ movdl    (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc())); break;
      case  8: __ movq     (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc())); break;
      case 16: __ movdqu   (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc())); break;
      case 32: __ vmovdqu  (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc())); break;
      case 64: __ evmovdqul(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), Assembler::AVX_512bit); break;
      default: ShouldNotReachHere();
    }
  
#line 33962 "ad_x86.cpp"
  }
}

void storeVNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3556 "src/hotspot/cpu/x86/x86.ad"

    switch (vector_length_in_bytes(this, opnd_array(2))) {
      case  4: __ movdl    (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case  8: __ movq     (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case 16: __ movdqu   (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case 32: __ vmovdqu  (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case 64: __ evmovdqul(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Assembler::AVX_512bit); break;
      default: ShouldNotReachHere();
    }
  
#line 33986 "ad_x86.cpp"
  }
}

void gatherNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3578 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "sanity");

    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = vector_element_basic_type(this);

    assert(vector_length_in_bytes(this) >= 16, "sanity");
    assert(!is_subword_type(elem_bt), "sanity"); // T_INT, T_LONG, T_FLOAT, T_DOUBLE

    if (vlen_enc == Assembler::AVX_128bit) {
      __ movdqu(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* mask */, ExternalAddress(vector_all_bits_set()));
    } else {
      __ vmovdqu(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* mask */, ExternalAddress(vector_all_bits_set()));
    }
    __ lea(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ vgather(elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* idx */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* mask */, vlen_enc);
  
#line 34020 "ad_x86.cpp"
  }
}

void evgatherNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3603 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "sanity");

    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = vector_element_basic_type(this);

    assert(!is_subword_type(elem_bt), "sanity"); // T_INT, T_LONG, T_FLOAT, T_DOUBLE

    KRegister ktmp = k2;
    __ kmovwl(k2, ExternalAddress(vector_all_bits_set()), opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */);
    __ lea(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ evgather(elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ktmp, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* idx */, vlen_enc);
  
#line 34049 "ad_x86.cpp"
  }
}

void scatterNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3627 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "sanity");

    int vlen_enc = vector_length_encoding(this, opnd_array(2));
    BasicType elem_bt = vector_element_basic_type(this, opnd_array(2));

    assert(vector_length_in_bytes(this, opnd_array(2)) >= 16, "sanity");
    assert(!is_subword_type(elem_bt), "sanity"); // T_INT, T_LONG, T_FLOAT, T_DOUBLE

    KRegister ktmp = k2;
    __ kmovwl(k2, ExternalAddress(vector_all_bits_set()), opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */);
    __ lea(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ evscatter(elem_bt, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* idx */, ktmp, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, vlen_enc);
  
#line 34079 "ad_x86.cpp"
  }
}

void ReplB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3650 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 64 || VM_Version::supports_avx512vlbw()) { // AVX512VL for <512bit operands
      assert(VM_Version::supports_avx512bw(), "required"); // 512-bit byte vectors assume AVX512BW
      int vlen_enc = vector_length_encoding(this);
      __ evpbroadcastb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, vlen_enc);
    } else if (VM_Version::supports_avx2()) {
      int vlen_enc = vector_length_encoding(this);
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ vpbroadcastb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ punpcklbw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
      __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
      if (vlen >= 16) {
        __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        if (vlen >= 32) {
          assert(vlen == 32, "sanity");
          __ vinserti128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        }
      }
    }
  
#line 34115 "ad_x86.cpp"
  }
}

void ReplB_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3680 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpbroadcastb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 34132 "ad_x86.cpp"
  }
}

void ReplB_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3690 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    InternalAddress const_addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    if (vlen == 4) {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
    } else {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      if (vlen >= 16) {
        if (VM_Version::supports_avx2()) {
          int vlen_enc = vector_length_encoding(this);
          __ vpbroadcastq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
        } else {
          assert(vlen == 16, "sanity");
          __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        }
      }
    }
  
#line 34163 "ad_x86.cpp"
  }
}

void ReplB_immNode::eval_constant(Compile* C) {
  {

#line 3692 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 1));
#line 34172 "ad_x86.cpp"
  }
}
void ReplB_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3715 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 16) {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      // Use vpxor since AVX512F does not have 512bit vxorpd (requires AVX512DQ).
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 34194 "ad_x86.cpp"
  }
}

void ReplS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3733 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 32 || VM_Version::supports_avx512vlbw()) { // AVX512VL for <512bit operands
      assert(VM_Version::supports_avx512bw(), "required"); // 512-bit short vectors assume AVX512BW
      int vlen_enc = vector_length_encoding(this);
      __ evpbroadcastw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, vlen_enc);
    } else if (VM_Version::supports_avx2()) {
      int vlen_enc = vector_length_encoding(this);
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ vpbroadcastw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
      if (vlen >= 8) {
        __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        if (vlen >= 16) {
          assert(vlen == 16, "sanity");
          __ vinserti128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        }
      }
    }
  
#line 34229 "ad_x86.cpp"
  }
}

void ReplS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3762 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpbroadcastw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 34246 "ad_x86.cpp"
  }
}

void ReplS_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3772 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    InternalAddress const_addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    if (vlen == 2) {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
    } else {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      if (vlen >= 8) {
        if (VM_Version::supports_avx2()) {
          int vlen_enc = vector_length_encoding(this);
          __ vpbroadcastw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
        } else {
          assert(vlen == 8, "sanity");
          __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        }
      }
    }
  
#line 34277 "ad_x86.cpp"
  }
}

void ReplS_immNode::eval_constant(Compile* C) {
  {

#line 3774 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 2));
#line 34286 "ad_x86.cpp"
  }
}
void ReplS_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3796 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 8) {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 34307 "ad_x86.cpp"
  }
}

void ReplI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3813 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 16 || VM_Version::supports_avx512vl()) { // AVX512VL for <512bit operands
      int vlen_enc = vector_length_encoding(this);
      __ evpbroadcastd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, vlen_enc);
    } else if (VM_Version::supports_avx2()) {
      int vlen_enc = vector_length_encoding(this);
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ vpbroadcastd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
      if (vlen >= 8) {
        assert(vlen == 8, "sanity");
        __ vinserti128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
      }
    }
  
#line 34338 "ad_x86.cpp"
  }
}

void ReplI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3837 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    } else {
      assert(VM_Version::supports_avx2(), "sanity");
      int vlen_enc = vector_length_encoding(this);
      __ vpbroadcastd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
    }
  
#line 34362 "ad_x86.cpp"
  }
}

void ReplI_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3854 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    InternalAddress const_addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    if (vlen <= 4) {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      if (vlen == 4) {
        __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
      }
    } else {
      assert(VM_Version::supports_avx2(), "sanity");
      int vlen_enc = vector_length_encoding(this);
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      __ vpbroadcastd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 34390 "ad_x86.cpp"
  }
}

void ReplI_immNode::eval_constant(Compile* C) {
  {

#line 3856 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 4));
#line 34399 "ad_x86.cpp"
  }
}
void ReplI_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3876 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 34420 "ad_x86.cpp"
  }
}

void ReplI_M1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3895 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vector_len);
  
#line 34438 "ad_x86.cpp"
  }
}

void ReplI_M1_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3895 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vector_len);
  
#line 34456 "ad_x86.cpp"
  }
}

void ReplI_M1_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3895 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vector_len);
  
#line 34474 "ad_x86.cpp"
  }
}

void ReplL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3938 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ punpcklqdq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
    } else if (VM_Version::supports_avx512vl()) { // AVX512VL for <512bit operands
      int vlen_enc = Assembler::AVX_256bit;
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ vpbroadcastq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vlen_enc);
    } else {
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ punpcklqdq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
      __ vinserti128_high(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
    }
  
#line 34510 "ad_x86.cpp"
  }
}

void ReplL_reg_legNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3967 "src/hotspot/cpu/x86/x86.ad"

    if (VM_Version::supports_avx512vl()) {
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ punpcklqdq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
      __ vinserti128_high(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
      __ vinserti64x4(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, 0x1);
    } else {
      int vlen_enc = Assembler::AVX_512bit;
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ vpbroadcastq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vlen_enc);
    }
  
#line 34541 "ad_x86.cpp"
  }
}

void ReplL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3990 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
      __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      assert(VM_Version::supports_avx2(), "sanity");
      int vlen_enc = vector_length_encoding(this);
      __ vpbroadcastq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
    }
  
#line 34565 "ad_x86.cpp"
  }
}

void ReplL_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4008 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    InternalAddress const_addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    if (vlen == 2) {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      assert(VM_Version::supports_avx2(), "sanity");
      int vlen_enc = vector_length_encoding(this);
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      __ vpbroadcastq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 34591 "ad_x86.cpp"
  }
}

void ReplL_immNode::eval_constant(Compile* C) {
  {

#line 4010 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 34600 "ad_x86.cpp"
  }
}
void ReplL_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4027 "src/hotspot/cpu/x86/x86.ad"

    int vlen = vector_length(this);
    if (vlen == 2) {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 34621 "ad_x86.cpp"
  }
}

void ReplL_M1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4044 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vector_len);
  
#line 34639 "ad_x86.cpp"
  }
}

void ReplF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4056 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x00);
   } else if (VM_Version::supports_avx2()) {
      int vlen_enc = vector_length_encoding(this);
      __ vbroadcastss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc); // reg-to-reg variant requires AVX2
    } else {
      assert(vlen == 8, "sanity");
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x00);
      __ vinsertf128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 34665 "ad_x86.cpp"
  }
}

void ReplF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4075 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    } else {
      assert(VM_Version::supports_avx(), "sanity");
      int vlen_enc = vector_length_encoding(this);
      __ vbroadcastss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
    }
  
#line 34689 "ad_x86.cpp"
  }
}

void ReplF_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4092 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ xorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */,opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc); // 512bit vxorps requires AVX512DQ
    }
  
#line 34711 "ad_x86.cpp"
  }
}

void ReplD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4110 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x44);
    } else if (VM_Version::supports_avx2()) {
      int vlen_enc = vector_length_encoding(this);
      __ vbroadcastsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc); // reg-to-reg variant requires AVX2
    } else {
      assert(vlen == 4, "sanity");
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x44);
      __ vinsertf128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 34737 "ad_x86.cpp"
  }
}

void ReplD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4129 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x44);
    } else {
      assert(VM_Version::supports_avx(), "sanity");
      int vlen_enc = vector_length_encoding(this);
      __ vbroadcastsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
    }
  
#line 34761 "ad_x86.cpp"
  }
}

void ReplD_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4146 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      __ xorpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc); // 512bit vxorps requires AVX512DQ
    }
  
#line 34783 "ad_x86.cpp"
  }
}

void insertNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// val
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4164 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");
    assert(vector_length_in_bytes(this) >= 8, "required");

    BasicType elem_bt = vector_element_basic_type(this);

    assert(is_integral_type(elem_bt), "");
    assert(opnd_array(3)->constant()< (int)vector_length(this), "out of bounds");

    __ insert(elem_bt, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* val */, opnd_array(3)->constant());
  
#line 34809 "ad_x86.cpp"
  }
}

void insert32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// val
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4183 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_256bit;
    BasicType elem_bt = vector_element_basic_type(this);
    int elem_per_lane = 16/type2aelembytes(elem_bt);
    int log2epr = log2(elem_per_lane);

    assert(is_integral_type(elem_bt), "sanity");
    assert(opnd_array(3)->constant()< (int)vector_length(this), "out of bounds");

    uint x_idx = opnd_array(3)->constant()& right_n_bits(log2epr);
    uint y_idx = (opnd_array(3)->constant()>> log2epr) & 1;
    __ vextracti128(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, y_idx);
    __ vinsert(elem_bt, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* val */, x_idx);
    __ vinserti128(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, y_idx);
  
#line 34840 "ad_x86.cpp"
  }
}

void insert64Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// val
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4206 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "sanity");

    BasicType elem_bt = vector_element_basic_type(this);
    int elem_per_lane = 16/type2aelembytes(elem_bt);
    int log2epr = log2(elem_per_lane);

    assert(is_integral_type(elem_bt), "");
    assert(opnd_array(3)->constant()< (int)vector_length(this), "out of bounds");

    uint x_idx = opnd_array(3)->constant()& right_n_bits(log2epr);
    uint y_idx = (opnd_array(3)->constant()>> log2epr) & 3;
    __ vextracti32x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, y_idx);
    __ vinsert(elem_bt, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* val */, x_idx);
    __ vinserti32x4(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, y_idx);
  
#line 34872 "ad_x86.cpp"
  }
}

void insertFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// val
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4282 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "sanity");

    assert(vector_element_basic_type(this) == T_FLOAT, "sanity");
    assert(opnd_array(3)->constant()< (int)vector_length(this), "out of bounds");

    __ insertps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* val */, opnd_array(3)->constant());
  
#line 34895 "ad_x86.cpp"
  }
}

void vinsertFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// val
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4298 "src/hotspot/cpu/x86/x86.ad"

    assert(vector_element_basic_type(this) == T_FLOAT, "sanity");
    assert(opnd_array(3)->constant()< (int)vector_length(this), "out of bounds");

    int vlen = vector_length(this);
    uint x_idx = opnd_array(3)->constant()& right_n_bits(2);
    if (vlen == 8) {
      uint y_idx = (opnd_array(3)->constant()>> 2) & 1;
      int vlen_enc = Assembler::AVX_256bit;
      __ vextracti128(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, y_idx);
      __ vinsertps(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* val */, x_idx);
      __ vinserti128(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, y_idx);
    } else {
      assert(vlen == 16, "sanity");
      uint y_idx = (opnd_array(3)->constant()>> 2) & 3;
      __ vextracti32x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, y_idx);
      __ vinsertps(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* val */, x_idx);
      __ vinserti32x4(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, y_idx);
    }
  
#line 34931 "ad_x86.cpp"
  }
}

void reductionINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4393 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 34952 "ad_x86.cpp"
  }
}

void reductionI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4393 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 34973 "ad_x86.cpp"
  }
}

void reductionI_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4393 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 34994 "ad_x86.cpp"
  }
}

void reductionI_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4393 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35015 "ad_x86.cpp"
  }
}

void reductionI_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4393 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35036 "ad_x86.cpp"
  }
}

void reductionI_4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4393 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35057 "ad_x86.cpp"
  }
}

void reductionI_5Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4393 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35078 "ad_x86.cpp"
  }
}

void reductionF128Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4451 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 35098 "ad_x86.cpp"
  }
}

void reductionF128_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4451 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 35118 "ad_x86.cpp"
  }
}

void reduction8FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4465 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35139 "ad_x86.cpp"
  }
}

void reduction8F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4465 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35160 "ad_x86.cpp"
  }
}

void reduction16FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4479 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35181 "ad_x86.cpp"
  }
}

void reduction16F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4479 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35202 "ad_x86.cpp"
  }
}

void reduction2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4495 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);

#line 35222 "ad_x86.cpp"
  }
}

void reduction2D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4495 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);

#line 35242 "ad_x86.cpp"
  }
}

void reduction4DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4509 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35263 "ad_x86.cpp"
  }
}

void reduction4D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4509 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35284 "ad_x86.cpp"
  }
}

void reduction8DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4523 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35305 "ad_x86.cpp"
  }
}

void reduction8D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4523 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35326 "ad_x86.cpp"
  }
}

void reductionSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4584 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35347 "ad_x86.cpp"
  }
}

void reductionS_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4584 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35368 "ad_x86.cpp"
  }
}

void reductionS_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4584 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35389 "ad_x86.cpp"
  }
}

void reductionS_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4584 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35410 "ad_x86.cpp"
  }
}

void reductionS_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4584 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35431 "ad_x86.cpp"
  }
}

void reductionS_4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4584 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35452 "ad_x86.cpp"
  }
}

void reductionS_5Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4584 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 35473 "ad_x86.cpp"
  }
}

void mul_reductionBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4600 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ mulreduceB(opcode, vlen, opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
  
#line 35495 "ad_x86.cpp"
  }
}

void mul_reduction64BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4614 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ mulreduceB(opcode, vlen, opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
  
#line 35517 "ad_x86.cpp"
  }
}

void minmax_reduction2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// atmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// btmp
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4634 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */,
                         opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* atmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* btmp */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_1 */);
  
#line 35544 "ad_x86.cpp"
  }
}

void minmax_reduction2F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// atmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// btmp
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4634 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */,
                         opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* atmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* btmp */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_1 */);
  
#line 35571 "ad_x86.cpp"
  }
}

void minmax_reductionFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// atmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// btmp
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_0
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4655 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */,
                         opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* atmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* btmp */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_0 */, opnd_array(8)->as_XMMRegister(ra_,this,idx8)/* xmm_1 */);
  
#line 35599 "ad_x86.cpp"
  }
}

void minmax_reductionF_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// atmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// btmp
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_0
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4655 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */,
                         opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* atmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* btmp */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_0 */, opnd_array(8)->as_XMMRegister(ra_,this,idx8)/* xmm_1 */);
  
#line 35627 "ad_x86.cpp"
  }
}

void minmax_reduction2F_avNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4674 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */,
                         opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xmm_1 */);
  
#line 35653 "ad_x86.cpp"
  }
}

void minmax_reduction2F_av_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4674 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */,
                         opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xmm_1 */);
  
#line 35679 "ad_x86.cpp"
  }
}

void minmax_reductionF_avNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xmm_0
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4694 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */,
                         opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xmm_0 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_1 */);
  
#line 35706 "ad_x86.cpp"
  }
}

void minmax_reductionF_av_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xmm_0
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4694 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */,
                         opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xmm_0 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_1 */);
  
#line 35733 "ad_x86.cpp"
  }
}

void minmax_reduction2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4718 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */);
  
#line 35760 "ad_x86.cpp"
  }
}

void minmax_reduction2D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4718 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */);
  
#line 35787 "ad_x86.cpp"
  }
}

void minmax_reductionDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4740 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, opnd_array(8)->as_XMMRegister(ra_,this,idx8)/* tmp5 */);
  
#line 35815 "ad_x86.cpp"
  }
}

void minmax_reductionD_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4740 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, opnd_array(8)->as_XMMRegister(ra_,this,idx8)/* tmp5 */);
  
#line 35843 "ad_x86.cpp"
  }
}

void minmax_reduction2D_avNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4761 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */,
                          opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp4 */);
  
#line 35869 "ad_x86.cpp"
  }
}

void minmax_reduction2D_av_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4761 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */,
                          opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp4 */);
  
#line 35895 "ad_x86.cpp"
  }
}

void minmax_reductionD_avNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4781 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */,
                          opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp4 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp5 */);
  
#line 35922 "ad_x86.cpp"
  }
}

void minmax_reductionD_av_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4781 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */,
                          opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp4 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp5 */);
  
#line 35949 "ad_x86.cpp"
  }
}

void vaddBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4801 "src/hotspot/cpu/x86/x86.ad"

    __ paddb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35966 "ad_x86.cpp"
  }
}

void vaddB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4811 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 35984 "ad_x86.cpp"
  }
}

void vaddB_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4822 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36002 "ad_x86.cpp"
  }
}

void vaddB_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4822 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36020 "ad_x86.cpp"
  }
}

void vaddSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4834 "src/hotspot/cpu/x86/x86.ad"

    __ paddw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36037 "ad_x86.cpp"
  }
}

void vaddS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4844 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36055 "ad_x86.cpp"
  }
}

void vaddS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4855 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36073 "ad_x86.cpp"
  }
}

void vaddS_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4855 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36091 "ad_x86.cpp"
  }
}

void vaddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4867 "src/hotspot/cpu/x86/x86.ad"

    __ paddd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36108 "ad_x86.cpp"
  }
}

void vaddI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4877 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36126 "ad_x86.cpp"
  }
}

void vaddI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4889 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36144 "ad_x86.cpp"
  }
}

void vaddI_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4889 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36162 "ad_x86.cpp"
  }
}

void vaddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4901 "src/hotspot/cpu/x86/x86.ad"

    __ paddq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36179 "ad_x86.cpp"
  }
}

void vaddL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4911 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36197 "ad_x86.cpp"
  }
}

void vaddL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4922 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36215 "ad_x86.cpp"
  }
}

void vaddL_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4922 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36233 "ad_x86.cpp"
  }
}

void vaddFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4934 "src/hotspot/cpu/x86/x86.ad"

    __ addps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36250 "ad_x86.cpp"
  }
}

void vaddF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4944 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36268 "ad_x86.cpp"
  }
}

void vaddF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4955 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36286 "ad_x86.cpp"
  }
}

void vaddF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4955 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36304 "ad_x86.cpp"
  }
}

void vaddDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4967 "src/hotspot/cpu/x86/x86.ad"

    __ addpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36321 "ad_x86.cpp"
  }
}

void vaddD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4977 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36339 "ad_x86.cpp"
  }
}

void vaddD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4988 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36357 "ad_x86.cpp"
  }
}

void vaddD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4988 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36375 "ad_x86.cpp"
  }
}

void vsubBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5002 "src/hotspot/cpu/x86/x86.ad"

    __ psubb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36392 "ad_x86.cpp"
  }
}

void vsubB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5012 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36410 "ad_x86.cpp"
  }
}

void vsubB_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5023 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36428 "ad_x86.cpp"
  }
}

void vsubSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5035 "src/hotspot/cpu/x86/x86.ad"

    __ psubw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36445 "ad_x86.cpp"
  }
}

void vsubS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5046 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36463 "ad_x86.cpp"
  }
}

void vsubS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5057 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36481 "ad_x86.cpp"
  }
}

void vsubINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5069 "src/hotspot/cpu/x86/x86.ad"

    __ psubd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36498 "ad_x86.cpp"
  }
}

void vsubI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5079 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36516 "ad_x86.cpp"
  }
}

void vsubI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5090 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36534 "ad_x86.cpp"
  }
}

void vsubLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5102 "src/hotspot/cpu/x86/x86.ad"

    __ psubq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36551 "ad_x86.cpp"
  }
}

void vsubL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5112 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36569 "ad_x86.cpp"
  }
}

void vsubL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5124 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36587 "ad_x86.cpp"
  }
}

void vsubFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5136 "src/hotspot/cpu/x86/x86.ad"

    __ subps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36604 "ad_x86.cpp"
  }
}

void vsubF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5146 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36622 "ad_x86.cpp"
  }
}

void vsubF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5157 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36640 "ad_x86.cpp"
  }
}

void vsubDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5169 "src/hotspot/cpu/x86/x86.ad"

    __ subpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36657 "ad_x86.cpp"
  }
}

void vsubD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5179 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vsubpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36675 "ad_x86.cpp"
  }
}

void vsubD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5190 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vsubpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36693 "ad_x86.cpp"
  }
}

void mulB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5206 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    __ pmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ pmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ pmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 36719 "ad_x86.cpp"
  }
}

void mul16B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5223 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    __ pmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ pmovsxbw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ pmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, 0xEE);
    __ pshufd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, 0xEE);
    __ pmovsxbw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ pmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pmullw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 36752 "ad_x86.cpp"
  }
}

void vmul16B_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5246 "src/hotspot/cpu/x86/x86.ad"

  int vlen_enc = Assembler::AVX_256bit;
    __ vpmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, vlen_enc);
    __ vpmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
    __ vpmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, 0);
  
#line 36779 "ad_x86.cpp"
  }
}

void vmul32B_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5264 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int vlen_enc = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ vpmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
    __ vpmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpmovsxbw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, vlen_enc);
    __ vpmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
    __ vpmullw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vlen_enc);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 36815 "ad_x86.cpp"
  }
}

void vmul64B_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5290 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = Assembler::AVX_512bit;
    __ vextracti64x4_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ vextracti64x4_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ vpmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
    __ vpmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpmovsxbw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, vlen_enc);
    __ vpmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
    __ vpmullw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vlen_enc);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vlen_enc, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 36852 "ad_x86.cpp"
  }
}

void vmulSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5317 "src/hotspot/cpu/x86/x86.ad"

    __ pmullw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36869 "ad_x86.cpp"
  }
}

void vmulS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5327 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36887 "ad_x86.cpp"
  }
}

void vmulS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5338 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36905 "ad_x86.cpp"
  }
}

void vmulS_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5338 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36923 "ad_x86.cpp"
  }
}

void vmulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5350 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    __ pmulld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36941 "ad_x86.cpp"
  }
}

void vmulI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5361 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 36959 "ad_x86.cpp"
  }
}

void vmulI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5372 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 36977 "ad_x86.cpp"
  }
}

void vmulI_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5372 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36995 "ad_x86.cpp"
  }
}

void vmulL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5384 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vpmullq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37014 "ad_x86.cpp"
  }
}

void vmulL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5396 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vpmullq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 37033 "ad_x86.cpp"
  }
}

void vmulL_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5396 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vpmullq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 37052 "ad_x86.cpp"
  }
}

void mul2L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5416 "src/hotspot/cpu/x86/x86.ad"

    assert(VM_Version::supports_sse4_1(), "required");
    int vlen_enc = Assembler::AVX_128bit;
    __ pshufd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, 177);
    __ pmulld(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */);
    __ phaddd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
    __ pmovzxdq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
    __ psllq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, 32);
    __ pmuludq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ paddq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 37078 "ad_x86.cpp"
  }
}

void vmul4L_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5441 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_256bit;
    __ vpshufd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, 177, vlen_enc);
    __ vpmulld(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, vlen_enc);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
    __ vphaddd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
    __ vpmovzxdq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, vlen_enc);
    __ vpsllq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, 32, vlen_enc);
    __ vpmuludq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
  
#line 37105 "ad_x86.cpp"
  }
}

void vmulFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5460 "src/hotspot/cpu/x86/x86.ad"

    __ mulps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 37122 "ad_x86.cpp"
  }
}

void vmulF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5470 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37140 "ad_x86.cpp"
  }
}

void vmulF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5481 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 37158 "ad_x86.cpp"
  }
}

void vmulF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5481 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 37176 "ad_x86.cpp"
  }
}

void vmulDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5493 "src/hotspot/cpu/x86/x86.ad"

    __ mulpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 37193 "ad_x86.cpp"
  }
}

void vmulD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5503 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37211 "ad_x86.cpp"
  }
}

void vmulD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5514 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 37229 "ad_x86.cpp"
  }
}

void vmulD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5514 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 37247 "ad_x86.cpp"
  }
}

void vcmov8F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// copnd
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cop
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5528 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    int vlen_enc = Assembler::AVX_256bit;
    int cond = (Assembler::Condition)(opnd_array(1)->ccode());
    __ vcmpps(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src2 */, cond, vlen_enc);
    __ vblendvps(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, vlen_enc);
  
#line 37272 "ad_x86.cpp"
  }
}

void vcmov4D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// copnd
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cop
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5546 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    int vlen_enc = Assembler::AVX_256bit;
    int cond = (Assembler::Condition)(opnd_array(1)->ccode());
    __ vcmppd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src2 */, cond, vlen_enc);
    __ vblendvpd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, vlen_enc);
  
#line 37297 "ad_x86.cpp"
  }
}

void vdivFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5564 "src/hotspot/cpu/x86/x86.ad"

    __ divps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 37314 "ad_x86.cpp"
  }
}

void vdivF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5574 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37332 "ad_x86.cpp"
  }
}

void vdivF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5585 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 37350 "ad_x86.cpp"
  }
}

void vdivDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5597 "src/hotspot/cpu/x86/x86.ad"

    __ divpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 37367 "ad_x86.cpp"
  }
}

void vdivD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5607 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vdivpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37385 "ad_x86.cpp"
  }
}

void vdivD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5618 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vdivpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 37403 "ad_x86.cpp"
  }
}

void minmax_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5634 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    int opcode = this->ideal_Opcode();
    BasicType elem_bt = vector_element_basic_type(this);
    __ pminmax(opcode, elem_bt, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 37424 "ad_x86.cpp"
  }
}

void minmax_reg_sse_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5634 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    int opcode = this->ideal_Opcode();
    BasicType elem_bt = vector_element_basic_type(this);
    __ pminmax(opcode, elem_bt, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 37445 "ad_x86.cpp"
  }
}

void vminmax_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5650 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = vector_element_basic_type(this);

    __ vpminmax(opcode, elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37466 "ad_x86.cpp"
  }
}

void vminmax_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5650 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = vector_element_basic_type(this);

    __ vpminmax(opcode, elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37487 "ad_x86.cpp"
  }
}

void minmaxL_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5668 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    int opcode = this->ideal_Opcode();
    BasicType elem_bt = vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ pminmax(opcode, elem_bt, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 37511 "ad_x86.cpp"
  }
}

void minmaxL_reg_sse_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5668 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    int opcode = this->ideal_Opcode();
    BasicType elem_bt = vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ pminmax(opcode, elem_bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 37535 "ad_x86.cpp"
  }
}

void vminmaxL_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5687 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    int opcode = this->ideal_Opcode();
    BasicType elem_bt = vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ vpminmax(opcode, elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37558 "ad_x86.cpp"
  }
}

void vminmaxL_reg_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5687 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    int opcode = this->ideal_Opcode();
    BasicType elem_bt = vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ vpminmax(opcode, elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37581 "ad_x86.cpp"
  }
}

void vminmaxL_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5704 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int vlen_enc = vector_length_encoding(this);
    int opcode = this->ideal_Opcode();
    BasicType elem_bt = vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ vpminmax(opcode, elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37605 "ad_x86.cpp"
  }
}

void vminmaxL_reg_evex_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5704 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int vlen_enc = vector_length_encoding(this);
    int opcode = this->ideal_Opcode();
    BasicType elem_bt = vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ vpminmax(opcode, elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 37629 "ad_x86.cpp"
  }
}

void minmaxFP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// a
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// b
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5726 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = vector_element_basic_type(this);

    __ vminmax_fp(opcode, elem_bt,
                  opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* a */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* b */,
                  opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, vlen_enc);
  
#line 37657 "ad_x86.cpp"
  }
}

void minmaxFP_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// a
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// b
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5726 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = vector_element_basic_type(this);

    __ vminmax_fp(opcode, elem_bt,
                  opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* a */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* b */,
                  opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, vlen_enc);
  
#line 37685 "ad_x86.cpp"
  }
}

void evminmaxFP_reg_eavxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// a
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// b
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5747 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = vector_element_basic_type(this);

    KRegister ktmp = k1;
    __ evminmax_fp(opcode, elem_bt,
                   opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* a */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* b */,
                   ktmp, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, vlen_enc);
  
#line 37714 "ad_x86.cpp"
  }
}

void evminmaxFP_reg_eavx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// a
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// b
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5747 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = vector_element_basic_type(this);

    KRegister ktmp = k1;
    __ evminmax_fp(opcode, elem_bt,
                   opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* a */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* b */,
                   ktmp, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, vlen_enc);
  
#line 37743 "ad_x86.cpp"
  }
}

void vsqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5767 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vsqrtps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 37761 "ad_x86.cpp"
  }
}

void vsqrtF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5778 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vsqrtps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 37779 "ad_x86.cpp"
  }
}

void vsqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5790 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vsqrtpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 37797 "ad_x86.cpp"
  }
}

void vsqrtD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5801 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vsqrtpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 37815 "ad_x86.cpp"
  }
}

void vshiftcntNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5817 "src/hotspot/cpu/x86/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
  
#line 37831 "ad_x86.cpp"
  }
}

void vshiftcnt_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5817 "src/hotspot/cpu/x86/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
  
#line 37847 "ad_x86.cpp"
  }
}

void vshiftBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5831 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 37874 "ad_x86.cpp"
  }
}

void vshiftB_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5831 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 37901 "ad_x86.cpp"
  }
}

void vshiftB_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5831 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 37928 "ad_x86.cpp"
  }
}

void vshift16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5852 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 37960 "ad_x86.cpp"
  }
}

void vshift16B_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5852 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 37992 "ad_x86.cpp"
  }
}

void vshift16B_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5852 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 38024 "ad_x86.cpp"
  }
}

void vshift16B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5877 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 38051 "ad_x86.cpp"
  }
}

void vshift16B_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5877 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 38078 "ad_x86.cpp"
  }
}

void vshift16B_avx_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5877 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 38105 "ad_x86.cpp"
  }
}

void vshift32B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5897 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vextendbw(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 38137 "ad_x86.cpp"
  }
}

void vshift32B_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5897 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vextendbw(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 38169 "ad_x86.cpp"
  }
}

void vshift32B_avx_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5897 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vextendbw(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 38201 "ad_x86.cpp"
  }
}

void vshift64B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5922 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_512bit;
    __ vextracti64x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vlen_enc);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vlen_enc, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 38237 "ad_x86.cpp"
  }
}

void vshift64B_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5922 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_512bit;
    __ vextracti64x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vlen_enc);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vlen_enc, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 38273 "ad_x86.cpp"
  }
}

void vshift64B_avx_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5922 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_512bit;
    __ vextracti64x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vlen_enc);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vlen_enc, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 38309 "ad_x86.cpp"
  }
}

void vshiftSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5955 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else if (vlen == 4) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert (vlen == 8, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 38344 "ad_x86.cpp"
  }
}

void vshiftS_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5955 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else if (vlen == 4) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert (vlen == 8, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 38379 "ad_x86.cpp"
  }
}

void vshiftS_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5955 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else if (vlen == 4) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert (vlen == 8, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 38414 "ad_x86.cpp"
  }
}

void vshiftINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5986 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 38446 "ad_x86.cpp"
  }
}

void vshiftI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5986 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 38478 "ad_x86.cpp"
  }
}

void vshiftI_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5986 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 38510 "ad_x86.cpp"
  }
}

void vshiftI_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6013 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftd_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->constant());
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->constant());
      }
    }
  
#line 38542 "ad_x86.cpp"
  }
}

void vshiftI_imm_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6013 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftd_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->constant());
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->constant());
      }
    }
  
#line 38574 "ad_x86.cpp"
  }
}

void vshiftI_imm_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6013 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftd_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->constant());
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->constant());
      }
    }
  
#line 38606 "ad_x86.cpp"
  }
}

void vshiftLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6040 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      assert(vector_length(this) == 2, "");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    }
  
#line 38632 "ad_x86.cpp"
  }
}

void vshiftL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6040 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      assert(vector_length(this) == 2, "");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    }
  
#line 38658 "ad_x86.cpp"
  }
}

void vshiftL_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6060 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftq_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
    } else {
      assert(vector_length(this) == 2, "");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vshiftq_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->constant());
    }
  
#line 38684 "ad_x86.cpp"
  }
}

void vshiftL_imm_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6060 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftq_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
    } else {
      assert(vector_length(this) == 2, "");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vshiftq_imm(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->constant());
    }
  
#line 38710 "ad_x86.cpp"
  }
}

void vshiftL_arith_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6081 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      assert(UseSSE >= 2, "required");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ psrlq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      __ movdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_long_sign_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
      __ psrlq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      __ pxor(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
      __ psubq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    } else {
      assert(vlen == 4, "sanity");
      assert(UseAVX > 1, "required");
      int vlen_enc = Assembler::AVX_256bit;
      __ vpsrlq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
      __ vmovdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_long_sign_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
      __ vpsrlq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
      __ vpxor(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
      __ vpsubq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    }
  
#line 38748 "ad_x86.cpp"
  }
}

void vshiftL_arith_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6109 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ evpsraq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 38766 "ad_x86.cpp"
  }
}

void vshift8B_var_nobwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6127 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 38791 "ad_x86.cpp"
  }
}

void vshift8B_var_nobw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6127 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 38816 "ad_x86.cpp"
  }
}

void vshift8B_var_nobw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6127 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 38841 "ad_x86.cpp"
  }
}

void vshift16B_var_nobwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6147 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Shift lower half and get word result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Shift upper half and get word result in vtmp1
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Merge and down convert the two word results to byte in dst
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);
  
#line 38875 "ad_x86.cpp"
  }
}

void vshift16B_var_nobw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6147 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Shift lower half and get word result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Shift upper half and get word result in vtmp1
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Merge and down convert the two word results to byte in dst
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);
  
#line 38909 "ad_x86.cpp"
  }
}

void vshift16B_var_nobw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6147 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Shift lower half and get word result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Shift upper half and get word result in vtmp1
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Merge and down convert the two word results to byte in dst
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);
  
#line 38943 "ad_x86.cpp"
  }
}

void vshift32B_var_nobwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vtmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// vtmp4
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6175 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Process lower 128 bits and get result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Process higher 128 bits and get result in vtmp3
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ varshiftbw(opcode, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* vtmp4 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpackuswb(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Merge the two results in dst
    __ vinserti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 38987 "ad_x86.cpp"
  }
}

void vshift32B_var_nobw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vtmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// vtmp4
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6175 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Process lower 128 bits and get result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Process higher 128 bits and get result in vtmp3
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ varshiftbw(opcode, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* vtmp4 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpackuswb(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Merge the two results in dst
    __ vinserti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 39031 "ad_x86.cpp"
  }
}

void vshift32B_var_nobw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vtmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// vtmp4
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6175 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Process lower 128 bits and get result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Process higher 128 bits and get result in vtmp3
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ varshiftbw(opcode, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* vtmp4 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(8)->as_Register(ra_,this,idx8)/* scratch */);
    __ vpackuswb(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Merge the two results in dst
    __ vinserti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 39075 "ad_x86.cpp"
  }
}

void vshiftB_var_evex_bwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6211 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
  
#line 39099 "ad_x86.cpp"
  }
}

void vshiftB_var_evex_bw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6211 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
  
#line 39123 "ad_x86.cpp"
  }
}

void vshiftB_var_evex_bw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6211 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
  
#line 39147 "ad_x86.cpp"
  }
}

void vshift64B_var_evex_bwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6230 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_256bit;
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vextracti64x4_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti64x4_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ evarshiftb(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vinserti64x4(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 39176 "ad_x86.cpp"
  }
}

void vshift64B_var_evex_bw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6230 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_256bit;
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vextracti64x4_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti64x4_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ evarshiftb(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vinserti64x4(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 39205 "ad_x86.cpp"
  }
}

void vshift64B_var_evex_bw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6230 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_256bit;
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vextracti64x4_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti64x4_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ evarshiftb(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vinserti64x4(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 39234 "ad_x86.cpp"
  }
}

void vshift8S_var_nobwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6254 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 1);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, 0);
  
#line 39264 "ad_x86.cpp"
  }
}

void vshift8S_var_nobw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6254 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 1);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, 0);
  
#line 39294 "ad_x86.cpp"
  }
}

void vshift8S_var_nobw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6254 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 1);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, 0);
  
#line 39324 "ad_x86.cpp"
  }
}

void vshift16S_var_nobwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6279 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    // Shift lower half, with result in vtmp2 usign vtmp1 as TEMP
    __ vextendwd(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ varshiftd(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Shift upper half, with result in dst usign vtmp1 as TEMP
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Merge lower and upper half result into dst
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 39366 "ad_x86.cpp"
  }
}

void vshift16S_var_nobw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6279 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    // Shift lower half, with result in vtmp2 usign vtmp1 as TEMP
    __ vextendwd(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ varshiftd(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Shift upper half, with result in dst usign vtmp1 as TEMP
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Merge lower and upper half result into dst
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 39408 "ad_x86.cpp"
  }
}

void vshift16S_var_nobw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6279 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    // Shift lower half, with result in vtmp2 usign vtmp1 as TEMP
    __ vextendwd(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ varshiftd(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Shift upper half, with result in dst usign vtmp1 as TEMP
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);

    // Merge lower and upper half result into dst
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 39450 "ad_x86.cpp"
  }
}

void vshift16S_var_evex_bwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6313 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ varshiftw(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 39474 "ad_x86.cpp"
  }
}

void vshift16S_var_evex_bw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6313 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ varshiftw(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 39498 "ad_x86.cpp"
  }
}

void vshift16S_var_evex_bw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6313 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ varshiftw(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 39522 "ad_x86.cpp"
  }
}

void vshiftI_varNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6333 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 39543 "ad_x86.cpp"
  }
}

void vshiftI_var_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6333 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 39564 "ad_x86.cpp"
  }
}

void vshiftI_var_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6333 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 39585 "ad_x86.cpp"
  }
}

void vshiftL_varNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6349 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftq(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 39606 "ad_x86.cpp"
  }
}

void vshiftL_var_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6349 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftq(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 39627 "ad_x86.cpp"
  }
}

void vshiftL_arith_varNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6367 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc,
                 opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */);
  
#line 39649 "ad_x86.cpp"
  }
}

void vshiftL_arith_var_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6381 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftq(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 39668 "ad_x86.cpp"
  }
}

void vandNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6395 "src/hotspot/cpu/x86/x86.ad"

    __ pand(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 39685 "ad_x86.cpp"
  }
}

void vand_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6405 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39703 "ad_x86.cpp"
  }
}

void vand_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6416 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 39721 "ad_x86.cpp"
  }
}

void vand_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6416 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 39739 "ad_x86.cpp"
  }
}

void vorNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6429 "src/hotspot/cpu/x86/x86.ad"

    __ por(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 39756 "ad_x86.cpp"
  }
}

void vor_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6439 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39774 "ad_x86.cpp"
  }
}

void vor_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6450 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 39792 "ad_x86.cpp"
  }
}

void vor_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6450 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 39810 "ad_x86.cpp"
  }
}

void vxorNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6463 "src/hotspot/cpu/x86/x86.ad"

    __ pxor(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 39827 "ad_x86.cpp"
  }
}

void vxor_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6473 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39845 "ad_x86.cpp"
  }
}

void vxor_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6484 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 39863 "ad_x86.cpp"
  }
}

void vxor_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6484 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 39881 "ad_x86.cpp"
  }
}

void vcastBtoXNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6496 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType to_elem_bt = vector_element_basic_type(this);
    int vlen_enc = vector_length_encoding(this);
    switch (to_elem_bt) {
      case T_SHORT:
        __ vpmovsxbw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_INT:
        __ vpmovsxbd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_FLOAT:
        __ vpmovsxbd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        __ vcvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
        break;
      case T_LONG:
        __ vpmovsxbq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_DOUBLE:
        __ vpmovsxbd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        __ vcvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
        break;

      default: assert(false, "%s", type2name(to_elem_bt));
    }
  
#line 39921 "ad_x86.cpp"
  }
}

void castStoXNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6533 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_short_to_byte_mask()), 0, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    __ vpackuswb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0);
  
#line 39941 "ad_x86.cpp"
  }
}

void vcastStoXNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6549 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    int vlen_enc = vector_length_encoding(vector_length_in_bytes(this, opnd_array(1)));
    __ vpand(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
    __ vextracti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, 0x1);
    __ vpackuswb(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 0);
  
#line 39965 "ad_x86.cpp"
  }
}

void vcastStoX_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6565 "src/hotspot/cpu/x86/x86.ad"

    BasicType to_elem_bt = vector_element_basic_type(this);
    int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
    int vlen_enc = vector_length_encoding(this);
    switch (to_elem_bt) {
      case T_BYTE:
        if (!VM_Version::supports_avx512vl()) {
          vlen_enc = Assembler::AVX_512bit;
        }
        __ evpmovwb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
        break;
      case T_INT:
        __ vpmovsxwd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_FLOAT:
        __ vpmovsxwd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        __ vcvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
        break;
      case T_LONG:
        __ vpmovsxwq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_DOUBLE:
        __ vpmovsxwd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        __ vcvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
        break;
      default:
        ShouldNotReachHere();
    }
  
#line 40007 "ad_x86.cpp"
  }
}

void castItoXNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6604 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType to_elem_bt = vector_element_basic_type(this);
    int vlen_enc = vector_length_encoding(this, opnd_array(1));

    if (to_elem_bt == T_BYTE) {
      __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_int_to_byte_mask()), vlen_enc, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
      __ vpackusdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
      __ vpackuswb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      assert(to_elem_bt == T_SHORT, "%s", type2name(to_elem_bt));
      __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
      __ vpackusdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 40037 "ad_x86.cpp"
  }
}

void vcastItoXNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6630 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType to_elem_bt = vector_element_basic_type(this);
    int vlen_enc = vector_length_encoding(this, opnd_array(1));

    if (to_elem_bt == T_BYTE) {
      __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_int_to_byte_mask()), vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
      __ vextracti128(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 0x1);
      __ vpackusdw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vlen_enc);
      __ vpackuswb(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Assembler::AVX_128bit);
    } else {
      assert(to_elem_bt == T_SHORT, "%s", type2name(to_elem_bt));
      __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
      __ vextracti128(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 0x1);
      __ vpackusdw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vlen_enc);
    }
  
#line 40071 "ad_x86.cpp"
  }
}

void vcastItoX_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6656 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType dst_elem_bt = vector_element_basic_type(this);
    int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
    int dst_vlen_enc = vector_length_encoding(this);
    switch (dst_elem_bt) {
      case T_BYTE:
        if (!VM_Version::supports_avx512vl()) {
          src_vlen_enc = Assembler::AVX_512bit;
        }
        __ evpmovdb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
        break;
      case T_SHORT:
        if (!VM_Version::supports_avx512vl()) {
          src_vlen_enc = Assembler::AVX_512bit;
        }
        __ evpmovdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
        break;
      case T_FLOAT:
        __ vcvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, dst_vlen_enc);
        break;
      case T_LONG:
        __ vpmovsxdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, dst_vlen_enc);
        break;
      case T_DOUBLE:
        __ vcvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, dst_vlen_enc);
        break;
      default:
        ShouldNotReachHere();
    }
  
#line 40116 "ad_x86.cpp"
  }
}

void vcastLtoBSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6697 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    int vlen = vector_length_in_bytes(this, opnd_array(1));
    BasicType to_elem_bt  = vector_element_basic_type(this);
    AddressLiteral mask_addr = (to_elem_bt == T_BYTE) ? ExternalAddress(vector_int_to_byte_mask())
                                                      : ExternalAddress(vector_int_to_short_mask());
    if (vlen <= 16) {
      __ vpshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 8, Assembler::AVX_128bit);
      __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, mask_addr, Assembler::AVX_128bit, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
      __ vpackusdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Assembler::AVX_128bit);
    } else {
      assert(vlen <= 32, "required");
      __ vpermilps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 8, Assembler::AVX_256bit);
      __ vpermpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 8, Assembler::AVX_256bit);
      __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, mask_addr, Assembler::AVX_128bit, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
      __ vpackusdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Assembler::AVX_128bit);
    }
    if (to_elem_bt == T_BYTE) {
      __ vpackuswb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Assembler::AVX_128bit);
    }
  
#line 40152 "ad_x86.cpp"
  }
}

void vcastLtoX_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6729 "src/hotspot/cpu/x86/x86.ad"

    BasicType to_elem_bt = vector_element_basic_type(this);
    int vlen = vector_length_in_bytes(this, opnd_array(1));
    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    switch (to_elem_bt) {
      case T_BYTE:
        if (UseAVX > 2 && !VM_Version::supports_avx512vl()) {
          vlen_enc = Assembler::AVX_512bit;
        }
        __ evpmovqb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_SHORT:
        if (UseAVX > 2 && !VM_Version::supports_avx512vl()) {
          vlen_enc = Assembler::AVX_512bit;
        }
        __ evpmovqw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_INT:
        if (vlen == 8) {
          if (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */!= opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */) {
            __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
          }
        } else if (vlen == 16) {
          __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 8);
        } else if (vlen == 32) {
          if (UseAVX > 2) {
            if (!VM_Version::supports_avx512vl()) {
              vlen_enc = Assembler::AVX_512bit;
            }
            __ evpmovqd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
          } else {
            __ vpermilps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 8, vlen_enc);
            __ vpermpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 8, vlen_enc);
          }
        } else { // vlen == 64
          __ evpmovqd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        }
        break;
      case T_FLOAT:
        assert(UseAVX > 2 && VM_Version::supports_avx512dq(), "required");
        __ evcvtqq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_DOUBLE:
        assert(UseAVX > 2 && VM_Version::supports_avx512dq(), "required");
        __ evcvtqq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;

      default: assert(false, "%s", type2name(to_elem_bt));
    }
  
#line 40215 "ad_x86.cpp"
  }
}

void vcastFtoD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6786 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vcvtps2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 40232 "ad_x86.cpp"
  }
}

void vcastDtoF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6797 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    __ vcvtpd2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 40249 "ad_x86.cpp"
  }
}

void vcmpFDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6812 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    Assembler::ComparisonPredicateFP cmp = booltest_pred_to_comparison_pred_fp(opnd_array(3)->constant());
    if (vector_element_basic_type(this, opnd_array(1)) == T_FLOAT) {
      __ vcmpps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
    } else {
      __ vcmppd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
    }
  
#line 40273 "ad_x86.cpp"
  }
}

void evcmpFDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6830 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_512bit;
    Assembler::ComparisonPredicateFP cmp = booltest_pred_to_comparison_pred_fp(opnd_array(3)->constant());
    KRegister ktmp = k2; // Use a hardcoded temp due to no k register allocation.
    KRegister mask = k0; // The comparison itself is not being masked.
    if (vector_element_basic_type(this, opnd_array(1)) == T_FLOAT) {
      __ evcmpps(ktmp, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
      __ evmovdqul(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, ktmp, ExternalAddress(vector_all_bits_set()), false, vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
    } else {
      __ evcmppd(ktmp, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
      __ evmovdquq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, ktmp, ExternalAddress(vector_all_bits_set()), false, vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
    }
  
#line 40302 "ad_x86.cpp"
  }
}

void vcmpNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6853 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
    Assembler::Width ww = widthForType(vector_element_basic_type(this, opnd_array(1)));
    __ vpcmpCCW(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, ww, vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
  
#line 40324 "ad_x86.cpp"
  }
}

void evcmpNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6868 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int vlen_enc = Assembler::AVX_512bit;
    Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
    KRegister ktmp = k2; // Use a hardcoded temp due to no k register allocation.
    KRegister mask = k0; // The comparison itself is not being masked.
    bool merge = false;
    BasicType src1_elem_bt = vector_element_basic_type(this, opnd_array(1));

    switch (src1_elem_bt) {
      case T_BYTE: {
        __ evpcmpb(ktmp, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
        __ evmovdqub(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, ktmp, ExternalAddress(vector_all_bits_set()), merge, vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
        break;
      }
      case T_SHORT: {
        __ evpcmpw(ktmp, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
        __ evmovdquw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, ktmp, ExternalAddress(vector_all_bits_set()), merge, vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
        break;
      }
      case T_INT: {
        __ evpcmpd(ktmp, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
        __ evmovdqul(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, ktmp, ExternalAddress(vector_all_bits_set()), merge, vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
        break;
      }
      case T_LONG: {
        __ evpcmpq(ktmp, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
        __ evmovdquq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, ktmp, ExternalAddress(vector_all_bits_set()), merge, vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
        break;
      }

      default: assert(false, "%s", type2name(src1_elem_bt));
    }
  
#line 40375 "ad_x86.cpp"
  }
}

void extractINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6916 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)vector_length(this, opnd_array(1)), "out of bounds");

    BasicType elem_bt = vector_element_basic_type(this, opnd_array(1));
    __ get_elem(elem_bt, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
  
#line 40395 "ad_x86.cpp"
  }
}

void extractI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6916 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)vector_length(this, opnd_array(1)), "out of bounds");

    BasicType elem_bt = vector_element_basic_type(this, opnd_array(1));
    __ get_elem(elem_bt, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
  
#line 40415 "ad_x86.cpp"
  }
}

void vextractINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6935 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)vector_length(this, opnd_array(1)), "out of bounds");

    BasicType elem_bt = vector_element_basic_type(this, opnd_array(1));
    XMMRegister lane_xmm = __ get_lane(elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
    __ get_elem(elem_bt, opnd_array(0)->as_Register(ra_,this)/* dst */, lane_xmm, opnd_array(2)->constant());
  
#line 40437 "ad_x86.cpp"
  }
}

void vextractI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6935 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)vector_length(this, opnd_array(1)), "out of bounds");

    BasicType elem_bt = vector_element_basic_type(this, opnd_array(1));
    XMMRegister lane_xmm = __ get_lane(elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
    __ get_elem(elem_bt, opnd_array(0)->as_Register(ra_,this)/* dst */, lane_xmm, opnd_array(2)->constant());
  
#line 40459 "ad_x86.cpp"
  }
}

void extractFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6980 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)vector_length(this, opnd_array(1)), "out of bounds");

    __ get_elem(T_FLOAT, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp */);
  
#line 40481 "ad_x86.cpp"
  }
}

void vextractFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6994 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)vector_length(this, opnd_array(1)), "out of bounds");

    XMMRegister lane_reg = __ get_lane(T_FLOAT, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
    __ get_elem(T_FLOAT, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, lane_reg, opnd_array(2)->constant(), opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 40503 "ad_x86.cpp"
  }
}

void extractDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7007 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)vector_length(this, opnd_array(1)), "out of bounds");

    __ get_elem(T_DOUBLE, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
  
#line 40522 "ad_x86.cpp"
  }
}

void vextractDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7021 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)vector_length(this, opnd_array(1)), "out of bounds");

    XMMRegister lane_reg = __ get_lane(T_DOUBLE, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
    __ get_elem(T_DOUBLE, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, lane_reg, opnd_array(2)->constant());
  
#line 40543 "ad_x86.cpp"
  }
}

void blendvpNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7037 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    if (opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */!= opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */) {
      __ movdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */);
    }
    __ pblendvb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); // uses xmm0 as mask
  
#line 40567 "ad_x86.cpp"
  }
}

void vblendvpINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7054 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpblendvb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */, vlen_enc);
  
#line 40586 "ad_x86.cpp"
  }
}

void vblendvpFDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7067 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vblendvps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */, vlen_enc);
  
#line 40605 "ad_x86.cpp"
  }
}

void evblendvp64Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7079 "src/hotspot/cpu/x86/x86.ad"

     int vlen_enc = Assembler::AVX_512bit;
     BasicType elem_bt = vector_element_basic_type(this);
     KRegister ktmp = k2;
    __ evpcmp(elem_bt, ktmp, k0, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */, ExternalAddress(vector_all_bits_set()), Assembler::eq, vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
    __ evpblend(elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, ktmp, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 40628 "ad_x86.cpp"
  }
}

void vabsB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7094 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 16) {
      __ pabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 40650 "ad_x86.cpp"
  }
}

void vabsS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7109 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 8) {
      __ pabsw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpabsw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 40672 "ad_x86.cpp"
  }
}

void vabsI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7124 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ pabsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpabsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 40694 "ad_x86.cpp"
  }
}

void vabsL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7139 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ evpabsq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 40715 "ad_x86.cpp"
  }
}

void vabsnegFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7159 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this);
    if (vlen == 2) {
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    } else {
      assert(vlen == 8 || vlen == 16, "required");
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    }
  
#line 40740 "ad_x86.cpp"
  }
}

void vabsnegF_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7159 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this);
    if (vlen == 2) {
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    } else {
      assert(vlen == 8 || vlen == 16, "required");
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    }
  
#line 40765 "ad_x86.cpp"
  }
}

void vabsneg4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7180 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    __ vabsnegf(opcode, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
  
#line 40783 "ad_x86.cpp"
  }
}

void vabsneg4F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7180 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    __ vabsnegf(opcode, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
  
#line 40801 "ad_x86.cpp"
  }
}

void vabsnegDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7192 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    uint vlen = vector_length(this);
    if (vlen == 2) {
      assert(UseSSE >= 2, "required");
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    }
  
#line 40826 "ad_x86.cpp"
  }
}

void vabsnegD_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7192 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    uint vlen = vector_length(this);
    if (vlen == 2) {
      assert(UseSSE >= 2, "required");
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    }
  
#line 40851 "ad_x86.cpp"
  }
}

void loadMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7304 "src/hotspot/cpu/x86/x86.ad"

    int vlen_in_bytes = vector_length_in_bytes(this);
    BasicType elem_bt = vector_element_basic_type(this);

    __ load_vector_mask(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_in_bytes, elem_bt);
  
#line 40871 "ad_x86.cpp"
  }
}

void storeMask1BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7319 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 3, "required");
    if (vector_length_in_bytes(this) <= 16) {
      __ pabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      assert(UseAVX >= 2, "required");
      int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
      __ vpabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
    }
  
#line 40895 "ad_x86.cpp"
  }
}

void storeMask2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7336 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 3, "required");
    __ pabsw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ packsswb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 40914 "ad_x86.cpp"
  }
}

void vstoreMask2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7349 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_128bit;
    __ vextracti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x1);
    __ vpacksswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */,vlen_enc);
    __ vpabsb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 40935 "ad_x86.cpp"
  }
}

void vstoreMask2B_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7362 "src/hotspot/cpu/x86/x86.ad"

    int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
    int dst_vlen_enc = vector_length_encoding(this);
    __ evpmovwb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
    __ vpabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, dst_vlen_enc);
  
#line 40955 "ad_x86.cpp"
  }
}

void storeMask4BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7375 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 3, "required");
    __ pabsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ packssdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ packsswb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 40975 "ad_x86.cpp"
  }
}

void vstoreMask4BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7389 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_128bit;
    __ vextracti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x1);
    __ vpackssdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpacksswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpabsb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 40997 "ad_x86.cpp"
  }
}

void vstoreMask4B_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7403 "src/hotspot/cpu/x86/x86.ad"

    int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
    int dst_vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      src_vlen_enc = Assembler::AVX_512bit;
    }
    __ evpmovdb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
    __ vpabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, dst_vlen_enc);
  
#line 41020 "ad_x86.cpp"
  }
}

void storeMask8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7419 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 3, "required");
    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x8);
    __ packssdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ packsswb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ pabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 41041 "ad_x86.cpp"
  }
}

void storeMask8B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7434 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_128bit;
    __ vpshufps(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x88, Assembler::AVX_256bit);
    __ vextracti128(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0x1);
    __ vblendps(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, 0xC, vlen_enc);
    __ vpackssdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpacksswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpabsb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 41066 "ad_x86.cpp"
  }
}

void vstoreMask8B_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7450 "src/hotspot/cpu/x86/x86.ad"

    int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
    int dst_vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      src_vlen_enc = Assembler::AVX_512bit;
    }
    __ evpmovqb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
    __ vpabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, dst_vlen_enc);
  
#line 41089 "ad_x86.cpp"
  }
}

void loadIotaIndicesNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7469 "src/hotspot/cpu/x86/x86.ad"

     int vlen_in_bytes = vector_length_in_bytes(this);
     __ load_iota_indices(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */, vlen_in_bytes);
  
#line 41107 "ad_x86.cpp"
  }
}

void loadShuffleBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7484 "src/hotspot/cpu/x86/x86.ad"

    // empty
  
#line 41123 "ad_x86.cpp"
  }
}

void rearrangeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7495 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");
    __ pshufb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */);
  
#line 41141 "ad_x86.cpp"
  }
}

void rearrangeB_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7507 "src/hotspot/cpu/x86/x86.ad"

    __ vpshufb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Assembler::AVX_256bit);
  
#line 41158 "ad_x86.cpp"
  }
}

void rearrangeB_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7518 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpermb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 41176 "ad_x86.cpp"
  }
}

void loadShuffleSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7533 "src/hotspot/cpu/x86/x86.ad"

    // Create a byte shuffle mask from short shuffle mask
    // only byte shuffle instruction available on these platforms

    // Multiply each shuffle by two to get byte index
    __ pmovzxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ psllw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 1);

    // Duplicate to create 2 copies of byte index
    __ movdqu(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
    __ psllw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, 8);
    __ por(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);

    // Add one to get alternate byte index
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, ExternalAddress(vector_short_shufflemask()), opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
    __ paddb(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 41209 "ad_x86.cpp"
  }
}

void rearrangeSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7558 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");
    __ pshufb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */);
  
#line 41227 "ad_x86.cpp"
  }
}

void loadShuffleS_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7570 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ vpmovzxbw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 41247 "ad_x86.cpp"
  }
}

void rearrangeS_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7585 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ vpermw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 41268 "ad_x86.cpp"
  }
}

void loadShuffleINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7603 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    // Create a byte shuffle mask from int shuffle mask
    // only byte shuffle instruction available on these platforms

    // Duplicate and multiply each shuffle by 4
    __ pmovzxbd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ pshuflw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 0xA0);
    __ pshufhw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 0xA0);
    __ psllw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 2);

    // Duplicate again to create 4 copies of byte index
    __ movdqu(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
    __ psllw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, 8);
    __ por(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);

    // Add 3,2,1,0 to get alternate byte index
    __ movdqu(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, ExternalAddress(vector_int_shufflemask()), opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
    __ paddb(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 41305 "ad_x86.cpp"
  }
}

void rearrangeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7632 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");
    __ pshufb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */);
  
#line 41323 "ad_x86.cpp"
  }
}

void loadShuffleI_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7644 "src/hotspot/cpu/x86/x86.ad"

  int vlen_enc = vector_length_encoding(this);
    __ vpmovzxbd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 41340 "ad_x86.cpp"
  }
}

void rearrangeI_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7656 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (vlen_enc == Assembler::AVX_128bit) {
      vlen_enc = Assembler::AVX_256bit;
    }
    __ vpermd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 41361 "ad_x86.cpp"
  }
}

void loadShuffleLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7674 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int vlen_enc = vector_length_encoding(this);
    // Create a double word shuffle mask from long shuffle mask
    // only double word shuffle instruction available on these platforms

    // Multiply each shuffle by two to get double word index
    __ vpmovzxbq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vpsllq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 1, vlen_enc);

    // Duplicate each double word shuffle
    __ vpsllq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 32, vlen_enc);
    __ vpor(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, vlen_enc);

    // Add one to get alternate double word index
    __ vpaddd(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, ExternalAddress(vector_long_shufflemask()), vlen_enc, opnd_array(4)->as_Register(ra_,this,idx4)/* scratch */);
  
#line 41395 "ad_x86.cpp"
  }
}

void rearrangeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7700 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int vlen_enc = vector_length_encoding(this);
    __ vpermd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 41415 "ad_x86.cpp"
  }
}

void loadShuffleL_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7714 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int vlen_enc = vector_length_encoding(this);
    __ vpmovzxbq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 41434 "ad_x86.cpp"
  }
}

void rearrangeL_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7728 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int vlen_enc = vector_length_encoding(this);
    if (vlen_enc == Assembler::AVX_128bit) {
      vlen_enc = Assembler::AVX_256bit;
    }
    __ vpermq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 41457 "ad_x86.cpp"
  }
}

void vfmaF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7747 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vlen_enc = vector_length_encoding(this);
    __ vfmaf(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vlen_enc);
  
#line 41477 "ad_x86.cpp"
  }
}

void vfmaF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7759 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vlen_enc = vector_length_encoding(this);
    __ vfmaf(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vlen_enc);
  
#line 41497 "ad_x86.cpp"
  }
}

void vfmaD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7771 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vlen_enc = vector_length_encoding(this);
    __ vfmad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vlen_enc);
  
#line 41517 "ad_x86.cpp"
  }
}

void vfmaD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7783 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vlen_enc = vector_length_encoding(this);
    __ vfmad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vlen_enc);
  
#line 41537 "ad_x86.cpp"
  }
}

void vmuladdS2I_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7797 "src/hotspot/cpu/x86/x86.ad"

    __ pmaddwd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */);
  
#line 41554 "ad_x86.cpp"
  }
}

void vmuladdS2I_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7807 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmaddwd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 41572 "ad_x86.cpp"
  }
}

void vmuladdaddS2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7820 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    __ evpdpwssd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 41592 "ad_x86.cpp"
  }
}

void vmuladdaddS2I_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7820 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    __ evpdpwssd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src2 */, vlen_enc);
  
#line 41612 "ad_x86.cpp"
  }
}

void vpopcountINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7834 "src/hotspot/cpu/x86/x86.ad"

    assert(UsePopCountInstruction, "not enabled");

    int vlen_enc = vector_length_encoding(this);
    __ vpopcntd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 41631 "ad_x86.cpp"
  }
}

void vpternlogNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// func
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7849 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpternlogd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(4)->constant(), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src3 */, vector_len);
  
#line 41651 "ad_x86.cpp"
  }
}

void vpternlog_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// func
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7860 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpternlogd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(4)->constant(), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), vector_len);
  
#line 41671 "ad_x86.cpp"
  }
}

void vprotate_immI8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7872 "src/hotspot/cpu/x86/x86.ad"

    int opcode      = this->ideal_Opcode();
    int vector_len  = vector_length_encoding(this);
    BasicType etype = this->bottom_type()->is_vect()->element_basic_type();
    __ vprotate_imm(opcode, etype, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
  
#line 41691 "ad_x86.cpp"
  }
}

void vprotate_immI8_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7872 "src/hotspot/cpu/x86/x86.ad"

    int opcode      = this->ideal_Opcode();
    int vector_len  = vector_length_encoding(this);
    BasicType etype = this->bottom_type()->is_vect()->element_basic_type();
    __ vprotate_imm(opcode, etype, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
  
#line 41711 "ad_x86.cpp"
  }
}

void vprorateNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7885 "src/hotspot/cpu/x86/x86.ad"

    int opcode      = this->ideal_Opcode();
    int vector_len  = vector_length_encoding(this);
    BasicType etype = this->bottom_type()->is_vect()->element_basic_type();
    __ vprotate_var(opcode, etype, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
  
#line 41731 "ad_x86.cpp"
  }
}

void vprorate_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7885 "src/hotspot/cpu/x86/x86.ad"

    int opcode      = this->ideal_Opcode();
    int vector_len  = vector_length_encoding(this);
    BasicType etype = this->bottom_type()->is_vect()->element_basic_type();
    __ vprotate_var(opcode, etype, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
  
#line 41751 "ad_x86.cpp"
  }
}

void compareAndSwapP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 42 "src/hotspot/cpu/x86/gc/shenandoah/shenandoah_x86_32.ad"

    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm,
                                                   opnd_array(0)->as_Register(ra_,this)/* res */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   false, // swap
                                                   opnd_array(4)->as_Register(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 41774 "ad_x86.cpp"
  }
}

void compareAndSwapP_shenandoah_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 42 "src/hotspot/cpu/x86/gc/shenandoah/shenandoah_x86_32.ad"

    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm,
                                                   opnd_array(0)->as_Register(ra_,this)/* res */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   false, // swap
                                                   opnd_array(4)->as_Register(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 41797 "ad_x86.cpp"
  }
}

void compareAndExchangeP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 63 "src/hotspot/cpu/x86/gc/shenandoah/shenandoah_x86_32.ad"

    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm,
                                                   NULL, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   true,  // exchange
                                                   opnd_array(4)->as_Register(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 41820 "ad_x86.cpp"
  }
}

const MachOper* popCountI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* popCountL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2L_immINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_immI_255Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_immINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2UBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2SNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2USNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immI_255Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immI_65535Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immU31Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLX_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLX_reg_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_partialNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAlloc0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL2INode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLX_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLX_reg_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmI16Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmPNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmCMNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDPR_roundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPR_roundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPR_DroundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPR_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* cmovI_memNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUCFNode::memory_operand() const { return _opnds[4]; }
const MachOper* addI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* incI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* decI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeIConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddB_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddBNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddS_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddSNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddI_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddINode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgBNode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgSNode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgINode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgPNode::memory_operand() const { return _opnds[1]; }
const MachOper* subI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulINode::memory_operand() const { return _opnds[2]; }
const MachOper* mulI_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* andnI_rReg_rReg_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* andnI_rReg_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsiI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* blsiI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskI_rReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsrI_rReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsrI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* addL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andnL_eReg_eReg_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* andnL_eReg_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsiL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* blsiL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskL_eReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsrL_eReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsrL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orl_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orl_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorl_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorl_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* cmpD_ccmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_ccmemCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_regmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* subDPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addDPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addDPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addDPR_mem_regNode::memory_operand() const { return _opnds[1]; }
const MachOper* addDPR_mem_reg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulDPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulDPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulDPR_reg_mem_ciscNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulDPR_reg_mem_cisc_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* cmpF_ccmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_ccmemCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_regmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR24_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR24_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addFPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addFPR24_mem_regNode::memory_operand() const { return _opnds[1]; }
const TypePtr *addFPR24_mem_ciscNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addFPR24_mem_ciscNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *addFPR24_mem_cisc_0Node::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addFPR24_mem_cisc_0Node::memory_operand() const { return (MachOper*)-1; }
const TypePtr *addFPR24_mem_memNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addFPR24_mem_memNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* mulFPR24_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulFPR24_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulFPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulFPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const TypePtr *mulFPR24_mem_memNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* mulFPR24_mem_memNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* mulFPR_reg_load1Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulFPR_reg_load1_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_mulFPR_reg_load1Node::memory_operand() const { return _opnds[1]; }
const MachOper* addFPR_mulFPR_reg_load1_1Node::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_mulFPR_reg_load1_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_mulFPR_reg_load1_2Node::memory_operand() const { return _opnds[3]; }
const MachOper* convI2D_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2DPR_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2FPR_SSF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2FPR_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* compI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testI_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compU_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_mem_eRegNode::memory_operand() const { return _opnds[2]; }
const MachOper* testP_Reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* cmovLL_mem_LTGENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_LTGENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_EQNENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_EQNENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_LEGTNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_LEGT_UNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_LEGTNode::memory_operand() const { return _opnds[4]; }
const MachOper* addF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* divF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* sqrtF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* sqrtD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadVNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeVNode::memory_operand() const { return _opnds[1]; }
const MachOper* gatherNode::memory_operand() const { return _opnds[1]; }
const MachOper* evgatherNode::memory_operand() const { return _opnds[1]; }
const MachOper* scatterNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplB_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplS_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* vaddB_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddB_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddS_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddS_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddI_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddL_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vsubB_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubS_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulS_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulS_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulI_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulL_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vdivF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vdivD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsqrtF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* vsqrtD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* vand_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vand_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vor_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vor_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vxor_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vxor_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vfmaF_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* vfmaD_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* vpternlog_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* compareAndSwapP_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapP_shenandoah_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeP_shenandoahNode::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // VecA
    false,  // VecS
    false,  // VecD
    false,  // VecX
    false,  // VecY
    false,  // VecZ
    false,  // RegVMask
    false,  // RegFlags
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    true ,  // AbsI
    false,  // AbsL
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    false,  // ArrayCopy
    true ,  // AryEq
    true ,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    true ,  // ReverseBytesUS
    true ,  // ReverseBytesS
    false,  // CProj
    false,  // CacheWB
    false,  // CacheWBPreSync
    false,  // CacheWBPostSync
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    false,  // CallNative
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastLL
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    true ,  // CMoveD
    true ,  // CMoveVD
    true ,  // CMoveF
    true ,  // CMoveVF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    false,  // CMoveN
    false,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CmpUL
    true ,  // CompareAndSwapB
    true ,  // CompareAndSwapS
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    false,  // CompareAndSwapN
    true ,  // WeakCompareAndSwapB
    true ,  // WeakCompareAndSwapS
    true ,  // WeakCompareAndSwapI
    true ,  // WeakCompareAndSwapL
    true ,  // WeakCompareAndSwapP
    false,  // WeakCompareAndSwapN
    true ,  // CompareAndExchangeB
    true ,  // CompareAndExchangeS
    true ,  // CompareAndExchangeI
    true ,  // CompareAndExchangeL
    true ,  // CompareAndExchangeP
    false,  // CompareAndExchangeN
    true ,  // GetAndAddB
    true ,  // GetAndAddS
    true ,  // GetAndAddI
    false,  // GetAndAddL
    true ,  // GetAndSetB
    true ,  // GetAndSetS
    true ,  // GetAndSetI
    false,  // GetAndSetL
    true ,  // GetAndSetP
    false,  // GetAndSetN
    false,  // Con
    false,  // ConN
    false,  // ConNKlass
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    false,  // OuterStripMinedLoop
    false,  // OuterStripMinedLoopEnd
    false,  // LongCountedLoop
    false,  // LongCountedLoopEnd
    true ,  // CountLeadingZerosI
    true ,  // CountLeadingZerosL
    true ,  // CountTrailingZerosI
    true ,  // CountTrailingZerosL
    true ,  // CreateEx
    false,  // DecodeN
    false,  // DecodeNKlass
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    true ,  // DivModI
    false,  // DivModL
    true ,  // EncodeISOArray
    false,  // EncodeP
    false,  // EncodePKlass
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // FmaD
    true ,  // FmaF
    true ,  // Goto
    true ,  // Halt
    true ,  // HasNegatives
    true ,  // If
    false,  // RangeCheck
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    true ,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadUB
    true ,  // LoadUS
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    false,  // LoadNKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadP
    false,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    false,  // Loop
    true ,  // LoopLimit
    false,  // Mach
    false,  // MachProj
    true ,  // MulAddS2I
    true ,  // MaxI
    false,  // MaxL
    false,  // MaxD
    false,  // MaxF
    true ,  // MemBarAcquire
    true ,  // LoadFence
    true ,  // SetVectMaskI
    true ,  // MemBarAcquireLock
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // StoreFence
    true ,  // MemBarReleaseLock
    true ,  // MemBarVolatile
    true ,  // MemBarStoreStore
    false,  // MergeMem
    true ,  // MinI
    false,  // MinL
    false,  // MinF
    false,  // MinD
    true ,  // ModD
    true ,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    false,  // MulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    false,  // NegI
    false,  // NegL
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    true ,  // OnSpinWait
    false,  // Opaque1
    false,  // OpaqueLoopInit
    false,  // OpaqueLoopStride
    false,  // Opaque2
    false,  // Opaque3
    false,  // Opaque4
    false,  // ProfileBoolean
    true ,  // OrI
    true ,  // OrL
    true ,  // OverflowAddI
    true ,  // OverflowSubI
    true ,  // OverflowMulI
    false,  // OverflowAddL
    false,  // OverflowSubL
    false,  // OverflowMulL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // SubTypeCheck
    false,  // Phi
    true ,  // PopCountI
    true ,  // PopCountL
    true ,  // PopCountVI
    true ,  // PrefetchAllocation
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    true ,  // RoundDouble
    false,  // RoundDoubleMode
    false,  // RoundDoubleModeV
    true ,  // RoundFloat
    false,  // RotateLeft
    true ,  // RotateLeftV
    false,  // RotateRight
    true ,  // RotateRightV
    true ,  // SafePoint
    false,  // SafePointScalarObject
    true ,  // ShenandoahCompareAndExchangeP
    false,  // ShenandoahCompareAndExchangeN
    false,  // ShenandoahCompareAndSwapN
    true ,  // ShenandoahCompareAndSwapP
    false,  // ShenandoahWeakCompareAndSwapN
    true ,  // ShenandoahWeakCompareAndSwapP
    false,  // ShenandoahIUBarrier
    false,  // ShenandoahLoadReferenceBarrier
    false,  // SCMemProj
    false,  // CopySignD
    false,  // CopySignF
    false,  // SignumD
    false,  // SignumF
    true ,  // SqrtD
    true ,  // SqrtF
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    false,  // StoreN
    false,  // StoreNKlass
    true ,  // StrComp
    true ,  // StrCompressedCopy
    true ,  // StrEquals
    true ,  // StrIndexOf
    true ,  // StrIndexOfChar
    true ,  // StrInflatedCopy
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    true ,  // MacroLogicV
    true ,  // ThreadLocal
    false,  // Unlock
    false,  // URShiftB
    false,  // URShiftS
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    true ,  // AddVB
    true ,  // AddVS
    true ,  // AddVI
    true ,  // AddReductionVI
    true ,  // AddVL
    false,  // AddReductionVL
    true ,  // AddVF
    true ,  // AddReductionVF
    true ,  // AddVD
    true ,  // AddReductionVD
    true ,  // SubVB
    true ,  // SubVS
    true ,  // SubVI
    true ,  // SubVL
    true ,  // SubVF
    true ,  // SubVD
    true ,  // MulVB
    true ,  // MulVS
    true ,  // MulVI
    true ,  // MulReductionVI
    true ,  // MulVL
    false,  // MulReductionVL
    true ,  // MulVF
    true ,  // MulReductionVF
    true ,  // MulVD
    true ,  // MulReductionVD
    true ,  // MulAddVS2VI
    true ,  // FmaVD
    true ,  // FmaVF
    true ,  // DivVF
    true ,  // DivVD
    true ,  // AbsVB
    true ,  // AbsVS
    true ,  // AbsVI
    true ,  // AbsVL
    true ,  // AbsVF
    true ,  // AbsVD
    false,  // NegVI
    true ,  // NegVF
    true ,  // NegVD
    true ,  // SqrtVD
    true ,  // SqrtVF
    true ,  // LShiftCntV
    true ,  // RShiftCntV
    true ,  // LShiftVB
    true ,  // LShiftVS
    true ,  // LShiftVI
    true ,  // LShiftVL
    true ,  // RShiftVB
    true ,  // RShiftVS
    true ,  // RShiftVI
    true ,  // RShiftVL
    true ,  // URShiftVB
    true ,  // URShiftVS
    true ,  // URShiftVI
    true ,  // URShiftVL
    true ,  // AndV
    true ,  // AndReductionV
    true ,  // OrV
    true ,  // OrReductionV
    true ,  // XorV
    true ,  // XorReductionV
    true ,  // MinV
    true ,  // MaxV
    true ,  // MinReductionV
    true ,  // MaxReductionV
    true ,  // LoadVector
    true ,  // LoadVectorGather
    true ,  // StoreVector
    true ,  // StoreVectorScatter
    false,  // LoadVectorMasked
    false,  // StoreVectorMasked
    false,  // VectorMaskGen
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2L
    false,  // Pack2D
    true ,  // ReplicateB
    true ,  // ReplicateS
    true ,  // ReplicateI
    true ,  // ReplicateL
    true ,  // ReplicateF
    true ,  // ReplicateD
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractUB
    false,  // ExtractC
    true ,  // ExtractS
    true ,  // ExtractI
    false,  // ExtractL
    true ,  // ExtractF
    true ,  // ExtractD
    false,  // Digit
    false,  // LowerCase
    false,  // UpperCase
    false,  // Whitespace
    false,  // VectorBox
    false,  // VectorBoxAllocate
    false,  // VectorUnbox
    false,  // VectorMaskWrapper
    true ,  // VectorMaskCmp
    false,  // VectorTest
    true ,  // VectorBlend
    true ,  // VectorRearrange
    true ,  // VectorLoadMask
    true ,  // VectorLoadShuffle
    true ,  // VectorLoadConst
    true ,  // VectorStoreMask
    true ,  // VectorReinterpret
    false,  // VectorCast
    true ,  // VectorCastB2X
    true ,  // VectorCastS2X
    true ,  // VectorCastI2X
    true ,  // VectorCastL2X
    true ,  // VectorCastF2X
    true ,  // VectorCastD2X
    true    // VectorInsert
};


int Compile::sync_stack_slots() const { return 1; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 1 +
              align_up((Compile::current()->in_preserve_stack_slots() +
                        Compile::current()->fixed_slots()),
                       stack_alignment_in_slots())); }

uint Compile::varargs_C_out_slots_killed() const { return 0; }

OptoRegPair Matcher::return_value(uint ideal_reg) {

#line 3171 "src/hotspot/cpu/x86/x86_32.ad"

    assert( ideal_reg >= Op_RegI && ideal_reg <= Op_RegL, "only return normal values" );
    static int lo[Op_RegL+1] = { 0, 0, OptoReg::Bad, EAX_num,      EAX_num,      FPR1L_num,    FPR1L_num, EAX_num };
    static int hi[Op_RegL+1] = { 0, 0, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, FPR1H_num, EDX_num };
    if( ideal_reg == Op_RegD && UseSSE>=2 )
      return OptoRegPair(XMM0b_num,XMM0_num);
    if( ideal_reg == Op_RegF && UseSSE>=1 )
      return OptoRegPair(OptoReg::Bad,XMM0_num);
    return OptoRegPair(hi[ideal_reg],lo[ideal_reg]);
  
#line 42577 "ad_x86.cpp"

}

OptoRegPair Matcher::c_return_value(uint ideal_reg) {

#line 3155 "src/hotspot/cpu/x86/x86_32.ad"

    assert( ideal_reg >= Op_RegI && ideal_reg <= Op_RegL, "only return normal values" );
    static int lo[Op_RegL+1] = { 0, 0, OptoReg::Bad, EAX_num,      EAX_num,      FPR1L_num,    FPR1L_num, EAX_num };
    static int hi[Op_RegL+1] = { 0, 0, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, FPR1H_num, EDX_num };

    // in SSE2+ mode we want to keep the FPU stack clean so pretend
    // that C functions return float and double results in XMM0.
    if( ideal_reg == Op_RegD && UseSSE>=2 )
      return OptoRegPair(XMM0b_num,XMM0_num);
    if( ideal_reg == Op_RegF && UseSSE>=2 )
      return OptoRegPair(OptoReg::Bad,XMM0_num);

    return OptoRegPair(hi[ideal_reg],lo[ideal_reg]);
  
#line 42598 "ad_x86.cpp"

}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(EAX_num); }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(EBP_num); }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(ESP_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

bool Compile::needs_clone_jvms() { return false; }

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DIA32=1
#ifndef IA32
#  error "IA32 must be defined"
#endif // IA32
// Check adlc -U_LP64
#ifdef _LP64
#  error "_LP64 must not be defined"
#endif // _LP64
